@@ -35,6 +35,10 @@
     } \
 } while(0)
 
+/* Warning messages */
+#define SAME_DOMAINS_ERROR_MSG "Domain '%s' is the same as or differs only "\
+                               "in case from domain '%s'.\n"
+
 static char *prepend_cn(char *str, int *slen, const char *comp, int clen)
 {
     char *ret;
@@ -196,6 +200,9 @@ int confdb_add_param(struct confdb_ctx *cdb,
 
         ret = ldb_modify(cdb->ldb, msg);
         if (ret != LDB_SUCCESS) {
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  "ldb_modify failed: [%s](%d)[%s]\n",
+                  ldb_strerror(ret), ret, ldb_errstring(cdb->ldb));
             ret = EIO;
             goto done;
         }
@@ -290,10 +297,10 @@ done:
     return ret;
 }
 
-int confdb_set_bool(struct confdb_ctx *cdb,
-                     const char *section,
-                     const char *attribute,
-                     bool val)
+int confdb_set_string(struct confdb_ctx *cdb,
+                      const char *section,
+                      const char *attribute,
+                      const char *val)
 {
     TALLOC_CTX *tmp_ctx;
     struct ldb_dn *dn;
@@ -302,8 +309,9 @@ int confdb_set_bool(struct confdb_ctx *cdb,
     int ret, lret;
 
     tmp_ctx = talloc_new(NULL);
-    if (!tmp_ctx)
+    if (!tmp_ctx) {
         return ENOMEM;
+    }
 
     ret = parse_section(tmp_ctx, section, &secdn, NULL);
     if (ret != EOK) {
@@ -332,11 +340,7 @@ int confdb_set_bool(struct confdb_ctx *cdb,
         goto done;
     }
 
-    if (val) {
-        lret = ldb_msg_add_string(msg, attribute, "True");
-    } else {
-        lret = ldb_msg_add_string(msg, attribute, "False");
-    }
+    lret = ldb_msg_add_string(msg, attribute, val);
     if (lret != LDB_SUCCESS) {
         DEBUG(SSSDBG_MINOR_FAILURE,
               "ldb_msg_add_string failed: [%s]\n", ldb_strerror(lret));
@@ -344,11 +348,11 @@ int confdb_set_bool(struct confdb_ctx *cdb,
         goto done;
     }
 
-
     lret = ldb_modify(cdb->ldb, msg);
     if (lret != LDB_SUCCESS) {
         DEBUG(SSSDBG_MINOR_FAILURE,
-              "ldb_modify failed: [%s]\n", ldb_strerror(lret));
+              "ldb_modify failed: [%s](%d)[%s]\n",
+              ldb_strerror(lret), lret, ldb_errstring(cdb->ldb));
         ret = EIO;
         goto done;
     }
@@ -1036,6 +1040,17 @@ static int confdb_get_domain_internal(struct confdb_ctx *cdb,
         goto done;
     }
 
+    /* Override the ssh known hosts timeout, if specified */
+    ret = get_entry_as_uint32(res->msgs[0], &domain->ssh_host_timeout,
+                              CONFDB_DOMAIN_SSH_HOST_CACHE_TIMEOUT,
+                              entry_cache_timeout);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              "Invalid value for [%s]\n",
+              CONFDB_DOMAIN_SSH_HOST_CACHE_TIMEOUT);
+        goto done;
+    }
+
     /* Set refresh_expired_interval, if specified */
     ret = get_entry_as_uint32(res->msgs[0], &domain->refresh_expired_interval,
                               CONFDB_DOMAIN_REFRESH_EXPIRED_INTERVAL,
@@ -1047,6 +1062,21 @@ static int confdb_get_domain_internal(struct confdb_ctx *cdb,
         goto done;
     }
 
+    /* detect and fix misconfiguration */
+    if (domain->refresh_expired_interval > entry_cache_timeout) {
+        DEBUG(SSSDBG_CONF_SETTINGS,
+              "refresh_expired_interval (%d) cannot be greater than "
+              "entry_cache_timeout (%u)\n",
+              domain->refresh_expired_interval, entry_cache_timeout);
+
+        domain->refresh_expired_interval = 0.75 * entry_cache_timeout;
+
+        DEBUG(SSSDBG_CONF_SETTINGS,
+              "refresh_expired_interval is being set to recommended value "
+              "entry_cache_timeout * 0.75 (%u).\n",
+              domain->refresh_expired_interval);
+    }
+
     /* Set the PAM warning time, if specified. If not specified, pass on
      * the "not set" value of "-1" which means "use provider default". The
      * value 0 means "always display the warning if server sends one" */
@@ -1143,12 +1173,27 @@ static int confdb_get_domain_internal(struct confdb_ctx *cdb,
         }
     }
 
-    ret = get_entry_as_bool(res->msgs[0], &domain->case_sensitive,
-                            CONFDB_DOMAIN_CASE_SENSITIVE, true);
-    if(ret != EOK) {
-        DEBUG(SSSDBG_FATAL_FAILURE,
-              "Invalid value for %s\n", CONFDB_DOMAIN_CASE_SENSITIVE);
-        goto done;
+    tmp = ldb_msg_find_attr_as_string(res->msgs[0],
+                                      CONFDB_DOMAIN_CASE_SENSITIVE, "true");
+    if (tmp != NULL) {
+        if (strcasecmp(tmp, "true") == 0) {
+            domain->case_sensitive = true;
+            domain->case_preserve = true;
+        } else if (strcasecmp(tmp, "false") == 0) {
+            domain->case_sensitive = false;
+            domain->case_preserve = false;
+        } else if (strcasecmp(tmp, "preserving") == 0) {
+            domain->case_sensitive = false;
+            domain->case_preserve = true;
+        } else {
+            DEBUG(SSSDBG_FATAL_FAILURE,
+                  "Invalid value for %s\n", CONFDB_DOMAIN_CASE_SENSITIVE);
+            goto done;
+        }
+    } else {
+        /* default */
+        domain->case_sensitive = true;
+        domain->case_preserve = true;
     }
     if (domain->case_sensitive == false &&
         strcasecmp(domain->provider, "local") == 0) {
@@ -1171,6 +1216,19 @@ static int confdb_get_domain_internal(struct confdb_ctx *cdb,
         }
     }
 
+    tmp = ldb_msg_find_attr_as_string(res->msgs[0],
+                                      CONFDB_DOMAIN_SUBDOMAIN_INHERIT,
+                                      NULL);
+    if (tmp != NULL) {
+        ret = split_on_separator(domain, tmp, ',', true, true,
+                                 &domain->sd_inherit, NULL);
+        if (ret != 0) {
+            DEBUG(SSSDBG_FATAL_FAILURE,
+                  "Cannot parse %s\n", CONFDB_SUBDOMAIN_ENUMERATE);
+            goto done;
+        }
+    }
+
     ret = get_entry_as_uint32(res->msgs[0], &domain->subdomain_refresh_interval,
                               CONFDB_DOMAIN_SUBDOMAIN_REFRESH, 14400);
     if (ret != EOK || domain->subdomain_refresh_interval == 0) {
@@ -1179,6 +1237,9 @@ static int confdb_get_domain_internal(struct confdb_ctx *cdb,
         goto done;
     }
 
+    domain->has_views = false;
+    domain->view_name = NULL;
+
     *_domain = domain;
     ret = EOK;
 done:
@@ -1216,6 +1277,20 @@ int confdb_get_domains(struct confdb_ctx *cdb,
     }
 
     for (i = 0; domlist[i]; i++) {
+        /* check if domain name is really unique */
+        DLIST_FOR_EACH(domain, cdb->doms) {
+            if (strcasecmp(domain->name, domlist[i]) == 0) {
+                DEBUG(SSSDBG_FATAL_FAILURE,
+                      SAME_DOMAINS_ERROR_MSG, domlist[i], domain->name);
+                sss_log(SSS_LOG_CRIT,
+                        SAME_DOMAINS_ERROR_MSG, domlist[i], domain->name);
+
+                ret = EINVAL;
+                goto done;
+            }
+        }
+
+        domain = NULL;
         ret = confdb_get_domain_internal(cdb, cdb, domlist[i], &domain);
         if (ret) {
             DEBUG(SSSDBG_FATAL_FAILURE,
@@ -1263,3 +1338,67 @@ int confdb_get_domain(struct confdb_ctx *cdb,
 
     return ENOENT;
 }
+
+int confdb_list_all_domain_names(TALLOC_CTX *mem_ctx,
+                                 struct confdb_ctx *cdb,
+                                 char ***_names)
+{
+    TALLOC_CTX *tmp_ctx = NULL;
+    struct ldb_dn *dn = NULL;
+    struct ldb_result *res = NULL;
+    static const char *attrs[] = {CONFDB_DOMAIN_ATTR, NULL};
+    const char *name = NULL;
+    char **names = NULL;
+    int i;
+    int ret;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        return ENOMEM;
+    }
+
+    dn = ldb_dn_new(tmp_ctx, cdb->ldb, CONFDB_DOMAIN_BASEDN);
+    if (dn == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = ldb_search(cdb->ldb, tmp_ctx, &res, dn, LDB_SCOPE_ONELEVEL,
+                     attrs, NULL);
+    if (ret != LDB_SUCCESS) {
+        ret = EIO;
+        goto done;
+    }
+
+    names = talloc_zero_array(tmp_ctx, char*, res->count + 1);
+    if (names == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    for (i = 0; i < res->count; i++) {
+        name = ldb_msg_find_attr_as_string(res->msgs[i], CONFDB_DOMAIN_ATTR,
+                                           NULL);
+        if (name == NULL) {
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  "The object [%s] doesn't have a name\n",
+                   ldb_dn_get_linearized(res->msgs[i]->dn));
+            ret = EINVAL;
+            goto done;
+        }
+
+        names[i] = talloc_strdup(names, name);
+        if (names[i] == NULL) {
+            ret = ENOMEM;
+            goto done;
+        }
+    }
+
+    *_names = talloc_steal(mem_ctx, names);
+
+    ret = EOK;
+
+done:
+    talloc_free(tmp_ctx);
+    return ret;
+}
