@@ -6,8 +6,9 @@ Created on Sep 18, 2009
 
 import os
 import gettext
-import exceptions
-from ipachangeconf import SSSDChangeConf
+import re
+import sys
+from .ipachangeconf import SSSDChangeConf
 
 # Exceptions
 class SSSDConfigException(Exception): pass
@@ -31,7 +32,10 @@ PACKAGE = 'sss_daemon'
 LOCALEDIR = '/usr/share/locale'
 
 translation = gettext.translation(PACKAGE, LOCALEDIR, fallback=True)
-_ = translation.ugettext
+if sys.version_info[0] > 2:
+    _ = translation.gettext
+else:
+    _ = translation.ugettext
 
 # TODO: This needs to be made external
 option_strings = {
@@ -55,6 +59,7 @@ option_strings = {
     'full_name_format' : _('Printf-compatible format for displaying fully-qualified names'),
     'krb5_rcache_dir' : _('Directory on the filesystem where SSSD should store Kerberos replay cache files.'),
     'default_domain_suffix' : _('Domain to add to names without a domain component.'),
+    'user' : _('The user to drop privileges to'),
 
     # [nss]
     'enum_cache_timeout' : _('Enumeration cache timeout length (seconds)'),
@@ -81,10 +86,12 @@ option_strings = {
     'pam_verbosity' : _('What kind of messages are displayed to the user during authentication'),
     'pam_id_timeout' : _('How many seconds to keep identity information cached for PAM requests'),
     'pam_pwd_expiration_warning' : _('How many days before password expiration a warning should be displayed'),
+    'pam_trusted_users' : _('List of trusted uids or user\'s name'),
+    'pam_public_domains' : _('List of domains accessible even for untrusted users.'),
+    'pam_account_expired_message' : _('Message printed when user account is expired.'),
 
     # [sudo]
     'sudo_timed' : _('Whether to evaluate the time-based attributes in sudo rules'),
-    'sudo_inverse_order' : _('If true, SSSD will switch back to lower-wins ordering logic'),
 
     # [autofs]
     'autofs_negative_timeout' : _('Negative cache timeout length (seconds)'),
@@ -141,6 +148,7 @@ option_strings = {
     'dyndns_auth' : _("What kind of authentication should be used to perform the DNS update"),
     'subdomain_enumerate' : _('Control enumeration of trusted domains'),
     'subdomain_refresh_interval' : _('How often should subdomains list be refreshed'),
+    'subdomain_inherit' : _('List of options that should be inherited into a subdomain'),
 
     # [provider/ipa]
     'ipa_domain' : _('IPA domain'),
@@ -159,6 +167,13 @@ option_strings = {
     'ipa_master_domain_search_base': _("Search base for object containing info about IPA domain"),
     'ipa_ranges_search_base': _("Search base for objects containing info about ID ranges"),
     'ipa_enable_dns_sites': _("Enable DNS sites - location based service discovery"),
+    'ipa_views_search_base': _("Search base for view containers"),
+    'ipa_view_class': _("Objectclass for view containers"),
+    'ipa_view_name': _("Attribute with the name of the view"),
+    'ipa_overide_object_class': _("Objectclass for override objects"),
+    'ipa_anchor_uuid': _("Attribute with the reference to the original object"),
+    'ipa_user_override_object_class': _("Objectclass for user override objects"),
+    'ipa_group_override_object_class': _("Objectclass for group override objects"),
 
     # [provider/ad]
     'ad_domain' : _('Active Directory domain'),
@@ -166,6 +181,19 @@ option_strings = {
     'ad_backup_server' : _('Active Directory backup server address'),
     'ad_hostname' : _('Active Directory client hostname'),
     'ad_enable_dns_sites' : _('Enable DNS sites - location based service discovery'),
+    'ad_access_filter' : _('LDAP filter to determine access privileges'),
+    'ad_enable_gc' : _('Whether to use the Global Catalog for lookups'),
+    'ad_gpo_access_control' : _('Operation mode for GPO-based access control'),
+    'ad_gpo_cache_timeout' : _("The amount of time between lookups of the GPO policy files against the AD server"),
+    'ad_gpo_map_interactive' : _('PAM service names that map to the GPO (Deny)InteractiveLogonRight policy settings'),
+    'ad_gpo_map_remote_interactive' : _('PAM service names that map to the GPO (Deny)RemoteInteractiveLogonRight policy settings'),
+    'ad_gpo_map_network' : _('PAM service names that map to the GPO (Deny)NetworkLogonRight policy settings'),
+    'ad_gpo_map_batch' : _('PAM service names that map to the GPO (Deny)BatchLogonRight policy settings'),
+    'ad_gpo_map_service' : _('PAM service names that map to the GPO (Deny)ServiceLogonRight policy settings'),
+    'ad_gpo_map_permit' : _('PAM service names for which GPO-based access is always granted'),
+    'ad_gpo_map_deny' : _('PAM service names for which GPO-based access is always denied'),
+    'ad_gpo_default_right' : _('Default logon right (or permit/deny) to use for unmapped PAM service names'),
+    'ad_site' : _('a particular site to be used by the client'),
 
     # [provider/krb5]
     'krb5_kdcip' : _('Kerberos server address'),
@@ -174,6 +202,7 @@ option_strings = {
     'krb5_realm' : _('Kerberos realm'),
     'krb5_auth_timeout' : _('Authentication timeout'),
     'krb5_use_kdcinfo' : _('Whether to create kdcinfo files'),
+    'krb5_confd_path' : _('Where to drop krb5 config snippets'),
 
     # [provider/krb5/auth]
     'krb5_ccachedir' : _('Directory to store credential caches'),
@@ -188,6 +217,7 @@ option_strings = {
     'krb5_fast_principal' : _("Selects the principal to use for FAST"),
     'krb5_canonicalize' : _("Enables principal canonicalization"),
     'krb5_use_enterprise_principal' : _("Enables enterprise principals"),
+    'krb5_map_user' : _('A mapping from user names to kerberos principal names'),
 
     # [provider/krb5/chpass]
     'krb5_kpasswd' : _('Server where the change password service is running if not on the KDC'),
@@ -251,6 +281,7 @@ option_strings = {
     'ldap_user_gecos' : _('GECOS attribute'),
     'ldap_user_home_directory' : _('Home directory attribute'),
     'ldap_user_shell' : _('Shell attribute'),
+    'ldap_user_uuid' : _('UUID attribute'),
     'ldap_user_objectsid' : _("objectSID attribute"),
     'ldap_user_primary_group' : _('Active Directory primary group attribute for ID-mapping'),
     'ldap_user_principal' : _('User principal attribute (for Kerberos)'),
@@ -287,6 +318,7 @@ option_strings = {
     'ldap_group_pwd' : _('Group password'),
     'ldap_group_gid_number' : _('GID attribute'),
     'ldap_group_member' : _('Group member attribute'),
+    'ldap_group_uuid' : _('Group UUID attribute'),
     'ldap_group_objectsid' : _("objectSID attribute"),
     'ldap_group_modify_timestamp' : _('Modification time attribute for groups'),
     'ldap_group_type' : _('Type of the group and other flags'),
@@ -406,7 +438,8 @@ class SSSDConfigSchema(SSSDChangeConf):
             self.readfp(fd)
             fd.close()
             # Read in the provider files
-            for file in os.listdir(schemaplugindir):
+            for file in filter(lambda f: re.search('^sssd-.*\.conf$', f),
+                                         os.listdir(schemaplugindir)):
                 fd = open(schemaplugindir+ "/" + file)
                 self.readfp(fd)
                 fd.close()
@@ -419,7 +452,7 @@ class SSSDConfigSchema(SSSDChangeConf):
         self.type_lookup = {
             'bool' : bool,
             'int'  : int,
-            'long' : long,
+            'long' : long if sys.version_info[0] == 2 else int,
             'float': float,
             'str'  : str,
             'list' : list,
@@ -454,7 +487,7 @@ class SSSDConfigSchema(SSSDChangeConf):
             subtype = self.type_lookup[split_option[SUBTYPE]]
             mandatory = self.bool_lookup[split_option[MANDATORY]]
 
-            if option_strings.has_key(option['name']):
+            if option['name'] in option_strings:
                 desc = option_strings[option['name']]
             else:
                 desc = None
@@ -500,7 +533,7 @@ class SSSDConfigSchema(SSSDChangeConf):
                                      mandatory,
                                      desc,
                                      [subtype(split_option[DEFAULT])])
-                        except ValueError, KeyError:
+                        except ValueError:
                             raise ParsingError
                 else:
                     try:
@@ -519,7 +552,7 @@ class SSSDConfigSchema(SSSDChangeConf):
                                  mandatory,
                                  desc,
                                  primarytype(split_option[DEFAULT]))
-                    except ValueError, KeyError:
+                    except ValueError:
                         raise ParsingError
 
             elif optionlen > 4:
@@ -534,7 +567,7 @@ class SSSDConfigSchema(SSSDChangeConf):
                             else:
                                 newvalue = subtype(x)
                             fixed_options.extend([newvalue])
-                        except ValueError, KeyError:
+                        except ValueError:
                             raise ParsingError
                     else:
                         fixed_options.extend([x])
@@ -583,7 +616,7 @@ class SSSDConfigSchema(SSSDChangeConf):
             splitsection = section['name'].split('/')
             if (splitsection[0] == 'provider'):
                 if(len(splitsection) == 3):
-                    if not providers.has_key(splitsection[1]):
+                    if splitsection[1] not in providers:
                         providers[splitsection[1]] = []
                     providers[splitsection[1]].extend([splitsection[2]])
         for key in providers.keys():
@@ -647,7 +680,7 @@ class SSSDConfigObject(object):
         === Errors ===
         No errors
         """
-        if self.options.has_key(optionname):
+        if optionname in self.options:
             del self.options[optionname]
 
 class SSSDService(SSSDConfigObject):
@@ -1282,12 +1315,12 @@ class SSSDDomain(SSSDConfigObject):
         # We should now have a list of options used only by this
         # provider. So we remove them.
         for option in options:
-            if self.options.has_key(option):
+            if option in self.options:
                 del self.options[option]
 
         # Remove this provider from the option list
         option = '%s_provider' % provider_type
-        if self.options.has_key(option):
+        if option in self.options:
             del self.options[option]
 
         self.providers.remove((provider, provider_type))
@@ -1425,9 +1458,9 @@ class SSSDConfig(SSSDChangeConf):
             outputfile = self.configfile
 
         # open() will raise IOError if it fails
-        old_umask = os.umask(0177)
+        old_umask = os.umask(0o177)
         of = open(outputfile, "wb")
-        output = self.dump(self.opts)
+        output = self.dump(self.opts).encode('utf-8')
         of.write(output)
         of.close()
         os.umask(old_umask)
@@ -1450,7 +1483,7 @@ class SSSDConfig(SSSDChangeConf):
         if (self.has_option('sssd', 'services')):
             active_services = striplist(self.get('sssd', 'services').split(','))
             service_dict = dict.fromkeys(active_services)
-            if service_dict.has_key(''):
+            if '' in service_dict:
                 del service_dict['']
 
             # Remove any entries in this list that don't
@@ -1606,7 +1639,7 @@ class SSSDConfig(SSSDChangeConf):
         # This guarantees uniqueness and makes it easy
         # to add a new value
         service_dict = dict.fromkeys(striplist(item['value'].split(',')))
-        if service_dict.has_key(''):
+        if '' in service_dict:
             del service_dict['']
 
         # Add a new key for the service being activated
@@ -1647,11 +1680,11 @@ class SSSDConfig(SSSDChangeConf):
         # This guarantees uniqueness and makes it easy
         # to remove the one unwanted value.
         service_dict = dict.fromkeys(striplist(item['value'].split(',')))
-        if service_dict.has_key(''):
+        if '' in service_dict:
             del service_dict['']
 
         # Remove the unwanted service from the lest
-        if service_dict.has_key(name):
+        if name in service_dict:
             del service_dict[name]
 
         # Write out the joined keys
@@ -1733,7 +1766,7 @@ class SSSDConfig(SSSDChangeConf):
         if (self.has_option('sssd', 'domains')):
             active_domains = striplist(self.get('sssd', 'domains').split(','))
             domain_dict = dict.fromkeys(active_domains)
-            if domain_dict.has_key(''):
+            if '' in domain_dict:
                 del domain_dict['']
 
             # Remove any entries in this list that don't
@@ -1928,7 +1961,7 @@ class SSSDConfig(SSSDChangeConf):
         # This guarantees uniqueness and makes it easy
         # to add a new value
         domain_dict = dict.fromkeys(striplist(item['value'].split(',')))
-        if domain_dict.has_key(''):
+        if '' in domain_dict:
             del domain_dict['']
 
         # Add a new key for the domain being activated
@@ -1969,11 +2002,11 @@ class SSSDConfig(SSSDChangeConf):
         # This guarantees uniqueness and makes it easy
         # to remove the one unwanted value.
         domain_dict = dict.fromkeys(striplist(item['value'].split(',')))
-        if domain_dict.has_key(''):
+        if '' in domain_dict:
             del domain_dict['']
 
         # Remove the unwanted domain from the lest
-        if domain_dict.has_key(name):
+        if name in domain_dict:
             del domain_dict[name]
 
         # Write out the joined keys
