@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 '''
 Created on Sep 18, 2009
 
@@ -12,7 +12,7 @@ import sys
 
 srcdir = os.getenv('srcdir')
 if srcdir:
-    sys.path.append("./src/config")
+    sys.path.insert(0, "./src/config")
     srcdir = srcdir + "/src/config"
 else:
     srcdir = "."
@@ -148,7 +148,7 @@ class SSSDConfigTestValid(unittest.TestCase):
 
         #Output files should not be readable or writable by
         #non-owners, and should not be executable by anyone
-        self.assertFalse(S_IMODE(mode) & 0177)
+        self.assertFalse(S_IMODE(mode) & 0o177)
 
         #Remove the output file
         os.unlink(of)
@@ -182,7 +182,7 @@ class SSSDConfigTestValid(unittest.TestCase):
 
         #Output files should not be readable or writable by
         #non-owners, and should not be executable by anyone
-        self.assertFalse(S_IMODE(mode) & 0177)
+        self.assertFalse(S_IMODE(mode) & 0o177)
 
         #Remove the output file
         os.unlink(of)
@@ -216,7 +216,7 @@ class SSSDConfigTestValid(unittest.TestCase):
 
         #Output files should not be readable or writable by
         #non-owners, and should not be executable by anyone
-        self.assertFalse(S_IMODE(mode) & 0177)
+        self.assertFalse(S_IMODE(mode) & 0o177)
 
         #Remove the output file
         os.unlink(of)
@@ -280,6 +280,7 @@ class SSSDConfigTestSSSDService(unittest.TestCase):
             're_expression',
             'full_name_format',
             'krb5_rcache_dir',
+            'user',
             'default_domain_suffix',
             'debug_level',
             'debug_timestamps',
@@ -488,6 +489,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'max_id',
             'timeout',
             'force_timeout',
+            'offline_timeout',
             'try_inotify',
             'command',
             'enumerate',
@@ -504,6 +506,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'entry_cache_service_timeout',
             'entry_cache_autofs_timeout',
             'entry_cache_sudo_timeout',
+            'entry_cache_ssh_host_timeout',
             'refresh_expired_interval',
             'lookup_family_order',
             'account_cache_expiration',
@@ -535,7 +538,8 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'hostid_provider',
             'subdomains_provider',
             'realmd_tags',
-            'subdomain_refresh_interval']
+            'subdomain_refresh_interval',
+            'subdomain_inherit']
 
         self.assertTrue(type(options) == dict,
                         "Options should be a dictionary")
@@ -618,7 +622,8 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
              'krb5_fast_principal',
              'krb5_canonicalize',
              'krb5_use_enterprise_principal',
-             'krb5_use_kdcinfo'])
+             'krb5_use_kdcinfo',
+             'krb5_map_user'])
 
         options = domain.list_options()
 
@@ -745,12 +750,12 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
         # Ensure that all of the expected defaults are there
         for provider in control_provider_dict.keys():
             for ptype in control_provider_dict[provider]:
-                self.assertTrue(providers.has_key(provider))
+                self.assertTrue(provider in providers)
                 self.assertTrue(ptype in providers[provider])
 
         for provider in providers.keys():
             for ptype in providers[provider]:
-                self.assertTrue(control_provider_dict.has_key(provider))
+                self.assertTrue(provider in control_provider_dict)
                 self.assertTrue(ptype in control_provider_dict[provider])
 
     def testListProviderOptions(self):
@@ -778,7 +783,8 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'krb5_fast_principal',
             'krb5_canonicalize',
             'krb5_use_enterprise_principal',
-            'krb5_use_kdcinfo']
+            'krb5_use_kdcinfo',
+            'krb5_map_user']
 
         self.assertTrue(type(options) == dict,
                         "Options should be a dictionary")
@@ -845,6 +851,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'max_id',
             'timeout',
             'force_timeout',
+            'offline_timeout',
             'try_inotify',
             'command',
             'enumerate',
@@ -861,6 +868,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'entry_cache_service_timeout',
             'entry_cache_autofs_timeout',
             'entry_cache_sudo_timeout',
+            'entry_cache_ssh_host_timeout',
             'refresh_expired_interval',
             'account_cache_expiration',
             'lookup_family_order',
@@ -892,7 +900,8 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'hostid_provider',
             'subdomains_provider',
             'realmd_tags',
-            'subdomain_refresh_interval']
+            'subdomain_refresh_interval',
+            'subdomain_inherit']
 
         self.assertTrue(type(options) == dict,
                         "Options should be a dictionary")
@@ -976,7 +985,8 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
              'krb5_fast_principal',
              'krb5_canonicalize',
              'krb5_use_enterprise_principal',
-             'krb5_use_kdcinfo'])
+             'krb5_use_kdcinfo',
+             'krb5_map_user'])
 
         options = domain.list_options()
 
@@ -998,7 +1008,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
         # Remove the local ID provider and add an LDAP one
         # LDAP ID providers can also use the krb5_realm
         domain.remove_provider('id')
-        self.assertFalse(domain.options.has_key('id_provider'))
+        self.assertFalse('id_provider' in domain.options)
 
         domain.add_provider('ldap', 'id')
 
@@ -1015,7 +1025,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
         domain.remove_provider('id')
         self.assertEquals(domain.get_option('krb5_realm'),
                   'EXAMPLE.COM')
-        self.assertFalse(domain.options.has_key('ldap_uri'))
+        self.assertFalse('ldap_uri' in domain.options)
 
         # Put the LOCAL provider back
         domain.add_provider('local', 'id')
@@ -1023,7 +1033,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
         # Remove the auth domain and verify that the options
         # revert to the backup_list
         domain.remove_provider('auth')
-        self.assertFalse(domain.options.has_key('auth_provider'))
+        self.assertFalse('auth_provider' in domain.options)
         options = domain.list_options()
 
         self.assertTrue(type(options) == dict,
@@ -1042,21 +1052,21 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
                             option)
 
         # Ensure that the krb5_realm option is now gone
-        self.assertFalse(domain.options.has_key('krb5_realm'))
+        self.assertFalse('krb5_realm' in domain.options)
 
         # Test removing nonexistent provider - Real
         domain.remove_provider('id')
-        self.assertFalse(domain.options.has_key('id_provider'))
+        self.assertFalse('id_provider' in domain.options)
 
         # Test removing nonexistent provider - Bad backend type
         # Should pass without complaint
         domain.remove_provider('id')
-        self.assertFalse(domain.options.has_key('id_provider'))
+        self.assertFalse('id_provider' in domain.options)
 
         # Test removing nonexistent provider - Bad provider type
         # Should pass without complaint
         domain.remove_provider('nosuchprovider')
-        self.assertFalse(domain.options.has_key('nosuchprovider_provider'))
+        self.assertFalse('nosuchprovider_provider' in domain.options)
 
     def testGetOption(self):
         domain = SSSDConfig.SSSDDomain('sssd', self.schema)
@@ -1362,7 +1372,7 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
         # Positive test - Service with invalid option loads
         # but ignores the invalid option
         service = sssdconfig.get_service('pam')
-        self.assertFalse(service.options.has_key('nosuchoption'))
+        self.assertFalse('nosuchoption' in service.options)
 
     def testNewService(self):
         sssdconfig = SSSDConfig.SSSDConfig(srcdir + "/etc/sssd.api.conf",
@@ -1593,13 +1603,13 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
         # Expected result: Domain is imported, but does not contain the
         # unknown provider entry
         domain = sssdconfig.get_domain('INVALIDPROVIDER')
-        self.assertFalse(domain.options.has_key('chpass_provider'))
+        self.assertFalse('chpass_provider' in domain.options)
 
         # Positive Test - Domain with unknown option
         # Expected result: Domain is imported, but does not contain the
         # unknown option entry
         domain = sssdconfig.get_domain('INVALIDOPTION')
-        self.assertFalse(domain.options.has_key('nosuchoption'))
+        self.assertFalse('nosuchoption' in domain.options)
 
     def testNewDomain(self):
         sssdconfig = SSSDConfig.SSSDConfig(srcdir + "/etc/sssd.api.conf",
@@ -1762,7 +1772,7 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
 
         #Output files should not be readable or writable by
         #non-owners, and should not be executable by anyone
-        self.assertFalse(S_IMODE(mode) & 0177)
+        self.assertFalse(S_IMODE(mode) & 0o177)
 
         #Remove the output file
         os.unlink(of)
