@@ -91,8 +91,7 @@ errno_t sysdb_dn_sanitize(TALLOC_CTX *mem_ctx, const char *input,
     return ret;
 }
 
-struct ldb_dn *sysdb_custom_subtree_dn(struct sysdb_ctx *sysdb,
-                                       TALLOC_CTX *mem_ctx,
+struct ldb_dn *sysdb_custom_subtree_dn(TALLOC_CTX *mem_ctx,
                                        struct sss_domain_info *dom,
                                        const char *subtree_name)
 {
@@ -110,7 +109,7 @@ struct ldb_dn *sysdb_custom_subtree_dn(struct sysdb_ctx *sysdb,
         return NULL;
     }
 
-    dn = ldb_dn_new_fmt(tmp_ctx, sysdb->ldb, SYSDB_TMPL_CUSTOM_SUBTREE,
+    dn = ldb_dn_new_fmt(tmp_ctx, dom->sysdb->ldb, SYSDB_TMPL_CUSTOM_SUBTREE,
                         clean_subtree, dom->name);
     if (dn) {
         talloc_steal(mem_ctx, dn);
@@ -120,7 +119,7 @@ struct ldb_dn *sysdb_custom_subtree_dn(struct sysdb_ctx *sysdb,
     return dn;
 }
 
-struct ldb_dn *sysdb_custom_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
+struct ldb_dn *sysdb_custom_dn(TALLOC_CTX *mem_ctx,
                                struct sss_domain_info *dom,
                                const char *object_name,
                                const char *subtree_name)
@@ -146,7 +145,7 @@ struct ldb_dn *sysdb_custom_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    dn = ldb_dn_new_fmt(mem_ctx, sysdb->ldb, SYSDB_TMPL_CUSTOM, clean_name,
+    dn = ldb_dn_new_fmt(mem_ctx, dom->sysdb->ldb, SYSDB_TMPL_CUSTOM, clean_name,
                         clean_subtree, dom->name);
 
 done:
@@ -154,8 +153,8 @@ done:
     return dn;
 }
 
-struct ldb_dn *sysdb_user_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
-                             struct sss_domain_info *dom, const char *name)
+struct ldb_dn *sysdb_user_dn(TALLOC_CTX *mem_ctx, struct sss_domain_info *dom,
+                             const char *name)
 {
     errno_t ret;
     char *clean_name;
@@ -166,14 +165,21 @@ struct ldb_dn *sysdb_user_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
         return NULL;
     }
 
-    dn = ldb_dn_new_fmt(mem_ctx, sysdb->ldb, SYSDB_TMPL_USER,
+    dn = ldb_dn_new_fmt(mem_ctx, dom->sysdb->ldb, SYSDB_TMPL_USER,
                         clean_name, dom->name);
     talloc_free(clean_name);
 
     return dn;
 }
 
-struct ldb_dn *sysdb_group_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
+struct ldb_dn *sysdb_user_base_dn(TALLOC_CTX *mem_ctx,
+                                  struct sss_domain_info *dom)
+{
+    return ldb_dn_new_fmt(mem_ctx, dom->sysdb->ldb,
+                          SYSDB_TMPL_USER_BASE, dom->name);
+}
+
+struct ldb_dn *sysdb_group_dn(TALLOC_CTX *mem_ctx,
                               struct sss_domain_info *dom, const char *name)
 {
     errno_t ret;
@@ -185,14 +191,22 @@ struct ldb_dn *sysdb_group_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
         return NULL;
     }
 
-    dn = ldb_dn_new_fmt(mem_ctx, sysdb->ldb, SYSDB_TMPL_GROUP,
+    dn = ldb_dn_new_fmt(mem_ctx, dom->sysdb->ldb, SYSDB_TMPL_GROUP,
                         clean_name, dom->name);
     talloc_free(clean_name);
 
     return dn;
 }
 
-struct ldb_dn *sysdb_netgroup_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
+struct ldb_dn *sysdb_group_base_dn(TALLOC_CTX *mem_ctx,
+                                   struct sss_domain_info *dom)
+{
+    return ldb_dn_new_fmt(mem_ctx, dom->sysdb->ldb,
+                          SYSDB_TMPL_GROUP_BASE, dom->name);
+}
+
+
+struct ldb_dn *sysdb_netgroup_dn(TALLOC_CTX *mem_ctx,
                                  struct sss_domain_info *dom, const char *name)
 {
     errno_t ret;
@@ -204,26 +218,25 @@ struct ldb_dn *sysdb_netgroup_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
         return NULL;
     }
 
-    dn = ldb_dn_new_fmt(mem_ctx, sysdb->ldb, SYSDB_TMPL_NETGROUP,
+    dn = ldb_dn_new_fmt(mem_ctx, dom->sysdb->ldb, SYSDB_TMPL_NETGROUP,
                         clean_name, dom->name);
     talloc_free(clean_name);
 
     return dn;
 }
 
-struct ldb_dn *sysdb_netgroup_base_dn(struct sysdb_ctx *sysdb,
-                                      TALLOC_CTX *mem_ctx,
+struct ldb_dn *sysdb_netgroup_base_dn(TALLOC_CTX *mem_ctx,
                                       struct sss_domain_info *dom)
 {
-    return ldb_dn_new_fmt(mem_ctx, sysdb->ldb,
+    return ldb_dn_new_fmt(mem_ctx, dom->sysdb->ldb,
                           SYSDB_TMPL_NETGROUP_BASE, dom->name);
 }
 
 errno_t sysdb_get_rdn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
-                      const char *_dn, char **_name, char **_val)
+                      const char *dn, char **_name, char **_val)
 {
     errno_t ret;
-    struct ldb_dn *dn;
+    struct ldb_dn *ldb_dn;
     const char *attr_name = NULL;
     const struct ldb_val *val;
     TALLOC_CTX *tmp_ctx;
@@ -236,14 +249,14 @@ errno_t sysdb_get_rdn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
-    dn = ldb_dn_new_fmt(tmp_ctx, sysdb->ldb, "%s", _dn);
-    if (dn == NULL) {
+    ldb_dn = ldb_dn_new_fmt(tmp_ctx, sysdb->ldb, "%s", dn);
+    if (ldb_dn == NULL) {
         ret = ENOMEM;
         goto done;
     }
 
     if (_name) {
-        attr_name = ldb_dn_get_rdn_name(dn);
+        attr_name = ldb_dn_get_rdn_name(ldb_dn);
         if (attr_name == NULL) {
             ret = EINVAL;
             goto done;
@@ -256,7 +269,7 @@ errno_t sysdb_get_rdn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
         }
     }
 
-    val = ldb_dn_get_rdn_val(dn);
+    val = ldb_dn_get_rdn_val(ldb_dn);
     if (val == NULL) {
         ret = EINVAL;
         if (_name) talloc_free(*_name);
@@ -278,15 +291,15 @@ done:
 }
 
 errno_t sysdb_group_dn_name(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
-                            const char *_dn, char **_name)
+                            const char *dn, char **_name)
 {
-    return sysdb_get_rdn(sysdb, mem_ctx, _dn, NULL, _name);
+    return sysdb_get_rdn(sysdb, mem_ctx, dn, NULL, _name);
 }
 
-struct ldb_dn *sysdb_domain_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx,
+struct ldb_dn *sysdb_domain_dn(TALLOC_CTX *mem_ctx,
                                struct sss_domain_info *dom)
 {
-    return ldb_dn_new_fmt(mem_ctx, sysdb->ldb, SYSDB_DOM_BASE, dom->name);
+    return ldb_dn_new_fmt(mem_ctx, dom->sysdb->ldb, SYSDB_DOM_BASE, dom->name);
 }
 
 struct ldb_dn *sysdb_base_dn(struct sysdb_ctx *sysdb, TALLOC_CTX *mem_ctx)
@@ -510,18 +523,31 @@ int sysdb_attrs_get_string_array(struct sysdb_attrs *attrs, const char *name,
     return EOK;
 }
 
-int sysdb_attrs_add_val(struct sysdb_attrs *attrs,
-                        const char *name, const struct ldb_val *val)
+
+static int sysdb_attrs_add_val_int(struct sysdb_attrs *attrs,
+                                   const char *name, bool check_values,
+                                   const struct ldb_val *val)
 {
     struct ldb_message_element *el = NULL;
     struct ldb_val *vals;
     int ret;
+    size_t c;
 
     ret = sysdb_attrs_get_el(attrs, name, &el);
     if (ret != EOK) {
         return ret;
     }
 
+    if (check_values) {
+        for (c = 0; c < el->num_values; c++) {
+            if (val->length == el->values[c].length
+                    && memcmp(val->data, el->values[c].data,
+                              val->length) == 0) {
+                return EOK;
+            }
+        }
+    }
+
     vals = talloc_realloc(attrs->a, el->values,
                           struct ldb_val, el->num_values+1);
     if (!vals) return ENOMEM;
@@ -537,6 +563,29 @@ int sysdb_attrs_add_val(struct sysdb_attrs *attrs,
 
     return EOK;
 }
+int sysdb_attrs_add_val(struct sysdb_attrs *attrs,
+                        const char *name, const struct ldb_val *val)
+{
+    return sysdb_attrs_add_val_int(attrs, name, false, val);
+}
+
+/* Check if the same value already exists. */
+int sysdb_attrs_add_val_safe(struct sysdb_attrs *attrs,
+                             const char *name, const struct ldb_val *val)
+{
+    return sysdb_attrs_add_val_int(attrs, name, true, val);
+}
+
+int sysdb_attrs_add_string_safe(struct sysdb_attrs *attrs,
+                                const char *name, const char *str)
+{
+    struct ldb_val v;
+
+    v.data = (uint8_t *)discard_const(str);
+    v.length = strlen(str);
+
+    return sysdb_attrs_add_val_safe(attrs, name, &v);
+}
 
 int sysdb_attrs_add_string(struct sysdb_attrs *attrs,
                            const char *name, const char *str)
@@ -549,6 +598,28 @@ int sysdb_attrs_add_string(struct sysdb_attrs *attrs,
     return sysdb_attrs_add_val(attrs, name, &v);
 }
 
+int sysdb_attrs_add_lower_case_string(struct sysdb_attrs *attrs,
+                                      const char *name, const char *str)
+{
+    char *lc_str;
+    int ret;
+
+    if (attrs == NULL || str == NULL) {
+        return EINVAL;
+    }
+
+    lc_str = sss_tc_utf8_str_tolower(attrs, str);
+    if (lc_str == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot convert name to lowercase.\n");
+        return ENOMEM;
+    }
+
+    ret = sysdb_attrs_add_string(attrs, name, lc_str);
+    talloc_free(lc_str);
+
+    return ret;
+}
+
 int sysdb_attrs_add_mem(struct sysdb_attrs *attrs, const char *name,
                         const void *mem, size_t size)
 {
@@ -658,23 +729,7 @@ int sysdb_attrs_add_time_t(struct sysdb_attrs *attrs,
 int sysdb_attrs_add_lc_name_alias(struct sysdb_attrs *attrs,
                                   const char *value)
 {
-    char *lc_str;
-    int ret;
-
-    if (attrs == NULL || value == NULL) {
-        return EINVAL;
-    }
-
-    lc_str = sss_tc_utf8_str_tolower(attrs, value);
-    if (lc_str == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, "Cannot convert name to lowercase\n");
-        return ENOMEM;
-    }
-
-    ret = sysdb_attrs_add_string(attrs, SYSDB_NAME_ALIAS, lc_str);
-    talloc_free(lc_str);
-
-    return ret;
+    return sysdb_attrs_add_lower_case_string(attrs, SYSDB_NAME_ALIAS, value);
 }
 
 int sysdb_attrs_copy_values(struct sysdb_attrs *src,
@@ -1203,6 +1258,13 @@ int sysdb_domain_init_internal(TALLOC_CTX *mem_ctx,
             }
         }
 
+        if (strcmp(version, SYSDB_VERSION_0_15) == 0) {
+            ret = sysdb_upgrade_15(sysdb, &version);
+            if (ret != EOK) {
+                goto done;
+            }
+        }
+
         /* The version should now match SYSDB_VERSION.
          * If not, it means we didn't match any of the
          * known older versions. The DB might be
@@ -1224,7 +1286,7 @@ int sysdb_domain_init_internal(TALLOC_CTX *mem_ctx,
 
         DEBUG(SSSDBG_FATAL_FAILURE,
               "Unknown DB version [%s], expected [%s] for domain %s!\n",
-                 version?version:"not found", SYSDB_VERSION, domain->name);
+              version, SYSDB_VERSION, domain->name);
         ret = sysdb_version_check(SYSDB_VERSION, version);
         goto done;
     }
@@ -1274,6 +1336,16 @@ done:
 int sysdb_init(TALLOC_CTX *mem_ctx,
                struct sss_domain_info *domains,
                bool allow_upgrade)
+{
+    return sysdb_init_ext(mem_ctx, domains, allow_upgrade, false, 0, 0);
+}
+
+int sysdb_init_ext(TALLOC_CTX *mem_ctx,
+                   struct sss_domain_info *domains,
+                   bool allow_upgrade,
+                   bool chown_dbfile,
+                   uid_t uid,
+                   gid_t gid)
 {
     struct sss_domain_info *dom;
     struct sysdb_ctx *sysdb;
@@ -1296,6 +1368,17 @@ int sysdb_init(TALLOC_CTX *mem_ctx,
             return ret;
         }
 
+        if (chown_dbfile) {
+            ret = chown(sysdb->ldb_file, uid, gid);
+            if (ret != 0) {
+                ret = errno;
+                DEBUG(SSSDBG_CRIT_FAILURE,
+                      "Cannot set sysdb ownership to %"SPRIuid":%"SPRIgid"\n",
+                      uid, gid);
+                return ret;
+            }
+        }
+
         dom->sysdb = talloc_move(dom, &sysdb);
     }
 
@@ -1440,6 +1523,7 @@ errno_t sysdb_get_bool(struct sysdb_ctx *sysdb,
     errno_t ret;
     int lret;
     const char *attrs[2] = {attr_name, NULL};
+    struct ldb_message_element *el;
 
     tmp_ctx = talloc_new(NULL);
     if (tmp_ctx == NULL) {
@@ -1462,7 +1546,7 @@ errno_t sysdb_get_bool(struct sysdb_ctx *sysdb,
          * to contain this attribute.
          */
         *value = false;
-        ret = EOK;
+        ret = ENOENT;
         goto done;
     } else if (res->count != 1) {
         DEBUG(SSSDBG_CRIT_FAILURE,
@@ -1471,6 +1555,12 @@ errno_t sysdb_get_bool(struct sysdb_ctx *sysdb,
         goto done;
     }
 
+    el = ldb_msg_find_element(res->msgs[0], attr_name);
+    if (el == NULL || el->num_values == 0) {
+        ret = ENOENT;
+        goto done;
+    }
+
     *value = ldb_msg_find_attr_as_bool(res->msgs[0], attr_name, false);
 
     ret = EOK;
@@ -1546,6 +1636,11 @@ errno_t sysdb_set_bool(struct sysdb_ctx *sysdb,
         lret = ldb_add(sysdb->ldb, msg);
     }
 
+    if (lret != LDB_SUCCESS) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "ldb operation failed: [%s](%d)[%s]\n",
+              ldb_strerror(lret), lret, ldb_errstring(sysdb->ldb));
+    }
     ret = sysdb_error_to_errno(lret);
 
 done:
@@ -1553,8 +1648,7 @@ done:
     return ret;
 }
 
-errno_t sysdb_has_enumerated(struct sysdb_ctx *sysdb,
-                             struct sss_domain_info *domain,
+errno_t sysdb_has_enumerated(struct sss_domain_info *domain,
                              bool *has_enumerated)
 {
     errno_t ret;
@@ -1568,21 +1662,22 @@ errno_t sysdb_has_enumerated(struct sysdb_ctx *sysdb,
         goto done;
     }
 
-    dn = ldb_dn_new_fmt(tmp_ctx, sysdb->ldb, SYSDB_DOM_BASE, domain->name);
+    dn = ldb_dn_new_fmt(tmp_ctx, domain->sysdb->ldb, SYSDB_DOM_BASE,
+                        domain->name);
     if (!dn) {
         ret = ENOMEM;
         goto done;
     }
 
-    ret = sysdb_get_bool(sysdb, dn, SYSDB_HAS_ENUMERATED, has_enumerated);
+    ret = sysdb_get_bool(domain->sysdb, dn, SYSDB_HAS_ENUMERATED,
+                         has_enumerated);
 
 done:
     talloc_free(tmp_ctx);
     return ret;
 }
 
-errno_t sysdb_set_enumerated(struct sysdb_ctx *sysdb,
-                             struct sss_domain_info *domain,
+errno_t sysdb_set_enumerated(struct sss_domain_info *domain,
                              bool enumerated)
 {
     errno_t ret;
@@ -1595,13 +1690,14 @@ errno_t sysdb_set_enumerated(struct sysdb_ctx *sysdb,
         goto done;
     }
 
-    dn = ldb_dn_new_fmt(tmp_ctx, sysdb->ldb, SYSDB_DOM_BASE, domain->name);
+    dn = ldb_dn_new_fmt(tmp_ctx, domain->sysdb->ldb, SYSDB_DOM_BASE,
+                        domain->name);
     if (!dn) {
         ret = ENOMEM;
         goto done;
     }
 
-    ret = sysdb_set_bool(sysdb, dn, domain->name,
+    ret = sysdb_set_bool(domain->sysdb, dn, domain->name,
                          SYSDB_HAS_ENUMERATED, enumerated);
 
 done:
@@ -1870,53 +1966,6 @@ done:
     return ret;
 }
 
-errno_t sysdb_get_real_name(TALLOC_CTX *mem_ctx,
-                            struct sysdb_ctx *sysdb,
-                            struct sss_domain_info *domain,
-                            const char *name,
-                            const char **_cname)
-{
-    errno_t ret;
-    TALLOC_CTX *tmp_ctx;
-    struct ldb_result *res;
-    const char *cname;
-
-    tmp_ctx = talloc_new(NULL);
-    if (!tmp_ctx) {
-        return ENOMEM;
-    }
-
-    ret = sysdb_getpwnam(tmp_ctx, sysdb, domain, name, &res);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "Cannot canonicalize username\n");
-        goto done;
-    }
-
-    if (res->count == 0) {
-        /* User is not cached yet */
-        ret = ENOENT;
-        goto done;
-    } else if (res->count != 1) {
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "sysdb_getpwnam returned count: [%d]\n", res->count);
-        ret = EIO;
-        goto done;
-    }
-
-    cname = ldb_msg_find_attr_as_string(res->msgs[0], SYSDB_NAME, NULL);
-    if (!cname) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "A user with no name?\n");
-        ret = ENOENT;
-        goto done;
-    }
-
-    ret = EOK;
-    *_cname = talloc_steal(mem_ctx, cname);
-done:
-    talloc_free(tmp_ctx);
-    return ret;
-}
-
 errno_t sysdb_msg2attrs(TALLOC_CTX *mem_ctx, size_t count,
                         struct ldb_message **msgs,
                         struct sysdb_attrs ***attrs)
