@@ -315,7 +315,7 @@ done:
 }
 
 errno_t
-sysdb_get_sudo_user_info(TALLOC_CTX *mem_ctx, struct sysdb_ctx *sysdb,
+sysdb_get_sudo_user_info(TALLOC_CTX *mem_ctx,
                          struct sss_domain_info *domain,
                          const char *username, uid_t *_uid,
                          char ***groupnames)
@@ -341,8 +341,7 @@ sysdb_get_sudo_user_info(TALLOC_CTX *mem_ctx, struct sysdb_ctx *sysdb,
     tmp_ctx = talloc_new(NULL);
     NULL_CHECK(tmp_ctx, ret, done);
 
-    ret = sysdb_search_user_by_name(tmp_ctx, sysdb, domain,
-                                    username, attrs, &msg);
+    ret = sysdb_search_user_by_name(tmp_ctx, domain, username, attrs, &msg);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Error looking up user %s\n", username);
         goto done;
@@ -371,7 +370,7 @@ sysdb_get_sudo_user_info(TALLOC_CTX *mem_ctx, struct sysdb_ctx *sysdb,
 
             /* Get a list of the groups by groupname only */
             for (i = 0; i < groups->num_values; i++) {
-                ret = sysdb_group_dn_name(sysdb,
+                ret = sysdb_group_dn_name(domain->sysdb,
                                           sysdb_groupnames,
                                           (const char *)groups->values[i].data,
                                           &sysdb_groupnames[i]);
@@ -387,8 +386,8 @@ sysdb_get_sudo_user_info(TALLOC_CTX *mem_ctx, struct sysdb_ctx *sysdb,
     /* resolve primary group */
     gid = ldb_msg_find_attr_as_uint64(msg, SYSDB_GIDNUM, 0);
     if (gid != 0) {
-        ret = sysdb_search_group_by_gid(tmp_ctx, sysdb, domain, gid,
-                                        group_attrs, &group_msg);
+        ret = sysdb_search_group_by_gid(tmp_ctx, domain, gid, group_attrs,
+                                        &group_msg);
         if (ret == EOK) {
             primary_group = ldb_msg_find_attr_as_string(group_msg, SYSDB_NAME,
                                                         NULL);
@@ -429,8 +428,7 @@ done:
 }
 
 errno_t
-sysdb_save_sudorule(struct sysdb_ctx *sysdb,
-                    struct sss_domain_info *domain,
+sysdb_save_sudorule(struct sss_domain_info *domain,
                     const char *rule_name,
                     struct sysdb_attrs *attrs)
 {
@@ -453,7 +451,8 @@ sysdb_save_sudorule(struct sysdb_ctx *sysdb,
         return ret;
     }
 
-    ret = sysdb_store_custom(sysdb, domain, rule_name, SUDORULE_SUBDIR, attrs);
+    ret = sysdb_store_custom(domain, rule_name,
+                             SUDORULE_SUBDIR, attrs);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "sysdb_store_custom failed [%d]: %s\n",
               ret, strerror(ret));
@@ -463,8 +462,7 @@ sysdb_save_sudorule(struct sysdb_ctx *sysdb,
     return EOK;
 }
 
-static errno_t sysdb_sudo_set_refresh_time(struct sysdb_ctx *sysdb,
-                                           struct sss_domain_info *domain,
+static errno_t sysdb_sudo_set_refresh_time(struct sss_domain_info *domain,
                                            const char *attr_name,
                                            time_t value)
 {
@@ -481,14 +479,15 @@ static errno_t sysdb_sudo_set_refresh_time(struct sysdb_ctx *sysdb,
         goto done;
     }
 
-    dn = ldb_dn_new_fmt(tmp_ctx, sysdb->ldb, SYSDB_TMPL_CUSTOM_SUBTREE,
+    dn = ldb_dn_new_fmt(tmp_ctx, domain->sysdb->ldb,
+                        SYSDB_TMPL_CUSTOM_SUBTREE,
                         SUDORULE_SUBDIR, domain->name);
     if (!dn) {
         ret = ENOMEM;
         goto done;
     }
 
-    lret = ldb_search(sysdb->ldb, tmp_ctx, &res, dn, LDB_SCOPE_BASE,
+    lret = ldb_search(domain->sysdb->ldb, tmp_ctx, &res, dn, LDB_SCOPE_BASE,
                       NULL, NULL);
     if (lret != LDB_SUCCESS) {
         ret = sysdb_error_to_errno(lret);
@@ -528,11 +527,16 @@ static errno_t sysdb_sudo_set_refresh_time(struct sysdb_ctx *sysdb,
     }
 
     if (res->count) {
-        lret = ldb_modify(sysdb->ldb, msg);
+        lret = ldb_modify(domain->sysdb->ldb, msg);
     } else {
-        lret = ldb_add(sysdb->ldb, msg);
+        lret = ldb_add(domain->sysdb->ldb, msg);
     }
 
+    if (lret != LDB_SUCCESS) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "ldb operation failed: [%s](%d)[%s]\n",
+              ldb_strerror(lret), lret, ldb_errstring(domain->sysdb->ldb));
+    }
     ret = sysdb_error_to_errno(lret);
 
 done:
@@ -540,8 +544,7 @@ done:
     return ret;
 }
 
-static errno_t sysdb_sudo_get_refresh_time(struct sysdb_ctx *sysdb,
-                                           struct sss_domain_info *domain,
+static errno_t sysdb_sudo_get_refresh_time(struct sss_domain_info *domain,
                                            const char *attr_name,
                                            time_t *value)
 {
@@ -557,14 +560,14 @@ static errno_t sysdb_sudo_get_refresh_time(struct sysdb_ctx *sysdb,
         return ENOMEM;
     }
 
-    dn = ldb_dn_new_fmt(tmp_ctx, sysdb->ldb, SYSDB_TMPL_CUSTOM_SUBTREE,
+    dn = ldb_dn_new_fmt(tmp_ctx, domain->sysdb->ldb, SYSDB_TMPL_CUSTOM_SUBTREE,
                         SUDORULE_SUBDIR, domain->name);
     if (!dn) {
         ret = ENOMEM;
         goto done;
     }
 
-    lret = ldb_search(sysdb->ldb, tmp_ctx, &res, dn, LDB_SCOPE_BASE,
+    lret = ldb_search(domain->sysdb->ldb, tmp_ctx, &res, dn, LDB_SCOPE_BASE,
                       attrs, NULL);
     if (lret != LDB_SUCCESS) {
         ret = sysdb_error_to_errno(lret);
@@ -596,26 +599,23 @@ done:
     return ret;
 }
 
-errno_t sysdb_sudo_set_last_full_refresh(struct sysdb_ctx *sysdb,
-                                         struct sss_domain_info *domain,
+errno_t sysdb_sudo_set_last_full_refresh(struct sss_domain_info *domain,
                                          time_t value)
 {
-    return sysdb_sudo_set_refresh_time(sysdb, domain,
+    return sysdb_sudo_set_refresh_time(domain,
                                        SYSDB_SUDO_AT_LAST_FULL_REFRESH, value);
 }
 
-errno_t sysdb_sudo_get_last_full_refresh(struct sysdb_ctx *sysdb,
-                                         struct sss_domain_info *domain,
+errno_t sysdb_sudo_get_last_full_refresh(struct sss_domain_info *domain,
                                          time_t *value)
 {
-    return sysdb_sudo_get_refresh_time(sysdb, domain,
+    return sysdb_sudo_get_refresh_time(domain,
                                        SYSDB_SUDO_AT_LAST_FULL_REFRESH, value);
 }
 
 /* ====================  Purge functions ==================== */
 
-static errno_t sysdb_sudo_purge_all(struct sysdb_ctx *sysdb,
-                                    struct sss_domain_info *domain)
+static errno_t sysdb_sudo_purge_all(struct sss_domain_info *domain)
 {
     struct ldb_dn *base_dn = NULL;
     TALLOC_CTX *tmp_ctx = NULL;
@@ -624,10 +624,10 @@ static errno_t sysdb_sudo_purge_all(struct sysdb_ctx *sysdb,
     tmp_ctx = talloc_new(NULL);
     NULL_CHECK(tmp_ctx, ret, done);
 
-    base_dn = sysdb_custom_subtree_dn(sysdb, tmp_ctx, domain, SUDORULE_SUBDIR);
+    base_dn = sysdb_custom_subtree_dn(tmp_ctx, domain, SUDORULE_SUBDIR);
     NULL_CHECK(base_dn, ret, done);
 
-    ret = sysdb_delete_recursive(sysdb, base_dn, true);
+    ret = sysdb_delete_recursive(domain->sysdb, base_dn, true);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "sysdb_delete_recursive failed.\n");
         goto done;
@@ -639,16 +639,14 @@ done:
     return ret;
 }
 
-errno_t sysdb_sudo_purge_byname(struct sysdb_ctx *sysdb,
-                                struct sss_domain_info *domain,
+errno_t sysdb_sudo_purge_byname(struct sss_domain_info *domain,
                                 const char *name)
 {
     DEBUG(SSSDBG_TRACE_INTERNAL, "Deleting sudo rule %s\n", name);
-    return sysdb_delete_custom(sysdb, domain, name, SUDORULE_SUBDIR);
+    return sysdb_delete_custom(domain, name, SUDORULE_SUBDIR);
 }
 
-errno_t sysdb_sudo_purge_byfilter(struct sysdb_ctx *sysdb,
-                                  struct sss_domain_info *domain,
+errno_t sysdb_sudo_purge_byfilter(struct sss_domain_info *domain,
                                   const char *filter)
 {
     TALLOC_CTX *tmp_ctx;
@@ -666,14 +664,14 @@ errno_t sysdb_sudo_purge_byfilter(struct sysdb_ctx *sysdb,
 
     /* just purge all if there's no filter */
     if (!filter) {
-        return sysdb_sudo_purge_all(sysdb, domain);
+        return sysdb_sudo_purge_all(domain);
     }
 
     tmp_ctx = talloc_new(NULL);
     NULL_CHECK(tmp_ctx, ret, done);
 
     /* match entries based on the filter and remove them one by one */
-    ret = sysdb_search_custom(tmp_ctx, sysdb, domain, filter,
+    ret = sysdb_search_custom(tmp_ctx, domain, filter,
                               SUDORULE_SUBDIR, attrs,
                               &count, &msgs);
     if (ret == ENOENT) {
@@ -681,11 +679,11 @@ errno_t sysdb_sudo_purge_byfilter(struct sysdb_ctx *sysdb,
         ret = EOK;
         goto done;
     } else if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Error looking up SUDO rules");
+        DEBUG(SSSDBG_CRIT_FAILURE, "Error looking up SUDO rules\n");
         goto done;
     }
 
-    ret = sysdb_transaction_start(sysdb);
+    ret = sysdb_transaction_start(domain->sysdb);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Failed to start transaction\n");
         goto done;
@@ -700,14 +698,14 @@ errno_t sysdb_sudo_purge_byfilter(struct sysdb_ctx *sysdb,
             continue;
         }
 
-        ret = sysdb_sudo_purge_byname(sysdb, domain, name);
+        ret = sysdb_sudo_purge_byname(domain, name);
         if (ret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE, "Could not delete rule %s\n", name);
             goto done;
         }
     }
 
-    ret = sysdb_transaction_commit(sysdb);
+    ret = sysdb_transaction_commit(domain->sysdb);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Failed to commit transaction\n");
         goto done;
@@ -716,7 +714,7 @@ errno_t sysdb_sudo_purge_byfilter(struct sysdb_ctx *sysdb,
 
 done:
     if (in_transaction) {
-        sret = sysdb_transaction_cancel(sysdb);
+        sret = sysdb_transaction_cancel(domain->sysdb);
         if (sret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE, "Could not cancel transaction\n");
         }
