@@ -34,19 +34,19 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <popt.h>
+#include <tevent.h>
+#include <dbus/dbus.h>
 
 /* Needed for res_init() */
 #include <netinet/in.h>
 #include <arpa/nameser.h>
 #include <resolv.h>
 
-#include "popt.h"
-#include "tevent.h"
 #include "confdb/confdb.h"
 #include "confdb/confdb_setup.h"
 #include "db/sysdb.h"
 #include "monitor/monitor.h"
-#include "dbus/dbus.h"
 #include "sbus/sssd_dbus.h"
 #include "monitor/monitor_interfaces.h"
 #include "responder/common/responder_sbus.h"
@@ -86,8 +86,8 @@
 
 /* Warning messages */
 #define CONF_FILE_PERM_ERROR_MSG "Cannot read config file %s. Please check "\
-                                 "if permissions are 0600 and the file is "\
-                                 "owned by root.root."
+                                 "that the file is accessible only by the "\
+                                 "owner and owned by root.root.\n"
 
 int cmdline_debug_level;
 int cmdline_debug_timestamps;
@@ -156,9 +156,7 @@ struct config_file_ctx {
 struct mt_ctx {
     struct tevent_context *ev;
     struct confdb_ctx *cdb;
-    TALLOC_CTX *domain_ctx; /* Memory context for domain list */
     struct sss_domain_info *domains;
-    TALLOC_CTX *service_ctx; /* Memory context for services */
     char **services;
     int num_services;
     int started_services;
@@ -174,6 +172,10 @@ struct mt_ctx {
     struct sss_sigchild_ctx *sigchld_ctx;
     bool is_daemon;
     pid_t parent_pid;
+
+    /* For running unprivileged services */
+    uid_t uid;
+    gid_t gid;
 };
 
 static int start_service(struct mt_svc *mt_svc);
@@ -515,7 +517,11 @@ static int monitor_dbus_init(struct mt_ctx *ctx)
         return ret;
     }
 
-    ret = sbus_new_server(ctx, ctx->ev, monitor_address,
+    /* If a service is running as unprivileged user, we need to make sure this
+     * user can access the monitor sbus server. root is still king, so we don't
+     * lose any access.
+     */
+    ret = sbus_new_server(ctx, ctx->ev, monitor_address, ctx->uid, ctx->gid,
                           false, &ctx->sbus_srv, monitor_service_init, ctx);
 
     talloc_free(monitor_address);
@@ -553,6 +559,9 @@ static void tasks_check_handler(struct tevent_context *ev,
         DEBUG(SSSDBG_CRIT_FAILURE,
               "Killing service [%s], not responding to pings!\n",
                svc->name);
+        sss_log(SSS_LOG_ERR,
+                "Killing service [%s], not responding to pings!\n",
+                svc->name);
 
         /* Kill the service. The SIGCHLD handler will restart it */
         monitor_kill_service(svc);
@@ -595,8 +604,8 @@ static int monitor_kill_service (struct mt_svc *svc)
         ret = errno;
         DEBUG(SSSDBG_FATAL_FAILURE,
               "Sending signal to child (%s:%d) failed: [%d]: %s! "
-              "Ignore and pretend child is dead.\n",
-              svc->name, svc->pid, ret, strerror(ret));
+               "Ignore and pretend child is dead.\n",
+               svc->name, svc->pid, ret, strerror(ret));
         /* The only thing we can try here is to launch a new process
          * and hope that it works.
          */
@@ -633,6 +642,9 @@ static void mt_svc_sigkill(struct tevent_context *ev,
     DEBUG(SSSDBG_FATAL_FAILURE,
           "[%s][%d] is not responding to SIGTERM. Sending SIGKILL.\n",
            svc->name, svc->pid);
+    sss_log(SSS_LOG_ERR,
+            "[%s][%d] is not responding to SIGTERM. Sending SIGKILL.\n",
+            svc->name, svc->pid);
 
     /* timer was succesfully executed and it will be released by tevent */
     svc->kill_timer = NULL;
@@ -745,7 +757,7 @@ static int service_signal(struct mt_svc *svc, const char *svc_signal)
     if (msg == NULL) {
         DEBUG(SSSDBG_FATAL_FAILURE,
               "Out of memory trying to allocate memory to invoke: %s\n",
-              svc_signal);
+               svc_signal);
         monitor_kill_service(svc);
         return ENOMEM;
     }
@@ -915,8 +927,7 @@ done:
 
 static char *check_services(char **services)
 {
-    const char *known_services[] = { "nss", "pam", "sudo", "autofs", "ssh",
-                                     "pac", "ifp", NULL };
+    const char * const *known_services = get_known_services();
     int i;
     int ii;
 
@@ -936,7 +947,30 @@ static char *check_services(char **services)
     return NULL;
 }
 
-int get_monitor_config(struct mt_ctx *ctx)
+static int get_service_user(struct mt_ctx *ctx)
+{
+    errno_t ret;
+    char *user_str;
+
+    ret = confdb_get_string(ctx->cdb, ctx, CONFDB_MONITOR_CONF_ENTRY,
+                            CONFDB_MONITOR_USER_RUNAS,
+                            SSSD_USER, &user_str);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Failed to get the user to run as\n");
+        return ret;
+    }
+
+    ret = sss_user_by_name_or_uid(user_str, &ctx->uid, &ctx->gid);
+    talloc_free(user_str);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Failed to set allowed UIDs.\n");
+        return ret;
+    }
+
+    return EOK;
+}
+
+static int get_monitor_config(struct mt_ctx *ctx)
 {
     int ret;
     int timeout_seconds;
@@ -953,11 +987,7 @@ int get_monitor_config(struct mt_ctx *ctx)
 
     ctx->service_id_timeout = timeout_seconds * 1000; /* service_id_timeout is in ms */
 
-    ctx->service_ctx = talloc_new(ctx);
-    if(!ctx->service_ctx) {
-        return ENOMEM;
-    }
-    ret = confdb_get_string_as_list(ctx->cdb, ctx->service_ctx,
+    ret = confdb_get_string_as_list(ctx->cdb, ctx,
                                     CONFDB_MONITOR_CONF_ENTRY,
                                     CONFDB_MONITOR_ACTIVE_SERVICES,
                                     &ctx->services);
@@ -966,11 +996,11 @@ int get_monitor_config(struct mt_ctx *ctx)
         return EINVAL;
     }
 
-    ret = add_implicit_services(ctx->cdb, ctx->service_ctx, &ctx->services);
+    ret = add_implicit_services(ctx->cdb, ctx, &ctx->services);
     if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "Failed to add implicit configured " \
-                                  "services. Some functionality might " \
-                                  "be missing");
+        DEBUG(SSSDBG_OP_FAILURE, "Failed to add implicit configured "
+                                 "services. Some functionality might "
+                                 "be missing\n");
     }
 
     badsrv = check_services(ctx->services);
@@ -985,10 +1015,12 @@ int get_monitor_config(struct mt_ctx *ctx)
         ctx->num_services++;
     }
 
-    ctx->domain_ctx = talloc_new(ctx);
-    if(!ctx->domain_ctx) {
-        return ENOMEM;
+    ret = get_service_user(ctx);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Failed to get the unprivileged user\n");
+        return ret;
     }
+
     ret = confdb_get_domains(ctx->cdb, &ctx->domains);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE, "No domains configured.\n");
@@ -1054,6 +1086,22 @@ static errno_t get_ping_config(struct mt_ctx *ctx, const char *path,
     return EOK;
 }
 
+/* This is a temporary function that returns false if the service
+ * being started was only tested when running as root.
+ */
+static bool svc_supported_as_nonroot(const char *svc_name)
+{
+    if ((strcmp(svc_name, "nss") == 0)
+        || (strcmp(svc_name, "pam") == 0)
+        || (strcmp(svc_name, "autofs") == 0)
+        || (strcmp(svc_name, "pac") == 0)
+        || (strcmp(svc_name, "sudo") == 0)
+        || (strcmp(svc_name, "ssh") == 0)) {
+        return true;
+    }
+    return false;
+}
+
 static int get_service_config(struct mt_ctx *ctx, const char *name,
                               struct mt_svc **svc_cfg)
 {
@@ -1061,6 +1109,8 @@ static int get_service_config(struct mt_ctx *ctx, const char *name,
     char *path;
     struct mt_svc *svc;
     time_t now = time(NULL);
+    uid_t uid = 0;
+    gid_t gid = 0;
 
     *svc_cfg = NULL;
 
@@ -1100,6 +1150,11 @@ static int get_service_config(struct mt_ctx *ctx, const char *name,
         return ret;
     }
 
+    if (svc_supported_as_nonroot(svc->name)) {
+        uid = ctx->uid;
+        gid = ctx->gid;
+    }
+
     if (!svc->command) {
         svc->command = talloc_asprintf(
             svc, "%s/sssd_%s", SSSD_LIBEXEC_PATH, svc->name
@@ -1109,6 +1164,14 @@ static int get_service_config(struct mt_ctx *ctx, const char *name,
             return ENOMEM;
         }
 
+        svc->command = talloc_asprintf_append(svc->command,
+                " --uid %"SPRIuid" --gid %"SPRIgid,
+                uid, gid);
+        if (!svc->command) {
+            talloc_free(svc);
+            return ENOMEM;
+        }
+
         if (cmdline_debug_level != SSSDBG_UNRESOLVED) {
             svc->command = talloc_asprintf_append(
                 svc->command, " -d %#.4x", cmdline_debug_level
@@ -1148,6 +1211,14 @@ static int get_service_config(struct mt_ctx *ctx, const char *name,
                 talloc_free(svc);
                 return ENOMEM;
             }
+        } else if (ctx->is_daemon == false) {
+            svc->command = talloc_strdup_append(
+                svc->command, " --debug-to-stderr"
+            );
+            if (!svc->command) {
+                talloc_free(svc);
+                return ENOMEM;
+            }
         }
     }
 
@@ -1272,6 +1343,14 @@ static int get_provider_config(struct mt_ctx *ctx, const char *name,
             return ENOMEM;
         }
 
+        svc->command = talloc_asprintf_append(svc->command,
+                " --uid %"SPRIuid" --gid %"SPRIgid,
+                ctx->uid, ctx->gid);
+        if (!svc->command) {
+            talloc_free(svc);
+            return ENOMEM;
+        }
+
         if (cmdline_debug_level != SSSDBG_UNRESOLVED) {
             svc->command = talloc_asprintf_append(
                 svc->command, " -d %#.4x", cmdline_debug_level
@@ -1311,6 +1390,14 @@ static int get_provider_config(struct mt_ctx *ctx, const char *name,
                 talloc_free(svc);
                 return ENOMEM;
             }
+        } else if (ctx->is_daemon == false) {
+            svc->command = talloc_strdup_append(
+                svc->command, " --debug-to-stderr"
+            );
+            if (!svc->command) {
+                talloc_free(svc);
+                return ENOMEM;
+            }
         }
     }
 
@@ -1590,9 +1677,17 @@ static int monitor_ctx_destructor(void *mem)
     return 0;
 }
 
-static errno_t load_configuration(TALLOC_CTX *mem_ctx,
-                                  const char *config_file,
-                                  struct mt_ctx **monitor)
+/*
+ * This function should not be static otherwise gcc does some special kind of
+ * optimisations which should not happen according to code: chown (unlink)
+ * failed (return -1) but errno was zero.
+ * As a result of this  * warning is printed ‘monitor’ may be used
+ * uninitialized in this function. Instead of checking errno for 0
+ * it's better to disable optimisation(in-lining) of this function.
+ */
+errno_t load_configuration(TALLOC_CTX *mem_ctx,
+                           const char *config_file,
+                           struct mt_ctx **monitor)
 {
     errno_t ret;
     struct mt_ctx *ctx;
@@ -1627,7 +1722,14 @@ static errno_t load_configuration(TALLOC_CTX *mem_ctx,
          * misconfiguration gets in the way
          */
         talloc_zfree(ctx->cdb);
-        unlink(cdb_file);
+        ret = unlink(cdb_file);
+        if (ret != EOK && errno != ENOENT) {
+            ret = errno;
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  "Purging existing confdb failed: %d [%s].\n",
+                  ret, sss_strerror(ret));
+            goto done;
+        }
 
         ret = confdb_init(ctx, &ctx->cdb, cdb_file);
         if (ret != EOK) {
@@ -1646,7 +1748,6 @@ static errno_t load_configuration(TALLOC_CTX *mem_ctx,
         DEBUG(SSSDBG_FATAL_FAILURE, "Fatal error initializing confdb\n");
         goto done;
     }
-    talloc_zfree(cdb_file);
 
     ret = confdb_init_db(config_file, ctx->cdb);
     if (ret != EOK) {
@@ -1662,22 +1763,34 @@ static errno_t load_configuration(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
+    /* Allow configuration database to be accessible
+     * when SSSD runs as nonroot */
+    ret = chown(cdb_file, ctx->uid, ctx->gid);
+    if (ret != 0) {
+        ret = errno;
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              "chown failed for [%s]: [%d][%s].\n",
+              cdb_file, ret, sss_strerror(ret));
+        goto done;
+    }
+
     *monitor = ctx;
 
     ret = EOK;
 
 done:
+    talloc_free(cdb_file);
     if (ret != EOK) {
         talloc_free(ctx);
     }
     return ret;
 }
 
-errno_t monitor_config_file_fallback(TALLOC_CTX *mem_ctx,
-                                     struct mt_ctx *ctx,
-                                     const char *file,
-                                     monitor_reconf_fn fn,
-                                     bool ignore_missing);
+static errno_t monitor_config_file_fallback(TALLOC_CTX *mem_ctx,
+                                            struct mt_ctx *ctx,
+                                            const char *file,
+                                            monitor_reconf_fn fn,
+                                            bool ignore_missing);
 
 #ifdef HAVE_INOTIFY
 static void process_config_file(struct tevent_context *ev,
@@ -1730,16 +1843,13 @@ static void process_config_file(struct tevent_context *ev,
 {
     TALLOC_CTX *tmp_ctx;
     struct inotify_event *in_event;
-    char *buf;
     char *name;
     ssize_t len;
-    ssize_t event_size;
     struct config_file_ctx *file_ctx;
     struct config_file_callback *cb;
     struct rewatch_ctx *rw_ctx;
     errno_t ret;
 
-    event_size = sizeof(struct inotify_event);
     file_ctx = talloc_get_type(ptr, struct config_file_ctx);
 
     DEBUG(SSSDBG_CRIT_FAILURE, "Processing config file changes\n");
@@ -1747,13 +1857,14 @@ static void process_config_file(struct tevent_context *ev,
     tmp_ctx = talloc_new(NULL);
     if (!tmp_ctx) return;
 
-    buf = talloc_size(tmp_ctx, event_size);
-    if (!buf) {
+    in_event = talloc(tmp_ctx, struct inotify_event);
+    if (!in_event) {
         goto done;
     }
 
     errno = 0;
-    len = sss_atomic_read_s(file_ctx->mt_ctx->inotify_fd, buf, event_size);
+    len = sss_atomic_read_s(file_ctx->mt_ctx->inotify_fd, in_event,
+                            sizeof(struct inotify_event));
     if (len == -1) {
         ret = errno;
         DEBUG(SSSDBG_CRIT_FAILURE,
@@ -1762,8 +1873,6 @@ static void process_config_file(struct tevent_context *ev,
         goto done;
     }
 
-    in_event = (struct inotify_event *)buf;
-
     if (in_event->len > 0) {
         /* Read in the name, even though we don't use it,
          * so that read ptr is in the right place
@@ -2005,7 +2114,7 @@ static int try_inotify(struct config_file_ctx *file_ctx, const char *filename,
     cb = talloc_zero(file_ctx, struct config_file_callback);
     if(!cb) {
         close(file_ctx->mt_ctx->inotify_fd);
-        return EIO;
+        return ENOMEM;
     }
 
     cb->filename = talloc_strdup(cb, filename);
@@ -2091,11 +2200,11 @@ static int monitor_config_file(TALLOC_CTX *mem_ctx,
     return ret;
 }
 
-errno_t monitor_config_file_fallback(TALLOC_CTX *mem_ctx,
-                                     struct mt_ctx *ctx,
-                                     const char *file,
-                                     monitor_reconf_fn fn,
-                                     bool ignore_missing)
+static errno_t monitor_config_file_fallback(TALLOC_CTX *mem_ctx,
+                                            struct mt_ctx *ctx,
+                                            const char *file,
+                                            monitor_reconf_fn fn,
+                                            bool ignore_missing)
 {
     struct config_file_callback *cb = NULL;
     struct stat file_stat;
@@ -2150,8 +2259,8 @@ errno_t monitor_config_file_fallback(TALLOC_CTX *mem_ctx,
     return EOK;
 }
 
-int monitor_process_init(struct mt_ctx *ctx,
-                         const char *config_file)
+static int monitor_process_init(struct mt_ctx *ctx,
+                                const char *config_file)
 {
     TALLOC_CTX *tmp_ctx;
     struct tevent_signal *tes;
@@ -2255,7 +2364,8 @@ int monitor_process_init(struct mt_ctx *ctx,
     if (!tmp_ctx) {
         return ENOMEM;
     }
-    ret = sysdb_init(tmp_ctx, ctx->domains, true);
+    ret = sysdb_init_ext(tmp_ctx, ctx->domains, true,
+                         true, ctx->uid, ctx->gid);
     if (ret != EOK) {
         SYSDB_VERSION_ERROR_DAEMON(ret);
         return ret;
@@ -2354,6 +2464,9 @@ static int monitor_service_init(struct sbus_connection *conn, void *data)
     mini->ctx = ctx;
     mini->conn = conn;
 
+    /* Allow access from the SSSD user */
+    sbus_allow_uid(conn, &ctx->uid);
+
     /* 10 seconds should be plenty */
     tv = tevent_timeval_current_ofs(10, 0);
 
@@ -2754,7 +2867,7 @@ int main(int argc, const char *argv[])
     DEBUG_INIT(debug_level);
 
     if (opt_version) {
-        puts(VERSION""DISTRO_VERSION""PRERELEASE_VERSION);
+        puts(VERSION""PRERELEASE_VERSION);
         return EXIT_SUCCESS;
     }
 
@@ -2791,7 +2904,10 @@ int main(int argc, const char *argv[])
     }
 
     if (opt_daemon) flags |= FLAGS_DAEMON;
-    if (opt_interactive) flags |= FLAGS_INTERACTIVE;
+    if (opt_interactive) {
+        flags |= FLAGS_INTERACTIVE;
+        debug_to_stderr = 1;
+    }
 
     if (opt_config_file) {
         config_file = talloc_strdup(tmp_ctx, opt_config_file);
@@ -2839,7 +2955,8 @@ int main(int argc, const char *argv[])
 #endif
 
     /* Warn if nscd seems to be running */
-    ret = check_file(NSCD_SOCKET_PATH, -1, -1, -1, CHECK_SOCK, NULL, false);
+    ret = check_file(NSCD_SOCKET_PATH,
+                     -1, -1, S_IFSOCK, S_IFMT, NULL, false);
     if (ret == EOK) {
         ret = sss_nscd_parse_conf(NSCD_CONF_PATH);
 
@@ -2901,7 +3018,8 @@ int main(int argc, const char *argv[])
     ret = close(STDIN_FILENO);
     if (ret != EOK) return 6;
 
-    ret = server_setup(MONITOR_NAME, flags, monitor->conf_path, &main_ctx);
+    ret = server_setup(MONITOR_NAME, flags, 0, 0,
+                       monitor->conf_path, &main_ctx);
     if (ret != EOK) return 2;
 
     monitor->is_daemon = !opt_interactive;
