@@ -263,6 +263,8 @@ ad_get_common_options(TALLOC_CTX *mem_ctx,
     char *realm;
     char *ad_hostname;
     char hostname[HOST_NAME_MAX + 1];
+    char *case_sensitive_opt;
+    const char *opt_override;
 
     opts = talloc_zero(mem_ctx, struct ad_options);
     if (!opts) return ENOMEM;
@@ -333,22 +335,46 @@ ad_get_common_options(TALLOC_CTX *mem_ctx,
     }
 
     /* Active Directory is always case-insensitive */
-    dom->case_sensitive = false;
+    ret = confdb_get_string(cdb, mem_ctx, conf_path,
+                            CONFDB_DOMAIN_CASE_SENSITIVE, "false",
+                            &case_sensitive_opt);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "condb_get_string failed.\n");
+        goto done;
+    }
+
+    if (strcasecmp(case_sensitive_opt, "true") == 0) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Warning: AD domain can not be set as case-sensitive.\n");
+        dom->case_sensitive = false;
+        dom->case_preserve = false;
+    } else if (strcasecmp(case_sensitive_opt, "false") == 0) {
+        dom->case_sensitive = false;
+        dom->case_preserve = false;
+    } else if (strcasecmp(case_sensitive_opt, "preserving") == 0) {
+        dom->case_sensitive = false;
+        dom->case_preserve = true;
+    } else {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              "Invalid value for %s\n", CONFDB_DOMAIN_CASE_SENSITIVE);
+        goto done;
+    }
+
+    opt_override = dom->case_preserve ? "preserving" : "false";
 
     /* Set this in the confdb so that the responders pick it
      * up when they start up.
      */
-    ret = confdb_set_bool(cdb, conf_path, "case_sensitive",
-                          dom->case_sensitive);
+    ret = confdb_set_string(cdb, conf_path, "case_sensitive", opt_override);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
-              "Could not set domain case-sensitive: [%s]\n",
+              "Could not set domain option case_sensitive: [%s]\n",
                strerror(ret));
         goto done;
     }
 
     DEBUG(SSSDBG_CONF_SETTINGS,
-          "Setting domain case-insensitive\n");
+          "Setting domain option case_sensitive to [%s]\n", opt_override);
 
     ret = EOK;
     *_opts = opts;
@@ -364,8 +390,7 @@ static void
 ad_resolve_callback(void *private_data, struct fo_server *server);
 
 static errno_t
-_ad_servers_init(TALLOC_CTX *mem_ctx,
-                 struct ad_service *service,
+_ad_servers_init(struct ad_service *service,
                  struct be_ctx *bectx,
                  const char *fo_service,
                  const char *fo_gc_service,
@@ -412,8 +437,8 @@ _ad_servers_init(TALLOC_CTX *mem_ctx,
                                        false, sdata);
             if (ret != EOK) {
                 DEBUG(SSSDBG_FATAL_FAILURE,
-                      "Failed to add service discovery to failover: [%s]",
-                       strerror(ret));
+                      "Failed to add service discovery to failover: [%s]\n",
+                      strerror(ret));
                 goto done;
             }
 
@@ -429,8 +454,8 @@ _ad_servers_init(TALLOC_CTX *mem_ctx,
                                        false, sdata);
             if (ret != EOK) {
                 DEBUG(SSSDBG_FATAL_FAILURE,
-                      "Failed to add service discovery to failover: [%s]",
-                       strerror(ret));
+                      "Failed to add service discovery to failover: [%s]\n",
+                      strerror(ret));
                 goto done;
             }
 
@@ -479,22 +504,22 @@ done:
 }
 
 static inline errno_t
-ad_primary_servers_init(TALLOC_CTX *mem_ctx, struct ad_service *service,
+ad_primary_servers_init(struct ad_service *service,
                         struct be_ctx *bectx, const char *servers,
                         const char *fo_service, const char *fo_gc_service,
                         const char *ad_domain)
 {
-    return _ad_servers_init(mem_ctx, service, bectx, fo_service,
+    return _ad_servers_init(service, bectx, fo_service,
                             fo_gc_service, servers, ad_domain, true);
 }
 
 static inline errno_t
-ad_backup_servers_init(TALLOC_CTX *mem_ctx, struct ad_service *service,
+ad_backup_servers_init(struct ad_service *service,
                         struct be_ctx *bectx, const char *servers,
                         const char *fo_service, const char *fo_gc_service,
                         const char *ad_domain)
 {
-    return _ad_servers_init(mem_ctx, service, bectx, fo_service,
+    return _ad_servers_init(service, bectx, fo_service,
                             fo_gc_service, servers, ad_domain, false);
 }
 
@@ -614,7 +639,7 @@ ad_failover_init(TALLOC_CTX *mem_ctx, struct be_ctx *bectx,
         primary_servers = BE_SRV_IDENTIFIER;
     }
 
-    ret = ad_primary_servers_init(mem_ctx, service, bectx,
+    ret = ad_primary_servers_init(service, bectx,
                                   primary_servers, ad_service,
                                   ad_gc_service, ad_domain);
     if (ret != EOK) {
@@ -622,7 +647,7 @@ ad_failover_init(TALLOC_CTX *mem_ctx, struct be_ctx *bectx,
     }
 
     if (backup_servers) {
-        ret = ad_backup_servers_init(mem_ctx, service, bectx,
+        ret = ad_backup_servers_init(service, bectx,
                                      backup_servers, ad_service,
                                      ad_gc_service, ad_domain);
         if (ret != EOK) {
@@ -1115,18 +1140,14 @@ ad_get_dom_ldap_conn(struct ad_id_ctx *ad_ctx, struct sss_domain_info *dom)
     struct sdap_domain *sdom;
     struct ad_id_ctx *subdom_id_ctx;
 
-    if (IS_SUBDOMAIN(dom)) {
-        sdom = sdap_domain_get(ad_ctx->sdap_id_ctx->opts, dom);
-        if (sdom == NULL || sdom->pvt == NULL) {
-            DEBUG(SSSDBG_CRIT_FAILURE, "No ID ctx available for [%s].\n",
-                                        dom->name);
-            return NULL;
-        }
-        subdom_id_ctx = talloc_get_type(sdom->pvt, struct ad_id_ctx);
-        conn = subdom_id_ctx->ldap_ctx;
-    } else {
-        conn = ad_ctx->ldap_ctx;
+    sdom = sdap_domain_get(ad_ctx->sdap_id_ctx->opts, dom);
+    if (sdom == NULL || sdom->pvt == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "No ID ctx available for [%s].\n",
+                                    dom->name);
+        return NULL;
     }
+    subdom_id_ctx = talloc_get_type(sdom->pvt, struct ad_id_ctx);
+    conn = subdom_id_ctx->ldap_ctx;
 
     return conn;
 }
