@@ -31,6 +31,13 @@
 /* The port the Global Catalog runs on */
 #define AD_GC_PORT      3268
 
+#define AD_AT_OBJECT_SID "objectSID"
+#define AD_AT_DNS_DOMAIN "DnsDomain"
+#define AD_AT_NT_VERSION "NtVer"
+#define AD_AT_NETLOGON   "netlogon"
+
+#define MASTER_DOMAIN_SID_FILTER "objectclass=domain"
+
 struct ad_options;
 
 enum ad_basic_opt {
@@ -43,6 +50,18 @@ enum ad_basic_opt {
     AD_ENABLE_DNS_SITES,
     AD_ACCESS_FILTER,
     AD_ENABLE_GC,
+    AD_GPO_ACCESS_CONTROL,
+    AD_GPO_CACHE_TIMEOUT,
+    AD_GPO_MAP_INTERACTIVE,
+    AD_GPO_MAP_REMOTE_INTERACTIVE,
+    AD_GPO_MAP_NETWORK,
+    AD_GPO_MAP_BATCH,
+    AD_GPO_MAP_SERVICE,
+    AD_GPO_MAP_PERMIT,
+    AD_GPO_MAP_DENY,
+    AD_GPO_DEFAULT_RIGHT,
+    AD_SITE,
+    AD_KRB5_CONFD_PATH,
 
     AD_OPTS_BASIC /* opts counter */
 };
@@ -132,4 +151,5 @@ int ad_sudo_init(struct be_ctx *be_ctx,
                  struct ad_id_ctx *id_ctx,
                  struct bet_ops **ops,
                  void **pvt_data);
+
 #endif /* AD_COMMON_H_ */
