@@ -120,12 +120,12 @@ ad_handle_acct_info_step(struct tevent_req *req)
         noexist_delete = true;
     }
 
-    subreq = sdap_handle_acct_req_send(state, state->breq,
+    subreq = sdap_handle_acct_req_send(state, state->ctx->be,
                                        state->ar, state->ctx,
                                        state->sdom,
                                        state->conn[state->cindex],
                                        noexist_delete);
-    if (req == NULL) {
+    if (subreq == NULL) {
         return ENOMEM;
     }
     tevent_req_set_callback(subreq, ad_handle_acct_info_done, req);
@@ -155,6 +155,10 @@ ad_handle_acct_info_done(struct tevent_req *subreq)
     }
     talloc_zfree(subreq);
     if (ret != EOK) {
+        /* if GC was not used dp error should be set */
+        state->dp_error = dp_error;
+        state->err = err;
+
         tevent_req_error(req, ret);
         return;
     }
@@ -256,12 +260,11 @@ get_conn_list(struct be_req *breq, struct ad_id_ctx *ad_ctx,
     return clist;
 }
 
-static errno_t ad_account_can_shortcut(struct be_ctx *be_ctx,
-                                       struct sdap_idmap_ctx *idmap_ctx,
-                                       int filter_type,
-                                       const char *filter_value,
-                                       const char *filter_domain,
-                                       bool *_shortcut)
+static bool ad_account_can_shortcut(struct be_ctx *be_ctx,
+                                    struct sdap_idmap_ctx *idmap_ctx,
+                                    int filter_type,
+                                    const char *filter_value,
+                                    const char *filter_domain)
 {
     struct sss_domain_info *domain = be_ctx->domain;
     struct sss_domain_info *req_dom = NULL;
@@ -274,8 +277,6 @@ static errno_t ad_account_can_shortcut(struct be_ctx *be_ctx,
 
     if (!sdap_idmap_domain_has_algorithmic_mapping(idmap_ctx, domain->name,
                                                    domain->domain_id)) {
-        shortcut = false;
-        ret = EOK;
         goto done;
     }
 
@@ -286,6 +287,8 @@ static errno_t ad_account_can_shortcut(struct be_ctx *be_ctx,
         id = strtouint32(filter_value, NULL, 10);
         if (errno != 0) {
             ret = errno;
+            DEBUG(SSSDBG_MINOR_FAILURE, "Unable to convert filter value to "
+                  "number [%d]: %s\n", ret, strerror(ret));
             goto done;
         }
 
@@ -294,43 +297,32 @@ static errno_t ad_account_can_shortcut(struct be_ctx *be_ctx,
         if (err != IDMAP_SUCCESS) {
             DEBUG(SSSDBG_MINOR_FAILURE, "Mapping ID [%s] to SID failed: "
                   "[%s]\n", filter_value, idmap_error_string(err));
-            ret = EIO;
             goto done;
         }
         /* fall through */
     case BE_FILTER_SECID:
         csid = sid == NULL ? filter_value : sid;
 
-        req_dom = find_subdomain_by_sid(domain, csid);
+        req_dom = find_domain_by_sid(domain, csid);
         if (req_dom == NULL) {
             DEBUG(SSSDBG_OP_FAILURE, "Invalid domain\n");
-            ret = ERR_DOMAIN_NOT_FOUND;
             goto done;
         }
 
         if (strcasecmp(req_dom->name, filter_domain) != 0) {
             shortcut = true;
-        } else {
-            shortcut = false;
         }
         break;
     default:
-        shortcut = false;
         break;
     }
 
-    ret = EOK;
-
 done:
     if (sid != NULL) {
         sss_idmap_free_sid(idmap_ctx->map, sid);
     }
 
-    if (ret == EOK) {
-        *_shortcut = shortcut;
-    }
-
-    return ret;
+    return shortcut;
 }
 
 static void ad_account_info_complete(struct tevent_req *req);
@@ -358,17 +350,16 @@ ad_account_info_handler(struct be_req *be_req)
         return be_req_terminate(be_req, DP_ERR_OFFLINE, EAGAIN, "Offline");
     }
 
-    /* Try to shortcut if this is ID or SID search and it belongs to
-     * other domain range than is in ar->domain. */
-    ret = ad_account_can_shortcut(be_ctx, sdap_id_ctx->opts->idmap_ctx,
-                                  ar->filter_type, ar->filter_value,
-                                  ar->domain, &shortcut);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_TRACE_FUNC,
-              "Cannot determine the right domain: %s\n", sss_strerror(ret));
-        shortcut = false;
+    if (sdap_is_enum_request(ar)) {
+        DEBUG(SSSDBG_TRACE_LIBS, "Skipping enumeration on demand\n");
+        return sdap_handler_done(be_req, DP_ERR_OK, EOK, "Success");
     }
 
+    /* Try to shortcut if this is ID or SID search and it belongs to
+     * other domain range than is in ar->domain. */
+    shortcut = ad_account_can_shortcut(be_ctx, sdap_id_ctx->opts->idmap_ctx,
+                                       ar->filter_type, ar->filter_value,
+                                       ar->domain);
     if (shortcut) {
         DEBUG(SSSDBG_TRACE_FUNC, "This ID is from different domain\n");
         be_req_terminate(be_req, DP_ERR_OK, EOK, NULL);
@@ -378,7 +369,7 @@ ad_account_info_handler(struct be_req *be_req)
     dom = be_ctx->domain;
     if (strcasecmp(ar->domain, be_ctx->domain->name) != 0) {
         /* Subdomain request, verify subdomain */
-        dom = find_subdomain_by_name(be_ctx->domain, ar->domain, true);
+        dom = find_domain_by_name(be_ctx->domain, ar->domain, true);
     }
 
     if (dom == NULL) {
@@ -576,7 +567,7 @@ ad_enumeration_master_done(struct tevent_req *subreq)
     char *forest;
 
     ret = ad_master_domain_recv(subreq, state,
-                                &flat_name, &master_sid, &forest);
+                                &flat_name, &master_sid, NULL, &forest);
     talloc_zfree(subreq);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "Cannot retrieve master domain info\n");
@@ -762,8 +753,7 @@ ad_enum_cross_dom_members(struct sdap_options *opts,
         goto done;
     }
 
-    ret = sysdb_search_groups(tmp_ctx, dom->sysdb, dom,
-                              filter, attrs, &count, &msgs);
+    ret = sysdb_search_groups(tmp_ctx, dom, filter, attrs, &count, &msgs);
     if (ret != EOK) {
         goto done;
     }
@@ -909,7 +899,7 @@ ad_group_stored_orig_members(TALLOC_CTX *mem_ctx, struct sss_domain_info *dom,
     if (tmp_ctx == NULL) return ENOMEM;
 
     /* Get all entries member element points to */
-    ret = sysdb_asq_search(tmp_ctx, dom->sysdb, dn, NULL, SYSDB_MEMBER,
+    ret = sysdb_asq_search(tmp_ctx, dom, dn, NULL, SYSDB_MEMBER,
                            attrs, &m_count, &members);
     if (ret != EOK) {
         goto done;
@@ -976,7 +966,7 @@ ad_group_add_member(struct sdap_options *opts,
         goto done;
     }
 
-    base_dn = sysdb_domain_dn(sd->dom->sysdb, tmp_ctx, sd->dom);
+    base_dn = sysdb_domain_dn(tmp_ctx, sd->dom);
     if (base_dn == NULL) {
         ret = ENOMEM;
         goto done;
@@ -1001,8 +991,7 @@ ad_group_add_member(struct sdap_options *opts,
         goto done;
     }
 
-    ret = sysdb_mod_group_member(group_domain->sysdb, msgs[0]->dn,
-                                 group_dn, SYSDB_MOD_ADD);
+    ret = sysdb_mod_group_member(group_domain, msgs[0]->dn, group_dn, SYSDB_MOD_ADD);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "Could not add [%s] as a member of [%s]\n",
               ldb_dn_get_linearized(msgs[0]->dn),
