@@ -159,6 +159,7 @@ sssm_ad_id_init(struct be_ctx *bectx,
     struct ad_id_ctx *ad_ctx;
     const char *hostname;
     const char *ad_domain;
+    const char *ad_site_override;
     struct ad_srv_plugin_ctx *srv_ctx;
 
     if (!ad_options) {
@@ -234,9 +235,12 @@ sssm_ad_id_init(struct be_ctx *bectx,
     if (dp_opt_get_bool(ad_options->basic, AD_ENABLE_DNS_SITES)) {
         /* use AD plugin */
         ad_domain = dp_opt_get_string(ad_options->basic, AD_DOMAIN);
+        ad_site_override = dp_opt_get_string(ad_options->basic, AD_SITE);
+
         srv_ctx = ad_srv_plugin_ctx_init(bectx, bectx->be_res,
                                          default_host_dbs, ad_options->id,
-                                         hostname, ad_domain);
+                                         hostname, ad_domain,
+                                         ad_site_override);
         if (srv_ctx == NULL) {
             DEBUG(SSSDBG_FATAL_FAILURE, "Out of memory?\n");
             ret = ENOMEM;
@@ -256,12 +260,9 @@ sssm_ad_id_init(struct be_ctx *bectx,
     }
 
     /* setup periodical refresh of expired records */
-    ret = be_refresh_add_cb(bectx->refresh_ctx, BE_REFRESH_TYPE_NETGROUPS,
-                            sdap_refresh_netgroups_send,
-                            sdap_refresh_netgroups_recv,
-                            ad_ctx->sdap_id_ctx);
+    ret = sdap_refresh_init(bectx->refresh_ctx, ad_ctx->sdap_id_ctx);
     if (ret != EOK && ret != EEXIST) {
-        DEBUG(SSSDBG_MINOR_FAILURE, "Periodical refresh of netgroups "
+        DEBUG(SSSDBG_MINOR_FAILURE, "Periodical refresh "
               "will not work [%d]: %s\n", ret, strerror(ret));
     }
 
@@ -361,6 +362,9 @@ sssm_ad_chpass_init(struct be_ctx *bectx,
     return ret;
 }
 
+/* GPO parsing of PAM service names to Windows Logon Rights*/
+errno_t ad_gpo_parse_map_options(struct ad_access_ctx *access_ctx);
+
 int
 sssm_ad_access_init(struct be_ctx *bectx,
                     struct bet_ops **ops,
@@ -370,6 +374,8 @@ sssm_ad_access_init(struct be_ctx *bectx,
     struct ad_access_ctx *access_ctx;
     struct ad_id_ctx *ad_id_ctx;
     const char *filter;
+    const char *gpo_access_control_mode;
+    int gpo_cache_timeout;
 
     access_ctx = talloc_zero(bectx, struct ad_access_ctx);
     if (!access_ctx) return ENOMEM;
@@ -421,6 +427,46 @@ sssm_ad_access_init(struct be_ctx *bectx,
         access_ctx->sdap_access_ctx->access_rule[1] = LDAP_ACCESS_EMPTY;
     }
 
+    /* GPO access control mode */
+    gpo_access_control_mode =
+        dp_opt_get_string(access_ctx->ad_options, AD_GPO_ACCESS_CONTROL);
+    if (strcasecmp(gpo_access_control_mode, "disabled") == 0) {
+        access_ctx->gpo_access_control_mode = GPO_ACCESS_CONTROL_DISABLED;
+    } else if (strcasecmp(gpo_access_control_mode, "permissive") == 0) {
+        access_ctx->gpo_access_control_mode = GPO_ACCESS_CONTROL_PERMISSIVE;
+    } else if (strcasecmp(gpo_access_control_mode, "enforcing") == 0) {
+        access_ctx->gpo_access_control_mode = GPO_ACCESS_CONTROL_ENFORCING;
+    } else {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              "Unrecognized GPO access control mode: %s\n",
+              gpo_access_control_mode);
+        ret = EINVAL;
+        goto fail;
+    }
+
+    /* GPO cache timeout */
+    gpo_cache_timeout =
+        dp_opt_get_int(access_ctx->ad_options, AD_GPO_CACHE_TIMEOUT);
+    access_ctx->gpo_cache_timeout = gpo_cache_timeout;
+
+    /* GPO logon maps */
+
+    ret = sss_hash_create(access_ctx, 10, &access_ctx->gpo_map_options_table);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              "Could not create gpo_map_options hash table: [%s]\n",
+              strerror(ret));
+        goto fail;
+    }
+
+    ret = ad_gpo_parse_map_options(access_ctx);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              "Could not parse gpo_map_options (invalid config): [%s]\n",
+              strerror(ret));
+        goto fail;
+    }
+
     *ops = &ad_access_ops;
     *pvt_data = access_ctx;
 
@@ -488,7 +534,7 @@ int sssm_ad_sudo_init(struct be_ctx *bectx,
     return ad_sudo_init(bectx, id_ctx, ops, pvt_data);
 #else
     DEBUG(SSSDBG_MINOR_FAILURE, "Sudo init handler called but SSSD is "
-                                 "built without sudo support, ignoring\n");
+                                "built without sudo support, ignoring\n");
     return EOK;
 #endif
 }
