@@ -26,17 +26,14 @@
 #include "providers/ad/ad_subdomains.h"
 #include "providers/ad/ad_domain_info.h"
 #include "providers/ad/ad_srv.h"
+#include "providers/ad/ad_common.h"
+
 #include "providers/ldap/sdap_idmap.h"
 #include "util/util_sss_idmap.h"
 #include <ctype.h>
 #include <ndr.h>
 #include <ndr/ndr_nbt.h>
 
-#define AD_AT_OBJECT_SID "objectSID"
-#define AD_AT_DNS_DOMAIN "DnsDomain"
-#define AD_AT_NT_VERSION "NtVer"
-#define AD_AT_NETLOGON "netlogon"
-
 /* Attributes of AD trusted domains */
 #define AD_AT_FLATNAME      "flatName"
 #define AD_AT_SID           "securityIdentifier"
@@ -44,8 +41,6 @@
 #define AD_AT_TRUST_PARTNER "trustPartner"
 #define AD_AT_TRUST_ATTRS   "trustAttributes"
 
-#define MASTER_DOMAIN_SID_FILTER "objectclass=domain"
-
 /* trustType=2 denotes uplevel (NT5 and later) trusted domains. See
  * http://msdn.microsoft.com/en-us/library/windows/desktop/ms680342%28v=vs.85%29.aspx
  * for example.
@@ -92,6 +87,7 @@ struct ad_subdomains_req_ctx {
 
     char *master_sid;
     char *flat_name;
+    char *site;
     char *forest;
 };
 
@@ -106,6 +102,7 @@ ad_subdom_ad_ctx_new(struct be_ctx *be_ctx,
     const char *gc_service_name;
     struct ad_srv_plugin_ctx *srv_ctx;
     char *ad_domain;
+    char *ad_site_override;
     struct sdap_domain *sdom;
     errno_t ret;
     const char *realm;
@@ -126,6 +123,7 @@ ad_subdom_ad_ctx_new(struct be_ctx *be_ctx,
     }
 
     ad_domain = subdom->name;
+    ad_site_override = dp_opt_get_string(ad_options->basic, AD_SITE);
 
     ret = dp_opt_set_string(ad_options->basic, AD_DOMAIN, ad_domain);
     if (ret != EOK) {
@@ -162,7 +160,8 @@ ad_subdom_ad_ctx_new(struct be_ctx *be_ctx,
                                      default_host_dbs,
                                      ad_id_ctx->ad_options->id,
                                      hostname,
-                                     ad_domain);
+                                     ad_domain,
+                                     ad_site_override);
     if (srv_ctx == NULL) {
         DEBUG(SSSDBG_FATAL_FAILURE, "Out of memory?\n");
         return ENOMEM;
@@ -184,6 +183,10 @@ ad_subdom_ad_ctx_new(struct be_ctx *be_ctx,
         return EFAULT;
     }
 
+    sdap_inherit_options(subdom->parent->sd_inherit,
+                         id_ctx->sdap_id_ctx->opts,
+                         ad_id_ctx->sdap_id_ctx->opts);
+
     /* Set up the ID mapping object */
     ad_id_ctx->sdap_id_ctx->opts->idmap_ctx =
         id_ctx->sdap_id_ctx->opts->idmap_ctx;
@@ -465,13 +468,21 @@ static errno_t ad_subdom_reinit(struct ad_subdomains_ctx *ctx)
 {
     errno_t ret;
 
+    ret = sss_write_krb5_conf_snippet(
+                            dp_opt_get_string(ctx->ad_id_ctx->ad_options->basic,
+                                              AD_KRB5_CONFD_PATH));
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE, "sss_write_krb5_conf_snippet failed.\n");
+        /* Just continue */
+    }
+
     ret = sysdb_update_subdomains(ctx->be_ctx->domain);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "sysdb_update_subdomains failed.\n");
         return ret;
     }
 
-    ret = sss_write_domain_mappings(ctx->be_ctx->domain, false);
+    ret = sss_write_domain_mappings(ctx->be_ctx->domain);
     if (ret != EOK) {
         DEBUG(SSSDBG_MINOR_FAILURE, "sss_krb5_write_mappings failed.\n");
         /* Just continue */
@@ -592,7 +603,7 @@ static void ad_subdomains_master_dom_done(struct tevent_req *req)
 
     ret = ad_master_domain_recv(req, ctx,
                                 &ctx->flat_name, &ctx->master_sid,
-                                &ctx->forest);
+                                &ctx->site, &ctx->forest);
     talloc_zfree(req);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "Cannot retrieve master domain info\n");
@@ -709,7 +720,7 @@ static void ad_subdomains_get_root_domain_done(struct tevent_req *req)
     } else if (reply_count > 1) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "Multiple results for root domain search, "
-               "domain list might be incomplete!\n");
+              "domain list might be incomplete!\n");
 
         ctx->root_op = ctx->sdap_op;
         ctx->root_id_ctx = ctx->sd_ctx->ad_id_ctx;
@@ -757,8 +768,7 @@ static void ad_subdomains_get_root_domain_done(struct tevent_req *req)
     ctx->root_domain = reply[0];
     ctx->root_id_ctx = ads_get_root_id_ctx(ctx);
     if (ctx->root_id_ctx == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE,
-              "Cannot create id ctx for the root domain\n");
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot create id ctx for the root domain\n");
         ret = EFAULT;
         goto fail;
     }
@@ -774,7 +784,7 @@ static void ad_subdomains_get_root_domain_done(struct tevent_req *req)
     req = sdap_id_op_connect_send(ctx->root_op, ctx, &ret);
     if (req == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "sdap_id_op_connect_send failed: %d(%s).\n",
-                                   ret, strerror(ret));
+                                  ret, strerror(ret));
         goto fail;
     }
 
@@ -804,8 +814,8 @@ static struct ad_id_ctx *ads_get_root_id_ctx(struct ad_subdomains_req_ctx *ctx)
     }
 
     /* With a subsequent run, the root should already be known */
-    root = find_subdomain_by_name(ctx->sd_ctx->be_ctx->domain,
-                                  name, false);
+    root = find_domain_by_name(ctx->sd_ctx->be_ctx->domain,
+                               name, false);
     if (root == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "Could not find the root domain\n");
         return NULL;
@@ -849,7 +859,7 @@ static void ad_subdomains_root_conn_done(struct tevent_req *req)
         if (dp_error == DP_ERR_OFFLINE) {
             DEBUG(SSSDBG_MINOR_FAILURE,
                   "No AD server is available, cannot get the "
-                   "subdomain list while offline\n");
+                  "subdomain list while offline\n");
         } else {
             DEBUG(SSSDBG_OP_FAILURE,
                   "Failed to connect to AD server: [%d](%s)\n",
@@ -1028,7 +1038,7 @@ static void ad_subdomains_get_slave_domain_done(struct tevent_req *req)
                                 ctx->reply_count, ctx->reply,
                                 ctx->root_domain, &nsubdoms, &subdoms);
     if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "Cannot process subdomain list\n");
+        DEBUG(SSSDBG_OP_FAILURE, ("Cannot process subdomain list\n"));
         tevent_req_error(req, ret);
         return;
     }
@@ -1180,12 +1190,12 @@ int ad_subdom_init(struct be_ctx *be_ctx,
 
     ret = be_add_online_cb(ctx, be_ctx, ad_subdom_online_cb, ctx, NULL);
     if (ret != EOK) {
-        DEBUG(SSSDBG_MINOR_FAILURE, "Failed to add subdom online callback");
+        DEBUG(SSSDBG_MINOR_FAILURE, "Failed to add subdom online callback\n");
     }
 
     ret = be_add_offline_cb(ctx, be_ctx, ad_subdom_offline_cb, ctx, NULL);
     if (ret != EOK) {
-        DEBUG(SSSDBG_MINOR_FAILURE, "Failed to add subdom offline callback");
+        DEBUG(SSSDBG_MINOR_FAILURE, "Failed to add subdom offline callback\n");
     }
 
     err = sss_idmap_init(sss_idmap_talloc, ctx, sss_idmap_talloc_free,
