@@ -32,12 +32,13 @@
 #include <sys/types.h>
 #include <keyutils.h>
 #endif
-#include "talloc.h"
-#include "tevent.h"
-#include "ldb.h"
+#include <talloc.h>
+#include <tevent.h>
+#include <ldb.h>
+#include <dbus/dbus.h>
+
 #include "util/util.h"
 #include "confdb/confdb.h"
-#include "dbus/dbus.h"
 #include "sbus/sssd_dbus.h"
 #include "sbus/sbus_client.h"
 #include "sss_client/sss_cli.h"
@@ -126,6 +127,7 @@
 #define BE_FILTER_IDNUM 2
 #define BE_FILTER_ENUM 3
 #define BE_FILTER_SECID 4
+#define BE_FILTER_UUID 5
 
 #define BE_REQ_USER          0x0001
 #define BE_REQ_GROUP         0x0002
@@ -138,6 +140,7 @@
 #define BE_REQ_HOST          0x0010
 #define BE_REQ_BY_SECID      0x0011
 #define BE_REQ_USER_AND_GROUP 0x0012
+#define BE_REQ_BY_UUID      0x0013
 #define BE_REQ_TYPE_MASK     0x00FF
 #define BE_REQ_FAST          0x1000
 
@@ -146,6 +149,9 @@
  * length */
 #define DP_SEC_ID_LEN (sizeof(DP_SEC_ID) - 1)
 
+#define EXTRA_NAME_IS_UPN "U"
+#define EXTRA_INPUT_MAYBE_WITH_VIEW "V"
+
 /* AUTH related common data and functions */
 
 #define DEBUG_PAM_DATA(level, pd) do { \
@@ -169,9 +175,12 @@ struct pam_data {
     char *tty;
     char *ruser;
     char *rhost;
+    char **requested_domains;
     struct sss_auth_token *authtok;
     struct sss_auth_token *newauthtok;
     uint32_t cli_pid;
+    char *logon_name;
+    bool name_is_upn;
 
     int pam_status;
     int response_delay;
@@ -268,6 +277,11 @@ struct dp_option {
 
 #define DP_OPTION_TERMINATOR { NULL, 0, NULL_STRING, NULL_STRING }
 
+void dp_option_inherit(char **inherit_opt_list,
+                       int option,
+                       struct dp_option *parent_opts,
+                       struct dp_option *subdom_opts);
+
 int dp_get_options(TALLOC_CTX *memctx,
                    struct confdb_ctx *cdb,
                    const char *conf_path,
