@@ -30,15 +30,15 @@
 #include <sys/time.h>
 #include <errno.h>
 #include <dlfcn.h>
+#include <popt.h>
+#include <dbus/dbus.h>
 
 #include <security/pam_appl.h>
 #include <security/pam_modules.h>
 
-#include "popt.h"
 #include "util/util.h"
 #include "confdb/confdb.h"
 #include "db/sysdb.h"
-#include "dbus/dbus.h"
 #include "sbus/sssd_dbus.h"
 #include "providers/dp_backend.h"
 #include "providers/fail_over.h"
@@ -105,6 +105,16 @@ static struct bet_data bet_data[] = {
     {BET_MAX, NULL, NULL}
 };
 
+struct bet_queue_item {
+    struct bet_queue_item *prev;
+    struct bet_queue_item *next;
+
+    TALLOC_CTX *mem_ctx;
+    struct be_req *be_req;
+    be_req_fn_t fn;
+
+};
+
 #define REQ_PHASE_ACCESS 0
 #define REQ_PHASE_SELINUX 1
 
@@ -161,7 +171,7 @@ static errno_t be_req_set_domain(struct be_req *be_req, const char *domain)
 {
     struct sss_domain_info *dom = NULL;
 
-    dom = find_subdomain_by_name(be_req->be_ctx->domain, domain, true);
+    dom = find_domain_by_name(be_req->be_ctx->domain, domain, true);
     if (dom == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Unknown domain [%s]!\n", domain);
         return ERR_DOMAIN_NOT_FOUND;
@@ -470,7 +480,7 @@ try_to_go_online(TALLOC_CTX *mem_ctx,
 
 void be_mark_offline(struct be_ctx *ctx)
 {
-    int offline_timeout = 60;
+    int offline_timeout;
     errno_t ret;
 
     DEBUG(SSSDBG_TRACE_INTERNAL, "Going offline!\n");
@@ -482,6 +492,16 @@ void be_mark_offline(struct be_ctx *ctx)
     if (ctx->check_if_online_ptask == NULL) {
         /* This is the first time we go offline - create a periodic task
          * to check if we can switch to online. */
+        DEBUG(SSSDBG_TRACE_INTERNAL, "Initialize check_if_online_ptask.\n");
+        ret = confdb_get_int(ctx->cdb, ctx->conf_path,
+                             CONFDB_DOMAIN_OFFLINE_TIMEOUT, 60,
+                             &offline_timeout);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "Failed to get offline_timeout from confdb. "
+                  "Will use 60 seconds.\n");
+            offline_timeout = 60;
+        }
         ret = be_ptask_create_sync(ctx, ctx,
                                    offline_timeout, offline_timeout,
                                    offline_timeout, 30, offline_timeout,
@@ -593,7 +613,6 @@ static int be_get_subdomains(struct sbus_request *dbus_req, void *user_data)
     struct be_subdom_req *req;
     struct be_req *be_req = NULL;
     struct be_client *becli;
-    dbus_bool_t force;
     char *domain_hint;
     dbus_uint16_t err_maj;
     dbus_uint32_t err_min;
@@ -604,7 +623,6 @@ static int be_get_subdomains(struct sbus_request *dbus_req, void *user_data)
     if (!becli) return EINVAL;
 
     if (!sbus_request_parse_or_finish(dbus_req,
-                                      DBUS_TYPE_BOOLEAN, &force,
                                       DBUS_TYPE_STRING, &domain_hint,
                                       DBUS_TYPE_INVALID))
         return EOK; /* handled */
@@ -618,9 +636,8 @@ static int be_get_subdomains(struct sbus_request *dbus_req, void *user_data)
         goto immediate;
     }
 
-    DEBUG(SSSDBG_TRACE_FUNC,
-          "Got get subdomains [%sforced][%s]\n", force ? "" : "not ",
-          domain_hint == NULL ? "no hint": domain_hint );
+    DEBUG(SSSDBG_TRACE_FUNC, "Got get subdomains [%s]\n",
+                             domain_hint == NULL ? "no hint": domain_hint );
 
     /* If we are offline return immediately
      */
@@ -650,7 +667,6 @@ static int be_get_subdomains(struct sbus_request *dbus_req, void *user_data)
         err_msg = "Out of memory";
         goto immediate;
     }
-    req->force = force;
     req->domain_hint = talloc_strdup(req, domain_hint);
     if (!req->domain_hint) {
         err_maj = DP_ERR_FATAL;
@@ -846,8 +862,8 @@ static errno_t be_initgroups_prereq(struct be_req *be_req)
     const char *tmpstr;
     int i;
 
-    ret = sysdb_initgroups(be_req, be_req->be_ctx->domain->sysdb,
-                           be_req->be_ctx->domain, ar->filter_value, &res);
+    ret = sysdb_initgroups(be_req, be_req->be_ctx->domain, ar->filter_value,
+                           &res);
     if (ret && ret != ENOENT) {
         return ret;
     }
@@ -910,7 +926,7 @@ be_file_account_request(struct be_req *be_req, struct be_acct_req *ar)
     if ((ar->entry_type & 0xFF) == BE_REQ_INITGROUPS) {
         ret = be_initgroups_prereq(be_req);
         if (ret) {
-            DEBUG(SSSDBG_CRIT_FAILURE, "Prerequest failed");
+            DEBUG(SSSDBG_CRIT_FAILURE, "Prerequest failed\n");
             return ret;
         }
     }
@@ -919,7 +935,7 @@ be_file_account_request(struct be_req *be_req, struct be_acct_req *ar)
     ret = be_file_request(be_ctx, be_req,
                           be_ctx->bet_info[BET_ID].bet_ops->handler);
     if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Failed to file request");
+        DEBUG(SSSDBG_CRIT_FAILURE, "Failed to file request\n");
         return ret;
     }
 
@@ -1086,8 +1102,8 @@ static int be_get_account_info(struct sbus_request *dbus_req, void *user_data)
                                       DBUS_TYPE_INVALID))
         return EOK; /* handled */
 
-    DEBUG(SSSDBG_CONF_SETTINGS,
-          "Got request for [%u][%d][%s]\n", type, attr_type, filter);
+    DEBUG(SSSDBG_FUNC_DATA,
+          "Got request for [%#x][%d][%s]\n", type, attr_type, filter);
 
     /* If we are offline and fast reply was requested
      * return offline immediately
@@ -1331,7 +1347,7 @@ static int be_pam_handler(struct sbus_request *dbus_req, void *user_data)
 
     ret = dp_unpack_pam_request(dbus_req->message, be_req, &pd, &dbus_error);
     if (!ret) {
-        DEBUG(SSSDBG_CRIT_FAILURE,"Failed, to parse message!\n");
+        DEBUG(SSSDBG_CRIT_FAILURE, "Failed to parse message!\n");
         talloc_free(be_req);
         return EIO;
     }
@@ -1737,13 +1753,6 @@ static int be_autofs_handler(struct sbus_request *dbus_req, void *user_data)
         goto done;
     }
 
-    /* If a request for auto.master comes in, the automounter deamon
-     * has been reloaded. Expire all autofs maps to force reload
-     */
-    if (strcmp(be_autofs_req->mapname, "auto.master") == 0) {
-        be_autofs_req->invalidate = true;
-    }
-
     be_req->req_data = be_autofs_req;
 
     if (!be_cli->bectx->bet_info[BET_AUTOFS].bet_ops) {
@@ -2228,6 +2237,9 @@ static int be_client_init(struct sbus_connection *conn, void *data)
     becli->conn = conn;
     becli->initialized = false;
 
+    /* Allow access from the SSSD user */
+    sbus_allow_uid(conn, &bectx->uid);
+
     /* 5 seconds should be plenty */
     tv = tevent_timeval_current_ofs(5, 0);
 
@@ -2253,7 +2265,8 @@ static int be_client_init(struct sbus_connection *conn, void *data)
 
 /* be_srv_init
  * set up per-domain sbus channel */
-static int be_srv_init(struct be_ctx *ctx)
+static int be_srv_init(struct be_ctx *ctx,
+                       uid_t uid, gid_t gid)
 {
     char *sbus_address;
     int ret;
@@ -2265,7 +2278,10 @@ static int be_srv_init(struct be_ctx *ctx)
         return ret;
     }
 
-    ret = sbus_new_server(ctx, ctx->ev, sbus_address,
+    ctx->uid = uid;
+    ctx->gid = gid;
+
+    ret = sbus_new_server(ctx, ctx->ev, sbus_address, uid, gid,
                           true, &ctx->sbus_srv, be_client_init, ctx);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE, "Could not set up sbus server.\n");
@@ -2481,6 +2497,17 @@ static void signal_be_offline(struct tevent_context *ev,
     be_mark_offline(ctx);
 }
 
+static void signal_be_reset_offline(struct tevent_context *ev,
+                                    struct tevent_signal *se,
+                                    int signum,
+                                    int count,
+                                    void *siginfo,
+                                    void *private_data)
+{
+    struct be_ctx *ctx = talloc_get_type(private_data, struct be_ctx);
+    check_if_online(ctx);
+}
+
 int be_process_init_sudo(struct be_ctx *be_ctx)
 {
     TALLOC_CTX *tmp_ctx = NULL;
@@ -2545,6 +2572,7 @@ done:
 
 int be_process_init(TALLOC_CTX *mem_ctx,
                     const char *be_domain,
+                    uid_t uid, gid_t gid,
                     struct tevent_context *ev,
                     struct confdb_ctx *cdb)
 {
@@ -2600,7 +2628,7 @@ int be_process_init(TALLOC_CTX *mem_ctx,
         goto fail;
     }
 
-    ret = be_srv_init(ctx);
+    ret = be_srv_init(ctx, uid, gid);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE, "fatal error setting up server bus\n");
         goto fail;
@@ -2769,6 +2797,15 @@ int be_process_init(TALLOC_CTX *mem_ctx,
         goto fail;
     }
 
+    /* Handle SIGUSR2 to force going online */
+    BlockSignals(false, SIGUSR2);
+    tes = tevent_add_signal(ctx->ev, ctx, SIGUSR2, 0,
+                            signal_be_reset_offline, ctx);
+    if (tes == NULL) {
+        ret = EIO;
+        goto fail;
+    }
+
     return EOK;
 
 fail:
@@ -2786,10 +2823,13 @@ int main(int argc, const char *argv[])
     struct main_context *main_ctx;
     char *confdb_path;
     int ret;
+    uid_t uid;
+    gid_t gid;
 
     struct poptOption long_options[] = {
         POPT_AUTOHELP
         SSSD_MAIN_OPTS
+        SSSD_SERVER_OPTS(uid, gid)
         {"domain", 0, POPT_ARG_STRING, &be_domain, 0,
          _("Domain of the information provider (mandatory)"), NULL },
         POPT_TABLEEND
@@ -2829,12 +2869,18 @@ int main(int argc, const char *argv[])
     confdb_path = talloc_asprintf(NULL, CONFDB_DOMAIN_PATH_TMPL, be_domain);
     if (!confdb_path) return 2;
 
-    ret = server_setup(srv_name, 0, confdb_path, &main_ctx);
+    ret = server_setup(srv_name, 0, 0, 0, confdb_path, &main_ctx);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE, "Could not set up mainloop [%d]\n", ret);
         return 2;
     }
 
+    ret = setenv(SSS_DOM_ENV, be_domain, 1);
+    if (ret != 0) {
+        DEBUG(SSSDBG_MINOR_FAILURE, "Setting "SSS_DOM_ENV" failed, journald "
+              "logging mightnot work as expected\n");
+    }
+
     ret = die_if_parent_died();
     if (ret != EOK) {
         /* This is not fatal, don't return */
@@ -2843,7 +2889,7 @@ int main(int argc, const char *argv[])
     }
 
     ret = be_process_init(main_ctx,
-                          be_domain,
+                          be_domain, uid, gid,
                           main_ctx->event_ctx,
                           main_ctx->confdb_ctx);
     if (ret != EOK) {
@@ -2851,6 +2897,19 @@ int main(int argc, const char *argv[])
         return 3;
     }
 
+    ret = chown_debug_file(NULL, uid, gid);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "Cannot chown the debug files, debugging might not work!\n");
+    }
+
+    ret = become_user(uid, gid);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FUNC_DATA,
+              "Cannot become user [%"SPRIuid"][%"SPRIgid"].\n", uid, gid);
+        return ret;
+    }
+
     DEBUG(SSSDBG_TRACE_FUNC, "Backend provider (%s) started!\n", be_domain);
 
     /* loop on main */
@@ -2892,7 +2951,7 @@ static int data_provider_logrotate(struct sbus_request *dbus_req, void *data)
     errno_t ret;
     struct be_ctx *be_ctx = talloc_get_type(data, struct be_ctx);
 
-    ret = monitor_common_rotate_logs(be_ctx->cdb, be_ctx->conf_path);
+    ret = server_common_rotate_logs(be_ctx->cdb, be_ctx->conf_path);
     if (ret != EOK) return ret;
 
     return sbus_request_return_and_finish(dbus_req, DBUS_TYPE_INVALID);
