@@ -147,55 +147,16 @@ fail:
 static bool
 ok_for_dns(struct sockaddr *sa)
 {
-    char straddr[INET6_ADDRSTRLEN];
-    struct in6_addr *addr6;
-    struct in_addr *addr;
+    struct sockaddr_in sa4;
+    struct sockaddr_in6 sa6;
 
     switch (sa->sa_family) {
     case AF_INET6:
-        addr6 = &((struct sockaddr_in6 *) sa)->sin6_addr;
-
-        if (inet_ntop(AF_INET6, addr6, straddr, INET6_ADDRSTRLEN) == NULL) {
-            DEBUG(SSSDBG_MINOR_FAILURE,
-                  "inet_ntop failed, won't log IP addresses\n");
-            snprintf(straddr, INET6_ADDRSTRLEN, "unknown");
-        }
-
-        if (IN6_IS_ADDR_LINKLOCAL(addr6)) {
-            DEBUG(SSSDBG_FUNC_DATA, "Link local IPv6 address %s\n", straddr);
-            return false;
-        } else if (IN6_IS_ADDR_LOOPBACK(addr6)) {
-            DEBUG(SSSDBG_FUNC_DATA, "Loopback IPv6 address %s\n", straddr);
-            return false;
-        } else if (IN6_IS_ADDR_MULTICAST(addr6)) {
-            DEBUG(SSSDBG_FUNC_DATA, "Multicast IPv6 address %s\n", straddr);
-            return false;
-        }
-        break;
+        memcpy(&sa6, sa, sizeof(struct sockaddr_in6));
+        return check_ipv6_addr(&sa6.sin6_addr, SSS_NO_SPECIAL);
     case AF_INET:
-        addr = &((struct sockaddr_in *) sa)->sin_addr;
-
-        if (inet_ntop(AF_INET, addr, straddr, INET6_ADDRSTRLEN) == NULL) {
-            DEBUG(SSSDBG_MINOR_FAILURE,
-                  "inet_ntop failed, won't log IP addresses\n");
-            snprintf(straddr, INET6_ADDRSTRLEN, "unknown");
-        }
-
-        if (IN_MULTICAST(ntohl(addr->s_addr))) {
-            DEBUG(SSSDBG_FUNC_DATA, "Multicast IPv4 address %s\n", straddr);
-            return false;
-        } else if (inet_netof(*addr) == IN_LOOPBACKNET) {
-            DEBUG(SSSDBG_FUNC_DATA, "Loopback IPv4 address %s\n", straddr);
-            return false;
-        } else if ((addr->s_addr & htonl(0xffff0000)) == htonl(0xa9fe0000)) {
-            /* 169.254.0.0/16 */
-            DEBUG(SSSDBG_FUNC_DATA, "Link-local IPv4 address %s\n", straddr);
-            return false;
-        } else if (addr->s_addr == htonl(INADDR_BROADCAST)) {
-            DEBUG(SSSDBG_FUNC_DATA, "Broadcast IPv4 address %s\n", straddr);
-            return false;
-        }
-        break;
+        memcpy(&sa4, sa, sizeof(struct sockaddr_in));
+        return check_ipv4_addr(&sa4.sin_addr, SSS_NO_SPECIAL);
     default:
         DEBUG(SSSDBG_CRIT_FAILURE, "Unknown address family\n");
         return false;
@@ -377,7 +338,9 @@ nsupdate_msg_add_ptr(char *update_msg, struct sss_iface_addr *addresses,
 
         /* example: update delete 38.78.16.10.in-addr.arpa. in PTR */
         update_msg = talloc_asprintf_append(update_msg,
-                                            "update delete %s in PTR\n", strptr);
+                                            "update delete %s in PTR\n"
+                                            "send\n",
+                                            strptr);
         talloc_free(strptr);
         if (update_msg == NULL) {
             return NULL;
@@ -406,7 +369,8 @@ nsupdate_msg_add_ptr(char *update_msg, struct sss_iface_addr *addresses,
 
         /* example: update delete 38.78.16.10.in-addr.arpa. in PTR */
         update_msg = talloc_asprintf_append(update_msg,
-                                            "update add %s %d in PTR %s.\n",
+                                            "update add %s %d in PTR %s.\n"
+                                            "send\n",
                                             strptr, ttl, hostname);
         talloc_free(strptr);
         if (update_msg == NULL) {
@@ -414,7 +378,7 @@ nsupdate_msg_add_ptr(char *update_msg, struct sss_iface_addr *addresses,
         }
     }
 
-    return talloc_asprintf_append(update_msg, "send\n");
+    return update_msg;
 }
 
 static char *
@@ -431,7 +395,7 @@ nsupdate_msg_create_common(TALLOC_CTX *mem_ctx, const char *realm,
 #ifdef HAVE_NSUPDATE_REALM
     realm_directive = talloc_asprintf(tmp_ctx, "realm %s\n", realm);
 #else
-    realm_directive = talloc_asprintf(tmp_ctx, "");
+    realm_directive = talloc_asprintf(tmp_ctx, "\n");
 #endif
     if (!realm_directive) {
         goto fail;
@@ -442,7 +406,7 @@ nsupdate_msg_create_common(TALLOC_CTX *mem_ctx, const char *realm,
      */
     if (servername) {
         DEBUG(SSSDBG_FUNC_DATA,
-              "Creating update message for server [%s] and realm [%s]\n.",
+              "Creating update message for server [%s] and realm [%s].\n",
                servername, realm);
 
         /* Add the server, realm and headers */
@@ -512,10 +476,10 @@ be_nsupdate_create_fwd_msg(TALLOC_CTX *mem_ctx, const char *realm,
     }
 
     DEBUG(SSSDBG_TRACE_FUNC,
-          " -- Begin nsupdate message -- \n%s",
-           update_msg);
-    DEBUG(SSSDBG_TRACE_FUNC,
-          " -- End nsupdate message -- \n");
+          " -- Begin nsupdate message -- \n"
+          "%s"
+          " -- End nsupdate message -- \n",
+          update_msg);
 
     ret = ERR_OK;
     *_update_msg = talloc_steal(mem_ctx, update_msg);
@@ -554,10 +518,10 @@ be_nsupdate_create_ptr_msg(TALLOC_CTX *mem_ctx, const char *realm,
     }
 
     DEBUG(SSSDBG_TRACE_FUNC,
-          " -- Begin nsupdate message -- \n%s",
-           update_msg);
-    DEBUG(SSSDBG_TRACE_FUNC,
-          " -- End nsupdate message -- \n");
+          " -- Begin nsupdate message -- \n"
+          "%s"
+          " -- End nsupdate message -- \n",
+          update_msg);
 
     ret = ERR_OK;
     *_update_msg = talloc_steal(mem_ctx, update_msg);
