@@ -89,7 +89,7 @@ void ipa_access_handler(struct be_req *be_req)
     dom = be_ctx->domain;
     if (strcasecmp(pd->domain, be_ctx->domain->name) != 0) {
         /* Subdomain request, verify subdomain */
-        dom = find_subdomain_by_name(be_ctx->domain, pd->domain, true);
+        dom = find_domain_by_name(be_ctx->domain, pd->domain, true);
     }
 
     /* First, verify that this account isn't locked.
@@ -178,6 +178,10 @@ static void ipa_hbac_check(struct tevent_req *req)
         hbac_ctx->get_deny_rules = false;
     } else {
         hbac_ctx->get_deny_rules = true;
+        sss_log(SSS_LOG_NOTICE,
+                "WARNING: Using deny rules is deprecated, the option "
+                "ipa_hbac_treat_deny_as will be removed in the next "
+                "upstream version\n");
     }
 
     ret = hbac_retry(hbac_ctx);
@@ -357,6 +361,12 @@ static int hbac_get_host_info_step(struct hbac_ctx *hbac_ctx)
          *    we want all hosts
          */
         hostname = NULL;
+
+        /* THIS FEATURE IS DEPRECATED */
+        DEBUG(SSSDBG_MINOR_FAILURE, "WARNING: Using deprecated option "
+                    "ipa_hbac_support_srchost.\n");
+        sss_log(SSS_LOG_NOTICE, "WARNING: Using deprecated option "
+                    "ipa_hbac_support_srchost.\n");
     } else {
         hostname = dp_opt_get_string(hbac_ctx->ipa_options, IPA_HOSTNAME);
     }
@@ -510,8 +520,7 @@ static void hbac_sysdb_save(struct tevent_req *req)
         /* Delete any rules in the sysdb so offline logins
          * are also denied.
          */
-        base_dn = sysdb_custom_subtree_dn(domain->sysdb, tmp_ctx,
-                                          domain, HBAC_RULES_SUBDIR);
+        base_dn = sysdb_custom_subtree_dn(tmp_ctx, domain, HBAC_RULES_SUBDIR);
         if (base_dn == NULL) {
             talloc_free(tmp_ctx);
             ipa_access_reply(hbac_ctx, PAM_SYSTEM_ERR);
@@ -702,11 +711,11 @@ errno_t hbac_get_cached_rules(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    ret = sysdb_search_custom(tmp_ctx, domain->sysdb, domain, filter,
+    ret = sysdb_search_custom(tmp_ctx, domain, filter,
                               HBAC_RULES_SUBDIR, attrs,
                               &rule_count, &msgs);
     if (ret != EOK && ret != ENOENT) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Error looking up HBAC rules");
+        DEBUG(SSSDBG_CRIT_FAILURE, "Error looking up HBAC rules\n");
         goto done;
     } if (ret == ENOENT) {
        rule_count = 0;
