@@ -29,6 +29,7 @@
 #include "providers/ldap/ldap_common.h"
 #include "providers/ldap/sdap_async.h"
 #include "providers/krb5/krb5_auth.h"
+#include "providers/ipa/ipa_auth.h"
 #include "providers/ipa/ipa_common.h"
 #include "providers/ipa/ipa_config.h"
 
@@ -156,7 +157,6 @@ done:
 }
 
 static int get_password_migration_flag_recv(struct tevent_req *req,
-                                            TALLOC_CTX *mem_ctx,
                                             bool *password_migration)
 {
     struct get_password_migration_flag_state *state = tevent_req_data(req,
@@ -289,8 +289,7 @@ static void ipa_get_migration_flag_done(struct tevent_req *req)
     int ret;
     int dp_err = DP_ERR_FATAL;
 
-    ret = get_password_migration_flag_recv(req, state,
-                                           &state->password_migration);
+    ret = get_password_migration_flag_recv(req, &state->password_migration);
     talloc_zfree(req);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "get_password_migration_flag "
@@ -354,9 +353,8 @@ static void ipa_migration_flag_connect_done(struct tevent_req *req)
     attrs[0] = SYSDB_ORIG_DN;
     attrs[1] = NULL;
 
-    ret = sysdb_search_user_by_name(state, be_ctx->domain->sysdb,
-                                    be_ctx->domain,
-                                    state->pd->user, attrs, &user_msg);
+    ret = sysdb_search_user_by_name(state, be_ctx->domain, state->pd->user,
+                                    attrs, &user_msg);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "sysdb_search_user_by_name failed.\n");
         goto done;
