@@ -44,8 +44,7 @@ ipa_hbac_save_list(struct sss_domain_info *domain,
     }
 
     if (delete_subdir) {
-        base_dn = sysdb_custom_subtree_dn(domain->sysdb, tmp_ctx,
-                                          domain, subdir);
+        base_dn = sysdb_custom_subtree_dn(tmp_ctx, domain, subdir);
         if (base_dn == NULL) {
             ret = ENOMEM;
             goto done;
@@ -78,8 +77,7 @@ ipa_hbac_save_list(struct sss_domain_info *domain,
         }
         DEBUG(SSSDBG_TRACE_ALL, "Object name: [%s].\n", object_name);
 
-        ret = sysdb_store_custom(domain->sysdb, domain,
-                                 object_name, subdir, list[c]);
+        ret = sysdb_store_custom(domain, object_name, subdir, list[c]);
         if (ret != EOK) {
             DEBUG(SSSDBG_CRIT_FAILURE, "sysdb_store_custom failed.\n");
             goto done;
@@ -403,23 +401,23 @@ done:
 
 static errno_t
 hbac_eval_user_element(TALLOC_CTX *mem_ctx,
-                       struct sysdb_ctx *sysdb,
                        struct sss_domain_info *domain,
                        const char *username,
+                       bool deny_rules,
                        struct hbac_request_element **user_element);
 
 static errno_t
 hbac_eval_service_element(TALLOC_CTX *mem_ctx,
-                          struct sysdb_ctx *sysdb,
                           struct sss_domain_info *domain,
                           const char *servicename,
+                          bool deny_rules,
                           struct hbac_request_element **svc_element);
 
 static errno_t
 hbac_eval_host_element(TALLOC_CTX *mem_ctx,
-                       struct sysdb_ctx *sysdb,
                        struct sss_domain_info *domain,
                        const char *hostname,
+                       bool deny_rules,
                        struct hbac_request_element **host_element);
 
 static errno_t
@@ -451,23 +449,26 @@ hbac_ctx_to_eval_request(TALLOC_CTX *mem_ctx,
     /* Get user the user name and groups,
      * take care of subdomain users as well */
     if (strcasecmp(pd->domain, domain->name) != 0) {
-        user_dom = find_subdomain_by_name(domain, pd->domain, true);
+        user_dom = find_domain_by_name(domain, pd->domain, true);
         if (user_dom == NULL) {
-            DEBUG(SSSDBG_OP_FAILURE, "find_subdomain_by_name failed.\n");
+            DEBUG(SSSDBG_OP_FAILURE, "find_domain_by_name failed.\n");
             ret = ENOMEM;
             goto done;
         }
-        ret = hbac_eval_user_element(eval_req, user_dom->sysdb, user_dom,
-                                     pd->user, &eval_req->user);
+        ret = hbac_eval_user_element(eval_req, user_dom, pd->user,
+                                     hbac_ctx->get_deny_rules,
+                                     &eval_req->user);
     } else {
-        ret = hbac_eval_user_element(eval_req, domain->sysdb, domain,
-                                     pd->user, &eval_req->user);
+        ret = hbac_eval_user_element(eval_req, domain, pd->user,
+                                     hbac_ctx->get_deny_rules,
+                                     &eval_req->user);
     }
     if (ret != EOK) goto done;
 
     /* Get the PAM service and service groups */
-    ret = hbac_eval_service_element(eval_req, domain->sysdb, domain,
-                                    pd->service, &eval_req->service);
+    ret = hbac_eval_service_element(eval_req, domain, pd->service,
+                                    hbac_ctx->get_deny_rules,
+                                    &eval_req->service);
     if (ret != EOK) goto done;
 
     /* Get the source host */
@@ -482,8 +483,9 @@ hbac_ctx_to_eval_request(TALLOC_CTX *mem_ctx,
         rhost = pd->rhost;
     }
 
-    ret = hbac_eval_host_element(eval_req, domain->sysdb, domain,
-                                 rhost, &eval_req->srchost);
+    ret = hbac_eval_host_element(eval_req, domain, rhost,
+                                 hbac_ctx->get_deny_rules,
+                                 &eval_req->srchost);
     if (ret != EOK) goto done;
 
     /* The target host is always the current machine */
@@ -495,8 +497,9 @@ hbac_ctx_to_eval_request(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    ret = hbac_eval_host_element(eval_req, domain->sysdb, domain,
-                                 thost, &eval_req->targethost);
+    ret = hbac_eval_host_element(eval_req, domain, thost,
+                                 hbac_ctx->get_deny_rules,
+                                 &eval_req->targethost);
     if (ret != EOK) goto done;
 
     *request = talloc_steal(mem_ctx, eval_req);
@@ -510,9 +513,9 @@ done:
 
 static errno_t
 hbac_eval_user_element(TALLOC_CTX *mem_ctx,
-                       struct sysdb_ctx *sysdb,
                        struct sss_domain_info *domain,
                        const char *username,
+                       bool deny_rules,
                        struct hbac_request_element **user_element)
 {
     errno_t ret;
@@ -540,8 +543,8 @@ hbac_eval_user_element(TALLOC_CTX *mem_ctx,
      * This will give us the list of both POSIX and
      * non-POSIX groups that this user belongs to.
      */
-    ret = sysdb_search_user_by_name(tmp_ctx, sysdb, domain,
-                                    users->name, attrs, &msg);
+    ret = sysdb_search_user_by_name(tmp_ctx, domain, users->name,
+                                    attrs, &msg);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "Could not determine user memberships for [%s]\n",
@@ -567,11 +570,18 @@ hbac_eval_user_element(TALLOC_CTX *mem_ctx,
     for (i = 0; i < el->num_values; i++) {
         member_dn = (const char *)el->values[i].data;
 
-        ret = get_ipa_groupname(users->groups, sysdb, member_dn,
+        ret = get_ipa_groupname(users->groups, domain->sysdb, member_dn,
                                 &users->groups[num_groups]);
-        if (ret != EOK && ret != ENOENT) {
-            DEBUG(SSSDBG_MINOR_FAILURE, "Parse error on [%s]\n", member_dn);
-            goto done;
+        if (ret != EOK && ret != ERR_UNEXPECTED_ENTRY_TYPE) {
+            if (deny_rules) {
+                DEBUG(SSSDBG_OP_FAILURE, "Parse error on [%s]: %s\n",
+                      member_dn, sss_strerror(ret));
+                goto done;
+            } else {
+                DEBUG(SSSDBG_MINOR_FAILURE,
+                      "Skipping malformed entry [%s]\n", member_dn);
+                continue;
+            }
         } else if (ret == EOK) {
             DEBUG(SSSDBG_TRACE_LIBS, "Added group [%s] for user [%s]\n",
                       users->groups[num_groups], users->name);
@@ -605,9 +615,9 @@ done:
 
 static errno_t
 hbac_eval_service_element(TALLOC_CTX *mem_ctx,
-                          struct sysdb_ctx *sysdb,
                           struct sss_domain_info *domain,
                           const char *servicename,
+                          bool deny_rules,
                           struct hbac_request_element **svc_element)
 {
     errno_t ret;
@@ -631,15 +641,14 @@ hbac_eval_service_element(TALLOC_CTX *mem_ctx,
 
     svc->name = servicename;
 
-    svc_dn = sysdb_custom_dn(sysdb, tmp_ctx,
-                             domain, svc->name, HBAC_SERVICES_SUBDIR);
+    svc_dn = sysdb_custom_dn(tmp_ctx, domain, svc->name, HBAC_SERVICES_SUBDIR);
     if (svc_dn == NULL) {
         ret = ENOMEM;
         goto done;
     }
 
     /* Look up the service to get its originalMemberOf entries */
-    ret = sysdb_search_entry(tmp_ctx, sysdb, svc_dn,
+    ret = sysdb_search_entry(tmp_ctx, domain->sysdb, svc_dn,
                              LDB_SCOPE_BASE, NULL,
                              memberof_attrs,
                              &count, &msgs);
@@ -676,12 +685,23 @@ hbac_eval_service_element(TALLOC_CTX *mem_ctx,
     }
 
     for (i = j = 0; i < el->num_values; i++) {
-        ret = get_ipa_servicegroupname(tmp_ctx, sysdb,
+        ret = get_ipa_servicegroupname(tmp_ctx, domain->sysdb,
                                        (const char *)el->values[i].data,
                                        &name);
-        if (ret != EOK && ret != ENOENT) goto done;
+        if (ret != EOK && ret != ERR_UNEXPECTED_ENTRY_TYPE) {
+            if (deny_rules) {
+                DEBUG(SSSDBG_OP_FAILURE, "Parse error on [%s]: %s\n",
+                                         (const char *)el->values[i].data,
+                                         sss_strerror(ret));
+                goto done;
+            } else {
+                DEBUG(SSSDBG_MINOR_FAILURE, "Skipping malformed entry [%s]\n",
+                                            (const char *)el->values[i].data);
+                continue;
+            }
+        }
 
-        /* ENOENT means we had a memberOf entry that wasn't a
+        /* ERR_UNEXPECTED_ENTRY_TYPE means we had a memberOf entry that wasn't a
          * service group. We'll just ignore those (could be
          * HBAC rules)
          */
@@ -705,9 +725,9 @@ done:
 
 static errno_t
 hbac_eval_host_element(TALLOC_CTX *mem_ctx,
-                       struct sysdb_ctx *sysdb,
                        struct sss_domain_info *domain,
                        const char *hostname,
+                       bool deny_rules,
                        struct hbac_request_element **host_element)
 {
     errno_t ret;
@@ -739,15 +759,14 @@ hbac_eval_host_element(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    host_dn = sysdb_custom_dn(sysdb, tmp_ctx,
-                              domain, host->name, HBAC_HOSTS_SUBDIR);
+    host_dn = sysdb_custom_dn(tmp_ctx, domain, host->name, HBAC_HOSTS_SUBDIR);
     if (host_dn == NULL) {
         ret = ENOMEM;
         goto done;
     }
 
     /* Look up the host to get its originalMemberOf entries */
-    ret = sysdb_search_entry(tmp_ctx, sysdb, host_dn,
+    ret = sysdb_search_entry(tmp_ctx, domain->sysdb, host_dn,
                              LDB_SCOPE_BASE, NULL,
                              memberof_attrs,
                              &count, &msgs);
@@ -784,12 +803,23 @@ hbac_eval_host_element(TALLOC_CTX *mem_ctx,
     }
 
     for (i = j = 0; i < el->num_values; i++) {
-        ret = get_ipa_hostgroupname(tmp_ctx, sysdb,
+        ret = get_ipa_hostgroupname(tmp_ctx, domain->sysdb,
                                     (const char *)el->values[i].data,
                                     &name);
-        if (ret != EOK && ret != ENOENT) goto done;
+        if (ret != EOK && ret != ERR_UNEXPECTED_ENTRY_TYPE) {
+            if (deny_rules) {
+                DEBUG(SSSDBG_OP_FAILURE, "Parse error on [%s]: %s\n",
+                                         (const char *)el->values[i].data,
+                                         sss_strerror(ret));
+                goto done;
+            } else {
+                DEBUG(SSSDBG_MINOR_FAILURE, "Skipping malformed entry [%s]\n",
+                                            (const char *)el->values[i].data);
+                continue;
+            }
+        }
 
-        /* ENOENT means we had a memberOf entry that wasn't a
+        /* ERR_UNEXPECTED_ENTRY_TYPE means we had a memberOf entry that wasn't a
          * host group. We'll just ignore those (could be
          * HBAC rules)
          */
