@@ -225,7 +225,13 @@ int sssm_ipa_id_init(struct be_ctx *bectx,
 
     /* Set up the ID mapping object */
     ret = ipa_idmap_init(sdap_ctx, sdap_ctx, &sdap_ctx->opts->idmap_ctx);
-    if (ret != EOK) goto done;
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              "Could not initialize ID mapping. In case ID mapping properties "
+              "changed on the server, please remove the SSSD database\n");
+        goto done;
+    }
+
 
     ret = ldap_id_setup_tasks(sdap_ctx);
     if (ret != EOK) {
@@ -243,24 +249,28 @@ int sssm_ipa_id_init(struct be_ctx *bectx,
     hostname = dp_opt_get_string(ipa_options->basic, IPA_HOSTNAME);
     server_mode = dp_opt_get_bool(ipa_options->basic, IPA_SERVER_MODE);
 
-    if (dp_opt_get_bool(ipa_options->basic, IPA_ENABLE_DNS_SITES)) {
-        /* use IPA plugin */
-        ipa_domain = dp_opt_get_string(ipa_options->basic, IPA_DOMAIN);
-        srv_ctx = ipa_srv_plugin_ctx_init(bectx, bectx->be_res->resolv,
-                                          hostname, ipa_domain);
-        if (srv_ctx == NULL) {
-            DEBUG(SSSDBG_FATAL_FAILURE, "Out of memory?\n");
+    if (server_mode == true) {
+        ipa_ctx->view_name = talloc_strdup(ipa_ctx, SYSDB_DEFAULT_VIEW_NAME);
+        if (ipa_ctx->view_name == NULL) {
+            DEBUG(SSSDBG_OP_FAILURE, "talloc_strdup failed.\n");
             ret = ENOMEM;
             goto done;
         }
 
-        be_fo_set_srv_lookup_plugin(bectx, ipa_srv_plugin_send,
-                                    ipa_srv_plugin_recv, srv_ctx, "IPA");
-    } else if (server_mode == true) {
+        ret = sysdb_update_view_name(bectx->domain->sysdb, ipa_ctx->view_name);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "Cannot add/update view name to sysdb.\n");
+            goto done;
+        }
+
         ipa_servers = dp_opt_get_string(ipa_options->basic, IPA_SERVER);
-        if (srv_in_server_list(ipa_servers) == true) {
-            DEBUG(SSSDBG_MINOR_FAILURE, "SRV resolution enabled on the IPA server. "
-                  "Site discovery of trusted AD servers might not work\n");
+        if (srv_in_server_list(ipa_servers) == true
+                || dp_opt_get_bool(ipa_options->basic,
+                                   IPA_ENABLE_DNS_SITES) == true) {
+            DEBUG(SSSDBG_MINOR_FAILURE, "SRV resolution or IPA sites enabled "
+                  "on the IPA server. Site discovery of trusted AD servers "
+                  "might not work\n");
 
             /* If SRV discovery is enabled on the server and
              * dns_discovery_domain is set explicitly, then
@@ -306,22 +316,47 @@ int sssm_ipa_id_init(struct be_ctx *bectx,
             }
         }
     } else {
-        /* fall back to standard plugin on clients. */
-        ret = be_fo_set_dns_srv_lookup_plugin(bectx, hostname);
+        ret = sysdb_get_view_name(ipa_ctx, bectx->domain->sysdb,
+                                  &ipa_ctx->view_name);
         if (ret != EOK) {
-            DEBUG(SSSDBG_CRIT_FAILURE, "Unable to set SRV lookup plugin "
-                                        "[%d]: %s\n", ret, strerror(ret));
-            goto done;
+            if (ret == ENOENT) {
+                DEBUG(SSSDBG_CRIT_FAILURE,
+                      "Cannot find view name in the cache. " \
+                      "Will do online lookup later.\n");
+            } else {
+                DEBUG(SSSDBG_OP_FAILURE, "sysdb_get_view_name failed.\n");
+                goto done;
+            }
+        }
+
+        if (dp_opt_get_bool(ipa_options->basic, IPA_ENABLE_DNS_SITES)) {
+            /* use IPA plugin */
+            ipa_domain = dp_opt_get_string(ipa_options->basic, IPA_DOMAIN);
+            srv_ctx = ipa_srv_plugin_ctx_init(bectx, bectx->be_res->resolv,
+                                              hostname, ipa_domain);
+            if (srv_ctx == NULL) {
+                DEBUG(SSSDBG_FATAL_FAILURE, "Out of memory?\n");
+                ret = ENOMEM;
+                goto done;
+            }
+
+            be_fo_set_srv_lookup_plugin(bectx, ipa_srv_plugin_send,
+                                        ipa_srv_plugin_recv, srv_ctx, "IPA");
+        } else {
+            /* fall back to standard plugin on clients. */
+            ret = be_fo_set_dns_srv_lookup_plugin(bectx, hostname);
+            if (ret != EOK) {
+                DEBUG(SSSDBG_CRIT_FAILURE, "Unable to set SRV lookup plugin "
+                                            "[%d]: %s\n", ret, strerror(ret));
+                goto done;
+            }
         }
     }
 
     /* setup periodical refresh of expired records */
-    ret = be_refresh_add_cb(bectx->refresh_ctx, BE_REFRESH_TYPE_NETGROUPS,
-                            sdap_refresh_netgroups_send,
-                            sdap_refresh_netgroups_recv,
-                            sdap_ctx);
+    ret = sdap_refresh_init(bectx->refresh_ctx, sdap_ctx);
     if (ret != EOK && ret != EEXIST) {
-        DEBUG(SSSDBG_MINOR_FAILURE, "Periodical refresh of netgroups "
+        DEBUG(SSSDBG_MINOR_FAILURE, "Periodical refresh "
               "will not work [%d]: %s\n", ret, strerror(ret));
     }
 
