@@ -196,8 +196,7 @@ static errno_t find_ipa_ext_memberships(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
-    ret = sysdb_initgroups(tmp_ctx, user_dom->sysdb, user_dom, user_name,
-                           &result);
+    ret = sysdb_initgroups(tmp_ctx, user_dom, user_name, &result);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "sysdb_initgroups failed.\n");
         goto done;
@@ -341,8 +340,8 @@ static errno_t add_ad_user_to_cached_groups(struct ldb_dn *user_dn,
             goto done;
         }
 
-        ret = sysdb_search_groups(tmp_ctx, group_dom->sysdb, group_dom,
-                                  subfilter, NULL, &msgs_count, &msgs);
+        ret = sysdb_search_groups(tmp_ctx, group_dom, subfilter, NULL,
+                                  &msgs_count, &msgs);
         if (ret != EOK) {
             if (ret == ENOENT) {
                 DEBUG(SSSDBG_TRACE_ALL, "Group [%s] not in the cache.\n",
@@ -358,7 +357,7 @@ static errno_t add_ad_user_to_cached_groups(struct ldb_dn *user_dn,
 /* TODO? Do we have to remove members as well? I think not because the AD
  * query before removes all memberships. */
 
-        ret = sysdb_mod_group_member(group_dom->sysdb, user_dn, msgs[0]->dn,
+        ret = sysdb_mod_group_member(group_dom, user_dn, msgs[0]->dn,
                                      LDB_FLAG_MOD_ADD);
         if (ret != EOK && ret != EEXIST) {
             DEBUG(SSSDBG_OP_FAILURE, "sysdb_mod_group_member failed.\n");
@@ -453,7 +452,8 @@ struct tevent_req *ipa_get_ad_memberships_send(TALLOC_CTX *mem_ctx,
     state->domain = domain;
     state->dp_error = -1;
 
-    if ((ar->entry_type & BE_REQ_TYPE_MASK) != BE_REQ_INITGROUPS
+    if (((ar->entry_type & BE_REQ_TYPE_MASK) != BE_REQ_INITGROUPS
+            && (ar->entry_type & BE_REQ_TYPE_MASK) != BE_REQ_USER)
             || ar->filter_type != BE_FILTER_NAME) {
         DEBUG(SSSDBG_OP_FAILURE, "Unsupported request type.\n");
         ret = EINVAL;
@@ -847,9 +847,9 @@ static void ipa_add_ad_memberships_get_next(struct tevent_req *req)
         }
 
         if (missing_groups) {
-            DEBUG(SSSDBG_CRIT_FAILURE, "There are unresolved external group " \
-                                        "memberships even after all groups have " \
-                                        "been looked up on the LDAP server.");
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "There are unresolved external group memberships even after "
+                  "all groups have been looked up on the LDAP server.\n");
         }
         tevent_req_done(req);
         return;
@@ -872,7 +872,7 @@ static void ipa_add_ad_memberships_get_next(struct tevent_req *req)
                                  state->sdap_id_ctx->conn,
                                  (const char *) val->data,
                                  BE_FILTER_NAME, BE_ATTR_CORE,
-                                 false);
+                                 false, false);
     if (subreq == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "groups_get_send failed.\n");
         ret = ENOMEM;
