@@ -33,6 +33,7 @@
 #include "providers/ldap/sdap_async.h"
 #include "providers/ldap/sdap_idmap.h"
 #include "providers/ldap/sdap_users.h"
+#include "providers/ad/ad_common.h"
 
 /* =Users-Related-Functions-(by-name,by-uid)============================== */
 
@@ -70,6 +71,7 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
                                   struct sdap_id_conn_ctx *conn,
                                   const char *name,
                                   int filter_type,
+                                  const char *extra_value,
                                   int attrs_type,
                                   bool noexist_delete)
 {
@@ -97,7 +99,7 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
     if (!state->op) {
         DEBUG(SSSDBG_OP_FAILURE, "sdap_id_op_create failed\n");
         ret = ENOMEM;
-        goto fail;
+        goto done;
     }
 
     state->domain = sdom->dom;
@@ -111,10 +113,14 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
                                                           sdom->dom->domain_id);
     switch (filter_type) {
     case BE_FILTER_NAME:
-        attr_name = ctx->opts->user_map[SDAP_AT_USER_NAME].name;
+        if (extra_value && strcmp(extra_value, EXTRA_NAME_IS_UPN) == 0) {
+            attr_name = ctx->opts->user_map[SDAP_AT_USER_PRINC].name;
+        } else {
+            attr_name = ctx->opts->user_map[SDAP_AT_USER_NAME].name;
+        }
         ret = sss_filter_sanitize(state, name, &clean_name);
         if (ret != EOK) {
-            goto fail;
+            goto done;
         }
         break;
     case BE_FILTER_IDNUM:
@@ -125,7 +131,7 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
             uid = strtouint32(name, &endptr, 10);
             if (errno != EOK) {
                 ret = EINVAL;
-                goto fail;
+                goto done;
             }
 
             /* Convert the UID to its objectSID */
@@ -136,34 +142,33 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
                       "[%s] did not match any configured ID mapping domain\n",
                        name);
 
-                ret = sysdb_delete_user(state->sysdb,
-                                        state->domain, NULL, uid);
+                ret = sysdb_delete_user(state->domain, NULL, uid);
                 if (ret == ENOENT) {
                     /* Ignore errors to remove users that were not cached previously */
                     ret = EOK;
                 }
 
-                goto fail;
+                goto done;
             } else if (err != IDMAP_SUCCESS) {
                 DEBUG(SSSDBG_MINOR_FAILURE,
                       "Mapping ID [%s] to SID failed: [%s]\n",
                        name, idmap_error_string(err));
                 ret = EIO;
-                goto fail;
+                goto done;
             }
 
             attr_name = ctx->opts->user_map[SDAP_AT_USER_OBJECTSID].name;
             ret = sss_filter_sanitize(state, sid, &clean_name);
             sss_idmap_free_sid(ctx->opts->idmap_ctx->map, sid);
             if (ret != EOK) {
-                goto fail;
+                goto done;
             }
 
         } else {
             attr_name = ctx->opts->user_map[SDAP_AT_USER_UID].name;
             ret = sss_filter_sanitize(state, name, &clean_name);
             if (ret != EOK) {
-                goto fail;
+                goto done;
             }
         }
         break;
@@ -172,18 +177,32 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
 
         ret = sss_filter_sanitize(state, name, &clean_name);
         if (ret != EOK) {
-            goto fail;
+            goto done;
+        }
+        break;
+    case BE_FILTER_UUID:
+        attr_name = ctx->opts->user_map[SDAP_AT_USER_UUID].name;
+        if (attr_name == NULL) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "UUID search not configured for this backend.\n");
+            ret = EINVAL;
+            goto done;
+        }
+
+        ret = sss_filter_sanitize(state, name, &clean_name);
+        if (ret != EOK) {
+            goto done;
         }
         break;
     default:
         ret = EINVAL;
-        goto fail;
+        goto done;
     }
 
     if (attr_name == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "Missing search attribute name.\n");
         ret = EINVAL;
-        goto fail;
+        goto done;
     }
 
     if (state->use_id_mapping || filter_type == BE_FILTER_SECID) {
@@ -212,30 +231,29 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
     if (!state->filter) {
         DEBUG(SSSDBG_OP_FAILURE, "Failed to build the base filter\n");
         ret = ENOMEM;
-        goto fail;
+        goto done;
     }
 
     /* TODO: handle attrs_type */
     ret = build_attrs_from_map(state, ctx->opts->user_map,
                                ctx->opts->user_map_cnt,
                                NULL, &state->attrs, NULL);
-    if (ret != EOK) goto fail;
+    if (ret != EOK) goto done;
 
     ret = users_get_retry(req);
     if (ret != EOK) {
-        goto fail;
+        goto done;
     }
 
     return req;
 
-fail:
+done:
     if (ret != EOK) {
         tevent_req_error(req, ret);
     } else {
         tevent_req_done(req);
     }
-    tevent_req_post(req, ev);
-    return req;
+    return tevent_req_post(req, ev);
 }
 
 static int users_get_retry(struct tevent_req *req)
@@ -414,9 +432,8 @@ static void users_get_done(struct tevent_req *subreq)
             ret = sdap_fallback_local_user(state, state->ctx->opts,
                                            name, uid, &usr_attrs);
             if (ret == EOK) {
-                ret = sdap_save_user(state, state->sysdb,
-                                     state->ctx->opts, state->domain,
-                                     usr_attrs[0], false, NULL, 0);
+                ret = sdap_save_user(state, state->ctx->opts, state->domain,
+                                     usr_attrs[0], NULL, 0);
             }
         }
     }
@@ -434,8 +451,7 @@ static void users_get_done(struct tevent_req *subreq)
             tevent_req_error(req, ret);
             return;
         case BE_FILTER_NAME:
-            ret = sysdb_delete_user(state->sysdb,
-                                    state->domain, state->name, 0);
+            ret = sysdb_delete_user(state->domain, state->name, 0);
             if (ret != EOK && ret != ENOENT) {
                 tevent_req_error(req, ret);
                 return;
@@ -449,7 +465,7 @@ static void users_get_done(struct tevent_req *subreq)
                 return;
             }
 
-            ret = sysdb_delete_user(state->sysdb, state->domain, NULL, uid);
+            ret = sysdb_delete_user(state->domain, NULL, uid);
             if (ret != EOK && ret != ENOENT) {
                 tevent_req_error(req, ret);
                 return;
@@ -457,8 +473,9 @@ static void users_get_done(struct tevent_req *subreq)
             break;
 
         case BE_FILTER_SECID:
-            /* Since it is not clear if the SID belongs to a user or a group
-             * we have nothing to do here. */
+        case BE_FILTER_UUID:
+            /* Since it is not clear if the SID/UUID belongs to a user or a
+             * group we have nothing to do here. */
             break;
 
         default:
@@ -511,6 +528,7 @@ struct groups_get_state {
     int dp_error;
     int sdap_ret;
     bool noexist_delete;
+    bool no_members;
 };
 
 static int groups_get_retry(struct tevent_req *req);
@@ -527,7 +545,8 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
                                    const char *name,
                                    int filter_type,
                                    int attrs_type,
-                                   bool noexist_delete)
+                                   bool noexist_delete,
+                                   bool no_members)
 {
     struct tevent_req *req;
     struct groups_get_state *state;
@@ -550,12 +569,13 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
     state->conn = conn;
     state->dp_error = DP_ERR_FATAL;
     state->noexist_delete = noexist_delete;
+    state->no_members = no_members;
 
     state->op = sdap_id_op_create(state, state->conn->conn_cache);
     if (!state->op) {
         DEBUG(SSSDBG_OP_FAILURE, "sdap_id_op_create failed\n");
         ret = ENOMEM;
-        goto fail;
+        goto done;
     }
 
     state->domain = sdom->dom;
@@ -574,7 +594,7 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
 
         ret = sss_filter_sanitize(state, name, &clean_name);
         if (ret != EOK) {
-            goto fail;
+            goto done;
         }
         break;
     case BE_FILTER_IDNUM:
@@ -585,7 +605,7 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
             gid = strtouint32(name, &endptr, 10);
             if (errno != EOK) {
                 ret = EINVAL;
-                goto fail;
+                goto done;
             }
 
             /* Convert the GID to its objectSID */
@@ -596,34 +616,33 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
                       "[%s] did not match any configured ID mapping domain\n",
                        name);
 
-                ret = sysdb_delete_group(state->sysdb,
-                                         state->domain, NULL, gid);
+                ret = sysdb_delete_group(state->domain, NULL, gid);
                 if (ret == ENOENT) {
                     /* Ignore errors to remove users that were not cached previously */
                     ret = EOK;
                 }
 
-                goto fail;
+                goto done;
             } else if (err != IDMAP_SUCCESS) {
                 DEBUG(SSSDBG_MINOR_FAILURE,
                       "Mapping ID [%s] to SID failed: [%s]\n",
                        name, idmap_error_string(err));
                 ret = EIO;
-                goto fail;
+                goto done;
             }
 
             attr_name = ctx->opts->group_map[SDAP_AT_GROUP_OBJECTSID].name;
             ret = sss_filter_sanitize(state, sid, &clean_name);
             sss_idmap_free_sid(ctx->opts->idmap_ctx->map, sid);
             if (ret != EOK) {
-                goto fail;
+                goto done;
             }
 
         } else {
             attr_name = ctx->opts->group_map[SDAP_AT_GROUP_GID].name;
             ret = sss_filter_sanitize(state, name, &clean_name);
             if (ret != EOK) {
-                goto fail;
+                goto done;
             }
         }
         break;
@@ -632,25 +651,39 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
 
         ret = sss_filter_sanitize(state, name, &clean_name);
         if (ret != EOK) {
-            goto fail;
+            goto done;
+        }
+        break;
+    case BE_FILTER_UUID:
+        attr_name = ctx->opts->group_map[SDAP_AT_GROUP_UUID].name;
+        if (attr_name == NULL) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "UUID search not configured for this backend.\n");
+            ret = EINVAL;
+            goto done;
+        }
+
+        ret = sss_filter_sanitize(state, name, &clean_name);
+        if (ret != EOK) {
+            goto done;
         }
         break;
     default:
         ret = EINVAL;
-        goto fail;
+        goto done;
     }
 
     if (attr_name == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "Missing search attribute name.\n");
         ret = EINVAL;
-        goto fail;
+        goto done;
     }
 
     oc_list = sdap_make_oc_list(state, ctx->opts->group_map);
     if (oc_list == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Failed to create objectClass list.\n");
         ret = ENOMEM;
-        goto fail;
+        goto done;
     }
 
     if (state->use_id_mapping || filter_type == BE_FILTER_SECID) {
@@ -675,7 +708,7 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
     if (!state->filter) {
         DEBUG(SSSDBG_OP_FAILURE, "Failed to build filter\n");
         ret = ENOMEM;
-        goto fail;
+        goto done;
     }
 
     member_filter[0] = (const char *)ctx->opts->group_map[SDAP_AT_GROUP_MEMBER].name;
@@ -683,27 +716,27 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
 
     /* TODO: handle attrs_type */
     ret = build_attrs_from_map(state, ctx->opts->group_map, SDAP_OPTS_GROUP,
-                               state->domain->ignore_group_members ?
+                               (state->domain->ignore_group_members
+                                    || state->no_members) ?
                                    (const char **)member_filter : NULL,
                                &state->attrs, NULL);
 
-    if (ret != EOK) goto fail;
+    if (ret != EOK) goto done;
 
     ret = groups_get_retry(req);
     if (ret != EOK) {
-        goto fail;
+        goto done;
     }
 
     return req;
 
-fail:
+done:
     if (ret != EOK) {
         tevent_req_error(req, ret);
     } else {
         tevent_req_done(req);
     }
-    tevent_req_post(req, ev);
-    return req;
+    return tevent_req_post(req, ev);
 }
 
 static int groups_get_retry(struct tevent_req *req)
@@ -816,7 +849,7 @@ static void groups_get_search(struct tevent_req *req)
                                   state->attrs, state->filter,
                                   dp_opt_get_int(state->ctx->opts->basic,
                                                  SDAP_SEARCH_TIMEOUT),
-                                  false);
+                                  false, state->no_members);
     if (!subreq) {
         tevent_req_error(req, ENOMEM);
         return;
@@ -863,8 +896,7 @@ static void groups_get_done(struct tevent_req *subreq)
             tevent_req_error(req, ret);
             return;
         case BE_FILTER_NAME:
-            ret = sysdb_delete_group(state->sysdb,
-                                     state->domain, state->name, 0);
+            ret = sysdb_delete_group(state->domain, state->name, 0);
             if (ret != EOK && ret != ENOENT) {
                 tevent_req_error(req, ret);
                 return;
@@ -878,8 +910,7 @@ static void groups_get_done(struct tevent_req *subreq)
                 return;
             }
 
-            ret = sysdb_delete_group(state->sysdb,
-                                     state->domain, NULL, gid);
+            ret = sysdb_delete_group(state->domain, NULL, gid);
             if (ret != EOK && ret != ENOENT) {
                 tevent_req_error(req, ret);
                 return;
@@ -887,8 +918,9 @@ static void groups_get_done(struct tevent_req *subreq)
             break;
 
         case BE_FILTER_SECID:
-            /* Since it is not clear if the SID belongs to a user or a group
-             * we have nothing to do here. */
+        case BE_FILTER_UUID:
+            /* Since it is not clear if the SID/UUID belongs to a user or a
+             * group we have nothing to do here. */
             break;
 
         default:
@@ -932,6 +964,8 @@ struct groups_by_user_state {
     struct sss_domain_info *domain;
 
     const char *name;
+    int name_type;
+    const char *extra_value;
     const char **attrs;
 
     int dp_error;
@@ -965,8 +999,7 @@ static errno_t set_initgroups_expire_attribute(struct sss_domain_info *domain,
         goto done;
     }
 
-    ret = sysdb_set_user_attr(domain->sysdb, domain, name, attrs,
-                              SYSDB_MOD_REP);
+    ret = sysdb_set_user_attr(domain, name, attrs, SYSDB_MOD_REP);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "Failed to set initgroups expire attribute\n");
@@ -984,6 +1017,8 @@ static struct tevent_req *groups_by_user_send(TALLOC_CTX *memctx,
                                               struct sdap_domain *sdom,
                                               struct sdap_id_conn_ctx *conn,
                                               const char *name,
+                                              int name_type,
+                                              const char *extra_value,
                                               bool noexist_delete)
 {
     struct tevent_req *req;
@@ -1008,6 +1043,8 @@ static struct tevent_req *groups_by_user_send(TALLOC_CTX *memctx,
     }
 
     state->name = name;
+    state->name_type = name_type;
+    state->extra_value = extra_value;
     state->domain = sdom->dom;
     state->sysdb = sdom->dom->sysdb;
 
@@ -1069,6 +1106,8 @@ static void groups_by_user_connect_done(struct tevent_req *subreq)
                                   state->ctx,
                                   state->conn,
                                   state->name,
+                                  state->name_type,
+                                  state->extra_value,
                                   state->attrs);
     if (!subreq) {
         tevent_req_error(req, ENOMEM);
@@ -1103,39 +1142,32 @@ static void groups_by_user_done(struct tevent_req *subreq)
     }
     state->sdap_ret = ret;
 
-    if (ret == EOK || ret == ENOENT) {
-        /* state->name is still the name used for the original req. The cached
-         * object might have a different name, e.g. a fully-qualified name. */
-        ret = sysdb_get_real_name(state, state->domain->sysdb, state->domain,
-                                  state->name, &cname);
-        if (ret != EOK) {
-            cname = state->name;
-            DEBUG(SSSDBG_OP_FAILURE,
-                  "Failed to canonicalize name, using [%s].\n", cname);
-        }
+    if (ret && ret != ENOENT) {
+        state->dp_error = dp_error;
+        tevent_req_error(req, ret);
+        return;
     }
 
-    switch (state->sdap_ret) {
-    case ENOENT:
-        if (state->noexist_delete == true) {
-            ret = sysdb_delete_user(state->domain->sysdb, state->domain,
-                                    cname, 0);
-            if (ret != EOK && ret != ENOENT) {
-                tevent_req_error(req, ret);
-                return;
-            }
-        }
-        break;
-    case EOK:
-        ret = set_initgroups_expire_attribute(state->domain, cname);
-        if (ret != EOK) {
-            state->dp_error = DP_ERR_FATAL;
+    /* state->name is still the name used for the original request. The cached
+     * object might have a different name, e.g. a fully-qualified name. */
+    ret = sysdb_get_real_name(state, state->domain, state->name, &cname);
+    if (ret != EOK) {
+        cname = state->name;
+        DEBUG(SSSDBG_OP_FAILURE, "Failed to canonicalize name, using [%s].\n",
+                                 cname);
+    }
+
+    if (ret == ENOENT && state->noexist_delete == true) {
+        ret = sysdb_delete_user(state->domain, cname, 0);
+        if (ret != EOK && ret != ENOENT) {
             tevent_req_error(req, ret);
             return;
         }
-        break;
-    default:
-        state->dp_error = dp_error;
+    }
+
+    ret = set_initgroups_expire_attribute(state->domain, cname);
+    if (ret != EOK) {
+        state->dp_error = DP_ERR_FATAL;
         tevent_req_error(req, ret);
         return;
     }
@@ -1335,9 +1367,22 @@ void sdap_account_info_handler(struct be_req *breq)
     return sdap_handle_account_info(breq, ctx, ctx->conn);
 }
 
+bool sdap_is_enum_request(struct be_acct_req *ar)
+{
+    switch (ar->entry_type & BE_REQ_TYPE_MASK) {
+    case BE_REQ_USER:
+    case BE_REQ_GROUP:
+    case BE_REQ_SERVICES:
+        if (ar->filter_type == BE_FILTER_ENUM) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 /* A generic LDAP account info handler */
 struct sdap_handle_acct_req_state {
-    struct be_req *breq;
     struct be_acct_req *ar;
     const char *err;
     int dp_error;
@@ -1348,7 +1393,7 @@ static void sdap_handle_acct_req_done(struct tevent_req *subreq);
 
 struct tevent_req *
 sdap_handle_acct_req_send(TALLOC_CTX *mem_ctx,
-                          struct be_req *breq,
+                          struct be_ctx *be_ctx,
                           struct be_acct_req *ar,
                           struct sdap_id_ctx *id_ctx,
                           struct sdap_domain *sdom,
@@ -1357,11 +1402,9 @@ sdap_handle_acct_req_send(TALLOC_CTX *mem_ctx,
 {
     struct tevent_req *req;
     struct tevent_req *subreq;
-    struct be_ctx *be_ctx;
     struct sdap_handle_acct_req_state *state;
     errno_t ret;
 
-    be_ctx = be_req_get_be_ctx(breq);
 
     req = tevent_req_create(mem_ctx, &state,
                             struct sdap_handle_acct_req_state);
@@ -1369,7 +1412,6 @@ sdap_handle_acct_req_send(TALLOC_CTX *mem_ctx,
         ret = ENOMEM;
         goto done;
     }
-    state->breq = breq;
     state->ar = ar;
 
     if (ar == NULL) {
@@ -1379,45 +1421,28 @@ sdap_handle_acct_req_send(TALLOC_CTX *mem_ctx,
 
     switch (ar->entry_type & BE_REQ_TYPE_MASK) {
     case BE_REQ_USER: /* user */
-
-        /* skip enumerations on demand */
-        if (ar->filter_type == BE_FILTER_ENUM) {
-            DEBUG(SSSDBG_TRACE_LIBS,
-                  "Skipping user enumeration on demand\n");
-            state->err = "Success";
-            ret = EOK;
-            goto done;
-        }
-
-        subreq = users_get_send(breq, be_ctx->ev, id_ctx,
+        subreq = users_get_send(state, be_ctx->ev, id_ctx,
                                 sdom, conn,
                                 ar->filter_value,
                                 ar->filter_type,
+                                ar->extra_value,
                                 ar->attr_type,
                                 noexist_delete);
         break;
 
     case BE_REQ_GROUP: /* group */
-
-        /* skip enumerations on demand */
-        if (ar->filter_type == BE_FILTER_ENUM) {
-            DEBUG(SSSDBG_TRACE_LIBS,
-                  "Skipping group enumeration on demand\n");
-            state->err = "Success";
-            ret = EOK;
-            goto done;
-        }
-
-        subreq = groups_get_send(breq, be_ctx->ev, id_ctx,
+        subreq = groups_get_send(state, be_ctx->ev, id_ctx,
                                  sdom, conn,
                                  ar->filter_value,
                                  ar->filter_type,
                                  ar->attr_type,
-                                 noexist_delete);
+                                 noexist_delete, false);
         break;
 
     case BE_REQ_INITGROUPS: /* init groups for user */
-        if (ar->filter_type != BE_FILTER_NAME) {
+        if (ar->filter_type != BE_FILTER_NAME
+                && ar->filter_type != BE_FILTER_SECID
+                && ar->filter_type != BE_FILTER_UUID) {
             ret = EINVAL;
             state->err = "Invalid filter type";
             goto done;
@@ -1428,9 +1453,11 @@ sdap_handle_acct_req_send(TALLOC_CTX *mem_ctx,
             goto done;
         }
 
-        subreq = groups_by_user_send(breq, be_ctx->ev, id_ctx,
+        subreq = groups_by_user_send(state, be_ctx->ev, id_ctx,
                                      sdom, conn,
                                      ar->filter_value,
+                                     ar->filter_type,
+                                     ar->extra_value,
                                      noexist_delete);
         break;
 
@@ -1441,29 +1468,21 @@ sdap_handle_acct_req_send(TALLOC_CTX *mem_ctx,
             goto done;
         }
 
-        subreq = ldap_netgroup_get_send(breq, be_ctx->ev, id_ctx,
+        subreq = ldap_netgroup_get_send(state, be_ctx->ev, id_ctx,
                                         sdom, conn,
                                         ar->filter_value,
                                         noexist_delete);
         break;
 
     case BE_REQ_SERVICES:
-        /* skip enumerations on demand */
-        if (ar->filter_type == BE_FILTER_ENUM) {
-            DEBUG(SSSDBG_TRACE_LIBS,
-                  "Skipping service enumeration on demand\n");
-            state->err = "Success";
-            ret = EOK;
-            goto done;
-        }
-
-        if (ar->filter_type == BE_FILTER_SECID) {
+        if (ar->filter_type == BE_FILTER_SECID
+                || ar->filter_type == BE_FILTER_UUID) {
             ret = EINVAL;
             state->err = "Invalid filter type";
             goto done;
         }
 
-        subreq = services_get_send(breq, be_ctx->ev, id_ctx,
+        subreq = services_get_send(state, be_ctx->ev, id_ctx,
                                    sdom, conn,
                                    ar->filter_value,
                                    ar->extra_value,
@@ -1478,7 +1497,22 @@ sdap_handle_acct_req_send(TALLOC_CTX *mem_ctx,
             goto done;
         }
 
-        subreq = get_user_and_group_send(breq, be_ctx->ev, id_ctx,
+        subreq = get_user_and_group_send(state, be_ctx->ev, id_ctx,
+                                         sdom, conn,
+                                         ar->filter_value,
+                                         ar->filter_type,
+                                         ar->attr_type,
+                                         noexist_delete);
+        break;
+
+    case BE_REQ_BY_UUID:
+        if (ar->filter_type != BE_FILTER_UUID) {
+            ret = EINVAL;
+            state->err = "Invalid filter type";
+            goto done;
+        }
+
+        subreq = get_user_and_group_send(state, be_ctx->ev, id_ctx,
                                          sdom, conn,
                                          ar->filter_value,
                                          ar->filter_type,
@@ -1494,7 +1528,7 @@ sdap_handle_acct_req_send(TALLOC_CTX *mem_ctx,
             goto done;
         }
 
-        subreq = get_user_and_group_send(breq, be_ctx->ev, id_ctx,
+        subreq = get_user_and_group_send(state, be_ctx->ev, id_ctx,
                                          sdom, conn,
                                          ar->filter_value,
                                          ar->filter_type,
@@ -1560,6 +1594,8 @@ sdap_handle_acct_req_done(struct tevent_req *subreq)
         break;
     case BE_REQ_BY_SECID:
         /* Fallthrough */
+    case BE_REQ_BY_UUID:
+        /* Fallthrough */
     case BE_REQ_USER_AND_GROUP:
         err = "Lookup by SID failed";
         ret = sdap_get_user_and_group_recv(subreq, &state->dp_error,
@@ -1624,7 +1660,12 @@ void sdap_handle_account_info(struct be_req *breq, struct sdap_id_ctx *ctx,
                                  EINVAL, "Invalid private data");
     }
 
-    req = sdap_handle_acct_req_send(breq, breq, ar, ctx,
+    if (sdap_is_enum_request(ar)) {
+        DEBUG(SSSDBG_TRACE_LIBS, "Skipping enumeration on demand\n");
+        return sdap_handler_done(breq, DP_ERR_OK, EOK, "Success");
+    }
+
+    req = sdap_handle_acct_req_send(breq, ctx->be, ar, ctx,
                                     ctx->opts->sdom, conn, true);
     if (req == NULL) {
         return sdap_handler_done(breq, DP_ERR_FATAL, ENOMEM, "Out of memory");
@@ -1646,7 +1687,7 @@ static void sdap_account_info_complete(struct tevent_req *req)
             error_text = NULL;
         } else {
             DEBUG(SSSDBG_CRIT_FAILURE,
-                  "Bug: dp_error is OK on failed request");
+                  "Bug: dp_error is OK on failed request\n");
             dp_error = DP_ERR_FATAL;
             error_text = req_error_text;
         }
@@ -1729,7 +1770,7 @@ static struct tevent_req *get_user_and_group_send(TALLOC_CTX *memctx,
     subreq = groups_get_send(req, state->ev, state->id_ctx,
                              state->sdom, state->conn,
                              state->filter_val, state->filter_type,
-                             state->attrs_type, state->noexist_delete);
+                             state->attrs_type, state->noexist_delete, false);
     if (subreq == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "users_get_send failed.\n");
         ret = ENOMEM;
@@ -1753,6 +1794,8 @@ static void get_user_and_group_groups_done(struct tevent_req *subreq)
     struct get_user_and_group_state *state = tevent_req_data(req,
                                                struct get_user_and_group_state);
     int ret;
+    struct ad_id_ctx *ad_id_ctx;
+    struct sdap_id_conn_ctx *user_conn;
 
     ret = groups_get_recv(subreq, &state->dp_error, &state->sdap_ret);
     talloc_zfree(subreq);
@@ -1772,9 +1815,23 @@ static void get_user_and_group_groups_done(struct tevent_req *subreq)
 
     /* Now the search finished fine but did not find an entry.
      * Retry with users. */
+
+    user_conn = state->conn;
+    /* Prefer LDAP over GC for users */
+    if (state->id_ctx->opts->schema_type == SDAP_SCHEMA_AD
+            && state->sdom->pvt != NULL) {
+        ad_id_ctx = talloc_get_type(state->sdom->pvt, struct ad_id_ctx);
+        if (ad_id_ctx != NULL &&  ad_id_ctx->ldap_ctx != NULL
+                && state->conn == ad_id_ctx->gc_ctx) {
+            DEBUG(SSSDBG_TRACE_ALL,
+                  "Switching to LDAP connection for user lookup.\n");
+            user_conn = ad_id_ctx->ldap_ctx;
+        }
+    }
+
     subreq = users_get_send(req, state->ev, state->id_ctx,
-                            state->sdom, state->conn,
-                            state->filter_val, state->filter_type,
+                            state->sdom, user_conn,
+                            state->filter_val, state->filter_type, NULL,
                             state->attrs_type, state->noexist_delete);
     if (subreq == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "groups_get_send failed.\n");
@@ -1800,16 +1857,17 @@ static void get_user_and_group_users_done(struct tevent_req *subreq)
         tevent_req_error(req, ret);
         return;
     }
-
     if (state->sdap_ret == ENOENT) {
-        /* The search ran to completion, but nothing was found.
-         * Delete the existing entry, if any. */
-        ret = sysdb_delete_by_sid(state->sysdb, state->domain,
-                                  state->filter_val);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_OP_FAILURE, "Could not delete entry by SID!\n");
-            tevent_req_error(req, ret);
-            return;
+        if (state->noexist_delete == true) {
+            /* The search ran to completion, but nothing was found.
+             * Delete the existing entry, if any. */
+            ret = sysdb_delete_by_sid(state->sysdb, state->domain,
+                                      state->filter_val);
+            if (ret != EOK) {
+                DEBUG(SSSDBG_OP_FAILURE, "Could not delete entry by SID!\n");
+                tevent_req_error(req, ret);
+                return;
+            }
         }
     } else if (state->sdap_ret != EOK) {
         tevent_req_error(req, EIO);
