@@ -28,6 +28,23 @@
 
 /* =Retrieve-Options====================================================== */
 
+errno_t sdap_copy_map_entry(const struct sdap_attr_map *src_map,
+                            struct sdap_attr_map *dst_map,
+                            int entry_index)
+{
+    if (src_map[entry_index].name != NULL) {
+        dst_map[entry_index].name = talloc_strdup(dst_map,
+                                                  src_map[entry_index].name);
+        if (dst_map[entry_index].name == NULL) {
+            return ENOMEM;
+        }
+    } else {
+        dst_map->name = NULL;
+    }
+
+    return EOK;
+}
+
 int sdap_copy_map(TALLOC_CTX *memctx,
                  struct sdap_attr_map *src_map,
                  int num_entries,
@@ -226,6 +243,64 @@ int sdap_extend_map_with_list(TALLOC_CTX *mem_ctx,
     return EOK;
 }
 
+static void sdap_inherit_basic_options(char **inherit_opt_list,
+                                       struct dp_option *parent_opts,
+                                       struct dp_option *subdom_opts)
+{
+    int inherit_options[] = {
+        SDAP_CACHE_PURGE_TIMEOUT,
+        SDAP_AD_USE_TOKENGROUPS,
+        SDAP_OPTS_BASIC     /* sentinel */
+    };
+    int i;
+
+    for (i = 0; inherit_options[i] != SDAP_OPTS_BASIC; i++) {
+        dp_option_inherit(inherit_opt_list,
+                          inherit_options[i],
+                          parent_opts,
+                          subdom_opts);
+    }
+}
+
+static void sdap_inherit_user_options(char **inherit_opt_list,
+                                      struct sdap_attr_map *parent_user_map,
+                                      struct sdap_attr_map *child_user_map)
+{
+    int inherit_options[] = {
+        SDAP_AT_USER_PRINC,
+        SDAP_OPTS_USER          /* sentinel */
+    };
+    int i;
+    int opt_index;
+    bool inherit_option;
+
+    for (i = 0; inherit_options[i] != SDAP_OPTS_USER; i++) {
+        opt_index = inherit_options[i];
+
+        inherit_option = string_in_list(parent_user_map[opt_index].opt_name,
+                                        inherit_opt_list,
+                                        false);
+        if (inherit_option == false) {
+            continue;
+        }
+
+        sdap_copy_map_entry(parent_user_map, child_user_map, opt_index);
+    }
+}
+
+void sdap_inherit_options(char **inherit_opt_list,
+                          struct sdap_options *parent_sdap_opts,
+                          struct sdap_options *child_sdap_opts)
+{
+    sdap_inherit_basic_options(inherit_opt_list,
+                               parent_sdap_opts->basic,
+                               child_sdap_opts->basic);
+
+    sdap_inherit_user_options(inherit_opt_list,
+                              parent_sdap_opts->user_map,
+                              child_sdap_opts->user_map);
+}
+
 int sdap_get_map(TALLOC_CTX *memctx,
                  struct confdb_ctx *cdb,
                  const char *conf_path,
@@ -295,7 +370,7 @@ static bool objectclass_matched(struct sdap_attr_map *map,
 int sdap_parse_entry(TALLOC_CTX *memctx,
                      struct sdap_handle *sh, struct sdap_msg *sm,
                      struct sdap_attr_map *map, int attrs_num,
-                     struct sysdb_attrs **_attrs, char **_dn,
+                     struct sysdb_attrs **_attrs,
                      bool disable_range_retrieval)
 {
     struct sysdb_attrs *attrs;
@@ -304,12 +379,12 @@ int sdap_parse_entry(TALLOC_CTX *memctx,
     struct ldb_val v;
     char *str;
     int lerrno;
-    int a, i, ret;
+    int i, ret, ai;
+    int base_attr_idx = 0;
     const char *name;
     bool store;
     bool base64;
     char *base_attr;
-    char *dn = NULL;
     uint32_t range_offset;
     TALLOC_CTX *tmp_ctx = talloc_new(NULL);
     if (!tmp_ctx) return ENOMEM;
@@ -317,8 +392,8 @@ int sdap_parse_entry(TALLOC_CTX *memctx,
     lerrno = 0;
     ret = ldap_set_option(sh->ldap, LDAP_OPT_RESULT_CODE, &lerrno);
     if (ret != LDAP_OPT_SUCCESS) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "ldap_set_option failed [%s], ignored.\n",
-                  sss_ldap_err2string(ret));
+        DEBUG(SSSDBG_MINOR_FAILURE, "ldap_set_option failed [%s], ignored.\n",
+              sss_ldap_err2string(ret));
     }
 
     attrs = sysdb_new_attrs(tmp_ctx);
@@ -331,23 +406,15 @@ int sdap_parse_entry(TALLOC_CTX *memctx,
     if (!str) {
         ldap_get_option(sh->ldap, LDAP_OPT_RESULT_CODE, &lerrno);
         DEBUG(SSSDBG_CRIT_FAILURE, "ldap_get_dn failed: %d(%s)\n",
-                  lerrno, sss_ldap_err2string(lerrno));
+              lerrno, sss_ldap_err2string(lerrno));
         ret = EIO;
         goto done;
     }
 
-    DEBUG(SSSDBG_TRACE_ALL, "OriginalDN: [%s].\n", str);
+    DEBUG(SSSDBG_TRACE_LIBS, "OriginalDN: [%s].\n", str);
     ret = sysdb_attrs_add_string(attrs, SYSDB_ORIG_DN, str);
-    if (ret) goto done;
-    if (_dn) {
-        dn = talloc_strdup(tmp_ctx, str);
-        if (!dn) {
-            ret = ENOMEM;
-            ldap_memfree(str);
-            goto done;
-        }
-    }
     ldap_memfree(str);
+    if (ret) goto done;
 
     if (map) {
         vals = ldap_get_values_len(sh->ldap, sm->msg, "objectClass");
@@ -366,7 +433,7 @@ int sdap_parse_entry(TALLOC_CTX *memctx,
         }
         if (!vals[i]) {
             DEBUG(SSSDBG_CRIT_FAILURE, "objectClass not matching: %s\n",
-                      map[0].name);
+                  map[0].name);
             ldap_value_free_len(vals);
             ret = EINVAL;
             goto done;
@@ -378,7 +445,7 @@ int sdap_parse_entry(TALLOC_CTX *memctx,
     if (!str) {
         ldap_get_option(sh->ldap, LDAP_OPT_RESULT_CODE, &lerrno);
         DEBUG(lerrno == LDAP_SUCCESS
-              ? SSSDBG_TRACE_INTERNAL
+              ? SSSDBG_TRACE_LIBS
               : SSSDBG_MINOR_FAILURE,
               "Entry has no attributes [%d(%s)]!?\n",
                lerrno, sss_ldap_err2string(lerrno));
@@ -406,22 +473,23 @@ int sdap_parse_entry(TALLOC_CTX *memctx,
         case EOK:
             break;
         default:
-            DEBUG(SSSDBG_MINOR_FAILURE,
+            DEBUG(SSSDBG_CRIT_FAILURE,
                   "Could not determine if attribute [%s] was ranged\n", str);
             goto done;
         }
 
         if (map) {
-            for (a = 1; a < attrs_num; a++) {
+            for (i = 1; i < attrs_num; i++) {
                 /* check if this attr is valid with the chosen schema */
-                if (!map[a].name) continue;
+                if (!map[i].name) continue;
                 /* check if it is an attr we are interested in */
-                if (strcasecmp(base_attr, map[a].name) == 0) break;
+                if (strcasecmp(base_attr, map[i].name) == 0) break;
             }
             /* interesting attr */
-            if (a < attrs_num) {
+            if (i < attrs_num) {
                 store = true;
-                name = map[a].sys_name;
+                name = map[i].sys_name;
+                base_attr_idx = i;
                 if (strcmp(name, SYSDB_SSH_PUBKEY) == 0) {
                     base64 = true;
                 }
@@ -444,33 +512,35 @@ int sdap_parse_entry(TALLOC_CTX *memctx,
             if (!vals) {
                 ldap_get_option(sh->ldap, LDAP_OPT_RESULT_CODE, &lerrno);
                 if (lerrno != LDAP_SUCCESS) {
-                    DEBUG(SSSDBG_CRIT_FAILURE, "LDAP Library error: %d(%s)",
-                              lerrno, sss_ldap_err2string(lerrno));
+                    DEBUG(SSSDBG_CRIT_FAILURE, "LDAP Library error: %d(%s)\n",
+                          lerrno, sss_ldap_err2string(lerrno));
                     ret = EIO;
                     goto done;
                 }
 
-                DEBUG(SSSDBG_FUNC_DATA,
+                DEBUG(SSSDBG_TRACE_LIBS,
                       "Attribute [%s] has no values, skipping.\n", str);
 
             } else {
                 if (!vals[0]) {
                     DEBUG(SSSDBG_CRIT_FAILURE,
                           "Missing value after ldap_get_values() ??\n");
+                    ldap_value_free_len(vals);
                     ret = EINVAL;
                     goto done;
                 }
                 for (i = 0; vals[i]; i++) {
                     if (vals[i]->bv_len == 0) {
-                        DEBUG(SSSDBG_MINOR_FAILURE,
+                        DEBUG(SSSDBG_TRACE_LIBS,
                               "Value of attribute [%s] is empty. "
                                "Skipping this value.\n", str);
                         continue;
                     }
                     if (base64) {
-                        v.data = (uint8_t *)sss_base64_encode(attrs,
-                                (uint8_t *)vals[i]->bv_val, vals[i]->bv_len);
+                        v.data = (uint8_t *) sss_base64_encode(attrs,
+                                 (uint8_t *) vals[i]->bv_val, vals[i]->bv_len);
                         if (!v.data) {
+                            ldap_value_free_len(vals);
                             ret = ENOMEM;
                             goto done;
                         }
@@ -480,8 +550,35 @@ int sdap_parse_entry(TALLOC_CTX *memctx,
                         v.length = vals[i]->bv_len;
                     }
 
-                    ret = sysdb_attrs_add_val(attrs, name, &v);
-                    if (ret) goto done;
+                    if (map) {
+                        /* The same LDAP attr might be used for more sysdb
+                         * attrs in case there is a map. Find all that match
+                         * and copy the value
+                         */
+                        for (ai = base_attr_idx; ai < attrs_num; ai++) {
+                            /* check if this attr is valid with the chosen
+                             * schema */
+                            if (!map[ai].name) continue;
+
+                            /* check if it is an attr we are interested in */
+                            if (strcasecmp(base_attr, map[ai].name) == 0) {
+                                ret = sysdb_attrs_add_val(attrs,
+                                                          map[ai].sys_name,
+                                                          &v);
+                                if (ret) {
+                                    ldap_value_free_len(vals);
+                                    goto done;
+                                }
+                            }
+                        }
+                    } else {
+                        /* No map, just store the attribute */
+                        ret = sysdb_attrs_add_val(attrs, name, &v);
+                        if (ret) {
+                            ldap_value_free_len(vals);
+                            goto done;
+                        }
+                    }
                 }
                 ldap_value_free_len(vals);
             }
@@ -495,14 +592,13 @@ int sdap_parse_entry(TALLOC_CTX *memctx,
 
     ldap_get_option(sh->ldap, LDAP_OPT_RESULT_CODE, &lerrno);
     if (lerrno) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "LDAP Library error: %d(%s)",
-                  lerrno, sss_ldap_err2string(lerrno));
+        DEBUG(SSSDBG_CRIT_FAILURE, "LDAP Library error: %d(%s)\n",
+              lerrno, sss_ldap_err2string(lerrno));
         ret = EIO;
         goto done;
     }
 
     *_attrs = talloc_steal(memctx, attrs);
-    if (_dn) *_dn = talloc_steal(memctx, dn);
     ret = EOK;
 
 done:
@@ -535,7 +631,7 @@ errno_t sdap_parse_deref(TALLOC_CTX *mem_ctx,
                          struct sdap_attr_map_info *minfo,
                          size_t num_maps,
                          LDAPDerefRes *dref,
-                         struct sdap_deref_attrs ***_res)
+                         struct sdap_deref_attrs ***_deref_res)
 {
     TALLOC_CTX *tmp_ctx;
     LDAPDerefVal *dval;
@@ -583,7 +679,7 @@ errno_t sdap_parse_deref(TALLOC_CTX *mem_ctx,
         DEBUG(SSSDBG_FUNC_DATA,
               "Dereferenced entry [%s] has no attributes, skipping\n",
               orig_dn);
-        *_res = NULL;
+        *_deref_res = NULL;
         ret = EOK;
         goto done;
     }
@@ -688,44 +784,13 @@ errno_t sdap_parse_deref(TALLOC_CTX *mem_ctx,
     }
 
 
-    *_res = talloc_steal(mem_ctx, res);
+    *_deref_res = talloc_steal(mem_ctx, res);
     ret = EOK;
 done:
     talloc_zfree(tmp_ctx);
     return ret;
 }
 
-/* =Get-DN-from-message=================================================== */
-
-int sdap_get_msg_dn(TALLOC_CTX *memctx, struct sdap_handle *sh,
-                    struct sdap_msg *sm, char **_dn)
-{
-    char *str;
-    int lerrno;
-    int ret;
-
-    lerrno = 0;
-    ret = ldap_set_option(sh->ldap, LDAP_OPT_RESULT_CODE, &lerrno);
-    if (ret != LDAP_OPT_SUCCESS) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "ldap_set_option failed [%s], ignored.\n",
-                  sss_ldap_err2string(ret));
-    }
-
-    str = ldap_get_dn(sh->ldap, sm->msg);
-    if (!str) {
-        ldap_get_option(sh->ldap, LDAP_OPT_RESULT_CODE, &lerrno);
-        DEBUG(SSSDBG_CRIT_FAILURE, "ldap_get_dn failed: %d(%s)\n",
-                  lerrno, sss_ldap_err2string(lerrno));
-        return EIO;
-    }
-
-    *_dn = talloc_strdup(memctx, str);
-    ldap_memfree(str);
-    if (!*_dn) return ENOMEM;
-
-    return EOK;
-}
-
 errno_t setup_tls_config(struct dp_option *basic_opts)
 {
     int ret;
