@@ -122,8 +122,6 @@ struct sdap_ppolicy_data {
 
 #define SYSDB_PWD_ATTRIBUTE "pwdAttribute"
 
-#define SYSDB_AD_ACCOUNT_EXPIRES "adAccountExpires"
-#define SYSDB_AD_USER_ACCOUNT_CONTROL "adUserAccountControl"
 #define SYSDB_NDS_LOGIN_DISABLED "ndsLoginDisabled"
 #define SYSDB_NDS_LOGIN_EXPIRATION_TIME "ndsLoginExpirationTime"
 #define SYSDB_NDS_LOGIN_ALLOWED_TIME_MAP "ndsLoginAllowedTimeMap"
@@ -173,6 +171,7 @@ enum sdap_basic_opt {
     SDAP_SUDO_INCLUDE_NETGROUPS,
     SDAP_SUDO_INCLUDE_REGEXP,
     SDAP_AUTOFS_SEARCH_BASE,
+    SDAP_AUTOFS_MAP_MASTER_NAME,
     SDAP_SCHEMA,
     SDAP_OFFLINE_TIMEOUT,
     SDAP_FORCE_UPPER_CASE_REALM,
@@ -257,6 +256,7 @@ enum sdap_user_attrs {
     SDAP_AT_USER_PRINC,
     SDAP_AT_USER_FULLNAME,
     SDAP_AT_USER_MEMBEROF,
+    SDAP_AT_USER_UUID,
     SDAP_AT_USER_OBJECTSID,
     SDAP_AT_USER_PRIMARY_GROUP,
     SDAP_AT_USER_MODSTAMP,
@@ -280,6 +280,7 @@ enum sdap_user_attrs {
     SDAP_AT_NDS_LOGIN_EXPIRATION_TIME,
     SDAP_AT_NDS_LOGIN_ALLOWED_TIME_MAP,
     SDAP_AT_USER_SSH_PUBLIC_KEY,
+    SDAP_AT_USER_AUTH_TYPE,
 
     SDAP_OPTS_USER /* attrs counter */
 };
@@ -295,6 +296,7 @@ enum sdap_group_attrs {
     SDAP_AT_GROUP_PWD,
     SDAP_AT_GROUP_GID,
     SDAP_AT_GROUP_MEMBER,
+    SDAP_AT_GROUP_UUID,
     SDAP_AT_GROUP_OBJECTSID,
     SDAP_AT_GROUP_MODSTAMP,
     SDAP_AT_GROUP_USN,
@@ -465,6 +467,10 @@ struct sdap_deref_attrs {
     struct sysdb_attrs *attrs;
 };
 
+errno_t sdap_copy_map_entry(const struct sdap_attr_map *src_map,
+                            struct sdap_attr_map *dst_map,
+                            int entry_index);
+
 int sdap_copy_map(TALLOC_CTX *memctx,
                  struct sdap_attr_map *src_map,
                  int num_entries,
@@ -485,6 +491,10 @@ int sdap_extend_map_with_list(TALLOC_CTX *mem_ctx,
                               struct sdap_attr_map **_map,
                               size_t *_new_size);
 
+void sdap_inherit_options(char **inherit_opt_list,
+                          struct sdap_options *parent_sdap_opts,
+                          struct sdap_options *child_sdap_opts);
+
 int sdap_get_map(TALLOC_CTX *memctx,
                  struct confdb_ctx *cdb,
                  const char *conf_path,
@@ -495,17 +505,14 @@ int sdap_get_map(TALLOC_CTX *memctx,
 int sdap_parse_entry(TALLOC_CTX *memctx,
                      struct sdap_handle *sh, struct sdap_msg *sm,
                      struct sdap_attr_map *map, int attrs_num,
-                     struct sysdb_attrs **_attrs, char **_dn,
+                     struct sysdb_attrs **_attrs,
                      bool disable_range_retrieval);
 
 errno_t sdap_parse_deref(TALLOC_CTX *mem_ctx,
                          struct sdap_attr_map_info *minfo,
                          size_t num_maps,
                          LDAPDerefRes *dref,
-                         struct sdap_deref_attrs ***_res);
-
-int sdap_get_msg_dn(TALLOC_CTX *memctx, struct sdap_handle *sh,
-                    struct sdap_msg *sm, char **_dn);
+                         struct sdap_deref_attrs ***_deref_res);
 
 errno_t setup_tls_config(struct dp_option *basic_opts);
 
