@@ -32,6 +32,7 @@
 #include <errno.h>
 
 #include "util/util.h"
+#include "util/strtonum.h"
 #include "db/sysdb.h"
 #include "providers/ldap/ldap_common.h"
 #include "providers/ldap/sdap.h"
@@ -39,10 +40,22 @@
 #include "providers/ldap/sdap_async.h"
 #include "providers/data_provider.h"
 #include "providers/dp_backend.h"
+#include "providers/ldap/ldap_auth.h"
 
 #define PERMANENTLY_LOCKED_ACCOUNT "000001010000Z"
 #define MALFORMED_FILTER "Malformed access control filter [%s]\n"
 
+enum sdap_pwpolicy_mode {
+    PWP_LOCKOUT_ONLY,
+    PWP_LOCKOUT_EXPIRE,
+    PWP_SENTINEL,
+};
+
+static errno_t perform_pwexpire_policy(TALLOC_CTX *mem_ctx,
+                                       struct sss_domain_info *domain,
+                                       struct pam_data *pd,
+                                       struct sdap_options *opts);
+
 static errno_t sdap_save_user_cache_bool(struct sss_domain_info *domain,
                                          const char *username,
                                          const char *attr_name,
@@ -53,14 +66,15 @@ static errno_t sdap_get_basedn_user_entry(struct ldb_message *user_entry,
                                           const char **_basedn);
 
 static struct tevent_req *
-sdap_access_lock_send(TALLOC_CTX *mem_ctx,
-                      struct tevent_context *ev,
-                      struct be_ctx *be_ctx,
-                      struct sss_domain_info *domain,
-                      struct sdap_access_ctx *access_ctx,
-                      struct sdap_id_conn_ctx *conn,
-                      const char *username,
-                      struct ldb_message *user_entry);
+sdap_access_ppolicy_send(TALLOC_CTX *mem_ctx,
+                         struct tevent_context *ev,
+                         struct be_ctx *be_ctx,
+                         struct sss_domain_info *domain,
+                         struct sdap_access_ctx *access_ctx,
+                         struct sdap_id_conn_ctx *conn,
+                         const char *username,
+                         struct ldb_message *user_entry,
+                         enum sdap_pwpolicy_mode pwpol_mod);
 
 static struct tevent_req *sdap_access_filter_send(TALLOC_CTX *mem_ctx,
                                              struct tevent_context *ev,
@@ -73,7 +87,7 @@ static struct tevent_req *sdap_access_filter_send(TALLOC_CTX *mem_ctx,
 
 static errno_t sdap_access_filter_recv(struct tevent_req *req);
 
-static errno_t sdap_access_lock_recv(struct tevent_req *req);
+static errno_t sdap_access_ppolicy_recv(struct tevent_req *req);
 
 static errno_t sdap_account_expired(struct sdap_access_ctx *access_ctx,
                                     struct pam_data *pd,
@@ -145,8 +159,7 @@ sdap_access_send(TALLOC_CTX *mem_ctx,
     }
 
     /* Get original user DN, domain already points to the right (sub)domain */
-    ret = sysdb_get_user_attr(state, domain->sysdb, domain,
-                              pd->user, attrs, &res);
+    ret = sysdb_get_user_attr(state, domain, pd->user, attrs, &res);
     if (ret != EOK) {
         if (ret == ENOENT) {
             /* If we can't find the user, return access denied */
@@ -199,15 +212,42 @@ static errno_t sdap_access_check_next_rule(struct sdap_access_req_ctx *state,
             /* we are done with no errors */
             return EOK;
 
+        /* This option is deprecated by LDAP_ACCESS_PPOLICY */
         case LDAP_ACCESS_LOCKOUT:
-            subreq = sdap_access_lock_send(state, state->ev, state->be_ctx,
-                                           state->domain,
-                                           state->access_ctx,
-                                           state->conn,
-                                           state->pd->user,
-                                           state->user_entry);
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  "WARNING: %s option is deprecated and might be removed in "
+                  "a future release. Please migrate to %s option instead.\n",
+                  LDAP_ACCESS_LOCK_NAME, LDAP_ACCESS_PPOLICY_NAME);
+
+            subreq = sdap_access_ppolicy_send(state, state->ev, state->be_ctx,
+                                              state->domain,
+                                              state->access_ctx,
+                                              state->conn,
+                                              state->pd->user,
+                                              state->user_entry,
+                                              PWP_LOCKOUT_ONLY);
             if (subreq == NULL) {
-                DEBUG(SSSDBG_CRIT_FAILURE, "sdap_access_lock_send failed.\n");
+                DEBUG(SSSDBG_CRIT_FAILURE,
+                      "sdap_access_ppolicy_send failed.\n");
+                return ENOMEM;
+            }
+
+            state->ac_type = SDAP_ACCESS_CONTROL_PPOLICY_LOCK;
+
+            tevent_req_set_callback(subreq, sdap_access_done, req);
+            return EAGAIN;
+
+        case LDAP_ACCESS_PPOLICY:
+            subreq = sdap_access_ppolicy_send(state, state->ev, state->be_ctx,
+                                              state->domain,
+                                              state->access_ctx,
+                                              state->conn,
+                                              state->pd->user,
+                                              state->user_entry,
+                                              PWP_LOCKOUT_EXPIRE);
+            if (subreq == NULL) {
+                DEBUG(SSSDBG_CRIT_FAILURE,
+                      "sdap_access_ppolicy_send failed.\n");
                 return ENOMEM;
             }
 
@@ -238,6 +278,30 @@ static errno_t sdap_access_check_next_rule(struct sdap_access_req_ctx *state,
                                        state->pd, state->user_entry);
             break;
 
+        case LDAP_ACCESS_EXPIRE_POLICY_REJECT:
+            ret = perform_pwexpire_policy(state, state->domain, state->pd,
+                                          state->access_ctx->id_ctx->opts);
+            if (ret == ERR_PASSWORD_EXPIRED) {
+                ret = ERR_PASSWORD_EXPIRED_REJECT;
+            }
+            break;
+
+        case LDAP_ACCESS_EXPIRE_POLICY_WARN:
+            ret = perform_pwexpire_policy(state, state->domain, state->pd,
+                                          state->access_ctx->id_ctx->opts);
+            if (ret == ERR_PASSWORD_EXPIRED) {
+                ret = ERR_PASSWORD_EXPIRED_WARN;
+            }
+            break;
+
+        case LDAP_ACCESS_EXPIRE_POLICY_RENEW:
+            ret = perform_pwexpire_policy(state, state->domain, state->pd,
+                                          state->access_ctx->id_ctx->opts);
+            if (ret == ERR_PASSWORD_EXPIRED) {
+                ret = ERR_PASSWORD_EXPIRED_RENEW;
+            }
+            break;
+
         case LDAP_ACCESS_SERVICE:
             ret = sdap_access_service( state->pd, state->user_entry);
             break;
@@ -273,11 +337,11 @@ static void sdap_access_done(struct tevent_req *subreq)
         ret = sdap_access_filter_recv(subreq);
         break;
     case SDAP_ACCESS_CONTROL_PPOLICY_LOCK:
-        ret = sdap_access_lock_recv(subreq);
+        ret = sdap_access_ppolicy_recv(subreq);
         break;
     default:
         ret = EINVAL;
-        DEBUG(SSSDBG_MINOR_FAILURE, "Unknown access control type: %d.",
+        DEBUG(SSSDBG_MINOR_FAILURE, "Unknown access control type: %d.\n",
               state->ac_type);
         break;
     }
@@ -652,7 +716,6 @@ static errno_t sdap_account_expired_nds(struct pam_data *pd,
     return EOK;
 }
 
-
 static errno_t sdap_account_expired(struct sdap_access_ctx *access_ctx,
                                     struct pam_data *pd,
                                     struct ldb_message *user_entry)
@@ -703,6 +766,37 @@ static errno_t sdap_account_expired(struct sdap_access_ctx *access_ctx,
     return ret;
 }
 
+static errno_t perform_pwexpire_policy(TALLOC_CTX *mem_ctx,
+                                       struct sss_domain_info *domain,
+                                       struct pam_data *pd,
+                                       struct sdap_options *opts)
+{
+    enum pwexpire pw_expire_type;
+    void *pw_expire_data;
+    errno_t ret;
+    char *dn;
+
+    ret = get_user_dn(mem_ctx, domain, opts, pd->user, &dn, &pw_expire_type,
+                      &pw_expire_data);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE, "get_user_dn returned %d:[%s].\n",
+              ret, sss_strerror(ret));
+        goto done;
+    }
+
+    ret = check_pwexpire_policy(pw_expire_type, pw_expire_data, pd,
+                                domain->pwd_expiration_warning);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "check_pwexpire_policy returned %d:[%s].\n",
+              ret, sss_strerror(ret));
+        goto done;
+    }
+
+done:
+    return ret;
+}
+
 struct sdap_access_filter_req_ctx {
     const char *username;
     const char *filter;
@@ -720,8 +814,8 @@ struct sdap_access_filter_req_ctx {
 
 static errno_t sdap_access_decide_offline(bool cached_ac);
 static int sdap_access_filter_retry(struct tevent_req *req);
-static void sdap_access_lock_connect_done(struct tevent_req *subreq);
-static errno_t sdap_access_lock_get_lockout_step(struct tevent_req *req);
+static void sdap_access_ppolicy_connect_done(struct tevent_req *subreq);
+static errno_t sdap_access_ppolicy_get_lockout_step(struct tevent_req *req);
 static void sdap_access_filter_connect_done(struct tevent_req *subreq);
 static void sdap_access_filter_done(struct tevent_req *req);
 static struct tevent_req *sdap_access_filter_send(TALLOC_CTX *mem_ctx,
@@ -1122,7 +1216,7 @@ static errno_t sdap_save_user_cache_bool(struct sss_domain_info *domain,
         goto done;
     }
 
-    ret = sysdb_set_user_attr(domain->sysdb, domain, username, attrs, SYSDB_MOD_REP);
+    ret = sysdb_set_user_attr(domain, username, attrs, SYSDB_MOD_REP);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Failed to set user access attribute\n");
         goto done;
@@ -1196,12 +1290,12 @@ static errno_t sdap_access_host(struct ldb_message *user_entry)
     return ret;
 }
 
-static void sdap_access_lock_get_lockout_done(struct tevent_req *subreq);
-static int sdap_access_lock_retry(struct tevent_req *req);
-static errno_t sdap_access_lock_step(struct tevent_req *req);
-static void sdap_access_lock_step_done(struct tevent_req *subreq);
+static void sdap_access_ppolicy_get_lockout_done(struct tevent_req *subreq);
+static int sdap_access_ppolicy_retry(struct tevent_req *req);
+static errno_t sdap_access_ppolicy_step(struct tevent_req *req);
+static void sdap_access_ppolicy_step_done(struct tevent_req *subreq);
 
-struct sdap_access_lock_req_ctx {
+struct sdap_access_ppolicy_req_ctx {
     const char *username;
     const char *filter;
     struct tevent_context *ev;
@@ -1217,24 +1311,26 @@ struct sdap_access_lock_req_ctx {
     /* default DNs to ppolicy */
     const char **ppolicy_dns;
     unsigned int ppolicy_dns_index;
+    enum sdap_pwpolicy_mode pwpol_mode;
 };
 
 static struct tevent_req *
-sdap_access_lock_send(TALLOC_CTX *mem_ctx,
-                      struct tevent_context *ev,
-                      struct be_ctx *be_ctx,
-                      struct sss_domain_info *domain,
-                      struct sdap_access_ctx *access_ctx,
-                      struct sdap_id_conn_ctx *conn,
-                      const char *username,
-                      struct ldb_message *user_entry)
+sdap_access_ppolicy_send(TALLOC_CTX *mem_ctx,
+                         struct tevent_context *ev,
+                         struct be_ctx *be_ctx,
+                         struct sss_domain_info *domain,
+                         struct sdap_access_ctx *access_ctx,
+                         struct sdap_id_conn_ctx *conn,
+                         const char *username,
+                         struct ldb_message *user_entry,
+                         enum sdap_pwpolicy_mode pwpol_mode)
 {
-    struct sdap_access_lock_req_ctx *state;
+    struct sdap_access_ppolicy_req_ctx *state;
     struct tevent_req *req;
     errno_t ret;
 
     req = tevent_req_create(mem_ctx,
-                            &state, struct sdap_access_lock_req_ctx);
+                            &state, struct sdap_access_ppolicy_req_ctx);
     if (req == NULL) {
         return NULL;
     }
@@ -1247,9 +1343,10 @@ sdap_access_lock_send(TALLOC_CTX *mem_ctx,
     state->access_ctx = access_ctx;
     state->domain = domain;
     state->ppolicy_dns_index = 0;
+    state->pwpol_mode = pwpol_mode;
 
     DEBUG(SSSDBG_TRACE_FUNC,
-          "Performing access lock check for user [%s]\n", username);
+          "Performing access ppolicy check for user [%s]\n", username);
 
     state->cached_access = ldb_msg_find_attr_as_bool(
         user_entry, SYSDB_LDAP_ACCESS_CACHED_LOCKOUT, false);
@@ -1267,7 +1364,7 @@ sdap_access_lock_send(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    DEBUG(SSSDBG_TRACE_FUNC, "Checking lock against LDAP\n");
+    DEBUG(SSSDBG_TRACE_FUNC, "Checking ppolicy against LDAP\n");
 
     state->sdap_op = sdap_id_op_create(state,
                                        state->conn->conn_cache);
@@ -1277,7 +1374,7 @@ sdap_access_lock_send(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    ret = sdap_access_lock_retry(req);
+    ret = sdap_access_ppolicy_retry(req);
     if (ret != EOK) {
         goto done;
     }
@@ -1294,21 +1391,22 @@ done:
     return req;
 }
 
-static int sdap_access_lock_retry(struct tevent_req *req)
+static int sdap_access_ppolicy_retry(struct tevent_req *req)
 {
-    struct sdap_access_lock_req_ctx *state;
+    struct sdap_access_ppolicy_req_ctx *state;
     struct tevent_req *subreq;
     int ret;
 
-    state = tevent_req_data(req, struct sdap_access_lock_req_ctx);
+    state = tevent_req_data(req, struct sdap_access_ppolicy_req_ctx);
     subreq = sdap_id_op_connect_send(state->sdap_op, state, &ret);
     if (!subreq) {
         DEBUG(SSSDBG_OP_FAILURE,
-              "sdap_id_op_connect_send failed: %d (%s)\n", ret, strerror(ret));
+              "sdap_id_op_connect_send failed: %d (%s)\n",
+              ret, sss_strerror(ret));
         return ret;
     }
 
-    tevent_req_set_callback(subreq, sdap_access_lock_connect_done, req);
+    tevent_req_set_callback(subreq, sdap_access_ppolicy_connect_done, req);
     return EOK;
 }
 
@@ -1335,15 +1433,15 @@ get_default_ppolicy_dns(TALLOC_CTX *mem_ctx, struct sdap_domain *sdom)
     return ppolicy_dns;
 }
 
-static void sdap_access_lock_connect_done(struct tevent_req *subreq)
+static void sdap_access_ppolicy_connect_done(struct tevent_req *subreq)
 {
     struct tevent_req *req;
-    struct sdap_access_lock_req_ctx *state;
+    struct sdap_access_ppolicy_req_ctx *state;
     int ret, dp_error;
     const char *ppolicy_dn;
 
     req = tevent_req_callback_data(subreq, struct tevent_req);
-    state = tevent_req_data(req, struct sdap_access_lock_req_ctx);
+    state = tevent_req_data(req, struct sdap_access_ppolicy_req_ctx);
 
     ret = sdap_id_op_connect_recv(subreq, &dp_error);
     talloc_zfree(subreq);
@@ -1369,7 +1467,7 @@ static void sdap_access_lock_connect_done(struct tevent_req *subreq)
         state->ppolicy_dns = talloc_array(state, const char*, 2);
         if (state->ppolicy_dns == NULL) {
             DEBUG(SSSDBG_CRIT_FAILURE, "Could not allocate ppolicy_dns.\n");
-            tevent_req_error(req, ERR_ACCESS_DENIED);
+            tevent_req_error(req, ERR_INTERNAL);
             return;
         }
 
@@ -1383,7 +1481,7 @@ static void sdap_access_lock_connect_done(struct tevent_req *subreq)
 
         state->ppolicy_dns = get_default_ppolicy_dns(state, state->opts->sdom);
         if (state->ppolicy_dns == NULL) {
-            tevent_req_error(req, ERR_ACCESS_DENIED);
+            tevent_req_error(req, ERR_INTERNAL);
             return;
         }
     }
@@ -1391,28 +1489,33 @@ static void sdap_access_lock_connect_done(struct tevent_req *subreq)
     /* Connection to LDAP succeeded
      * Send 'pwdLockout' request
      */
-    ret = sdap_access_lock_get_lockout_step(req);
+    ret = sdap_access_ppolicy_get_lockout_step(req);
     if (ret != EOK && ret != EAGAIN) {
         DEBUG(SSSDBG_CRIT_FAILURE,
-              "sdap_access_lock_get_lockout_step failed: [%d][%s]\n",
-              ret, strerror(ret));
-        tevent_req_error(req, ERR_ACCESS_DENIED);
+              "sdap_access_ppolicy_get_lockout_step failed: [%d][%s]\n",
+              ret, sss_strerror(ret));
+        tevent_req_error(req, ERR_INTERNAL);
         return;
     }
+
+    if (ret == EOK) {
+        tevent_req_done(req);
+    }
 }
 
 static errno_t
-sdap_access_lock_get_lockout_step(struct tevent_req *req)
+sdap_access_ppolicy_get_lockout_step(struct tevent_req *req)
 {
     const char *attrs[] = { SYSDB_LDAP_ACCESS_LOCKOUT, NULL };
-    struct sdap_access_lock_req_ctx *state;
+    struct sdap_access_ppolicy_req_ctx *state;
     struct tevent_req *subreq;
     errno_t ret;
 
-    state = tevent_req_data(req, struct sdap_access_lock_req_ctx);
+    state = tevent_req_data(req, struct sdap_access_ppolicy_req_ctx);
 
     /* no more DNs to try */
     if (state->ppolicy_dns[state->ppolicy_dns_index] == NULL) {
+        DEBUG(SSSDBG_TRACE_FUNC, "No more DNs to try.\n");
         ret = EOK;
         goto done;
     }
@@ -1434,14 +1537,13 @@ sdap_access_lock_get_lockout_step(struct tevent_req *req)
                                    false);
     if (subreq == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Could not start LDAP communication\n");
-        tevent_req_error(req, EIO);
         ret = EIO;
         goto done;
     }
 
     /* try next basedn */
     state->ppolicy_dns_index++;
-    tevent_req_set_callback(subreq, sdap_access_lock_get_lockout_done, req);
+    tevent_req_set_callback(subreq, sdap_access_ppolicy_get_lockout_done, req);
 
     ret = EAGAIN;
 
@@ -1449,20 +1551,25 @@ done:
     return ret;
 }
 
-static void sdap_access_lock_get_lockout_done(struct tevent_req *subreq)
+static void sdap_access_ppolicy_get_lockout_done(struct tevent_req *subreq)
 {
     int ret, tret, dp_error;
     size_t num_results;
     bool pwdLockout = false;
     struct sysdb_attrs **results;
     struct tevent_req *req;
-    struct sdap_access_lock_req_ctx *state;
+    struct sdap_access_ppolicy_req_ctx *state;
 
     req = tevent_req_callback_data(subreq, struct tevent_req);
-    state = tevent_req_data(req, struct sdap_access_lock_req_ctx);
+    state = tevent_req_data(req, struct sdap_access_ppolicy_req_ctx);
 
     ret = sdap_get_generic_recv(subreq, state, &num_results, &results);
     talloc_zfree(subreq);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot retrieve ppolicy\n");
+        ret = ERR_NETWORK_IO;
+        goto done;
+    }
 
     /* Check the number of responses we got
      * If it's exactly 1, we passed the check
@@ -1472,7 +1579,7 @@ static void sdap_access_lock_get_lockout_done(struct tevent_req *subreq)
     /* Didn't find ppolicy attribute */
     if (num_results < 1) {
         /* Try using next $search_base */
-        ret = sdap_access_lock_get_lockout_step(req);
+        ret = sdap_access_ppolicy_get_lockout_step(req);
         if (ret == EOK) {
             /* No more search bases to try */
             DEBUG(SSSDBG_CONF_SETTINGS,
@@ -1481,8 +1588,9 @@ static void sdap_access_lock_get_lockout_done(struct tevent_req *subreq)
         } else {
             if (ret != EAGAIN) {
                 DEBUG(SSSDBG_CRIT_FAILURE,
-                      "sdap_access_lock_get_lockout_step failed: [%d][%s]\n",
-                      ret, strerror(ret));
+                      "sdap_access_ppolicy_get_lockout_step failed: "
+                      "[%d][%s]\n",
+                      ret, sss_strerror(ret));
             }
             goto done;
         }
@@ -1503,7 +1611,7 @@ static void sdap_access_lock_get_lockout_done(struct tevent_req *subreq)
         if (ret != EOK) {
             DEBUG(SSSDBG_MINOR_FAILURE,
                   "Error reading %s: [%s]\n", SYSDB_LDAP_ACCESS_LOCKOUT,
-                  strerror(ret));
+                  sss_strerror(ret));
             ret = ERR_INTERNAL;
             goto done;
         }
@@ -1514,11 +1622,11 @@ static void sdap_access_lock_get_lockout_done(struct tevent_req *subreq)
               "Password policy is enabled on LDAP server.\n");
 
         /* ppolicy is enabled => find out if account is locked */
-        ret = sdap_access_lock_step(req);
+        ret = sdap_access_ppolicy_step(req);
         if (ret != EOK && ret != EAGAIN) {
             DEBUG(SSSDBG_CRIT_FAILURE,
-                  "sdap_access_lock_step failed: [%d][%s].\n",
-                  ret, strerror(ret));
+                  "sdap_access_ppolicy_step failed: [%d][%s].\n",
+                  ret, sss_strerror(ret));
         }
         goto done;
     } else {
@@ -1559,14 +1667,16 @@ done:
     }
 }
 
-errno_t sdap_access_lock_step(struct tevent_req *req)
+errno_t sdap_access_ppolicy_step(struct tevent_req *req)
 {
     errno_t ret;
     struct tevent_req *subreq;
-    struct sdap_access_lock_req_ctx *state;
-    const char *attrs[] = { SYSDB_LDAP_ACCESS_LOCKED_TIME, NULL };
+    struct sdap_access_ppolicy_req_ctx *state;
+    const char *attrs[] = { SYSDB_LDAP_ACCESS_LOCKED_TIME,
+                            SYSDB_LDAP_ACESS_LOCKOUT_DURATION,
+                            NULL };
 
-    state = tevent_req_data(req, struct sdap_access_lock_req_ctx);
+    state = tevent_req_data(req, struct sdap_access_ppolicy_req_ctx);
 
     subreq = sdap_get_generic_send(state,
                                    state->ev,
@@ -1581,30 +1691,116 @@ errno_t sdap_access_lock_step(struct tevent_req *req)
                                    false);
 
     if (subreq == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "sdap_access_lock_send failed.\n");
+        DEBUG(SSSDBG_CRIT_FAILURE, "sdap_access_ppolicy_send failed.\n");
         ret = ENOMEM;
         goto done;
     }
 
-    tevent_req_set_callback(subreq, sdap_access_lock_step_done, req);
+    tevent_req_set_callback(subreq, sdap_access_ppolicy_step_done, req);
     ret = EAGAIN;
 
 done:
     return ret;
 }
 
-static void sdap_access_lock_step_done(struct tevent_req *subreq)
+static errno_t
+is_account_locked(const char *pwdAccountLockedTime,
+                  const char *pwdAccountLockedDurationTime,
+                  enum sdap_pwpolicy_mode pwpol_mode,
+                  const char *username,
+                  bool *_locked)
+{
+    errno_t ret;
+    time_t lock_time;
+    time_t duration;
+    time_t now;
+    bool locked;
+
+    /* Default action is to consider account to be locked. */
+    locked = true;
+
+    /* account is permanently locked */
+    if (strcasecmp(pwdAccountLockedTime,
+                   PERMANENTLY_LOCKED_ACCOUNT) == 0) {
+        ret = EOK;
+        goto done;
+    }
+
+    switch(pwpol_mode) {
+    case PWP_LOCKOUT_ONLY:
+        /* We do *not* care about exact value of account locked time, we
+         * only *do* care if the value is equal to
+         * PERMANENTLY_LOCKED_ACCOUNT, which means that account is locked
+         * permanently.
+         */
+        DEBUG(SSSDBG_TRACE_FUNC,
+              "Account of: %s is being blocked by password policy, "
+              "but value: [%s] value is ignored by SSSD.\n",
+              username, pwdAccountLockedTime);
+        locked = false;
+        break;
+    case PWP_LOCKOUT_EXPIRE:
+        /* Account may be locked out from natural reasons (too many attempts,
+         * expired password). In this case, pwdAccountLockedTime is also set,
+         * to the time of lock out.
+         */
+        ret = sss_utc_to_time_t(pwdAccountLockedTime, "%Y%m%d%H%M%SZ",
+                                &lock_time);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_TRACE_FUNC, "sss_utc_to_time_t failed with %d:%s.\n",
+                  ret, sss_strerror(ret));
+            goto done;
+        }
+
+        now = time(NULL);
+
+        /* Account was NOT locked in past. */
+        if (difftime(lock_time, now) > 0.0) {
+            locked = false;
+        } else if (pwdAccountLockedDurationTime != NULL) {
+            errno = 0;
+            duration = strtouint32(pwdAccountLockedDurationTime, NULL, 0);
+            if (errno) {
+                ret = errno;
+                goto done;
+            }
+            /* Lockout has expired */
+            if (duration != 0 && difftime(now, lock_time) > duration) {
+                locked = false;
+            }
+        }
+        break;
+    case PWP_SENTINEL:
+    default:
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "Unexpected value of password policy mode: %d.\n", pwpol_mode);
+        ret = EINVAL;
+        goto done;
+    }
+
+    ret = EOK;
+
+done:
+    if (ret == EOK) {
+        *_locked = locked;
+    }
+
+    return ret;
+}
+
+static void sdap_access_ppolicy_step_done(struct tevent_req *subreq)
 {
     int ret, tret, dp_error;
     size_t num_results;
     bool locked = false;
     const char *pwdAccountLockedTime;
+    const char *pwdAccountLockedDurationTime;
     struct sysdb_attrs **results;
     struct tevent_req *req;
-    struct sdap_access_lock_req_ctx *state;
+    struct sdap_access_ppolicy_req_ctx *state;
 
     req = tevent_req_callback_data(subreq, struct tevent_req);
-    state = tevent_req_data(req, struct sdap_access_lock_req_ctx);
+    state = tevent_req_data(req, struct sdap_access_ppolicy_req_ctx);
 
     ret = sdap_get_generic_recv(subreq, state, &num_results, &results);
     talloc_zfree(subreq);
@@ -1613,7 +1809,7 @@ static void sdap_access_lock_step_done(struct tevent_req *subreq)
     if (ret != EOK) {
         if (dp_error == DP_ERR_OK) {
             /* retry */
-            tret = sdap_access_lock_retry(req);
+            tret = sdap_access_ppolicy_retry(req);
             if (tret == EOK) {
                 return;
             }
@@ -1636,7 +1832,7 @@ static void sdap_access_lock_step_done(struct tevent_req *subreq)
     if (num_results < 1) {
         DEBUG(SSSDBG_CONF_SETTINGS,
               "User [%s] was not found with the specified filter. "
-                  "Denying access.\n", state->username);
+              "Denying access.\n", state->username);
     } else if (results == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE, "num_results > 0, but results is NULL\n");
         ret = ERR_INTERNAL;
@@ -1649,22 +1845,35 @@ static void sdap_access_lock_step_done(struct tevent_req *subreq)
         ret = ERR_INTERNAL;
         goto done;
     } else { /* Ok, we got a single reply */
+        ret = sysdb_attrs_get_string(results[0], SYSDB_LDAP_ACESS_LOCKOUT_DURATION,
+                                     &pwdAccountLockedDurationTime);
+        if (ret != EOK) {
+            /* This attribute might not be set even if account is locked */
+            pwdAccountLockedDurationTime = NULL;
+        }
+
         ret = sysdb_attrs_get_string(results[0], SYSDB_LDAP_ACCESS_LOCKED_TIME,
                                      &pwdAccountLockedTime);
         if (ret == EOK) {
-            /* We do *not* care about exact value of account locked time, we
-             * only *do* care if the value is equal to
-             * PERMANENTLY_LOCKED_ACCOUNT, which means that account is locked
-             * permanently.
-             */
-            if (strcasecmp(pwdAccountLockedTime,
-                           PERMANENTLY_LOCKED_ACCOUNT) == 0) {
+
+            ret = is_account_locked(pwdAccountLockedTime,
+                                    pwdAccountLockedDurationTime,
+                                    state->pwpol_mode,
+                                    state->username,
+                                    &locked);
+            if (ret != EOK) {
+                if (ret == ERR_TIMESPEC_NOT_SUPPORTED) {
+                    DEBUG(SSSDBG_MINOR_FAILURE,
+                          "timezone specifier in ppolicy is not supported\n");
+                } else {
+                    DEBUG(SSSDBG_MINOR_FAILURE,
+                          "is_account_locked failed: %d:[%s].\n",
+                          ret, sss_strerror(ret));
+                }
+
+                DEBUG(SSSDBG_MINOR_FAILURE,
+                      "Account will be considered to be locked.\n");
                 locked = true;
-            } else {
-                DEBUG(SSSDBG_TRACE_FUNC,
-                      "Account of: %s is beeing blocked by password policy, "
-                      "but value: [%s] value is ignored by SSSD.\n",
-                      state->username, pwdAccountLockedTime);
             }
         } else {
             /* Attribute SYSDB_LDAP_ACCESS_LOCKED_TIME in not be present unless
@@ -1710,7 +1919,7 @@ done:
     }
 }
 
-static errno_t sdap_access_lock_recv(struct tevent_req *req)
+static errno_t sdap_access_ppolicy_recv(struct tevent_req *req)
 {
     TEVENT_REQ_RETURN_ON_ERROR(req);
 
