@@ -19,31 +19,14 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+
 #include <ctype.h>
 #include "util/util.h"
 #include "util/strtonum.h"
 #include "providers/ldap/sdap_async_private.h"
 
-#define REALM_SEPARATOR '@'
 #define REPLY_REALLOC_INCREMENT 10
 
-void make_realm_upper_case(const char *upn)
-{
-    char *c;
-
-    c = strchr(upn, REALM_SEPARATOR);
-    if (c == NULL) {
-        DEBUG(SSSDBG_TRACE_ALL, "No realm delimiter found in upn [%s].\n", upn);
-        return;
-    }
-
-    while(*(++c) != '\0') {
-        c[0] = toupper(*c);
-    }
-
-    return;
-}
-
 /* ==LDAP-Memory-Handling================================================= */
 
 static int lmsg_destructor(void *mem)
@@ -1106,6 +1089,40 @@ static errno_t add_to_deref_reply(TALLOC_CTX *mem_ctx,
     return EOK;
 }
 
+static void sdap_print_server(struct sdap_handle *sh)
+{
+    int ret;
+    int fd;
+    struct sockaddr_storage ss;
+    socklen_t ss_len = sizeof(ss);
+    char ip[NI_MAXHOST];
+
+    if (!DEBUG_IS_SET(SSSDBG_TRACE_INTERNAL)) {
+        return;
+    }
+
+    ret = get_fd_from_ldap(sh->ldap, &fd);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE, "cannot get sdap fd\n");
+        return;
+    }
+
+    ret = getpeername(fd, (struct sockaddr *) &ss, &ss_len);
+    if (ret == -1) {
+        DEBUG(SSSDBG_MINOR_FAILURE, "getsockname failed\n");
+        return;
+    }
+
+    ret = getnameinfo((struct sockaddr *) &ss, ss_len,
+                      ip, sizeof(ip), NULL, 0, NI_NUMERICHOST);
+    if (ret != 0) {
+        DEBUG(SSSDBG_MINOR_FAILURE, "getnameinfo failed\n");
+        return;
+    }
+
+    DEBUG(SSSDBG_TRACE_INTERNAL, "Searching %s\n", ip);
+}
+
 /* ==Generic Search exposing all options======================= */
 typedef errno_t (*sdap_parse_cb)(struct sdap_handle *sh,
                                  struct sdap_msg *msg,
@@ -1139,9 +1156,9 @@ struct sdap_get_generic_ext_state {
 
 static errno_t sdap_get_generic_ext_step(struct tevent_req *req);
 
-static void sdap_get_generic_ext_done(struct sdap_op *op,
-                                      struct sdap_msg *reply,
-                                      int error, void *pvt);
+static void sdap_get_generic_op_finished(struct sdap_op *op,
+                                         struct sdap_msg *reply,
+                                         int error, void *pvt);
 
 static struct tevent_req *
 sdap_get_generic_ext_send(TALLOC_CTX *memctx,
@@ -1195,6 +1212,8 @@ sdap_get_generic_ext_send(TALLOC_CTX *memctx,
         return req;
     }
 
+    sdap_print_server(sh);
+
     /* Be extra careful and never allow paging for BASE searches,
      * even if requested.
      */
@@ -1338,7 +1357,7 @@ static errno_t sdap_get_generic_ext_step(struct tevent_req *req)
     DEBUG(SSSDBG_TRACE_INTERNAL, "ldap_search_ext called, msgid = %d\n", msgid);
 
     ret = sdap_op_add(state, state->ev, state->sh, msgid,
-                      sdap_get_generic_ext_done, req,
+                      sdap_get_generic_op_finished, req,
                       state->timeout,
                       &state->op);
     if (ret != EOK) {
@@ -1350,14 +1369,16 @@ done:
     return ret;
 }
 
-static void sdap_get_generic_ext_done(struct sdap_op *op,
-                                      struct sdap_msg *reply,
-                                      int error, void *pvt)
+static void sdap_get_generic_op_finished(struct sdap_op *op,
+                                         struct sdap_msg *reply,
+                                         int error, void *pvt)
 {
     struct tevent_req *req = talloc_get_type(pvt, struct tevent_req);
     struct sdap_get_generic_ext_state *state = tevent_req_data(req,
                                             struct sdap_get_generic_ext_state);
     char *errmsg = NULL;
+    int i;
+    char **refs = NULL;
     int result;
     int ret;
     int lret;
@@ -1393,7 +1414,7 @@ static void sdap_get_generic_ext_done(struct sdap_op *op,
 
     case LDAP_RES_SEARCH_RESULT:
         ret = ldap_parse_result(state->sh->ldap, reply->msg,
-                                &result, NULL, &errmsg, NULL,
+                                &result, NULL, &errmsg, &refs,
                                 &returned_controls, 0);
         if (ret != LDAP_SUCCESS) {
             DEBUG(SSSDBG_OP_FAILURE,
@@ -1427,6 +1448,12 @@ static void sdap_get_generic_ext_done(struct sdap_op *op,
             tevent_req_error(req, ENOTSUP);
             return;
         } else if (result == LDAP_REFERRAL) {
+            if (refs != NULL) {
+                for (i = 0; refs[i]; i++) {
+                    DEBUG(SSSDBG_TRACE_LIBS, "Ref: %s\n", refs[i]);
+                }
+                ldap_memvfree((void **) refs);
+            }
             ldap_memfree(errmsg);
             tevent_req_error(req, ERR_REFERRAL);
             return;
@@ -1454,7 +1481,7 @@ static void sdap_get_generic_ext_done(struct sdap_op *op,
                                                &total_count, &cookie);
         ldap_controls_free(returned_controls);
         if (lret != LDAP_SUCCESS) {
-            DEBUG(SSSDBG_CRIT_FAILURE, "Could not determine page control");
+            DEBUG(SSSDBG_CRIT_FAILURE, "Could not determine page control\n");
             tevent_req_error(req, EIO);
             return;
         }
@@ -1505,6 +1532,31 @@ sdap_get_generic_ext_recv(struct tevent_req *req)
     return EOK;
 }
 
+static void generic_ext_search_handler(struct tevent_req *subreq,
+                                       struct sdap_options *opts)
+{
+    struct tevent_req *req = tevent_req_callback_data(subreq,
+                                                      struct tevent_req);
+    int ret;
+
+    ret = sdap_get_generic_ext_recv(subreq);
+    talloc_zfree(subreq);
+    if (ret == ERR_REFERRAL) {
+        if (dp_opt_get_bool(opts->basic, SDAP_REFERRALS)) {
+            tevent_req_error(req, ret);
+            return;
+        }
+    } else if (ret) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "sdap_get_generic_ext_recv failed [%d]: %s\n",
+              ret, sss_strerror(ret));
+        tevent_req_error(req, ret);
+        return;
+    }
+
+    tevent_req_done(req);
+}
+
 /* ==Generic Search============================================ */
 struct sdap_get_generic_state {
     struct sdap_attr_map *map;
@@ -1570,7 +1622,7 @@ static errno_t sdap_get_generic_parse_entry(struct sdap_handle *sh,
 
     ret = sdap_parse_entry(state, sh, msg,
                            state->map, state->map_num_attrs,
-                           &attrs, NULL, disable_range_rtrvl);
+                           &attrs, disable_range_rtrvl);
     if (ret != EOK) {
         DEBUG(SSSDBG_MINOR_FAILURE,
               "sdap_parse_entry failed [%d]: %s\n", ret, strerror(ret));
@@ -1594,24 +1646,8 @@ static void sdap_get_generic_done(struct tevent_req *subreq)
                                                       struct tevent_req);
     struct sdap_get_generic_state *state =
                 tevent_req_data(req, struct sdap_get_generic_state);
-    int ret;
 
-    ret = sdap_get_generic_ext_recv(subreq);
-    talloc_zfree(subreq);
-    if (ret == ERR_REFERRAL) {
-        if (dp_opt_get_bool(state->opts->basic, SDAP_REFERRALS)) {
-            tevent_req_error(req, ret);
-            return;
-        }
-    } else if (ret) {
-        DEBUG(SSSDBG_CONF_SETTINGS,
-              "sdap_get_generic_ext_recv failed [%d]: %s\n",
-                  ret, sss_strerror(ret));
-        tevent_req_error(req, ret);
-        return;
-    }
-
-    tevent_req_done(req);
+    return generic_ext_search_handler(subreq, state->opts);
 }
 
 int sdap_get_generic_recv(struct tevent_req *req,
@@ -1645,9 +1681,9 @@ static errno_t sdap_x_deref_parse_entry(struct sdap_handle *sh,
 struct sdap_x_deref_search_state {
     struct sdap_handle *sh;
     struct sdap_op *op;
-    struct sdap_options *opts;
     struct sdap_attr_map_info *maps;
     LDAPControl **ctrls;
+    struct sdap_options *opts;
 
     struct sdap_deref_reply dreply;
     int num_maps;
@@ -1656,9 +1692,10 @@ struct sdap_x_deref_search_state {
 static struct tevent_req *
 sdap_x_deref_search_send(TALLOC_CTX *memctx, struct tevent_context *ev,
                          struct sdap_options *opts, struct sdap_handle *sh,
-                         const char *base_dn, const char *deref_attr,
-                         const char **attrs, struct sdap_attr_map_info *maps,
-                         int num_maps, int timeout)
+                         const char *base_dn, const char *filter,
+                         const char *deref_attr, const char **attrs,
+                         struct sdap_attr_map_info *maps, int num_maps,
+                         int timeout)
 {
     struct tevent_req *req = NULL;
     struct tevent_req *subreq = NULL;
@@ -1692,7 +1729,9 @@ sdap_x_deref_search_send(TALLOC_CTX *memctx, struct tevent_context *ev,
     DEBUG(SSSDBG_TRACE_FUNC,
           "Dereferencing entry [%s] using OpenLDAP deref\n", base_dn);
     subreq = sdap_get_generic_ext_send(state, ev, opts, sh, base_dn,
-                                       LDAP_SCOPE_BASE, NULL, attrs,
+                                       filter == NULL ? LDAP_SCOPE_BASE
+                                                      : LDAP_SCOPE_SUBTREE,
+                                       filter, attrs,
                                        false, state->ctrls, NULL, 0, timeout,
                                        true, sdap_x_deref_parse_entry,
                                        state);
@@ -1763,8 +1802,13 @@ static errno_t sdap_x_deref_parse_entry(struct sdap_handle *sh,
     }
 
     if (!ctrls) {
+        /* When we attempt to request attributes that are not present in
+         * the dereferenced links, some serves might not send the dereference
+         * control back at all. Be permissive and treat the search as if
+         * it didn't find anything.
+         */
         DEBUG(SSSDBG_MINOR_FAILURE, "No controls found for entry\n");
-        ret = ENOENT;
+        ret = EOK;
         goto done;
     }
 
@@ -1823,26 +1867,10 @@ static void sdap_x_deref_search_done(struct tevent_req *subreq)
 {
     struct tevent_req *req = tevent_req_callback_data(subreq,
                                                       struct tevent_req);
-    struct sdap_x_deref_search_state *state = tevent_req_data(req,
-                                            struct sdap_x_deref_search_state);
-    int ret;
+    struct sdap_x_deref_search_state *state =
+                tevent_req_data(req, struct sdap_x_deref_search_state);
 
-    ret = sdap_get_generic_ext_recv(subreq);
-    talloc_zfree(subreq);
-    if (ret == ERR_REFERRAL) {
-        if (dp_opt_get_bool(state->opts->basic, SDAP_REFERRALS)) {
-            tevent_req_error(req, ret);
-            return;
-        }
-    } else if (ret) {
-        DEBUG(SSSDBG_CONF_SETTINGS,
-              "sdap_get_generic_ext_recv failed [%d]: %s\n",
-                  ret, sss_strerror(ret));
-        tevent_req_error(req, ret);
-        return;
-    }
-
-    tevent_req_done(req);
+    return generic_ext_search_handler(subreq, state->opts);
 }
 
 static int sdap_x_deref_search_ctrls_destructor(void *ptr)
@@ -1873,6 +1901,177 @@ sdap_x_deref_search_recv(struct tevent_req *req,
     return EOK;
 }
 
+/* ==Security Descriptor (ACL) search=================================== */
+struct sdap_sd_search_state {
+  LDAPControl **ctrls;
+  struct sdap_options *opts;
+  size_t reply_count;
+  struct sysdb_attrs **reply;
+  struct sdap_reply sreply;
+};
+
+static int sdap_sd_search_create_control(struct sdap_handle *sh,
+					 int val,
+					 LDAPControl **ctrl);
+static int sdap_sd_search_ctrls_destructor(void *ptr);
+static errno_t sdap_sd_search_parse_entry(struct sdap_handle *sh,
+                                          struct sdap_msg *msg,
+                                          void *pvt);
+static void sdap_sd_search_done(struct tevent_req *subreq);
+
+struct tevent_req *
+sdap_sd_search_send(TALLOC_CTX *memctx, struct tevent_context *ev,
+                     struct sdap_options *opts, struct sdap_handle *sh,
+                     const char *base_dn, int sd_flags,
+                     const char **attrs, int timeout)
+{
+    struct tevent_req *req = NULL;
+    struct tevent_req *subreq = NULL;
+    struct sdap_sd_search_state *state;
+    int ret;
+
+    req = tevent_req_create(memctx, &state, struct sdap_sd_search_state);
+    if (!req) return NULL;
+
+    state->ctrls = talloc_zero_array(state, LDAPControl *, 2);
+    state->opts = opts;
+    if (state->ctrls == NULL) {
+        ret = EIO;
+        goto fail;
+    }
+    talloc_set_destructor((TALLOC_CTX *) state->ctrls,
+                          sdap_sd_search_ctrls_destructor);
+
+    ret = sdap_sd_search_create_control(sh, sd_flags, &state->ctrls[0]);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Could not create SD control\n");
+        ret = EIO;
+        goto fail;
+    }
+
+    DEBUG(SSSDBG_TRACE_FUNC, "Searching entry [%s] using SD\n", base_dn);
+    subreq = sdap_get_generic_ext_send(state, ev, opts, sh, base_dn,
+                                       LDAP_SCOPE_BASE, "(objectclass=*)", attrs,
+                                       false, state->ctrls, NULL, 0, timeout,
+                                       true, sdap_sd_search_parse_entry,
+                                       state);
+    if (!subreq) {
+        ret = EIO;
+        goto fail;
+    }
+    tevent_req_set_callback(subreq, sdap_sd_search_done, req);
+    return req;
+
+fail:
+    tevent_req_error(req, ret);
+    tevent_req_post(req, ev);
+    return req;
+}
+
+static int sdap_sd_search_create_control(struct sdap_handle *sh,
+					 int val,
+					 LDAPControl **ctrl)
+{
+    struct berval *sdval;
+    int ret;
+    BerElement *ber = NULL;
+    ber = ber_alloc_t(LBER_USE_DER);
+    if (ber == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "ber_alloc_t failed.\n");
+        return ENOMEM;
+    }
+
+    ret = ber_printf(ber, "{i}", val);
+    if (ret == -1) {
+        DEBUG(SSSDBG_OP_FAILURE, "ber_printf failed.\n");
+        ber_free(ber, 1);
+        return EIO;
+    }
+
+    ret = ber_flatten(ber, &sdval);
+    ber_free(ber, 1);
+    if (ret == -1) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "ber_flatten failed.\n");
+        return EIO;
+    }
+
+    ret = sdap_control_create(sh, LDAP_SERVER_SD_OID, 1, sdval, 1, ctrl);
+    ber_bvfree(sdval);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "sdap_control_create failed\n");
+        return ret;
+    }
+
+    return EOK;
+}
+
+static errno_t sdap_sd_search_parse_entry(struct sdap_handle *sh,
+                                          struct sdap_msg *msg,
+                                          void *pvt)
+{
+    errno_t ret;
+    struct sysdb_attrs *attrs;
+    struct sdap_sd_search_state *state =
+                talloc_get_type(pvt, struct sdap_sd_search_state);
+
+    bool disable_range_rtrvl = dp_opt_get_bool(state->opts->basic,
+                                               SDAP_DISABLE_RANGE_RETRIEVAL);
+
+    ret = sdap_parse_entry(state, sh, msg,
+                           NULL, 0,
+                           &attrs, disable_range_rtrvl);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "sdap_parse_entry failed [%d]: %s\n", ret, strerror(ret));
+        return ret;
+    }
+
+    ret = add_to_reply(state, &state->sreply, attrs);
+    if (ret != EOK) {
+        talloc_free(attrs);
+        DEBUG(SSSDBG_CRIT_FAILURE, "add_to_reply failed.\n");
+        return ret;
+    }
+
+    /* add_to_reply steals attrs, no need to free them here */
+    return EOK;
+}
+
+static void sdap_sd_search_done(struct tevent_req *subreq)
+{
+    struct tevent_req *req = tevent_req_callback_data(subreq,
+                                                      struct tevent_req);
+    struct sdap_sd_search_state *state =
+                tevent_req_data(req, struct sdap_sd_search_state);
+
+    return generic_ext_search_handler(subreq, state->opts);
+}
+
+static int sdap_sd_search_ctrls_destructor(void *ptr)
+{
+    LDAPControl **ctrls = talloc_get_type(ptr, LDAPControl *);;
+    if (ctrls && ctrls[0]) {
+        ldap_control_free(ctrls[0]);
+    }
+
+    return 0;
+}
+
+int sdap_sd_search_recv(struct tevent_req *req,
+                        TALLOC_CTX *mem_ctx,
+                        size_t *_reply_count,
+                        struct sysdb_attrs ***_reply)
+{
+    struct sdap_sd_search_state *state = tevent_req_data(req,
+                                            struct sdap_sd_search_state);
+    TEVENT_REQ_RETURN_ON_ERROR(req);
+
+    *_reply_count = state->sreply.reply_count;
+    *_reply = talloc_steal(mem_ctx, state->sreply.reply);
+
+    return EOK;
+}
+
 /* ==Attribute scoped search============================================ */
 struct sdap_asq_search_state {
     struct sdap_attr_map_info *maps;
@@ -2065,7 +2264,7 @@ static errno_t sdap_asq_search_parse_entry(struct sdap_handle *sh,
 
         ret = sdap_parse_entry(res[mi], sh, msg,
                                map, num_attrs,
-                               &res[mi]->attrs, NULL, disable_range_rtrvl);
+                               &res[mi]->attrs, disable_range_rtrvl);
         if (ret != EOK) {
             DEBUG(SSSDBG_MINOR_FAILURE,
                   "sdap_parse_entry failed [%d]: %s\n", ret, strerror(ret));
@@ -2093,24 +2292,8 @@ static void sdap_asq_search_done(struct tevent_req *subreq)
                                                       struct tevent_req);
     struct sdap_asq_search_state *state =
                 tevent_req_data(req, struct sdap_asq_search_state);
-    int ret;
 
-    ret = sdap_get_generic_ext_recv(subreq);
-    talloc_zfree(subreq);
-    if (ret == ERR_REFERRAL) {
-        if (dp_opt_get_bool(state->opts->basic, SDAP_REFERRALS)) {
-            tevent_req_error(req, ret);
-            return;
-        }
-    } else if (ret) {
-        DEBUG(SSSDBG_CONF_SETTINGS,
-              "sdap_get_generic_ext_recv failed [%d]: %s\n",
-                  ret, sss_strerror(ret));
-        tevent_req_error(req, ret);
-        return;
-    }
-
-    tevent_req_done(req);
+    return generic_ext_search_handler(subreq, state->opts);
 }
 
 static int sdap_asq_search_ctrls_destructor(void *ptr)
@@ -2190,12 +2373,9 @@ sdap_posix_check_send(TALLOC_CTX *memctx, struct tevent_context *ev,
     state->attrs[2] = opts->group_map[SDAP_AT_GROUP_GID].name;
     state->attrs[3] = NULL;
 
-    state->filter = talloc_asprintf(state,
-        "(|(&(%s=*)(objectclass=%s))(&(%s=*)(objectclass=%s)))",
+    state->filter = talloc_asprintf(state, "(|(%s=*)(%s=*))",
                                     opts->user_map[SDAP_AT_USER_UID].name,
-                                    opts->user_map[SDAP_OC_USER].name,
-                                    opts->group_map[SDAP_AT_GROUP_GID].name,
-                                    opts->group_map[SDAP_OC_GROUP].name);
+                                    opts->group_map[SDAP_AT_GROUP_GID].name);
     if (state->filter == NULL) {
         ret = ENOMEM;
         goto fail;
@@ -2278,8 +2458,9 @@ static errno_t sdap_posix_check_parse(struct sdap_handle *sh,
     errno = 0;
     strtouint32(vals[0]->bv_val, &endptr, 10);
     if (errno || *endptr || (vals[0]->bv_val == endptr)) {
-        DEBUG(SSSDBG_MINOR_FAILURE,
+        DEBUG(SSSDBG_OP_FAILURE,
               "POSIX attribute is not a number: %s\n", vals[0]->bv_val);
+        goto done;
     }
 
     state->has_posix = true;
@@ -2354,6 +2535,69 @@ struct sdap_deref_search_state {
 };
 
 static void sdap_deref_search_done(struct tevent_req *subreq);
+static void sdap_deref_search_with_filter_done(struct tevent_req *subreq);
+
+struct tevent_req *
+sdap_deref_search_with_filter_send(TALLOC_CTX *memctx,
+                                   struct tevent_context *ev,
+                                   struct sdap_options *opts,
+                                   struct sdap_handle *sh,
+                                   const char *search_base,
+                                   const char *filter,
+                                   const char *deref_attr,
+                                   const char **attrs,
+                                   int num_maps,
+                                   struct sdap_attr_map_info *maps,
+                                   int timeout)
+{
+    struct tevent_req *req = NULL;
+    struct tevent_req *subreq = NULL;
+    struct sdap_deref_search_state *state;
+
+    req = tevent_req_create(memctx, &state, struct sdap_deref_search_state);
+    if (!req) return NULL;
+
+    state->sh = sh;
+    state->reply_count = 0;
+    state->reply = NULL;
+
+    if (sdap_is_control_supported(sh, LDAP_CONTROL_X_DEREF)) {
+        DEBUG(SSSDBG_TRACE_INTERNAL, "Server supports OpenLDAP deref\n");
+        state->deref_type = SDAP_DEREF_OPENLDAP;
+
+        subreq = sdap_x_deref_search_send(state, ev, opts, sh, search_base,
+                                          filter, deref_attr, attrs, maps,
+                                          num_maps, timeout);
+        if (!subreq) {
+            DEBUG(SSSDBG_OP_FAILURE, "Cannot start OpenLDAP deref search\n");
+            goto fail;
+        }
+    } else {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "Server does not support any known deref method!\n");
+        goto fail;
+    }
+
+    tevent_req_set_callback(subreq, sdap_deref_search_with_filter_done, req);
+    return req;
+
+fail:
+    talloc_zfree(req);
+    return NULL;
+}
+
+static void sdap_deref_search_with_filter_done(struct tevent_req *subreq)
+{
+    sdap_deref_search_done(subreq);
+}
+
+int sdap_deref_search_with_filter_recv(struct tevent_req *req,
+                                       TALLOC_CTX *mem_ctx,
+                                       size_t *reply_count,
+                                       struct sdap_deref_attrs ***reply)
+{
+    return sdap_deref_search_recv(req, mem_ctx, reply_count, reply);
+}
 
 struct tevent_req *
 sdap_deref_search_send(TALLOC_CTX *memctx,
@@ -2393,7 +2637,7 @@ sdap_deref_search_send(TALLOC_CTX *memctx,
         DEBUG(SSSDBG_TRACE_INTERNAL, "Server supports OpenLDAP deref\n");
         state->deref_type = SDAP_DEREF_OPENLDAP;
 
-        subreq = sdap_x_deref_search_send(state, ev, opts, sh, base_dn,
+        subreq = sdap_x_deref_search_send(state, ev, opts, sh, base_dn, NULL,
                                           deref_attr, attrs, maps, num_maps,
                                           timeout);
         if (!subreq) {
@@ -2501,103 +2745,3 @@ bool sdap_has_deref_support(struct sdap_handle *sh, struct sdap_options *opts)
 
     return false;
 }
-
-errno_t
-sdap_attrs_add_ldap_attr(struct sysdb_attrs *ldap_attrs,
-                         const char *attr_name,
-                         const char *attr_desc,
-                         bool multivalued,
-                         const char *name,
-                         struct sysdb_attrs *attrs)
-{
-    errno_t ret;
-    struct ldb_message_element *el;
-    const char *objname = name ?: "object";
-    const char *desc = attr_desc ?: attr_name;
-    unsigned int num_values, i;
-
-    ret = sysdb_attrs_get_el(ldap_attrs, attr_name, &el);
-    if (ret) {
-        DEBUG(SSSDBG_OP_FAILURE, "Could not get %s from the "
-              "list of the LDAP attributes [%d]: %s\n",
-              attr_name, ret, strerror(ret));
-        return ret;
-    }
-
-    if (el->num_values == 0) {
-        DEBUG(SSSDBG_TRACE_INTERNAL, "%s is not available "
-              "for [%s].\n", desc, objname);
-    } else {
-        num_values = multivalued ? el->num_values : 1;
-        for (i = 0; i < num_values; i++) {
-            DEBUG(SSSDBG_TRACE_INTERNAL, "Adding %s [%s] to attributes "
-                  "of [%s].\n", desc, el->values[i].data, objname);
-
-            ret = sysdb_attrs_add_mem(attrs, attr_name, el->values[i].data,
-                                      el->values[i].length);
-            if (ret) {
-                return ret;
-            }
-        }
-    }
-
-    return EOK;
-}
-
-errno_t
-sdap_save_all_names(const char *name,
-                    struct sysdb_attrs *ldap_attrs,
-                    struct sss_domain_info *dom,
-                    struct sysdb_attrs *attrs)
-{
-    const char **aliases = NULL;
-    const char *domname;
-    errno_t ret;
-    TALLOC_CTX *tmp_ctx;
-    int i;
-    bool lowercase = !dom->case_sensitive;
-
-    tmp_ctx = talloc_new(NULL);
-    if (!tmp_ctx) {
-        ret = ENOMEM;
-        goto done;
-    }
-
-    ret = sysdb_attrs_get_aliases(tmp_ctx, ldap_attrs, name,
-                                  lowercase, &aliases);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "Failed to get the alias list");
-        goto done;
-    }
-
-    for (i = 0; aliases[i]; i++) {
-        domname = sss_get_domain_name(tmp_ctx, aliases[i], dom);
-        if (domname == NULL) {
-            ret = ENOMEM;
-            goto done;
-        }
-
-        if (lowercase) {
-            ret = sysdb_attrs_add_lc_name_alias(attrs, domname);
-            if (ret) {
-                DEBUG(SSSDBG_OP_FAILURE, "Failed to add lower-cased version "
-                                          "of alias [%s] into the "
-                                          "attribute list\n", aliases[i]);
-                goto done;
-            }
-        } else {
-            ret = sysdb_attrs_add_string(attrs, SYSDB_NAME_ALIAS, domname);
-            if (ret) {
-                DEBUG(SSSDBG_OP_FAILURE, "Failed to add alias [%s] into the "
-                                          "attribute list\n", aliases[i]);
-                goto done;
-            }
-        }
-
-    }
-
-    ret = EOK;
-done:
-    talloc_free(tmp_ctx);
-    return ret;
-}
