@@ -96,7 +96,8 @@ struct tevent_req *sdap_get_groups_send(TALLOC_CTX *memctx,
                                        const char **attrs,
                                        const char *filter,
                                        int timeout,
-                                       bool enumeration);
+                                       bool enumeration,
+                                       bool no_members);
 int sdap_get_groups_recv(struct tevent_req *req,
                          TALLOC_CTX *mem_ctx, char **timestamp);
 
@@ -134,6 +135,8 @@ struct tevent_req *sdap_get_initgr_send(TALLOC_CTX *memctx,
                                         struct sdap_id_ctx *id_ctx,
                                         struct sdap_id_conn_ctx *conn,
                                         const char *name,
+                                        int name_type,
+                                        const char *extra_value,
                                         const char **grp_attrs);
 int sdap_get_initgr_recv(struct tevent_req *req);
 
@@ -194,6 +197,23 @@ int sdap_get_generic_recv(struct tevent_req *req,
 
 bool sdap_has_deref_support(struct sdap_handle *sh, struct sdap_options *opts);
 
+struct tevent_req *
+sdap_deref_search_with_filter_send(TALLOC_CTX *memctx,
+                                   struct tevent_context *ev,
+                                   struct sdap_options *opts,
+                                   struct sdap_handle *sh,
+                                   const char *search_base,
+                                   const char *filter,
+                                   const char *deref_attr,
+                                   const char **attrs,
+                                   int num_maps,
+                                   struct sdap_attr_map_info *maps,
+                                   int timeout);
+int sdap_deref_search_with_filter_recv(struct tevent_req *req,
+                                       TALLOC_CTX *mem_ctx,
+                                       size_t *reply_count,
+                                       struct sdap_deref_attrs ***reply);
+
 struct tevent_req *
 sdap_deref_search_send(TALLOC_CTX *memctx,
                        struct tevent_context *ev,
@@ -219,6 +239,20 @@ sdap_posix_check_send(TALLOC_CTX *memctx, struct tevent_context *ev,
 int sdap_posix_check_recv(struct tevent_req *req,
                           bool *_has_posix);
 
+struct tevent_req *
+sdap_sd_search_send(TALLOC_CTX *memctx,
+		    struct tevent_context *ev,
+		    struct sdap_options *opts,
+		    struct sdap_handle *sh,
+		    const char *base_dn,
+		    int sd_flags,
+		    const char **attrs,
+		    int timeout);
+int sdap_sd_search_recv(struct tevent_req *req,
+			TALLOC_CTX *mem_ctx,
+			size_t *reply_count,
+			struct sysdb_attrs ***reply);
+
 errno_t
 sdap_attrs_add_ldap_attr(struct sysdb_attrs *ldap_attrs,
                          const char *attr_name,
