@@ -481,8 +481,7 @@ static errno_t sdap_get_ad_tokengroups_recv(TALLOC_CTX *mem_ctx,
 }
 
 static errno_t
-sdap_ad_tokengroups_update_members(TALLOC_CTX *mem_ctx,
-                                   const char *username,
+sdap_ad_tokengroups_update_members(const char *username,
                                    struct sysdb_ctx *sysdb,
                                    struct sss_domain_info *domain,
                                    char **ldap_groups)
@@ -518,8 +517,7 @@ sdap_ad_tokengroups_update_members(TALLOC_CTX *mem_ctx,
 
     DEBUG(SSSDBG_TRACE_LIBS, "Updating memberships for [%s]\n", username);
 
-    ret = sysdb_update_members_dn(domain->sysdb, domain, username,
-                                  SYSDB_MEMBER_USER,
+    ret = sysdb_update_members_dn(domain, username, SYSDB_MEMBER_USER,
                                   (const char *const *) add_groups,
                                   (const char *const *) del_groups);
     if (ret != EOK) {
@@ -632,7 +630,7 @@ static errno_t sdap_ad_resolve_sids_step(struct tevent_req *req)
 
     subreq = groups_get_send(state, state->ev, state->id_ctx, sdap_domain,
                              state->conn, state->current_sid,
-                             BE_FILTER_SECID, BE_ATTR_CORE, false);
+                             BE_FILTER_SECID, BE_ATTR_CORE, false, true);
     if (subreq == NULL) {
         return ENOMEM;
     }
@@ -899,7 +897,6 @@ static void sdap_ad_tokengroups_initgr_mapping_done(struct tevent_req *subreq)
         ret = sdap_idmap_sid_to_unix(state->idmap_ctx, sid, &gid);
         if (ret == ENOTSUP) {
             DEBUG(SSSDBG_TRACE_FUNC, "Skipping built-in object.\n");
-            ret = EOK;
             continue;
         } else if (ret != EOK) {
             DEBUG(SSSDBG_MINOR_FAILURE, "Could not convert SID to GID: [%s]. "
@@ -917,8 +914,7 @@ static void sdap_ad_tokengroups_initgr_mapping_done(struct tevent_req *subreq)
                                   sid, gid);
 
         /* Check whether this GID already exists in the sysdb */
-        ret = sysdb_search_group_by_gid(tmp_ctx, domain->sysdb, domain,
-                                        gid, attrs, &msg);
+        ret = sysdb_search_group_by_gid(tmp_ctx, domain, gid, attrs, &msg);
         if (ret == EOK) {
             name = ldb_msg_find_attr_as_string(msg, SYSDB_NAME, NULL);
             if (name == NULL) {
@@ -932,8 +928,8 @@ static void sdap_ad_tokengroups_initgr_mapping_done(struct tevent_req *subreq)
              * of its SID. When a direct lookup of the group or its GID occurs,
              * it will replace this temporary entry. */
             name = sid;
-            ret = sysdb_add_incomplete_group(domain->sysdb, domain, name, gid,
-                                             NULL, sid, false, now);
+            ret = sysdb_add_incomplete_group(domain, name, gid,
+                                             NULL, sid, NULL, false, now);
             if (ret != EOK) {
                 DEBUG(SSSDBG_MINOR_FAILURE, "Could not create incomplete "
                                              "group: [%s]\n", strerror(ret));
@@ -956,7 +952,7 @@ static void sdap_ad_tokengroups_initgr_mapping_done(struct tevent_req *subreq)
 
     groups[num_groups] = NULL;
 
-    ret = sdap_ad_tokengroups_update_members(state, state->username,
+    ret = sdap_ad_tokengroups_update_members(state->username,
                                              state->sysdb, state->domain,
                                              groups);
     if (ret != EOK) {
@@ -1203,8 +1199,7 @@ sdap_ad_tokengroups_get_posix_members(TALLOC_CTX *mem_ctx,
             continue;
         }
 
-        ret = sysdb_search_group_by_sid_str(tmp_ctx, domain->sysdb, domain,
-                                            sid, attrs, &msg);
+        ret = sysdb_search_group_by_sid_str(tmp_ctx, domain, sid, attrs, &msg);
         if (ret == EOK) {
             /* we will update membership of this group */
             name = ldb_msg_find_attr_as_string(msg, SYSDB_NAME, NULL);
@@ -1368,8 +1363,7 @@ sdap_ad_tokengroups_initgr_posix_sids_done(struct tevent_req *subreq)
     }
 
     /* update membership of existing groups */
-    ret = sdap_ad_tokengroups_update_members(state,
-                                             state->username,
+    ret = sdap_ad_tokengroups_update_members(state->username,
                                              state->sysdb, state->domain,
                                              state->cached_groups);
     if (ret != EOK) {
