@@ -143,7 +143,6 @@ sdap_idmap_init(TALLOC_CTX *mem_ctx,
     id_t rangesize;
     bool autorid_mode;
     struct sdap_idmap_ctx *idmap_ctx = NULL;
-    struct sysdb_ctx *sysdb = id_ctx->be->domain->sysdb;
 
     tmp_ctx = talloc_new(NULL);
     if (!tmp_ctx) return ENOMEM;
@@ -223,7 +222,7 @@ sdap_idmap_init(TALLOC_CTX *mem_ctx,
     }
 
     /* Read in any existing mappings from the cache */
-    ret = sysdb_idmap_get_mappings(tmp_ctx, sysdb, id_ctx->be->domain, &res);
+    ret = sysdb_idmap_get_mappings(tmp_ctx, id_ctx->be->domain, &res);
     if (ret != EOK && ret != ENOENT) {
         DEBUG(SSSDBG_FATAL_FAILURE,
               "Could not read ID mappings from the cache: [%s]\n",
@@ -231,7 +230,7 @@ sdap_idmap_init(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    if (ret == EOK && res->count > 0) {
+    if (ret == EOK) {
         DEBUG(SSSDBG_CONF_SETTINGS,
               "Initializing [%d] domains for ID-mapping\n", res->count);
 
@@ -289,6 +288,13 @@ sdap_idmap_init(TALLOC_CTX *mem_ctx,
 
         sid_str = dp_opt_get_string(idmap_ctx->id_ctx->opts->basic, SDAP_IDMAP_DEFAULT_DOMAIN_SID);
         if (sid_str) {
+            struct sss_domain_info *domain = idmap_ctx->id_ctx->be->domain;
+            domain->domain_id = talloc_strdup(domain, sid_str);
+            if (domain->domain_id == NULL) {
+                ret = ENOMEM;
+                goto done;
+            }
+
             /* Set the default domain as slice 0 */
             ret = sdap_idmap_add_domain(idmap_ctx, dom_name,
                                         sid_str, 0);
@@ -383,8 +389,7 @@ sdap_idmap_add_domain(struct sdap_idmap_ctx *idmap_ctx,
     /* If algorithmic mapping is used add this domain to the SYSDB cache so it
      * will survive reboot */
     if (!external_mapping) {
-        ret = sysdb_idmap_store_mapping(idmap_ctx->id_ctx->be->domain->sysdb,
-                                        idmap_ctx->id_ctx->be->domain,
+        ret = sysdb_idmap_store_mapping(idmap_ctx->id_ctx->be->domain,
                                         dom_name, dom_sid,
                                         slice);
         if (ret != EOK) {
