@@ -43,13 +43,24 @@ static char **PyList_AsStringList(TALLOC_CTX *mem_ctx, PyObject *list,
 
     ret = talloc_array(mem_ctx, char *, PyList_Size(list)+1);
     for (i = 0; i < PyList_Size(list); i++) {
+        char *itemstr;
+        Py_ssize_t itemlen;
         PyObject *item = PyList_GetItem(list, i);
+#ifdef IS_PY3K
+        if (!PyUnicode_Check(item)) {
+#else
         if (!PyString_Check(item)) {
+#endif
             PyErr_Format(PyExc_TypeError, "%s should be strings", paramname);
             return NULL;
         }
-        ret[i] = talloc_strndup(ret, PyString_AsString(item),
-                                PyString_Size(item));
+#ifdef IS_PY3K
+        itemstr = PyUnicode_AsUTF8AndSize(item, &itemlen);
+#else
+        itemstr = PyString_AsString(item);
+        itemlen = strlen(itemstr);
+#endif
+        ret[i] = talloc_strndup(ret, itemstr, itemlen);
     }
 
     ret[i] = NULL;
@@ -75,17 +86,6 @@ typedef struct {
     int unlock;
 } PySssLocalObject;
 
-/*
- * The transaction object
- */
-struct py_sss_transaction {
-    PySssLocalObject *self;
-    struct ops_ctx *ops;
-
-    bool transaction_done;
-    int error;
-};
-
 /*
  * Error reporting
  */
@@ -236,7 +236,7 @@ static PyObject *py_sss_useradd(PySssLocalObject *self,
     in_transaction = true;
 
     /* useradd */
-    tctx->error = useradd(tctx, tctx->sysdb, tctx->octx);
+    tctx->error = useradd(tctx, tctx->octx);
     if (tctx->error) {
         PyErr_SetSssError(tctx->error);
         goto fail;
@@ -255,7 +255,6 @@ static PyObject *py_sss_useradd(PySssLocalObject *self,
          * sysdb did assign it automatically, do a lookup */
         if (tctx->octx->uid == 0 || tctx->octx->gid == 0) {
             ret = sysdb_getpwnam_sync(tctx,
-                                      tctx->sysdb,
                                       tctx->octx->name,
                                       tctx->octx);
             if (ret != EOK) {
@@ -364,7 +363,6 @@ static PyObject *py_sss_userdel(PySssLocalObject *self,
 
     if (tctx->octx->remove_homedir) {
         ret = sysdb_getpwnam_sync(tctx,
-                                  tctx->sysdb,
                                   tctx->octx->name,
                                   tctx->octx);
         if (ret != EOK) {
@@ -502,7 +500,7 @@ static PyObject *py_sss_usermod(PySssLocalObject *self,
     in_transaction = true;
 
     /* usermod */
-    tctx->error = usermod(tctx, tctx->sysdb, tctx->octx);
+    tctx->error = usermod(tctx, tctx->octx);
     if (tctx->error) {
         PyErr_SetSssError(tctx->error);
         goto fail;
@@ -576,7 +574,7 @@ static PyObject *py_sss_groupadd(PySssLocalObject *self,
     in_transaction = true;
 
     /* groupadd */
-    tctx->error = groupadd(tctx->sysdb, tctx->octx);
+    tctx->error = groupadd(tctx->octx);
     if (tctx->error) {
         PyErr_SetSssError(tctx->error);
         goto fail;
@@ -719,7 +717,7 @@ static PyObject *py_sss_groupmod(PySssLocalObject *self,
     in_transaction = true;
 
     /* groupmod */
-    tctx->error = groupmod(tctx, tctx->sysdb, tctx->octx);
+    tctx->error = groupmod(tctx, tctx->octx);
     if (tctx->error) {
         PyErr_SetSssError(tctx->error);
         goto fail;
@@ -784,7 +782,11 @@ static PyObject *py_sss_getgrouplist(PyObject *self, PyObject *args)
     do {
         ret = getgrouplist(username, pw->pw_gid, groups, &ngroups);
         if (ret < ngroups) {
-            groups = realloc(groups, ngroups * sizeof(gid_t));
+            gid_t *tmp_groups = realloc(groups, ngroups * sizeof(gid_t));
+            if (tmp_groups == NULL) {
+                goto fail;
+            }
+            groups = tmp_groups;
         }
     } while (ret != ngroups);
 
@@ -800,7 +802,13 @@ static PyObject *py_sss_getgrouplist(PyObject *self, PyObject *args)
     for (i = 0; i < ngroups; i++) {
         gr = getgrgid(groups[i]);
         if (gr) {
-            PyTuple_SetItem(groups_tuple, idx, PyString_FromString(gr->gr_name));
+            PyTuple_SetItem(groups_tuple, idx,
+#ifdef IS_PY3K
+                    PyUnicode_FromString(gr->gr_name)
+#else
+                    PyString_FromString(gr->gr_name)
+#endif
+                    );
             idx++;
         }
     }
@@ -826,7 +834,7 @@ fail:
 static void PySssLocalObject_dealloc(PySssLocalObject *self)
 {
     talloc_free(self->mem_ctx);
-    self->ob_type->tp_free((PyObject*) self);
+    Py_TYPE(self)->tp_free((PyObject *)self);
 }
 
 /*
@@ -857,27 +865,24 @@ static PyObject *PySssLocalObject_new(PyTypeObject *type,
 
     confdb_path = talloc_asprintf(self->mem_ctx, "%s/%s", DB_PATH, CONFDB_FILE);
     if (confdb_path == NULL) {
-        talloc_free(mem_ctx);
         PyErr_NoMemory();
-        return NULL;
+        goto fail;
     }
 
     /* Connect to the conf db */
     ret = confdb_init(self->mem_ctx, &self->confdb, confdb_path);
     if (ret != EOK) {
-        talloc_free(mem_ctx);
         PyErr_SetSssErrorWithMessage(ret,
                 "Could not initialize connection to the confdb\n");
-        return NULL;
+        goto fail;
     }
 
     ret = sssd_domain_init(self->mem_ctx, self->confdb, "local",
                            DB_PATH, &self->local);
     if (ret != EOK) {
-        talloc_free(mem_ctx);
         PyErr_SetSssErrorWithMessage(ret,
                 "Could not initialize connection to the sysdb\n");
-        return NULL;
+        goto fail;
     }
     self->sysdb = self->local->sysdb;
 
@@ -885,6 +890,10 @@ static PyObject *PySssLocalObject_new(PyTypeObject *type,
     self->unlock = DO_UNLOCK;
 
     return (PyObject *) self;
+
+fail:
+    Py_DECREF(self);
+    return NULL;
 }
 
 /*
@@ -914,9 +923,9 @@ static PyMethodDef sss_local_methods[] = {
 
 static PyMemberDef sss_local_members[] = {
     { discard_const_p(char, "lock"), T_INT,
-      offsetof(PySssLocalObject, lock), RO, NULL},
+      offsetof(PySssLocalObject, lock), READONLY, NULL},
     { discard_const_p(char, "unlock"), T_INT,
-      offsetof(PySssLocalObject, unlock), RO, NULL},
+      offsetof(PySssLocalObject, unlock), READONLY, NULL},
     {NULL, 0, 0, 0, NULL} /* Sentinel */
 };
 
@@ -924,7 +933,7 @@ static PyMemberDef sss_local_members[] = {
  * sss.local object properties
  */
 static PyTypeObject pysss_local_type = {
-    PyObject_HEAD_INIT(NULL)
+    PyVarObject_HEAD_INIT(NULL, 0)
     .tp_name = sss_py_const_p(char, "sss.local"),
     .tp_basicsize = sizeof(PySssLocalObject),
     .tp_new = PySssLocalObject_new,
@@ -1041,7 +1050,7 @@ fail:
  */
 static void PySssPasswordObject_dealloc(PySssPasswordObject *self)
 {
-    self->ob_type->tp_free((PyObject*) self);
+    Py_TYPE(self)->tp_free((PyObject*) self);
 }
 
 /*
@@ -1084,7 +1093,7 @@ static PyMethodDef sss_password_methods[] = {
  */
 static PyMemberDef sss_password_members[] = {
     { discard_const_p(char, "AES_256"), T_INT,
-      offsetof(PySssPasswordObject, aes_256), RO, NULL},
+      offsetof(PySssPasswordObject, aes_256), READONLY, NULL},
     {NULL, 0, 0, 0, NULL} /* Sentinel */
 };
 
@@ -1092,7 +1101,7 @@ static PyMemberDef sss_password_members[] = {
  * sss.password object properties
  */
 static PyTypeObject pysss_password_type = {
-    PyObject_HEAD_INIT(NULL)
+    PyVarObject_HEAD_INIT(NULL, 0)
     .tp_name = sss_py_const_p(char, "sss.password"),
     .tp_basicsize = sizeof(PySssPasswordObject),
     .tp_new = PySssPasswordObject_new,
@@ -1116,23 +1125,48 @@ static PyMethodDef module_methods[] = {
 /*
  * Module initialization
  */
+#ifdef IS_PY3K
+static struct PyModuleDef pysssdef = {
+    PyModuleDef_HEAD_INIT,
+    "pysss",
+    NULL,
+    -1,
+    module_methods,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+};
+
+PyMODINIT_FUNC
+PyInit_pysss(void)
+#else
 PyMODINIT_FUNC
 initpysss(void)
+#endif
 {
     PyObject *m;
 
     if (PyType_Ready(&pysss_local_type) < 0)
-        return;
+        MODINITERROR;
     if (PyType_Ready(&pysss_password_type) < 0)
-        return;
+        MODINITERROR;
 
+#ifdef IS_PY3K
+    m = PyModule_Create(&pysssdef);
+#else
     m = Py_InitModule(discard_const_p(char, "pysss"), module_methods);
+#endif
     if (m == NULL)
-        return;
+        MODINITERROR;
 
     Py_INCREF(&pysss_local_type);
     PyModule_AddObject(m, discard_const_p(char, "local"), (PyObject *)&pysss_local_type);
     Py_INCREF(&pysss_password_type);
     PyModule_AddObject(m, discard_const_p(char, "password"), (PyObject *)&pysss_password_type);
+
+#ifdef IS_PY3K
+    return m;
+#endif
 }
 
