@@ -188,9 +188,9 @@ set_autofs_map(struct autofs_ctx *actx,
     hret = hash_enter(actx->maps, &key, &value);
     if (hret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
-              "Unable to add hash table entry for [%s]", key.str);
+              "Unable to add hash table entry for [%s]\n", key.str);
         DEBUG(SSSDBG_MINOR_FAILURE,
-              "Hash error [%d][%s]", hret, hash_error_string(hret));
+              "Hash error [%d][%s]\n", hret, hash_error_string(hret));
         return EIO;
     }
     talloc_steal(actx->maps, map);
@@ -326,8 +326,12 @@ static void sss_autofs_cmd_setautomntent_done(struct tevent_req *req)
             }
 
             sss_packet_get_body(packet, &body, &blen);
-            ((uint32_t *)body)[0] = 1; /* Got some results */
-            ((uint32_t *)body)[1] = 0; /* reserved */
+
+            /* Got some results */
+            SAFEALIGN_SETMEM_UINT32(body, 1, NULL);
+
+            /* Reserved padding */
+            SAFEALIGN_SETMEM_UINT32(body + sizeof(uint32_t), 0, NULL);
         }
 
         sss_cmd_done(cmdctx->cctx, NULL);
@@ -645,7 +649,7 @@ lookup_automntmap_step(struct setautomntent_lookup_ctx *lookup_ctx)
 
         /* Look into the cache */
         talloc_free(dctx->map);
-        ret = sysdb_get_map_byname(dctx, sysdb, dom, lookup_ctx->mapname,
+        ret = sysdb_get_map_byname(dctx, dom, lookup_ctx->mapname,
                                    &dctx->map);
         if (ret != EOK && ret != ENOENT) {
             DEBUG(SSSDBG_OP_FAILURE, "Could not check cache\n");
@@ -697,7 +701,7 @@ lookup_automntmap_step(struct setautomntent_lookup_ctx *lookup_ctx)
         /* OK, the map is in cache and valid.
          * Let's get all members and return it
          */
-        ret = sysdb_autofs_entries_by_map(map, sysdb, dom, map->mapname,
+        ret = sysdb_autofs_entries_by_map(map, dom, map->mapname,
                                           &map->entry_count,
                                           &map->entries);
         if (ret != EOK && ret != ENOENT) {
