@@ -22,6 +22,7 @@
 #include "util/util.h"
 #include "confdb/confdb.h"
 #include "responder/common/responder.h"
+#include "responder/common/negcache.h"
 #include <fcntl.h>
 #include <time.h>
 #include "tdb.h"
@@ -115,7 +116,7 @@ static int sss_ncache_check_str(struct sss_nc_ctx *ctx, char *str, int ttl)
         goto done;
     }
 
-    if (timestamp + ttl > time(NULL)) {
+    if (timestamp + ttl >= time(NULL)) {
         /* still valid */
         ret = EEXIST;
         goto done;
@@ -581,7 +582,7 @@ done:
     return 0;
 }
 
-int sss_ncache_reset_permament(struct sss_nc_ctx *ctx)
+int sss_ncache_reset_permanent(struct sss_nc_ctx *ctx)
 {
     int ret;
 
