@@ -684,8 +684,8 @@ sss_dp_internal_get_send(struct resp_ctx *rctx,
     ret = sss_dp_get_domain_conn(rctx, dom->conn_name, &be_conn);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
-              "BUG: The Data Provider connection for %s is not available!",
-               dom->name);
+              "BUG: The Data Provider connection for %s is not available!\n",
+              dom->name);
         ret = EIO;
         goto error;
     }
