@@ -38,6 +38,8 @@
 #include "monitor/monitor_interfaces.h"
 #include "confdb/confdb.h"
 #include "responder/ifp/ifp_private.h"
+#include "responder/ifp/ifp_domains.h"
+#include "responder/ifp/ifp_components.h"
 #include "responder/common/responder_sbus.h"
 
 #define DEFAULT_ALLOWED_UIDS "0"
@@ -69,8 +71,64 @@ static struct data_provider_iface ifp_dp_methods = {
 struct infopipe_iface ifp_iface = {
     { &infopipe_iface_meta, 0 },
     .Ping = ifp_ping,
+
+    /* components */
+    .ListComponents = ifp_list_components,
+    .ListResponders = ifp_list_responders,
+    .ListBackends = ifp_list_backends,
+    .FindMonitor = ifp_find_monitor,
+    .FindResponderByName = ifp_find_responder_by_name,
+    .FindBackendByName = ifp_find_backend_by_name,
+
     .GetUserAttr = ifp_user_get_attr,
     .GetUserGroups = ifp_user_get_groups,
+    .ListDomains = ifp_list_domains,
+    .FindDomainByName = ifp_find_domain_by_name,
+};
+
+struct infopipe_component ifp_component = {
+    { &infopipe_component_meta, 0 },
+    .Enable = ifp_component_enable,
+    .Disable = ifp_component_disable,
+    .ChangeDebugLevel = ifp_component_change_debug_level,
+    .ChangeDebugLevelTemporarily = ifp_component_change_debug_level_tmp,
+    .infopipe_component_get_name = ifp_component_get_name,
+    .infopipe_component_get_debug_level = ifp_component_get_debug_level,
+    .infopipe_component_get_enabled = ifp_component_get_enabled,
+    .infopipe_component_get_type = ifp_component_get_type,
+    /* FIXME: This should be part of Components.Backends interface, onece
+     * SSSD supports multiple interfaces per object path. */
+    .infopipe_component_get_providers = ifp_backend_get_providers
+};
+
+struct infopipe_domain ifp_domain = {
+    { &infopipe_domain_meta, 0 },
+    .infopipe_domain_get_name = ifp_dom_get_name,
+    .infopipe_domain_get_provider = ifp_dom_get_provider,
+    .infopipe_domain_get_primary_servers = ifp_dom_get_primary_servers,
+    .infopipe_domain_get_backup_servers = ifp_dom_get_backup_servers,
+    .infopipe_domain_get_min_id = ifp_dom_get_min_id,
+    .infopipe_domain_get_max_id = ifp_dom_get_max_id,
+    .infopipe_domain_get_realm = ifp_dom_get_realm,
+    .infopipe_domain_get_forest = ifp_dom_get_forest,
+    .infopipe_domain_get_login_format = ifp_dom_get_login_format,
+    .infopipe_domain_get_fully_qualified_name_format = ifp_dom_get_fqdn_format,
+    .infopipe_domain_get_enumerable = ifp_dom_get_enumerable,
+    .infopipe_domain_get_use_fully_qualified_names = ifp_dom_get_use_fqdn,
+    .infopipe_domain_get_subdomain = ifp_dom_get_subdomain,
+    .infopipe_domain_get_parent_domain = ifp_dom_get_parent_domain
+};
+
+struct sysbus_iface {
+    const char *path;
+    struct sbus_vtable *iface_vtable;
+};
+
+static struct sysbus_iface ifp_ifaces[] = {
+    { INFOPIPE_PATH, &ifp_iface.vtable },
+    { INFOPIPE_DOMAIN_PATH, &ifp_domain.vtable },
+    { INFOPIPE_COMPONENT_PATH, &ifp_component.vtable },
+    { NULL, NULL },
 };
 
 struct sss_cmd_table *get_ifp_cmds(void)
@@ -113,8 +171,7 @@ static errno_t
 sysbus_init(TALLOC_CTX *mem_ctx,
             struct tevent_context *ev,
             const char *dbus_name,
-            const char *dbus_path,
-            struct sbus_vtable *iface_vtable,
+            struct sysbus_iface *sysbus_ifaces,
             void *pvt,
             struct sysbus_ctx **sysbus)
 {
@@ -122,6 +179,7 @@ sysbus_init(TALLOC_CTX *mem_ctx,
     DBusConnection *conn = NULL;
     struct sysbus_ctx *system_bus = NULL;
     struct sbus_interface *sif;
+    int i;
     errno_t ret;
 
     system_bus = talloc_zero(mem_ctx, struct sysbus_ctx);
@@ -167,21 +225,23 @@ sysbus_init(TALLOC_CTX *mem_ctx,
         goto fail;
     }
 
-    sif = sbus_new_interface(system_bus->conn,
-                             dbus_path,
-                             iface_vtable,
-                             pvt);
-    if (sif == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "Could not add the sbus interface\n");
-        goto fail;
-    }
+    for (i = 0; sysbus_ifaces[i].path != NULL; i++) {
+        sif = sbus_new_interface(system_bus->conn,
+                                 sysbus_ifaces[i].path,
+                                 sysbus_ifaces[i].iface_vtable,
+                                 pvt);
+        if (sif == NULL) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "Could not add the sbus interface\n");
+            goto fail;
+        }
 
-    ret = sbus_conn_add_interface(system_bus->conn, sif);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "Could not add the interface\n");
-        goto fail;
+        ret = sbus_conn_add_interface(system_bus->conn, sif);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "Could not add the interface\n");
+            goto fail;
+        }
     }
 
     *sysbus = system_bus;
@@ -209,14 +269,14 @@ static int ifp_sysbus_reconnect(struct sbus_request *dbus_req, void *data)
     DEBUG(SSSDBG_TRACE_FUNC, "Attempting to reconnect to the system bus\n");
 
     if (ifp_ctx->sysbus) {
+        DEBUG(SSSDBG_TRACE_LIBS, "Already connected to sysbus\n");
         goto done;
     }
 
     /* Connect to the D-BUS system bus and set up methods */
     ret = sysbus_init(ifp_ctx, ifp_ctx->rctx->ev,
                       INFOPIPE_IFACE,
-                      INFOPIPE_PATH,
-                      &ifp_iface.vtable,
+                      ifp_ifaces,
                       ifp_ctx, &ifp_ctx->sysbus);
     if (ret == ERR_NO_SYSBUS) {
         DEBUG(SSSDBG_MINOR_FAILURE,
@@ -250,7 +310,7 @@ int ifp_process_init(TALLOC_CTX *mem_ctx,
     ifp_cmds = get_ifp_cmds();
     ret = sss_process_init(mem_ctx, ev, cdb,
                            ifp_cmds,
-                           NULL, NULL,
+                           NULL, -1, NULL, -1,
                            CONFDB_IFP_CONF_ENTRY,
                            SSS_IFP_SBUS_SERVICE_NAME,
                            SSS_IFP_SBUS_SERVICE_VERSION,
@@ -316,7 +376,7 @@ int ifp_process_init(TALLOC_CTX *mem_ctx,
                             CONFDB_IFP_CONF_ENTRY, CONFDB_IFP_USER_ATTR_LIST,
                             NULL, &attr_list_str);
     if (ret != EOK) {
-        DEBUG(SSSDBG_FATAL_FAILURE, "Failed to get allowed UIDs.\n");
+        DEBUG(SSSDBG_FATAL_FAILURE, "Failed to get user attribute list.\n");
         goto fail;
     }
 
@@ -347,8 +407,7 @@ int ifp_process_init(TALLOC_CTX *mem_ctx,
     /* Connect to the D-BUS system bus and set up methods */
     ret = sysbus_init(ifp_ctx, ifp_ctx->rctx->ev,
                       INFOPIPE_IFACE,
-                      INFOPIPE_PATH,
-                      &ifp_iface.vtable,
+                      ifp_ifaces,
                       ifp_ctx, &ifp_ctx->sysbus);
     if (ret == ERR_NO_SYSBUS) {
         DEBUG(SSSDBG_MINOR_FAILURE,
@@ -382,16 +441,21 @@ int main(int argc, const char *argv[])
     poptContext pc;
     struct main_context *main_ctx;
     int ret;
+    uid_t uid;
+    gid_t gid;
 
     struct poptOption long_options[] = {
         POPT_AUTOHELP
         SSSD_MAIN_OPTS
+        SSSD_SERVER_OPTS(uid, gid)
         POPT_TABLEEND
     };
 
     /* Set debug level to invalid value so we can deside if -d 0 was used. */
     debug_level = SSSDBG_INVALID;
 
+    umask(DFL_RSP_UMASK);
+
     pc = poptGetContext(argv[0], argc, argv, long_options, 0);
     while((opt = poptGetNextOpt(pc)) != -1) {
         switch(opt) {
@@ -410,7 +474,8 @@ int main(int argc, const char *argv[])
     /* set up things like debug, signals, daemonization, etc... */
     debug_log_file = "sssd_ifp";
 
-    ret = server_setup("sssd[ifp]", 0, CONFDB_IFP_CONF_ENTRY, &main_ctx);
+    ret = server_setup("sssd[ifp]", 0, 0, 0,
+                       CONFDB_IFP_CONF_ENTRY, &main_ctx);
     if (ret != EOK) return 2;
 
     ret = die_if_parent_died();
