@@ -106,6 +106,178 @@ const char *ifp_path_strip_prefix(const char *path, const char *prefix)
     return NULL;
 }
 
+/* The following path related functions are based on similar code in
+ * storaged, just tailored to use talloc instead of glib
+ */
+char *ifp_bus_path_escape(TALLOC_CTX *mem_ctx, const char *path)
+{
+    size_t n;
+    char *safe_path = NULL;
+    TALLOC_CTX *tmp_ctx = NULL;
+
+    /* The path must be valid */
+    if (path == NULL) {
+        return NULL;
+    }
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        return NULL;
+    }
+
+    safe_path = talloc_strdup(tmp_ctx, "");
+    if (safe_path == NULL) {
+        goto done;
+    }
+
+    /* Special case for an empty string */
+    if (strcmp(path, "") == 0) {
+        /* the for loop would just fall through */
+        safe_path = talloc_asprintf_append_buffer(safe_path, "_");
+    }
+
+    for (n = 0; path[n]; n++) {
+        int c = path[n];
+        /* D-Bus spec says:
+         * *
+         * * Each element must only contain the ASCII characters
+         * "[A-Z][a-z][0-9]_"
+         * */
+        if ((c >= 'A' && c <= 'Z')
+                || (c >= 'a' && c <= 'z')
+                || (c >= '0' && c <= '9')) {
+            safe_path = talloc_asprintf_append_buffer(safe_path, "%c", c);
+            if (safe_path == NULL) {
+                goto done;
+            }
+        } else {
+            safe_path = talloc_asprintf_append_buffer(safe_path, "_%02x", c);
+            if (safe_path == NULL) {
+                goto done;
+            }
+        }
+    }
+
+    safe_path = talloc_steal(mem_ctx, safe_path);
+done:
+    talloc_free(tmp_ctx);
+    return safe_path;
+}
+
+static inline int unhexchar(char c)
+{
+    if (c >= '0' && c <= '9') {
+        return c - '0';
+    }
+
+    if (c >= 'a' && c <= 'f') {
+        return c - 'a' + 10;
+    }
+
+    if (c >= 'A' && c <= 'F') {
+        return c - 'A' + 10;
+    }
+
+    return -1;
+}
+
+char *ifp_bus_path_unescape(TALLOC_CTX *mem_ctx, const char *path)
+{
+    char *safe_path;
+    const char *p;
+    int a, b, c;
+    TALLOC_CTX *tmp_ctx = NULL;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        return NULL;
+    }
+
+    safe_path = talloc_strdup(tmp_ctx, "");
+    if (safe_path == NULL) {
+        goto done;
+    }
+
+    /* Special case for the empty string */
+    if (strcmp(path, "_") == 0) {
+        safe_path = talloc_steal(mem_ctx, safe_path);
+        goto done;
+    }
+
+    for (p = path; *p; p++) {
+        if (*p == '_') {
+            /* There must be at least two more chars after underscore */
+            if (p[1] == '\0' || p[2] == '\0') {
+                safe_path = NULL;
+                goto done;
+            }
+
+            if ((a = unhexchar(p[1])) < 0
+                    || (b = unhexchar(p[2])) < 0) {
+                /* Invalid escape code, let's take it literal then */
+                c = '_';
+            } else {
+                c = ((a << 4) | b);
+                p += 2;
+            }
+        } else  {
+            c = *p;
+        }
+
+        safe_path = talloc_asprintf_append_buffer(safe_path, "%c", c);
+        if (safe_path == NULL) {
+            goto done;
+        }
+    }
+
+    safe_path = talloc_steal(mem_ctx, safe_path);
+done:
+    talloc_free(tmp_ctx);
+    return safe_path;
+}
+
+char *
+_ifp_reply_objpath(TALLOC_CTX *mem_ctx, const char *base,
+                   const char *part, ...)
+{
+    char *safe_part;
+    char *path = NULL;
+    va_list va;
+
+    if (base == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "Wrong object path base!\n");
+        return NULL;
+    }
+
+    path = talloc_strdup(mem_ctx, base);
+    if (path == NULL) return NULL;
+
+    va_start(va, part);
+    while (part != NULL) {
+        safe_part = ifp_bus_path_escape(mem_ctx, part);
+        if (safe_part == NULL) {
+            DEBUG(SSSDBG_OP_FAILURE, "Could not add [%s] to objpath\n", part);
+            goto fail;
+        }
+
+        path = talloc_asprintf_append(path, "/%s", safe_part);
+        talloc_free(safe_part);
+        if (path == NULL) {
+            goto fail;
+        }
+
+        part = va_arg(va, const char *);
+    }
+    va_end(va);
+
+    return path;
+
+fail:
+    va_end(va);
+    talloc_free(path);
+    return NULL;
+}
+
 errno_t ifp_add_ldb_el_to_dict(DBusMessageIter *iter_dict,
                                struct ldb_message_element *el)
 {
@@ -184,127 +356,12 @@ errno_t ifp_add_ldb_el_to_dict(DBusMessageIter *iter_dict,
     return EOK;
 }
 
-static inline bool
-attr_in_list(const char **list, size_t nlist, const char *str)
-{
-    size_t i;
-
-    for (i = 0; i < nlist; i++) {
-        if (strcasecmp(list[i], str) == 0) {
-            break;
-        }
-    }
-
-    return (i < nlist) ? true : false;
-}
-
 const char **
 ifp_parse_attr_list(TALLOC_CTX *mem_ctx, const char *conf_str)
 {
-    TALLOC_CTX *tmp_ctx;
-    errno_t ret;
-    const char **list = NULL;
-    const char **res = NULL;
-    int list_size;
-    char **conf_list = NULL;
-    int conf_list_size = 0;
-    const char **allow = NULL;
-    const char **deny = NULL;
-    int ai = 0, di = 0, li = 0;
-    int i;
     const char *defaults[] = IFP_DEFAULT_ATTRS;
 
-    tmp_ctx = talloc_new(NULL);
-    if (tmp_ctx == NULL) {
-        return NULL;
-    }
-
-    if (conf_str) {
-        ret = split_on_separator(tmp_ctx, conf_str, ',', true, true,
-                                 &conf_list, &conf_list_size);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_OP_FAILURE,
-                  "Cannot parse attribute ACL list  %s: %d\n", conf_str, ret);
-            goto done;
-        }
-
-        allow = talloc_zero_array(tmp_ctx, const char *, conf_list_size);
-        deny = talloc_zero_array(tmp_ctx, const char *, conf_list_size);
-        if (allow == NULL || deny == NULL) {
-            goto done;
-        }
-    }
-
-    for (i = 0; i < conf_list_size; i++) {
-        switch (conf_list[i][0]) {
-            case '+':
-                allow[ai] = conf_list[i] + 1;
-                ai++;
-                continue;
-            case '-':
-                deny[di] = conf_list[i] + 1;
-                di++;
-                continue;
-            default:
-                DEBUG(SSSDBG_CRIT_FAILURE, "ACL values must start with "
-                      "either '+' (allow) or '-' (deny), got '%s'\n",
-                      conf_list[i]);
-                goto done;
-        }
-    }
-
-    /* Assume the output will have to hold defauls and all the configured,
-     * values, resize later
-     */
-    list_size = 0;
-    while (defaults[list_size]) {
-        list_size++;
-    }
-    list_size += conf_list_size;
-
-    list = talloc_zero_array(tmp_ctx, const char *, list_size + 1);
-    if (list == NULL) {
-        goto done;
-    }
-
-    /* Start by copying explicitly allowed attributes */
-    for (i = 0; i < ai; i++) {
-        /* if the attribute is explicitly denied, skip it */
-        if (attr_in_list(deny, di, allow[i])) {
-            continue;
-        }
-
-        list[li] = talloc_strdup(list, allow[i]);
-        if (list[li] == NULL) {
-            goto done;
-        }
-        li++;
-
-        DEBUG(SSSDBG_TRACE_INTERNAL,
-              "Added allowed attr %s to whitelist\n", allow[i]);
-    }
-
-    /* Add defaults */
-    for (i = 0; defaults[i]; i++) {
-        /* if the attribute is explicitly denied, skip it */
-        if (attr_in_list(deny, di, defaults[i])) {
-            continue;
-        }
-
-        list[li] = talloc_strdup(list, defaults[i]);
-        if (list[li] == NULL) {
-            goto done;
-        }
-        li++;
-
-        DEBUG(SSSDBG_TRACE_INTERNAL,
-              "Added default attr %s to whitelist\n", defaults[i]);
-    }
-
-    res = talloc_steal(mem_ctx, list);
-done:
-    talloc_free(tmp_ctx);
-    return res;
+    return parse_attr_list_ex(mem_ctx, conf_str, defaults);
 }
 
 bool
