@@ -93,6 +93,34 @@ struct sss_mc_ctx {
     else used = false; \
 } while (0)
 
+static inline
+uint32_t sss_mc_next_slot_with_hash(struct sss_mc_rec *rec,
+                                    uint32_t hash)
+{
+    if (rec->hash1 == hash) {
+        return rec->next1;
+    } else if (rec->hash2 == hash) {
+        return rec->next2;
+    } else {
+        /* it should never happen. */
+        return MC_INVALID_VAL;
+    }
+}
+
+static inline
+void sss_mc_chain_slot_to_record_with_hash(struct sss_mc_rec *rec,
+                                           uint32_t hash,
+                                           uint32_t slot)
+{
+    /* changing a single uint32_t is atomic, so there is no
+     * need to use barriers in this case */
+    if (rec->hash1 == hash) {
+        rec->next1 = slot;
+    } else if (rec->hash2 == hash) {
+        rec->next2 = slot;
+    }
+}
+
 /* This function will store corrupted memcache to disk for later
  * analysis. */
 static void  sss_mc_save_corrupted(struct sss_mc_ctx *mc_ctx)
@@ -191,74 +219,33 @@ static void sss_mc_add_rec_to_chain(struct sss_mc_ctx *mcc,
     }
 
     do {
-        if (!MC_SLOT_WITHIN_BOUNDS(slot, mcc->dt_size)) {
-            DEBUG(SSSDBG_FATAL_FAILURE,
-                  "Corrupted fastcache. Slot number too big.\n");
-            sss_mmap_cache_reset(mcc);
-            return;
-        }
-
         cur = MC_SLOT_TO_PTR(mcc->data_table, slot, struct sss_mc_rec);
         if (cur == rec) {
             /* rec already stored in hash chain */
             return;
         }
-        slot = cur->next;
+        slot = sss_mc_next_slot_with_hash(cur, hash);
     } while (slot != MC_INVALID_VAL);
     /* end of chain, append our record here */
 
-    /* changing a single uint32_t is atomic, so there is no
-     * need to use barriers in this case */
-    cur->next = MC_PTR_TO_SLOT(mcc->data_table, rec);
+    slot = MC_PTR_TO_SLOT(mcc->data_table, rec);
+    sss_mc_chain_slot_to_record_with_hash(cur, hash, slot);
 }
 
-static inline errno_t
-sss_mc_get_next_slot_with_hash(struct sss_mc_ctx *mcc,
-                               struct sss_mc_rec *start_rec,
-                               uint32_t hash,
-                               uint32_t *_slot)
-{
-    struct sss_mc_rec *rec;
-    uint32_t slot;
-
-    slot = start_rec->next;
-    while (slot != MC_INVALID_VAL) {
-        if (!MC_SLOT_WITHIN_BOUNDS(slot, mcc->dt_size)) {
-            DEBUG(SSSDBG_FATAL_FAILURE,
-                  "Corrupted fastcache. Slot number too big.\n");
-            sss_mmap_cache_reset(mcc);
-            return EINVAL;
-        }
-
-        rec = MC_SLOT_TO_PTR(mcc->data_table, slot, struct sss_mc_rec);
-        if (rec->hash1 == hash || rec->hash2 == hash) {
-            break;
-        }
-
-        slot = rec->next;
-    }
-
-    *_slot = slot;
-
-    return EOK;
-}
-
-static errno_t sss_mc_rm_rec_from_chain(struct sss_mc_ctx *mcc,
-                                        struct sss_mc_rec *rec,
-                                        uint32_t hash)
+static void sss_mc_rm_rec_from_chain(struct sss_mc_ctx *mcc,
+                                     struct sss_mc_rec *rec,
+                                     uint32_t hash)
 {
     struct sss_mc_rec *prev = NULL;
     struct sss_mc_rec *cur = NULL;
     uint32_t slot;
 
     if (hash > MC_HT_ELEMS(mcc->ht_size)) {
-        if (hash == MC_INVALID_VAL) {
-            /* This can happen if rec->hash1 and rec->hash2 was the same. */
-            return EOK;
-        }
-
-        /* The hash is invalid. */
-        return EINVAL;
+        /* It can happen if rec->hash1 and rec->hash2 was the same.
+         * or it is invalid hash. It is better to return
+         * than trying to access out of bounds memory
+         */
+        return;
     }
 
     slot = mcc->hash_table[hash];
@@ -266,55 +253,26 @@ static errno_t sss_mc_rm_rec_from_chain(struct sss_mc_ctx *mcc,
         /* record has already been removed. It may happen if rec->hash1 and
          * rec->has2 are the same. (It is not very likely).
          */
-        return EOK;
+        return;
     }
-
-    if (!MC_SLOT_WITHIN_BOUNDS(slot, mcc->dt_size)) {
-        DEBUG(SSSDBG_FATAL_FAILURE,
-              "Corrupted fastcache. Slot number too big.\n");
-        sss_mmap_cache_reset(mcc);
-        return EINVAL;
-    }
-
     cur = MC_SLOT_TO_PTR(mcc->data_table, slot, struct sss_mc_rec);
     if (cur == rec) {
-        /* rec->next can refer to record without matching hashes.
-         * We need to skip this(those) records, because
-         * mcc->hash_table[hash] have to refer to valid start of the chain.
-         */
-        return sss_mc_get_next_slot_with_hash(mcc, rec, hash,
-                                              &mcc->hash_table[hash]);
+        mcc->hash_table[hash] = sss_mc_next_slot_with_hash(rec, hash);
     } else {
-        slot = cur->next;
+        slot = sss_mc_next_slot_with_hash(cur, hash);
         while (slot != MC_INVALID_VAL) {
-            if (!MC_SLOT_WITHIN_BOUNDS(slot, mcc->dt_size)) {
-                DEBUG(SSSDBG_FATAL_FAILURE,
-                      "Corrupted fastcache. Slot number too big.\n");
-                sss_mmap_cache_reset(mcc);
-                return EINVAL;
-            }
-
             prev = cur;
             cur = MC_SLOT_TO_PTR(mcc->data_table, slot, struct sss_mc_rec);
             if (cur == rec) {
-                /* changing a single uint32_t is atomic, so there is no
-                 * need to use barriers in this case.
-                 *
-                 * This situation is different to the removing record from
-                 * the beggining of the chain. The record have to only be
-                 * removed from chain, because this chain can be
-                 * subset or supperset of another chain and we don't want
-                 * to break another chains.
-                 */
-                prev->next = cur->next;
+                slot = sss_mc_next_slot_with_hash(cur, hash);
+
+                sss_mc_chain_slot_to_record_with_hash(prev, hash, slot);
                 slot = MC_INVALID_VAL;
             } else {
-                slot = cur->next;
+                slot = sss_mc_next_slot_with_hash(cur, hash);
             }
         }
     }
-
-    return EOK;
 }
 
 static void sss_mc_free_slots(struct sss_mc_ctx *mcc, struct sss_mc_rec *rec)
@@ -333,8 +291,6 @@ static void sss_mc_free_slots(struct sss_mc_ctx *mcc, struct sss_mc_rec *rec)
 static void sss_mc_invalidate_rec(struct sss_mc_ctx *mcc,
                                   struct sss_mc_rec *rec)
 {
-    errno_t ret;
-
     if (rec->b1 == MC_INVALID_VAL) {
         /* record already invalid */
         return;
@@ -342,16 +298,9 @@ static void sss_mc_invalidate_rec(struct sss_mc_ctx *mcc,
 
     /* Remove from hash chains */
     /* hash chain 1 */
-    ret = sss_mc_rm_rec_from_chain(mcc, rec, rec->hash1);
-    if (ret != EOK) {
-        return;
-    }
-
+    sss_mc_rm_rec_from_chain(mcc, rec, rec->hash1);
     /* hash chain 2 */
-    ret = sss_mc_rm_rec_from_chain(mcc, rec, rec->hash2);
-    if (ret != EOK) {
-        return;
-    }
+    sss_mc_rm_rec_from_chain(mcc, rec, rec->hash2);
 
     /* Clear from free_table */
     sss_mc_free_slots(mcc, rec);
@@ -362,7 +311,8 @@ static void sss_mc_invalidate_rec(struct sss_mc_ctx *mcc,
                                         - sizeof(struct sss_mc_rec)));
     rec->len = MC_INVALID_VAL32;
     rec->expire = MC_INVALID_VAL64;
-    rec->next = MC_INVALID_VAL32;
+    rec->next1 = MC_INVALID_VAL32;
+    rec->next2 = MC_INVALID_VAL32;
     rec->hash1 = MC_INVALID_VAL32;
     rec->hash2 = MC_INVALID_VAL32;
     MC_LOWER_BARRIER(rec);
@@ -391,7 +341,7 @@ static bool sss_mc_is_valid_rec(struct sss_mc_ctx *mcc, struct sss_mc_rec *rec)
         return false;
     }
 
-    /* rec->next can be invalid if there are no next records */
+    /* next record can be invalid if there are no next records */
 
     if (rec->hash1 == MC_INVALID_VAL32) {
         return false;
@@ -399,15 +349,8 @@ static bool sss_mc_is_valid_rec(struct sss_mc_ctx *mcc, struct sss_mc_rec *rec)
         self = NULL;
         slot = mcc->hash_table[rec->hash1];
         while (slot != MC_INVALID_VAL32 && self != rec) {
-            if (!MC_SLOT_WITHIN_BOUNDS(slot, mcc->dt_size)) {
-                DEBUG(SSSDBG_FATAL_FAILURE,
-                      "Corrupted fastcache. Slot number too big.\n");
-                sss_mmap_cache_reset(mcc);
-                return false;
-            }
-
             self = MC_SLOT_TO_PTR(mcc->data_table, slot, struct sss_mc_rec);
-            slot = self->next;
+            slot = sss_mc_next_slot_with_hash(self, rec->hash1);
         }
         if (self != rec) {
             return false;
@@ -417,15 +360,8 @@ static bool sss_mc_is_valid_rec(struct sss_mc_ctx *mcc, struct sss_mc_rec *rec)
         self = NULL;
         slot = mcc->hash_table[rec->hash2];
         while (slot != MC_INVALID_VAL32 && self != rec) {
-            if (!MC_SLOT_WITHIN_BOUNDS(slot, mcc->dt_size)) {
-                DEBUG(SSSDBG_FATAL_FAILURE,
-                      "Corrupted fastcache. Slot number too big.\n");
-                sss_mmap_cache_reset(mcc);
-                return false;
-            }
-
             self = MC_SLOT_TO_PTR(mcc->data_table, slot, struct sss_mc_rec);
-            slot = self->next;
+            slot = sss_mc_next_slot_with_hash(self, rec->hash2);
         }
         if (self != rec) {
             return false;
@@ -603,7 +539,7 @@ static struct sss_mc_rec *sss_mc_find_record(struct sss_mc_ctx *mcc,
             return NULL;
         }
 
-        name_ptr = *((rel_ptr_t *)rec->data);
+        safealign_memcpy(&name_ptr, rec->data, sizeof(rel_ptr_t), NULL);
         if (key->len > strs_len
             || (name_ptr + key->len) > (strs_offset + strs_len)
             || (uint8_t *)rec->data + strs_offset + strs_len > max_addr) {
@@ -619,7 +555,7 @@ static struct sss_mc_rec *sss_mc_find_record(struct sss_mc_ctx *mcc,
             break;
         }
 
-        slot = rec->next;
+        slot = sss_mc_next_slot_with_hash(rec, hash);
     }
 
     if (slot == MC_INVALID_VAL) {
@@ -675,7 +611,9 @@ static errno_t sss_mc_get_record(struct sss_mc_ctx **_mcc,
     /* mark as not valid yet */
     MC_RAISE_INVALID_BARRIER(rec);
     rec->len = rec_len;
-    rec->next = MC_INVALID_VAL;
+    rec->next1 = MC_INVALID_VAL;
+    rec->next2 = MC_INVALID_VAL;
+    rec->padding = MC_INVALID_VAL;
     MC_LOWER_BARRIER(rec);
 
     /* and now mark slots as used */
@@ -861,7 +799,7 @@ errno_t sss_mmap_cache_pw_invalidate_uid(struct sss_mc_ctx *mcc, uid_t uid)
             break;
         }
 
-        slot = rec->next;
+        slot = sss_mc_next_slot_with_hash(rec, hash);
     }
 
     if (slot == MC_INVALID_VAL) {
@@ -1000,7 +938,7 @@ errno_t sss_mmap_cache_gr_invalidate_gid(struct sss_mc_ctx *mcc, gid_t gid)
             break;
         }
 
-        slot = rec->next;
+        slot = sss_mc_next_slot_with_hash(rec, hash);
     }
 
     if (slot == MC_INVALID_VAL) {
@@ -1031,7 +969,7 @@ static errno_t sss_mc_set_recycled(int fd)
     struct sss_mc_header h;
     off_t offset;
     off_t pos;
-    int ret;
+    ssize_t written;
 
     offset = MC_PTR_DIFF(&h.status, &h);
 
@@ -1042,12 +980,12 @@ static errno_t sss_mc_set_recycled(int fd)
     }
 
     errno = 0;
-    ret = sss_atomic_write_s(fd, (uint8_t *)&w, sizeof(h.status));
-    if (ret == -1) {
+    written = sss_atomic_write_s(fd, (uint8_t *)&w, sizeof(h.status));
+    if (written == -1) {
         return errno;
     }
 
-    if (ret != sizeof(h.status)) {
+    if (written != sizeof(h.status)) {
         /* Write error */
         return EIO;
     }
@@ -1303,6 +1241,7 @@ done:
         if (mc_ctx && mc_ctx->file && mc_ctx->fd != -1) {
             dret = unlink(mc_ctx->file);
             if (dret == -1) {
+                dret = errno;
                 DEBUG(SSSDBG_CRIT_FAILURE,
                       "Failed to rm mmap file %s: %d(%s)\n", mc_ctx->file,
                        dret, strerror(dret));
