@@ -25,12 +25,13 @@
 #ifndef NSSSRV_PRIVATE_H_
 #define NSSSRV_PRIVATE_H_
 
-#include "dhash.h"
+#include <dhash.h>
 
 struct nss_cmd_ctx {
     struct cli_ctx *cctx;
     enum sss_cli_command cmd;
     char *name;
+    bool name_is_upn;
     uint32_t id;
     char *secid;
 
@@ -119,9 +120,10 @@ void nss_setent_notify_done(struct getent_ctx *getent_ctx);
 errno_t check_cache(struct nss_dom_ctx *dctx,
                     struct nss_ctx *nctx,
                     struct ldb_result *res,
-                    int req_type,
+                    enum sss_dp_acct_type req_type,
                     const char *opt_name,
                     uint32_t opt_id,
+                    const char *extra,
                     sss_dp_callback_t callback,
                     void *pvt);
 
