@@ -131,7 +131,7 @@ static errno_t pac_add_pac_user(struct cli_ctx *cctx)
 
     pr_ctx->domain_name = pr_ctx->logon_info->info3.base.logon_domain.string;
     if (pr_ctx->domain_name == NULL) {
-        DEBUG(SSSDBG_FATAL_FAILURE, "No domain name in PAC");
+        DEBUG(SSSDBG_FATAL_FAILURE, "No domain name in PAC\n");
         ret = EINVAL;
         goto done;
     }
@@ -276,6 +276,14 @@ static void pac_lookup_sids_done(struct tevent_req *req)
     value.type = HASH_VALUE_ULONG;
 
     ret = hash_entries(pr_ctx->sid_table, &count, &entries);
+    if (ret != HASH_SUCCESS) {
+        DEBUG(SSSDBG_OP_FAILURE, "hash_entries failed [%d][%s].\n",
+                                 ret, hash_error_string(ret));
+        talloc_free(pr_ctx);
+        pac_cmd_done(cctx, ret);
+        return;
+    }
+
     for (c = 0; c < count; c++) {
         if (entries[c].value.ul == 0) {
             ret =responder_get_domain_by_id(cctx->rctx,
@@ -287,19 +295,19 @@ static void pac_lookup_sids_done(struct tevent_req *req)
             }
 
             msg = NULL;
-            ret = sysdb_search_object_by_sid(pr_ctx, dom->sysdb, dom,
-                                             entries[c].key.str, NULL, &msg);
-            if (ret != EOK) {
-                DEBUG(SSSDBG_OP_FAILURE, "sysdb_search_object_by_sid " \
-                                          "failed.\n");
+            ret = sysdb_search_object_by_sid(pr_ctx, dom, entries[c].key.str,
+                                             NULL, &msg);
+            if (ret == ENOENT) {
+                DEBUG(SSSDBG_OP_FAILURE, "No entry found for SID [%s].\n",
+                      entries[c].key.str);
+                continue;
+            } else if (ret != EOK) {
+                DEBUG(SSSDBG_OP_FAILURE,
+                      "sysdb_search_object_by_sid failed.\n");
                 continue;
             }
 
-            if (msg->count == 0) {
-                DEBUG(SSSDBG_OP_FAILURE, "No entry found for SID [%s].\n",
-                                          entries[c].key.str);
-                continue;
-            } else if (msg->count > 1) {
+            if (msg->count > 1) {
                 DEBUG(SSSDBG_CRIT_FAILURE, "More then one result returned " \
                                             "for SID [%s].\n",
                                             entries[c].key.str);
@@ -416,8 +424,7 @@ static errno_t pac_user_get_grp_info(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    ret = sysdb_initgroups(tmp_ctx, sysdb,
-                           pr_ctx->dom, pr_ctx->user_name, &res);
+    ret = sysdb_initgroups(tmp_ctx, pr_ctx->dom, pr_ctx->user_name, &res);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "sysdb_initgroups failed.\n");
         goto done;
@@ -573,24 +580,22 @@ static errno_t save_pac_user(struct pac_req_ctx *pr_ctx)
         goto done;
     }
 
-    ret = sysdb_search_user_by_uid(tmp_ctx, sysdb, pr_ctx->dom,
-                                   pwd->pw_uid, attrs, &msg);
+    ret = sysdb_search_user_by_uid(tmp_ctx, pr_ctx->dom, pwd->pw_uid, attrs,
+                                   &msg);
     if (ret == ENOENT) {
         if (pwd->pw_gid == 0 && !pr_ctx->dom->mpg) {
-            DEBUG(SSSDBG_CRIT_FAILURE, "Primary group RID from the PAC " \
-                                        "cannot be translated into a GID for " \
-                                        "user [%s]. Typically this happens " \
-                                        "when UIDs and GIDs are read from AD " \
-                                        "and the primary AD group does not " \
-                                        "have a GID assigned. Make sure the " \
-                                        "user is created by the ID provider " \
-                                        "before GSSAPI based authentication " \
-                                        "is used in this case.", pwd->pw_name);
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "Primary group RID from the PAC cannot be translated into "
+                  "a GID for user [%s]. Typically this happens when UIDs and "
+                  "GIDs are read from AD and the primary AD group does not "
+                  "have a GID assigned. Make sure the user is created by the "
+                  "ID provider before GSSAPI based authentication "
+                  "is used in this case.\n", pwd->pw_name);
             ret = EINVAL;
             goto done;
         }
 
-        ret = sysdb_store_user(sysdb, pr_ctx->dom, pwd->pw_name, NULL,
+        ret = sysdb_store_user(pr_ctx->dom, pwd->pw_name, NULL,
                                pwd->pw_uid, pwd->pw_gid, pwd->pw_gecos,
                                pwd->pw_dir,
                                pwd->pw_shell, NULL, user_attrs, NULL,
@@ -645,8 +650,7 @@ struct tevent_req *pac_save_memberships_send(struct pac_req_ctx *pr_ctx)
         goto done;
     }
 
-    ret = sysdb_search_user_by_name(state, dom->sysdb, dom, dom_name, NULL,
-                                    &msg);
+    ret = sysdb_search_user_by_name(state, dom, dom_name, NULL, &msg);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "sysdb_search_user_by_name failed " \
                                   "[%d][%s].\n", ret, strerror(ret));
@@ -716,7 +720,7 @@ pac_save_memberships_delete(struct pac_save_memberships_state *state)
     for (c = 0; c < pr_ctx->del_grp_count; c++) {
         /* If there is a failure for one group we still try to remove the
          * remaining groups. */
-        ret = sysdb_mod_group_member(pr_ctx->dom->sysdb, state->user_dn,
+        ret = sysdb_mod_group_member(pr_ctx->dom, state->user_dn,
                                      pr_ctx->del_grp_list[c].dn,
                                      LDB_FLAG_MOD_DELETE);
         if (ret != EOK) {
@@ -903,12 +907,15 @@ pac_store_membership(struct pac_req_ctx *pr_ctx,
         return ENOMEM;
     }
 
-    ret = sysdb_search_object_by_sid(tmp_ctx, grp_dom->sysdb, grp_dom,
-                                     grp_sid_str, group_attrs, &group);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_TRACE_INTERNAL, "sysdb_search_object_by_sid " \
-                                      "for SID [%s] failed [%d][%s].\n",
-                                      grp_sid_str, ret, strerror(ret));
+    ret = sysdb_search_object_by_sid(tmp_ctx, grp_dom, grp_sid_str,
+                                     group_attrs, &group);
+    if (ret == ENOENT) {
+        DEBUG(SSSDBG_OP_FAILURE, "Unexpected number of groups returned.\n");
+        goto done;
+    } else if (ret != EOK) {
+        DEBUG(SSSDBG_TRACE_INTERNAL,
+              "sysdb_search_object_by_sid for SID [%s] failed [%d][%s].\n",
+              grp_sid_str, ret, strerror(ret));
         goto done;
     }
 
@@ -929,7 +936,7 @@ pac_store_membership(struct pac_req_ctx *pr_ctx,
     DEBUG(SSSDBG_TRACE_ALL, "Adding user [%s] to group [%s][%s].\n",
                              ldb_dn_get_linearized(user_dn), grp_sid_str,
                              ldb_dn_get_linearized(group->msgs[0]->dn));
-    ret = sysdb_mod_group_member(grp_dom->sysdb, user_dn, group->msgs[0]->dn,
+    ret = sysdb_mod_group_member(grp_dom, user_dn, group->msgs[0]->dn,
                                  LDB_FLAG_MOD_ADD);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "sysdb_mod_group_member failed user [%s] " \
@@ -1086,6 +1093,14 @@ static void pac_lookup_sids_next_done(struct tevent_req *subreq)
     ret = sss_dp_get_account_recv(req, subreq,
                                   &err_maj, &err_min,
                                   &err_msg);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "Unable to get information from Data Provider\n"
+              "dp_error: [%u], errno: [%u], error_msg: [%s]\n",
+              (unsigned int)err_maj, (unsigned int)err_min,
+              err_msg ? err_msg : "none");
+    }
+
     talloc_zfree(subreq);
     talloc_zfree(err_msg);
     /* Errors during individual lookups are ignored. */
