@@ -150,8 +150,7 @@ errno_t get_sids_from_pac(TALLOC_CTX *mem_ctx,
     key.str = sid_str;
     value.ul = 0;
 
-    ret = sysdb_search_object_by_sid(mem_ctx, user_dom->sysdb, user_dom,
-                                     sid_str, NULL, &msg);
+    ret = sysdb_search_object_by_sid(mem_ctx, user_dom, sid_str, NULL, &msg);
     if (ret == EOK && msg->count == 1) {
         value.ul = ldb_msg_find_attr_as_uint64(msg->msgs[0], SYSDB_UIDNUM, 0);
     }
@@ -185,8 +184,7 @@ errno_t get_sids_from_pac(TALLOC_CTX *mem_ctx,
     key.str = sid_str;
     value.ul = 0;
 
-    ret = sysdb_search_object_by_sid(mem_ctx, user_dom->sysdb, user_dom,
-                                     sid_str, NULL, &msg);
+    ret = sysdb_search_object_by_sid(mem_ctx, user_dom, sid_str, NULL, &msg);
     if (ret == EOK && msg->count == 1) {
         value.ul = ldb_msg_find_attr_as_uint64(msg->msgs[0], SYSDB_GIDNUM, 0);
     }
@@ -214,8 +212,8 @@ errno_t get_sids_from_pac(TALLOC_CTX *mem_ctx,
         key.str = sid_str;
         value.ul = 0;
 
-        ret = sysdb_search_object_by_sid(mem_ctx, user_dom->sysdb, user_dom,
-                                         sid_str, NULL, &msg);
+        ret = sysdb_search_object_by_sid(mem_ctx, user_dom, sid_str,
+                                         NULL, &msg);
         if (ret == EOK && msg->count == 1) {
             value.ul = ldb_msg_find_attr_as_uint64(msg->msgs[0],
                                                    SYSDB_GIDNUM, 0);
@@ -246,8 +244,8 @@ errno_t get_sids_from_pac(TALLOC_CTX *mem_ctx,
 
         ret = responder_get_domain_by_id(pac_ctx->rctx, msid_str, &group_dom);
         if (ret == EOK) {
-            ret = sysdb_search_object_by_sid(mem_ctx, group_dom->sysdb,
-                                             group_dom, msid_str, NULL, &msg);
+            ret = sysdb_search_object_by_sid(mem_ctx, group_dom, msid_str,
+                                             NULL, &msg);
             if (ret == EOK && msg->count == 1 ) {
                 value.ul = ldb_msg_find_attr_as_uint64(msg->msgs[0],
                                                        SYSDB_GIDNUM, 0);
@@ -435,8 +433,8 @@ errno_t get_pwd_from_pac(TALLOC_CTX *mem_ctx,
             goto done;
         }
     } else {
-        DEBUG(SSSDBG_OP_FAILURE, "Missing full name in PAC, "
-                                  "gecos field will by empty.\n ");
+        DEBUG(SSSDBG_OP_FAILURE,
+              "Missing full name in PAC, gecos field will by empty.\n");
     }
 
     /* Check if there is a special homedir template for sub-domains. If not a
