@@ -37,6 +37,12 @@ struct pam_ctx {
     int neg_timeout;
     time_t id_timeout;
     hash_table_t *id_table;
+    size_t trusted_uids_count;
+    uid_t *trusted_uids;
+
+    /* List of domains that are accessible even for untrusted users. */
+    char **public_domains;
+    int public_domains_count;
 };
 
 struct pam_auth_dp_req {
@@ -51,7 +57,7 @@ struct pam_auth_req {
 
     pam_dp_callback_t *callback;
 
-    struct ldb_result *res;
+    bool is_uid_trusted;
     bool check_provider;
     void *data;
 
