@@ -30,6 +30,26 @@ struct sbus_request;
 #include <sys/types.h>
 #include "util/util.h"
 
+/* Older platforms (such as RHEL-6) might not have these error constants
+ * defined */
+#ifndef DBUS_ERROR_UNKNOWN_INTERFACE
+/** Interface you invoked a method on isn't known by the object. */
+#define DBUS_ERROR_UNKNOWN_INTERFACE \
+    "org.freedesktop.DBus.Error.UnknownInterface"
+#endif /* DBUS_ERROR_UNKNOWN_INTERFACE */
+
+#ifndef DBUS_ERROR_UNKNOWN_PROPERTY
+/** Property you tried to access isn't known by the object. */
+#define DBUS_ERROR_UNKNOWN_PROPERTY \
+    "org.freedesktop.DBus.Error.UnknownProperty"
+#endif /* DBUS_ERROR_UNKNOWN_PROPERTY */
+
+#ifndef DBUS_ERROR_PROPERTY_READ_ONLY
+/** Property you tried to set is read-only. */
+#define DBUS_ERROR_PROPERTY_READ_ONLY \
+    "org.freedesktop.DBus.Error.PropertyReadOnly"
+#endif /* DBUS_ERROR_PROPERTY_READ_ONLY */
+
 #ifndef DBUS_ERROR_INIT
 #define DBUS_ERROR_INIT { NULL, NULL, TRUE, 0, 0, 0, 0, NULL }
 #endif /* DBUS_ERROR_INIT */
@@ -84,6 +104,9 @@ struct sbus_vtable {
 #define DBUS_INTROSPECT_INTERFACE "org.freedesktop.DBus.Introspectable"
 #define DBUS_INTROSPECT_METHOD "Introspect"
 
+/* Special interface and method for D-BUS properties */
+#define DBUS_PROPERTIES_INTERFACE "org.freedesktop.DBus.Properties"
+
 struct sbus_interface {
     const char *path;
     struct sbus_vtable *vtable;
@@ -109,6 +132,7 @@ sbus_new_interface(TALLOC_CTX *mem_ctx,
 int sbus_new_server(TALLOC_CTX *mem_ctx,
                     struct tevent_context *ev,
                     const char *address,
+                    uid_t uid, gid_t gid,
                     bool use_symlink,
                     struct sbus_connection **server,
                     sbus_server_conn_init_fn init_fn, void *init_pvt_data);
@@ -185,6 +209,10 @@ int sbus_conn_send(struct sbus_connection *conn,
 void sbus_conn_send_reply(struct sbus_connection *conn,
                           DBusMessage *reply);
 
+/* Set up D-BUS access control. If there is a SSSD user, we must allow
+ * him to connect. root is always allowed */
+void sbus_allow_uid(struct sbus_connection *conn, uid_t *uid);
+
 /*
  * This structure is passed to all dbus method and property
  * handlers. It is a talloc context which will be valid until
@@ -250,7 +278,30 @@ int sbus_request_return_and_finish(struct sbus_request *dbus_req,
                                    int first_arg_type,
                                    ...);
 
+int sbus_add_variant_to_dict(DBusMessageIter *iter_dict,
+                             const char *key,
+                             int type,
+                             const void *value);
+
+int sbus_add_array_as_variant_to_dict(DBusMessageIter *iter_dict,
+                                      const char *key,
+                                      int type,
+                                      uint8_t *values,
+                                      const int len,
+                                      const unsigned int item_size);
+
+int sbus_request_return_as_variant(struct sbus_request *dbus_req,
+                                   int type,
+                                   const void *value);
+
+int sbus_request_return_array_as_variant(struct sbus_request *dbus_req,
+                                         int type,
+                                         uint8_t *values,
+                                         const int len,
+                                         const size_t item_size);
+
 /*
+
  * Return an error for a DBus method call request. The @error is a normal
  * DBusError.
  *
