@@ -79,6 +79,7 @@ sbus_request_invoke_or_finish(struct sbus_request *dbus_req,
         sbus_request_finish(dbus_req, NULL);
         break;
     default:
+        dbus_error_init(&error);
         dbus_set_error_const(&error, DBUS_ERROR_FAILED, INTERNAL_ERROR);
         sbus_request_fail_and_finish(dbus_req, &error);
         break;
@@ -127,6 +128,190 @@ int sbus_request_return_and_finish(struct sbus_request *dbus_req,
     return ret;
 }
 
+int sbus_request_return_as_variant(struct sbus_request *dbus_req,
+                                   int type,
+                                   const void *value)
+{
+    DBusMessage *reply;
+    dbus_bool_t dbret;
+    DBusMessageIter iter;
+    DBusMessageIter valiter;
+    char strtype[2];
+    int ret;
+
+    snprintf(strtype, 2, "%c", type);
+
+    reply = dbus_message_new_method_return(dbus_req->message);
+    if (reply == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Out of memory allocating DBus message\n");
+        sbus_request_finish(dbus_req, NULL);
+        return ENOMEM;
+    }
+
+    dbus_message_iter_init_append(reply, &iter);
+    dbret = dbus_message_iter_open_container(&iter, DBUS_TYPE_VARIANT,
+                                             strtype, &valiter);
+    if (!dbret) {
+        sbus_request_fail_and_finish(
+                            dbus_req,
+                            sbus_error_new(dbus_req,
+                                        DBUS_ERROR_FAILED,
+                                        "Could not open variant for [%s]\n",
+                                        strtype));
+        ret = EINVAL;
+        goto done;
+    }
+
+    dbret = dbus_message_iter_append_basic(&valiter, type, value);
+    if (!dbret) {
+        sbus_request_fail_and_finish(
+                            dbus_req,
+                            sbus_error_new(dbus_req,
+                                        DBUS_ERROR_FAILED,
+                                        "Could not append [%s] to variant\n",
+                                        strtype));
+        ret = EINVAL;
+        goto done;
+    }
+
+    dbret = dbus_message_iter_close_container(&iter, &valiter);
+    if (dbret) {
+        ret = sbus_request_finish(dbus_req, reply);
+    } else {
+        sbus_request_fail_and_finish(
+                            dbus_req,
+                            sbus_error_new(dbus_req,
+                                        DBUS_ERROR_FAILED,
+                                        "Could not close variant\n"));
+        ret = EINVAL;
+    }
+
+done:
+    dbus_message_unref(reply);
+    return ret;
+}
+
+
+int sbus_request_return_array_as_variant(struct sbus_request *dbus_req,
+                                         int type,
+                                         uint8_t *values,
+                                         const int len,
+                                         const size_t item_size)
+{
+    TALLOC_CTX *tmp_ctx = NULL;
+    DBusMessage *reply = NULL;
+    dbus_bool_t dbret;
+    DBusMessageIter iter;
+    DBusMessageIter variant_iter;
+    DBusMessageIter array_iter;
+    char *variant_type = NULL;
+    char *array_type = NULL;
+    void *addr = NULL;
+    int ret;
+    int i;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        return ENOMEM;
+    }
+
+    variant_type = talloc_asprintf(tmp_ctx, DBUS_TYPE_ARRAY_AS_STRING "%c",
+                                   type);
+    if (variant_type == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    array_type = talloc_asprintf(tmp_ctx, "%c", type);
+    if (array_type == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    reply = dbus_message_new_method_return(dbus_req->message);
+    if (reply == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Out of memory allocating DBus message\n");
+        sbus_request_finish(dbus_req, NULL);
+        ret = ENOMEM;
+        goto done;
+    }
+
+    dbus_message_iter_init_append(reply, &iter);
+    dbret = dbus_message_iter_open_container(&iter, DBUS_TYPE_VARIANT,
+                                             variant_type, &variant_iter);
+    if (!dbret) {
+        sbus_request_fail_and_finish(
+                            dbus_req,
+                            sbus_error_new(dbus_req,
+                                        DBUS_ERROR_FAILED,
+                                        "Could not open variant for [%s]\n",
+                                        variant_type));
+        ret = EINVAL;
+        goto done;
+    }
+
+    /* Open container for values */
+    dbret = dbus_message_iter_open_container(&variant_iter, DBUS_TYPE_ARRAY,
+                                             array_type, &array_iter);
+    if (!dbret) {
+        sbus_request_fail_and_finish(
+                            dbus_req,
+                            sbus_error_new(dbus_req,
+                                        DBUS_ERROR_FAILED,
+                                        "Could not open array for [%s]\n",
+                                        array_type));
+        ret = EINVAL;
+        goto done;
+    }
+
+    for (i = 0; i < len; i++) {
+        addr = values + i * item_size;
+        dbret = dbus_message_iter_append_basic(&array_iter, type, addr);
+        if (!dbret) {
+            sbus_request_fail_and_finish(
+                                dbus_req,
+                                sbus_error_new(dbus_req,
+                                            DBUS_ERROR_FAILED,
+                                            "Could not append [%s] to variant\n",
+                                            array_type));
+            ret = EINVAL;
+            goto done;
+        }
+    }
+
+    dbret = dbus_message_iter_close_container(&variant_iter, &array_iter);
+    if (!dbret) {
+        sbus_request_fail_and_finish(
+                            dbus_req,
+                            sbus_error_new(dbus_req,
+                                        DBUS_ERROR_FAILED,
+                                        "Could not close array\n"));
+        ret = EINVAL;
+        goto done;
+    }
+
+    dbret = dbus_message_iter_close_container(&iter, &variant_iter);
+    if (dbret) {
+        ret = sbus_request_finish(dbus_req, reply);
+    } else {
+        sbus_request_fail_and_finish(
+                            dbus_req,
+                            sbus_error_new(dbus_req,
+                                        DBUS_ERROR_FAILED,
+                                        "Could not close variant\n"));
+        ret = EINVAL;
+        goto done;
+    }
+
+done:
+    if (reply != NULL) {
+        dbus_message_unref(reply);
+    }
+
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
 int sbus_request_fail_and_finish(struct sbus_request *dbus_req,
                                  const DBusError *error)
 {
