@@ -424,7 +424,7 @@ static bool sss_cli_check_version(const char *socket_name)
         return false;
     }
 
-    obtained_version = ((uint32_t *)repbuf)[0];
+    SAFEALIGN_COPY_UINT32(&obtained_version, repbuf, NULL);
     free(repbuf);
 
     return (obtained_version == expected_version);
@@ -724,7 +724,13 @@ enum nss_status sss_nss_make_request(enum sss_cli_command cmd,
 
     ret = sss_cli_check_socket(errnop, SSS_NSS_SOCKET_NAME);
     if (ret != SSS_STATUS_SUCCESS) {
+#ifdef NONSTANDARD_SSS_NSS_BEHAVIOUR
+        *errnop = 0;
+        errno = 0;
+        return NSS_STATUS_NOTFOUND;
+#else
         return NSS_STATUS_UNAVAIL;
+#endif
     }
 
     ret = sss_cli_make_request_nochecks(cmd, rd, repbuf, replen, errnop);
@@ -735,10 +741,29 @@ enum nss_status sss_nss_make_request(enum sss_cli_command cmd,
         return NSS_STATUS_SUCCESS;
     case SSS_STATUS_UNAVAIL:
     default:
+#ifdef NONSTANDARD_SSS_NSS_BEHAVIOUR
+        *errnop = 0;
+        errno = 0;
+        return NSS_STATUS_NOTFOUND;
+#else
         return NSS_STATUS_UNAVAIL;
+#endif
     }
 }
 
+int sss_pac_check_and_open(void)
+{
+    enum sss_status ret;
+    int errnop;
+
+    ret = sss_cli_check_socket(&errnop, SSS_PAC_SOCKET_NAME);
+    if (ret != SSS_STATUS_SUCCESS) {
+        return EIO;
+    }
+
+    return EOK;
+}
+
 int sss_pac_make_request(enum sss_cli_command cmd,
                          struct sss_cli_req_data *rd,
                          uint8_t **repbuf, size_t *replen,
@@ -1022,7 +1047,7 @@ static int sss_mutexattr_setrobust (pthread_mutexattr_t *attr)
 {
 #ifdef HAVE_PTHREAD_MUTEXATTR_SETROBUST
     return pthread_mutexattr_setrobust(attr, PTHREAD_MUTEX_ROBUST);
-#elif  HAVE_PTHREAD_MUTEXATTR_SETROBUST_NP
+#elif defined(HAVE_PTHREAD_MUTEXATTR_SETROBUST_NP)
     return pthread_mutexattr_setrobust_np(attr, PTHREAD_MUTEX_ROBUST_NP);
 #else
 #warning Robust mutexes are not supported on this platform.
@@ -1034,7 +1059,7 @@ static int sss_mutex_consistent(pthread_mutex_t *mtx)
 {
 #ifdef HAVE_PTHREAD_MUTEX_CONSISTENT
     return pthread_mutex_consistent(mtx);
-#elif HAVE_PTHREAD_MUTEX_CONSISTENT_NP
+#elif defined(HAVE_PTHREAD_MUTEX_CONSISTENT_NP)
     return pthread_mutex_consistent_np(mtx);
 #else
 #warning Robust mutexes are not supported on this platform.
