@@ -250,10 +250,6 @@ errno_t sss_nss_mc_get_record(struct sss_cli_mc_ctx *ctx,
     int count;
     int ret;
 
-    if (!MC_SLOT_WITHIN_BOUNDS(slot, ctx->dt_size)) {
-        return EINVAL;
-    }
-
     /* try max 5 times */
     for (count = 5; count > 0; count--) {
         rec = MC_SLOT_TO_PTR(ctx->data_table, slot, struct sss_mc_rec);
@@ -355,3 +351,16 @@ errno_t sss_nss_str_ptr_from_buffer(char **str, void **cookie,
     return 0;
 }
 
+uint32_t sss_nss_mc_next_slot_with_hash(struct sss_mc_rec *rec,
+                                        uint32_t hash)
+{
+    if (rec->hash1 == hash) {
+        return rec->next1;
+    } else if (rec->hash2 == hash) {
+        return rec->next2;
+    } else {
+        /* it should never happen. */
+        return MC_INVALID_VAL;
+    }
+
+}
