@@ -244,6 +244,13 @@ SSS_NSS_GETIDBYSID   = 0x0114, /**< Takes the zero terminated string
                                     another unsigned 32bit integer value
                                     indicating the type (unknown, user, group,
                                     both) of the object. */
+SSS_NSS_GETORIGBYNAME = 0x0115, /**< Takes a zero terminated fully qualified
+                                     name and returns a list of zero
+                                     terminated strings with key-value pairs
+                                     where the first string is the key and
+                                     second the value. Hence the list should
+                                     have an even number of strings, if not
+                                     the whole list is invalid. */
 };
 
 /**
@@ -314,6 +321,7 @@ enum pam_item_type {
     SSS_PAM_ITEM_NEWAUTHTOK,
     SSS_PAM_ITEM_CLI_LOCALE,
     SSS_PAM_ITEM_CLI_PID,
+    SSS_PAM_ITEM_REQUESTED_DOMAINS,
 };
 
 #define SSS_NSS_MAX_ENTRIES 256
@@ -354,12 +362,9 @@ enum sss_status {
  *
  * Examples:
  *  - #SSS_PAM_ENV_ITEM,
- *    <pre>
- *    ------------------------------------
- *    | uint32_t | uint32_t | uint8_t[4] |
- *    | 0x03     | 0x04     | a=b\\0      |
- *    ------------------------------------
- *    </pre>
+       uint32_t | uint32_t | uint8_t[4]
+      ----------|----------|------------
+       0x03     | 0x04     | a=b\\0
  * @{
  */
 
@@ -408,19 +413,14 @@ enum response_type {
  *
  * Examples:
  *  - #SSS_PAM_USER_INFO_OFFLINE_CHPASS
- *    <pre>
- *    ----------------------------------
- *    | uint32_t | uint32_t | uint32_t |
- *    | 0x06     | 0x01     | 0x03     |
- *    ----------------------------------
- *    </pre>
+ *     uint32_t | uint32_t | uint32_t
+ *    ----------|----------|----------
+ *     0x06     | 0x01     | 0x03
+ *
  *  - #SSS_PAM_USER_INFO_CHPASS_ERROR
- *    <pre>
- *    ----------------------------------------------------------
- *    | uint32_t | uint32_t | uint32_t | uint32_t | uint8_t[3] |
- *    | 0x06     | 0x05     | 0x04     | 0x03     | abc        |
- *    ----------------------------------------------------------
- *    </pre>
+ *     uint32_t | uint32_t | uint32_t | uint32_t | uint8_t[3]
+ *    ----------|----------|----------|----------|------------
+ *     0x06     | 0x05     | 0x04     | 0x03     | abc
  * @{
  */
 
@@ -461,15 +461,25 @@ enum user_info_type {
                                      * indicates that no message is following.
                                      * @param String with the specified
                                      * length. */
+
     SSS_PAM_USER_INFO_GRACE_LOGIN, /**< Warn the user that the password is
                                     * expired and inform about the remaining
                                     * number of grace logins.
                                     * @param The number of remaining grace
                                     * logins as uint32_t */
-    SSS_PAM_USER_INFO_EXPIRE_WARN /**< Warn the user that the password will
-                                   * expire soon.
-                                   * @param Number of seconds before the user's
-                                   * password will expire. */
+    SSS_PAM_USER_INFO_EXPIRE_WARN, /**< Warn the user that the password will
+                                    * expire soon.
+                                    * @param Number of seconds before the
+                                    * user's password will expire. */
+
+    SSS_PAM_USER_INFO_ACCOUNT_EXPIRED, /**< Tell the user that the account
+                                        * has expired and optionally give
+                                        * a reason.
+                                        * @param Size of the message as
+                                        * unsigned 32-bit integer value. A
+                                        * value of 0 indicates that no message
+                                        * is following. @param String with the
+                                        * specified length. */
 };
 /**
  * @}
@@ -511,6 +521,12 @@ int sss_pam_make_request(enum sss_cli_command cmd,
                          int *errnop);
 void sss_pam_close_fd(void);
 
+/* Checks access to the PAC responder and opens the socket, if available.
+ * Required for processes like krb5_child that need to open the socket
+ * before dropping privs.
+ */
+int sss_pac_check_and_open(void);
+
 int sss_pac_make_request(enum sss_cli_command cmd,
                          struct sss_cli_req_data *rd,
                          uint8_t **repbuf, size_t *replen,
