@@ -38,11 +38,18 @@ struct so {
     const char *libs[6];
 } so[] = {
     { "libsss_debug.so", { LIBPFX"libsss_debug.so", NULL } },
+    { "libsss_semanage.so", { LIBPFX"libsss_semanage.so", NULL } },
     { "libipa_hbac.so", { LIBPFX"libipa_hbac.so", NULL } },
     { "libsss_idmap.so", { LIBPFX"libsss_idmap.so", NULL } },
     { "libsss_nss_idmap.so", { LIBPFX"libsss_nss_idmap.so", NULL } },
     { "libnss_sss.so", { LIBPFX"libnss_sss.so", NULL } },
     { "pam_sss.so", { LIBPFX"pam_sss.so", NULL } },
+#ifdef BUILD_LIBWBCLIENT
+    { "libwbclient.so", { LIBPFX"libwbclient.so", NULL } },
+#endif /* BUILD_LIBWBCLIENT */
+#ifdef BUILD_IFP
+    { "libsss_simpleifp.so", { LIBPFX"libsss_simpleifp.so", NULL } },
+#endif /* BUILD_IFP */
 #ifdef BUILD_SUDO
     { "libsss_sudo.so", { LIBPFX"libsss_sudo.so", NULL } },
 #endif
@@ -53,22 +60,21 @@ struct so {
     { "sssd_krb5_locator_plugin.so", { LIBPFX"sssd_krb5_locator_plugin.so",
                                        NULL } },
 #endif
+#ifdef HAVE_KRB5_LOCALAUTH_PLUGIN
+    { "sssd_krb5_localauth_plugin.so", { LIBPFX"sssd_krb5_localauth_plugin.so",
+                                       NULL } },
+#endif
 #ifdef HAVE_PAC_RESPONDER
     { "sssd_pac_plugin.so", { LIBPFX"sssd_pac_plugin.so", NULL } },
+#endif
+#ifdef HAVE_CIFS_IDMAP_PLUGIN
+    { "cifs_idmap_sss.so", { LIBPFX"cifs_idmap_sss.so", NULL } },
 #endif
     { "memberof.so", { LIBPFX"memberof.so", NULL } },
-    { "libsss_child.so", { "libtevent.so",
-                           LIBPFX"libsss_debug.so",
-                           LIBPFX"libsss_crypt.so",
-                           LIBPFX"libsss_util.so",
+    { "libsss_child.so", { LIBPFX"libsss_util.so",
                            LIBPFX"libsss_child.so", NULL } },
-    { "libsss_crypt.so", { "libtalloc.so",
-                           LIBPFX"libsss_debug.so",
-                           LIBPFX"libsss_crypt.so", NULL } },
-    { "libsss_util.so", { "libtalloc.so",
-                           LIBPFX"libsss_debug.so",
-                           LIBPFX"libsss_crypt.so",
-                           LIBPFX"libsss_util.so", NULL } },
+    { "libsss_crypt.so", { LIBPFX"libsss_crypt.so", NULL } },
+    { "libsss_util.so", { LIBPFX"libsss_util.so", NULL } },
     { "libsss_simple.so", { LIBPFX"libdlopen_test_providers.so",
                             LIBPFX"libsss_simple.so", NULL } },
 #ifdef BUILD_SAMBA
@@ -79,15 +85,24 @@ struct so {
 #endif /* BUILD_SAMBA */
     { "libsss_krb5.so", { LIBPFX"libdlopen_test_providers.so",
                           LIBPFX"libsss_krb5.so", NULL } },
+    { "libsss_krb5_common.so", { LIBPFX"libdlopen_test_providers.so",
+                                 LIBPFX"libsss_krb5_common.so", NULL } },
     { "libsss_ldap.so", { LIBPFX"libdlopen_test_providers.so",
                           LIBPFX"libsss_ldap.so", NULL } },
+    { "libsss_ldap_common.so", { LIBPFX"libdlopen_test_providers.so",
+                                 LIBPFX"libsss_ldap_common.so", NULL } },
     { "libsss_proxy.so", { LIBPFX"libdlopen_test_providers.so",
                            LIBPFX"libsss_proxy.so", NULL } },
+    { "libdlopen_test_providers.so", { LIBPFX"libdlopen_test_providers.so",
+                                       NULL } },
 #ifdef HAVE_PYTHON_BINDINGS
     { "pyhbac.so", { LIBPFX"pyhbac.so", NULL } },
     { "pysss.so", { LIBPFX"pysss.so", NULL } },
     { "pysss_murmur.so", { LIBPFX"pysss_murmur.so", NULL } },
     { "pysss_nss_idmap.so", { LIBPFX"pysss_nss_idmap.so", NULL } },
+#endif
+#ifdef HAVE_CONFIG_LIB
+    { "libsss_config.so", { LIBPFX"libsss_config.so", NULL } },
 #endif
     { NULL }
 };
