@@ -28,6 +28,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <stdlib.h>
+
 #include "util/util.h"
 #include "util/sss_utf8.h"
 #include "util/murmurhash3.h"
@@ -141,6 +143,9 @@ START_TEST(test_parse_args)
     const char *parsed6[] = { "foo", "a", "e", NULL };
     const char *parsed7[] = { "foo", "a", "f", NULL };
     const char *parsed8[] = { "foo", "a\tg", NULL };
+    const char *parsed9[] = { "foo", NULL };
+    const char *parsed10[] = { " ", "foo", "\t", "\\'", NULL };
+    const char *parsed11[] = { "a", NULL };
     struct pa_testcase tc[] = {
         { "foo", parsed1 },
         { "foo a", parsed2 },
@@ -148,15 +153,22 @@ START_TEST(test_parse_args)
         { "foo a\\ c", parsed4 },
         { "foo a d ", parsed5 },
         { "foo   a   e   ", parsed6 },
-        { "foo	a	 	f 	", parsed7 },
+        { "foo\ta\t \tf \t", parsed7 },
         { "foo a\\\tg", parsed8 },
+        { "   foo  ", parsed9 },
+        { "\\   foo \\\t \\'  ", parsed10 },
+        { "a", parsed11 },
+        { " ", NULL },
+        { "", NULL },
+        { "  \t  ", NULL },
         { NULL, NULL }
     };
 
     for (i=0; tc[i].argstr != NULL; i++) {
         parsed = parse_args(tc[i].argstr);
         fail_if(parsed == NULL && tc[i].parsed != NULL,
-                "Could not parse correct argument string '%s'\n");
+                "Could not parse correct %d argument string '%s'\n",
+                i, tc[i].argstr);
 
         ret = diff_string_lists(test_ctx, parsed, discard_const(tc[i].parsed),
                                 &only_ret, &only_exp, &both);
@@ -164,8 +176,20 @@ START_TEST(test_parse_args)
         fail_unless(only_ret[0] == NULL, "The parser returned more data than expected\n");
         fail_unless(only_exp[0] == NULL, "The parser returned less data than expected\n");
 
-        for (ii = 0; parsed[ii]; ii++) free(parsed[ii]);
-        free(parsed);
+        if (parsed) {
+            int parsed_len;
+            int expected_len;
+
+            for (parsed_len=0; parsed[parsed_len]; ++parsed_len);
+            for (expected_len=0; tc[i].parsed[expected_len]; ++expected_len);
+
+            fail_unless(parsed_len == expected_len,
+                        "Test %d: length of 1st array [%d] != length of 2nd "
+                        "array[%d]\n", i, parsed_len, expected_len);
+
+            for (ii = 0; parsed[ii]; ii++) free(parsed[ii]);
+            free(parsed);
+        }
     }
 
     talloc_free(test_ctx);
@@ -522,7 +546,7 @@ START_TEST(test_murmurhash3_random)
     }
     test[len] = '\0'; /* null terminate */
 
-    fprintf(stdout, "test_murmurhash3_random seed: %d\n", init_seed);
+    fprintf(stdout, "test_murmurhash3_random seed: %u\n", init_seed);
 
     result1 = murmurhash3(test, len + 1, init_seed);
     result2 = murmurhash3(test, len + 1, init_seed);
@@ -808,6 +832,128 @@ START_TEST(test_split_on_separator)
 }
 END_TEST
 
+struct check_ip_test_data {
+    const char *str_ipaddr;
+    uint8_t flags;
+    bool expected_ret;
+};
+
+START_TEST(test_check_ipv4_addr)
+{
+    int a;
+    int num_of_tests;
+    int ret;
+    bool bret;
+    struct in_addr addr;
+    struct check_ip_test_data tst_data[] = {
+        {
+            "192.168.100.1", /* input IPv4 address */
+            0, /* flags value */
+            true /* Expected return value */
+        },
+        {
+            "224.0.0.22", /* multicast address */
+            SSS_NO_MULTICAST,
+            false
+        },
+        {
+            "192.186.0.224",
+            SSS_NO_MULTICAST,
+            true
+        },
+        {
+            "127.0.0.1",
+            SSS_NO_LOOPBACK,
+            false
+        },
+        {
+            "169.254.0.11",
+            SSS_NO_LINKLOCAL,
+            false
+        },
+        {
+            "255.255.255.255",
+            SSS_NO_BROADCAST,
+            false
+        },
+        {
+            "255.255.255.255",
+            SSS_NO_SPECIAL,
+            false
+        },
+        {
+            "192.168.254.169",
+            SSS_NO_SPECIAL,
+            true
+        },
+    };
+
+    num_of_tests = sizeof(tst_data) / sizeof(struct check_ip_test_data);
+
+    for (a = 0; a < num_of_tests; a++) {
+        /* fill sockaddr_in structure */
+
+        ret = inet_pton(AF_INET, tst_data[a].str_ipaddr, &addr);
+        fail_if(ret != 1, "inet_pton failed.");
+
+        bret = check_ipv4_addr(&addr, tst_data[a].flags);
+        fail_unless(bret == tst_data[a].expected_ret,
+                    "check_ipv4_addr failed (iteration %d)", a);
+    }
+}
+END_TEST
+
+START_TEST(test_check_ipv6_addr)
+{
+    int a;
+    int num_of_tests;
+    int ret;
+    bool bret;
+    struct in6_addr addr;
+    struct check_ip_test_data tst_data[] = {
+        {
+            "fde9:7e3f:1ed3:24a5::4", /* input IPv6 address */
+            0, /* flags value */
+            true /* Expected return value */
+        },
+        {
+            "fe80::f2de:f1ff:fefa:67f0",
+            SSS_NO_LINKLOCAL,
+            false
+        },
+        {
+            "::1",
+            SSS_NO_LOOPBACK,
+            false
+        },
+        {
+            "ff00::123",
+            SSS_NO_MULTICAST,
+            false
+        },
+        {
+            "ff00::321",
+            SSS_NO_SPECIAL,
+            false
+        },
+    };
+
+    num_of_tests = sizeof(tst_data) / sizeof(struct check_ip_test_data);
+
+    for (a = 0; a < num_of_tests; a++) {
+        /* fill sockaddr_in structure */
+
+        ret = inet_pton(AF_INET6, tst_data[a].str_ipaddr, &addr);
+        fail_if(ret != 1, "inet_pton failed.");
+
+        bret = check_ipv6_addr(&addr, tst_data[a].flags);
+        fail_unless(bret == tst_data[a].expected_ret,
+                    "check_ipv6_addr failed (iteration %d)", a);
+
+    }
+}
+END_TEST
+
 START_TEST(test_is_host_in_domain)
 {
     struct {
@@ -835,6 +981,81 @@ START_TEST(test_is_host_in_domain)
 }
 END_TEST
 
+START_TEST(test_known_service)
+{
+    const char * const * svcs;
+    bool found_nss;
+    int i;
+
+    /* Just make sure we can't find a bogus service and nss
+     * is always available
+     */
+    svcs = get_known_services();
+    for (i = 0; svcs[i]; i++) {
+        ck_assert_str_ne(svcs[i], "nosuchservice");
+        if (strcmp(svcs[i], "nss")) {
+            found_nss = true;
+        }
+    }
+
+    ck_assert(found_nss == true);
+}
+END_TEST
+
+static void convert_time_tz(const char* tz)
+{
+    errno_t ret, ret2;
+    time_t unix_time;
+    const char *orig_tz = NULL;
+
+    orig_tz = getenv("TZ");
+    if (orig_tz == NULL) {
+        orig_tz = "";
+    }
+
+    if (tz) {
+        ret = setenv("TZ", tz, 1);
+        fail_if(ret == -1);
+    }
+
+    ret = sss_utc_to_time_t("20140801115742Z", "%Y%m%d%H%M%SZ", &unix_time);
+
+    /* restore */
+    if (orig_tz != NULL) {
+        ret2 = setenv("TZ", orig_tz, 1);
+        fail_if(ret2 == -1);
+    }
+    fail_unless(ret == EOK && difftime(1406894262, unix_time) == 0);
+}
+
+START_TEST(test_convert_time)
+{
+    const char *format = "%Y%m%d%H%M%SZ";
+    time_t unix_time;
+    errno_t ret;
+
+    ret = sss_utc_to_time_t("20150127133540P", format, &unix_time);
+    fail_unless(ret == ERR_TIMESPEC_NOT_SUPPORTED);
+    ret = sss_utc_to_time_t("0Z", format, &unix_time);
+    fail_unless(ret == EINVAL);
+    ret = sss_utc_to_time_t("000001010000Z", format, &unix_time);
+    fail_unless(ret == EINVAL);
+
+    /* test that results are still same no matter what timezone is set */
+    convert_time_tz(NULL);
+
+    convert_time_tz("GST-1");
+
+    convert_time_tz("GST-2");
+}
+END_TEST
+
+START_TEST(test_sss_strerror_err_last)
+{
+    ck_assert_str_eq(sss_strerror(ERR_LAST), "ERR_LAST");
+}
+END_TEST
+
 Suite *util_suite(void)
 {
     Suite *s = suite_create("util");
@@ -851,7 +1072,10 @@ Suite *util_suite(void)
     tcase_add_test (tc_util, test_add_string_to_list);
     tcase_add_test (tc_util, test_string_in_list);
     tcase_add_test (tc_util, test_split_on_separator);
+    tcase_add_test (tc_util, test_check_ipv4_addr);
+    tcase_add_test (tc_util, test_check_ipv6_addr);
     tcase_add_test (tc_util, test_is_host_in_domain);
+    tcase_add_test (tc_util, test_known_service);
     tcase_set_timeout(tc_util, 60);
 
     TCase *tc_utf8 = tcase_create("utf8");
@@ -878,10 +1102,21 @@ Suite *util_suite(void)
     tcase_add_test(tc_atomicio, test_atomicio_read_exact_sized_file);
     tcase_add_test(tc_atomicio, test_atomicio_read_from_empty_file);
 
+    TCase *tc_convert_time = tcase_create("convert_time");
+    tcase_add_checked_fixture(tc_convert_time,
+                              ck_leak_check_setup,
+                              ck_leak_check_teardown);
+    tcase_add_test(tc_convert_time, test_convert_time);
+
+    TCase *tc_sss_strerror = tcase_create("sss_strerror");
+    tcase_add_test(tc_sss_strerror, test_sss_strerror_err_last);
+
     suite_add_tcase (s, tc_util);
     suite_add_tcase (s, tc_utf8);
     suite_add_tcase (s, tc_mh3);
     suite_add_tcase (s, tc_atomicio);
+    suite_add_tcase (s, tc_convert_time);
+    suite_add_tcase (s, tc_sss_strerror);
 
     return s;
 }
@@ -915,7 +1150,7 @@ int main(int argc, const char *argv[])
     }
     poptFreeContext(pc);
 
-    DEBUG_INIT(debug_level);
+    DEBUG_CLI_INIT(debug_level);
 
     tests_set_cwd();
 
