@@ -30,6 +30,7 @@
 #include "db/sysdb.h"
 #include "db/sysdb_services.h"
 #include "db/sysdb_autofs.h"
+#include "db/sysdb_ssh.h"
 
 #define INVALIDATE_NONE 0
 #define INVALIDATE_USERS 1
@@ -37,25 +38,39 @@
 #define INVALIDATE_NETGROUPS 4
 #define INVALIDATE_SERVICES 8
 #define INVALIDATE_AUTOFSMAPS 16
+#define INVALIDATE_SSH_HOSTS 32
 
 #ifdef BUILD_AUTOFS
+#ifdef BUILD_SSH
 #define INVALIDATE_EVERYTHING (INVALIDATE_USERS | INVALIDATE_GROUPS | \
                                INVALIDATE_NETGROUPS | INVALIDATE_SERVICES | \
-                               INVALIDATE_AUTOFSMAPS)
-#else
+                               INVALIDATE_AUTOFSMAPS | INVALIDATE_SSH_HOSTS )
+#else  /* BUILD_SSH */
 #define INVALIDATE_EVERYTHING (INVALIDATE_USERS | INVALIDATE_GROUPS | \
-                               INVALIDATE_NETGROUPS | INVALIDATE_SERVICES)
-#endif
+                               INVALIDATE_NETGROUPS | INVALIDATE_SERVICES | \
+                               INVALIDATE_AUTOFSMAPS )
+#endif /* BUILD_SSH */
+#else  /* BUILD_AUTOFS */
+#ifdef BUILD_SSH
+#define INVALIDATE_EVERYTHING (INVALIDATE_USERS | INVALIDATE_GROUPS | \
+                               INVALIDATE_NETGROUPS | INVALIDATE_SERVICES | \
+                               INVALIDATE_SSH_HOSTS )
+#else  /* BUILD_SSH */
+#define INVALIDATE_EVERYTHING (INVALIDATE_USERS | INVALIDATE_GROUPS | \
+                               INVALIDATE_NETGROUPS | INVALIDATE_SERVICES )
+#endif /* BUILD_SSH */
+#endif /* BUILD_AUTOFS */
 
 enum sss_cache_entry {
     TYPE_USER=0,
     TYPE_GROUP,
     TYPE_NETGROUP,
     TYPE_SERVICE,
-    TYPE_AUTOFSMAP
+    TYPE_AUTOFSMAP,
+    TYPE_SSH_HOST
 };
 
-static errno_t search_autofsmaps(TALLOC_CTX *mem_ctx, struct sysdb_ctx *sysdb,
+static errno_t search_autofsmaps(TALLOC_CTX *mem_ctx,
                                  struct sss_domain_info *domain,
                                  const char *sub_filter, const char **attrs,
                                  size_t *msgs_count, struct ldb_message ***msgs);
@@ -69,28 +84,30 @@ struct cache_tool_ctx {
     char *netgroup_filter;
     char *service_filter;
     char *autofs_filter;
+    char *ssh_host_filter;
 
     char *user_name;
     char *group_name;
     char *netgroup_name;
     char *service_name;
     char *autofs_name;
+    char *ssh_host_name;
 
     bool update_user_filter;
     bool update_group_filter;
     bool update_netgroup_filter;
     bool update_service_filter;
     bool update_autofs_filter;
+    bool update_ssh_host_filter;
 };
 
 errno_t init_domains(struct cache_tool_ctx *ctx, const char *domain);
 errno_t init_context(int argc, const char *argv[], struct cache_tool_ctx **tctx);
-static errno_t invalidate_entry(TALLOC_CTX *ctx, struct sysdb_ctx *sysdb,
+static errno_t invalidate_entry(TALLOC_CTX *ctx,
                                 struct sss_domain_info *domain,
                                 const char *name, int entry_type);
 static bool invalidate_entries(TALLOC_CTX *ctx,
                                struct sss_domain_info *dinfo,
-                               struct sysdb_ctx *sysdb,
                                enum sss_cache_entry entry_type,
                                const char *filter, const char *name);
 static errno_t update_all_filters(struct cache_tool_ctx *tctx,
@@ -112,8 +129,6 @@ int main(int argc, const char *argv[])
     }
 
     for (dinfo = tctx->domains; dinfo; dinfo = get_next_domain(dinfo, true)) {
-         sysdb = dinfo->sysdb;
-
         if (!IS_SUBDOMAIN(dinfo)) {
             /* Update list of subdomains for this domain */
             ret = sysdb_update_subdomains(dinfo);
@@ -138,21 +153,24 @@ int main(int argc, const char *argv[])
             goto done;
         }
 
-        skipped &= !invalidate_entries(tctx, dinfo, sysdb, TYPE_USER,
+        skipped &= !invalidate_entries(tctx, dinfo, TYPE_USER,
                                        tctx->user_filter,
                                        tctx->user_name);
-        skipped &= !invalidate_entries(tctx, dinfo, sysdb, TYPE_GROUP,
+        skipped &= !invalidate_entries(tctx, dinfo, TYPE_GROUP,
                                        tctx->group_filter,
                                        tctx->group_name);
-        skipped &= !invalidate_entries(tctx, dinfo, sysdb, TYPE_NETGROUP,
+        skipped &= !invalidate_entries(tctx, dinfo, TYPE_NETGROUP,
                                        tctx->netgroup_filter,
                                        tctx->netgroup_name);
-        skipped &= !invalidate_entries(tctx, dinfo, sysdb, TYPE_SERVICE,
+        skipped &= !invalidate_entries(tctx, dinfo, TYPE_SERVICE,
                                        tctx->service_filter,
                                        tctx->service_name);
-        skipped &= !invalidate_entries(tctx, dinfo, sysdb, TYPE_AUTOFSMAP,
+        skipped &= !invalidate_entries(tctx, dinfo, TYPE_AUTOFSMAP,
                                        tctx->autofs_filter,
                                        tctx->autofs_name);
+        skipped &= !invalidate_entries(tctx, dinfo, TYPE_SSH_HOST,
+                                       tctx->ssh_host_filter,
+                                       tctx->ssh_host_name);
 
         ret = sysdb_transaction_commit(sysdb);
         if (ret != EOK) {
@@ -329,12 +347,19 @@ static errno_t update_all_filters(struct cache_tool_ctx *tctx,
         return ret;
     }
 
+    /* Update ssh host filter */
+    ret = update_filter(tctx, dinfo, tctx->ssh_host_name,
+                        tctx->update_ssh_host_filter, "(%s=%s)", false,
+                        &tctx->ssh_host_filter);
+    if (ret != EOK) {
+        return ret;
+    }
+
     return EOK;
 }
 
 static bool invalidate_entries(TALLOC_CTX *ctx,
                                struct sss_domain_info *dinfo,
-                               struct sysdb_ctx *sysdb,
                                enum sss_cache_entry entry_type,
                                const char *filter, const char *name)
 {
@@ -351,28 +376,36 @@ static bool invalidate_entries(TALLOC_CTX *ctx,
     switch (entry_type) {
     case TYPE_USER:
         type_string = "user";
-        ret = sysdb_search_users(ctx, sysdb, dinfo,
+        ret = sysdb_search_users(ctx, dinfo,
                                  filter, attrs, &msg_count, &msgs);
         break;
     case TYPE_GROUP:
         type_string = "group";
-        ret = sysdb_search_groups(ctx, sysdb, dinfo,
+        ret = sysdb_search_groups(ctx, dinfo,
                                   filter, attrs, &msg_count, &msgs);
         break;
     case TYPE_NETGROUP:
         type_string = "netgroup";
-        ret = sysdb_search_netgroups(ctx, sysdb, dinfo,
+        ret = sysdb_search_netgroups(ctx, dinfo,
                                      filter, attrs, &msg_count, &msgs);
         break;
     case TYPE_SERVICE:
         type_string = "service";
-        ret = sysdb_search_services(ctx, sysdb, dinfo,
+        ret = sysdb_search_services(ctx, dinfo,
                                     filter, attrs, &msg_count, &msgs);
         break;
     case TYPE_AUTOFSMAP:
         type_string = "autofs map";
-        ret = search_autofsmaps(ctx, sysdb, dinfo,
-                                filter, attrs, &msg_count, &msgs);
+        ret = search_autofsmaps(ctx, dinfo, filter, attrs, &msg_count, &msgs);
+        break;
+    case TYPE_SSH_HOST:
+        type_string = "ssh_host";
+#ifdef BUILD_SSH
+        ret = sysdb_search_ssh_hosts(ctx, dinfo,
+                                     filter, attrs, &msg_count, &msgs);
+#else  /* BUILD_SSH */
+        ret = ENOSYS;
+#endif /* BUILD_SSH */
         break;
     }
 
@@ -393,15 +426,16 @@ static bool invalidate_entries(TALLOC_CTX *ctx,
         c_name = ldb_msg_find_attr_as_string(msgs[i], SYSDB_NAME, NULL);
         if (c_name == NULL) {
             DEBUG(SSSDBG_MINOR_FAILURE,
-                  "Something bad happened, can't find attribute %s", SYSDB_NAME);
-            ERROR("Couldn't invalidate %1$s", type_string);
+                  "Something bad happened, can't find attribute %s\n",
+                  SYSDB_NAME);
+            ERROR("Couldn't invalidate %1$s\n", type_string);
             iret = false;
         } else {
-            ret = invalidate_entry(ctx, sysdb, dinfo, c_name, entry_type);
+            ret = invalidate_entry(ctx, dinfo, c_name, entry_type);
             if (ret != EOK) {
                 DEBUG(SSSDBG_MINOR_FAILURE,
-                      "Couldn't invalidate %s %s", type_string, c_name);
-                ERROR("Couldn't invalidate %1$s %2$s", type_string, c_name);
+                      "Couldn't invalidate %s %s\n", type_string, c_name);
+                ERROR("Couldn't invalidate %1$s %2$s\n", type_string, c_name);
                 iret = false;
             }
         }
@@ -410,7 +444,7 @@ static bool invalidate_entries(TALLOC_CTX *ctx,
     return iret;
 }
 
-static errno_t invalidate_entry(TALLOC_CTX *ctx, struct sysdb_ctx *sysdb,
+static errno_t invalidate_entry(TALLOC_CTX *ctx,
                                 struct sss_domain_info *domain,
                                 const char *name, int entry_type)
 {
@@ -430,25 +464,33 @@ static errno_t invalidate_entry(TALLOC_CTX *ctx, struct sysdb_ctx *sysdb,
                             SYSDB_INITGR_EXPIRE, 1);
                     if (ret != EOK) return ret;
 
-                    ret = sysdb_set_user_attr(sysdb, domain, name, sys_attrs,
+                    ret = sysdb_set_user_attr(domain, name, sys_attrs,
                                               SYSDB_MOD_REP);
                     break;
                 case TYPE_GROUP:
-                    ret = sysdb_set_group_attr(sysdb, domain, name, sys_attrs,
+                    ret = sysdb_set_group_attr(domain, name, sys_attrs,
                                                SYSDB_MOD_REP);
                     break;
                 case TYPE_NETGROUP:
-                    ret = sysdb_set_netgroup_attr(sysdb, domain, name,
-                            sys_attrs, SYSDB_MOD_REP);
+                    ret = sysdb_set_netgroup_attr(domain, name, sys_attrs,
+                                                  SYSDB_MOD_REP);
                     break;
                 case TYPE_SERVICE:
-                    ret = sysdb_set_service_attr(sysdb, domain, name,
+                    ret = sysdb_set_service_attr(domain, name,
                                                  sys_attrs, SYSDB_MOD_REP);
                     break;
                 case TYPE_AUTOFSMAP:
-                    ret = sysdb_set_autofsmap_attr(sysdb, domain, name,
+                    ret = sysdb_set_autofsmap_attr(domain, name,
                                                    sys_attrs, SYSDB_MOD_REP);
                     break;
+                case TYPE_SSH_HOST:
+#ifdef BUILD_SSH
+                    ret = sysdb_set_ssh_host_attr(domain, name,
+                                                  sys_attrs, SYSDB_MOD_REP);
+#else  /* BUILD_SSH */
+                    ret = ENOSYS;
+#endif /* BUILD_SSH */
+                    break;
                 default:
                     return EINVAL;
             }
@@ -533,6 +575,7 @@ errno_t init_context(int argc, const char *argv[], struct cache_tool_ctx **tctx)
     char *netgroup = NULL;
     char *service = NULL;
     char *map = NULL;
+    char *ssh_host = NULL;
     char *domain = NULL;
     int debug = SSSDBG_DEFAULT;
     errno_t ret = EOK;
@@ -566,6 +609,12 @@ errno_t init_context(int argc, const char *argv[], struct cache_tool_ctx **tctx)
         { "autofs-maps", 'A', POPT_ARG_NONE, NULL, 'a',
             _("Invalidate all autofs maps"), NULL },
 #endif /* BUILD_AUTOFS */
+#ifdef BUILD_SSH
+        { "ssh-host", 'h', POPT_ARG_STRING, &ssh_host, 0,
+            _("Invalidate particular SSH host"), NULL },
+        { "ssh-hosts", 'H', POPT_ARG_NONE, NULL, 'h',
+            _("Invalidate all SSH hosts"), NULL },
+#endif /* BUILD_SSH */
         { "domain", 'd', POPT_ARG_STRING, &domain, 0,
             _("Only invalidate entries from a particular domain"), NULL },
         POPT_TABLEEND
@@ -597,13 +646,16 @@ errno_t init_context(int argc, const char *argv[], struct cache_tool_ctx **tctx)
             case 'a':
                 idb |= INVALIDATE_AUTOFSMAPS;
                 break;
+            case 'h':
+                idb |= INVALIDATE_SSH_HOSTS;
+                break;
             case 'e':
                 idb = INVALIDATE_EVERYTHING;
                 break;
         }
     }
 
-    DEBUG_INIT(debug);
+    DEBUG_CLI_INIT(debug);
     debug_prg_name = argv[0];
 
     if (ret != -1) {
@@ -611,7 +663,7 @@ errno_t init_context(int argc, const char *argv[], struct cache_tool_ctx **tctx)
     }
 
     if (idb == INVALIDATE_NONE && !user && !group &&
-        !netgroup && !service && !map) {
+        !netgroup && !service && !map && !ssh_host) {
         BAD_POPT_PARAMS(pc,
                 _("Please select at least one object to invalidate\n"),
                 ret, fini);
@@ -668,14 +720,26 @@ errno_t init_context(int argc, const char *argv[], struct cache_tool_ctx **tctx)
         ctx->update_autofs_filter = true;
     }
 
+    if (idb & INVALIDATE_SSH_HOSTS) {
+        ctx->ssh_host_filter = talloc_asprintf(ctx, "(%s=*)", SYSDB_NAME);
+        ctx->update_ssh_host_filter = false;
+    } else if (ssh_host) {
+        ctx->ssh_host_name = talloc_strdup(ctx, ssh_host);
+        ctx->update_ssh_host_filter = true;
+    }
+
     if (((idb & INVALIDATE_USERS) && !ctx->user_filter) ||
         ((idb & INVALIDATE_GROUPS) && !ctx->group_filter) ||
         ((idb & INVALIDATE_NETGROUPS) && !ctx->netgroup_filter) ||
         ((idb & INVALIDATE_SERVICES) && !ctx->service_filter) ||
         ((idb & INVALIDATE_AUTOFSMAPS) && !ctx->autofs_filter) ||
-         (user && !ctx->user_name) || (group && !ctx->group_name) ||
-         (netgroup && !ctx->netgroup_name) || (map && !ctx->autofs_name) ||
-         (service && !ctx->service_name)) {
+        ((idb & INVALIDATE_SSH_HOSTS) && !ctx->ssh_host_filter) ||
+         (user && !ctx->user_name) ||
+         (group && !ctx->group_name) ||
+         (netgroup && !ctx->netgroup_name) ||
+         (service && !ctx->service_name) ||
+         (map && !ctx->autofs_name) ||
+         (ssh_host && !ctx->ssh_host_name)) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Construction of filters failed\n");
         ret = ENOMEM;
         goto fini;
@@ -713,13 +777,13 @@ fini:
 }
 
 static errno_t
-search_autofsmaps(TALLOC_CTX *mem_ctx, struct sysdb_ctx *sysdb,
+search_autofsmaps(TALLOC_CTX *mem_ctx,
                   struct sss_domain_info *domain,
                   const char *sub_filter, const char **attrs,
                   size_t *msgs_count, struct ldb_message ***msgs)
 {
 #ifdef BUILD_AUTOFS
-    return sysdb_search_custom(mem_ctx, sysdb, domain, sub_filter,
+    return sysdb_search_custom(mem_ctx, domain, sub_filter,
                                AUTOFS_MAP_SUBDIR, attrs,
                                msgs_count, msgs);
 #else
