@@ -313,7 +313,6 @@ int group_show_recurse(TALLOC_CTX *mem_ctx,
                        struct group_info ***up_members);
 
 static int group_show_trim_memberof(TALLOC_CTX *mem_ctx,
-                                    struct sysdb_ctx *sysdb,
                                     struct sss_domain_info *domain,
                                     const char *name,
                                     const char **memberofs,
@@ -335,7 +334,7 @@ int group_show(TALLOC_CTX *mem_ctx,
     int i;
 
     /* First, search for the root group */
-    ret = sysdb_search_group_by_name(mem_ctx, sysdb, domain, name, attrs, &msg);
+    ret = sysdb_search_group_by_name(mem_ctx, domain, name, attrs, &msg);
     if (ret) {
         DEBUG(SSSDBG_OP_FAILURE,
               "Search failed: %s (%d)\n", strerror(ret), ret);
@@ -382,7 +381,7 @@ int group_show(TALLOC_CTX *mem_ctx,
         }
 
         /* if not recursive, only show the direct parent */
-        ret = group_show_trim_memberof(mem_ctx, sysdb, domain, root->name,
+        ret = group_show_trim_memberof(mem_ctx, domain, root->name,
                                        root->memberofs, &root->memberofs);
         goto done;
     }
@@ -412,7 +411,6 @@ done:
 /*=========Nonrecursive search should only show direct parent========== */
 
 static int group_show_trim_memberof(TALLOC_CTX *mem_ctx,
-                                    struct sysdb_ctx *sysdb,
                                     struct sss_domain_info *domain,
                                     const char *name,
                                     const char **memberofs,
@@ -427,7 +425,7 @@ static int group_show_trim_memberof(TALLOC_CTX *mem_ctx,
     int ret;
     int i;
 
-    dn = sysdb_group_dn(sysdb, mem_ctx, domain, name);
+    dn = sysdb_group_dn(mem_ctx, domain, name);
     if (!dn) {
         return ENOMEM;
     }
@@ -441,7 +439,7 @@ static int group_show_trim_memberof(TALLOC_CTX *mem_ctx,
             return ENOMEM;
         }
 
-        ret = sysdb_search_groups(mem_ctx, sysdb, domain,
+        ret = sysdb_search_groups(mem_ctx, domain,
                                   filter, NULL, &count, &msgs);
         /* ENOENT is OK, the group is just not a direct parent */
         if (ret != EOK && ret != ENOENT) {
@@ -510,8 +508,8 @@ int group_show_recurse(TALLOC_CTX *mem_ctx,
             continue;
         }
 
-        ret = sysdb_search_group_by_name(mem_ctx, sysdb, domain,
-                                         group_members[i], attrs, &msg);
+        ret = sysdb_search_group_by_name(mem_ctx, domain, group_members[i],
+                                         attrs, &msg);
         if (ret) {
             DEBUG(SSSDBG_OP_FAILURE,
                   "Search failed: %s (%d)\n", strerror(ret), ret);
@@ -549,7 +547,6 @@ int group_show_recurse(TALLOC_CTX *mem_ctx,
 /*==================Get info about MPG================================= */
 
 static int group_show_mpg(TALLOC_CTX *mem_ctx,
-                          struct sysdb_ctx *sysdb,
                           struct sss_domain_info *domain,
                           const char *name,
                           struct group_info **res)
@@ -565,7 +562,7 @@ static int group_show_mpg(TALLOC_CTX *mem_ctx,
         goto fail;
     }
 
-    ret = sysdb_search_user_by_name(info, sysdb, domain, name, attrs, &msg);
+    ret = sysdb_search_user_by_name(info, domain, name, attrs, &msg);
     if (ret) {
         DEBUG(SSSDBG_OP_FAILURE,
               "Search failed: %s (%d)\n", strerror(ret), ret);
@@ -683,7 +680,7 @@ int main(int argc, const char **argv)
         }
     }
 
-    DEBUG_INIT(pc_debug);
+    DEBUG_CLI_INIT(pc_debug);
 
     if (ret != -1) {
         BAD_POPT_PARAMS(pc, poptStrerror(ret), ret, fini);
@@ -722,8 +719,7 @@ int main(int argc, const char **argv)
                      tctx->local, pc_recursive, tctx->octx->name, &root);
     /* Also show MPGs */
     if (ret == ENOENT) {
-        ret = group_show_mpg(tctx, tctx->sysdb, tctx->local,
-                             tctx->octx->name, &root);
+        ret = group_show_mpg(tctx, tctx->local, tctx->octx->name, &root);
     }
 
     /* Process result */
