@@ -45,6 +45,11 @@ struct io_buffer {
     size_t size;
 };
 
+struct child_io_fds {
+    int read_from_child_fd;
+    int write_to_child_fd;
+};
+
 /* COMMON SIGCHLD HANDLING */
 typedef void (*sss_child_fn_t)(int pid, int wait_status, void *pvt);
 
@@ -78,9 +83,15 @@ typedef void (*sss_child_callback_t)(int child_status,
                                      struct tevent_signal *sige,
                                      void *pvt);
 
+struct sss_child_ctx_old;
+
 /* Set up child termination signal handler */
 int child_handler_setup(struct tevent_context *ev, int pid,
-                        sss_child_callback_t cb, void *pvt);
+                        sss_child_callback_t cb, void *pvt,
+                        struct sss_child_ctx_old **_child_ctx);
+
+/* Destroy child termination signal handler */
+void child_handler_destroy(struct sss_child_ctx_old *ctx);
 
 /* Async communication with the child process via a pipe */
 struct tevent_req *write_pipe_send(TALLOC_CTX *mem_ctx,
@@ -101,10 +112,23 @@ void child_sig_handler(struct tevent_context *ev,
                        int count, void *__siginfo, void *pvt);
 
 /* Never returns EOK, ether returns an error, or doesn't return on success */
+errno_t exec_child_ex(TALLOC_CTX *mem_ctx,
+                      int *pipefd_to_child, int *pipefd_from_child,
+                      const char *binary, int debug_fd,
+                      const char *extra_argv[],
+                      int child_in_fd, int child_out_fd);
+
+/* Same as exec_child_ex() except child_in_fd is set to STDIN_FILENO and
+ * child_out_fd is set to STDOUT_FILENO and extra_argv is always NULL.
+ */
 errno_t exec_child(TALLOC_CTX *mem_ctx,
                    int *pipefd_to_child, int *pipefd_from_child,
                    const char *binary, int debug_fd);
 
 void child_cleanup(int readfd, int writefd);
 
+int child_io_destructor(void *ptr);
+
+errno_t child_debug_init(const char *logfile, int *debug_fd);
+
 #endif /* __CHILD_COMMON_H__ */
