@@ -24,9 +24,6 @@
 #include "db/sysdb.h"
 #include "util/util.h"
 
-/* the directory domain - realm mappings are written to */
-#define KRB5_MAPPING_DIR PUBCONF_PATH"/krb5.include.d"
-
 struct sss_domain_info *get_domains_head(struct sss_domain_info *domain)
 {
     struct sss_domain_info *dom = NULL;
@@ -84,12 +81,16 @@ bool subdomain_enumerates(struct sss_domain_info *parent,
     return false;
 }
 
-struct sss_domain_info *find_subdomain_by_name(struct sss_domain_info *domain,
-                                               const char *name,
-                                               bool match_any)
+struct sss_domain_info *find_domain_by_name(struct sss_domain_info *domain,
+                                            const char *name,
+                                            bool match_any)
 {
     struct sss_domain_info *dom = domain;
 
+    if (name == NULL) {
+        return NULL;
+    }
+
     while (dom && dom->disabled) {
         dom = get_next_domain(dom, true);
     }
@@ -105,7 +106,7 @@ struct sss_domain_info *find_subdomain_by_name(struct sss_domain_info *domain,
     return NULL;
 }
 
-struct sss_domain_info *find_subdomain_by_sid(struct sss_domain_info *domain,
+struct sss_domain_info *find_domain_by_sid(struct sss_domain_info *domain,
                                               const char *sid)
 {
     struct sss_domain_info *dom = domain;
@@ -156,13 +157,13 @@ sss_get_domain_by_sid_ldap_fallback(struct sss_domain_info *domain,
     if (strcmp(domain->provider, "ldap") == 0) {
         return domain;
     } else {
-        return find_subdomain_by_sid(get_domains_head(domain), sid);
+        return find_domain_by_sid(get_domains_head(domain), sid);
     }
 }
 
 struct sss_domain_info *
-find_subdomain_by_object_name(struct sss_domain_info *domain,
-                              const char *object_name)
+find_domain_by_object_name(struct sss_domain_info *domain,
+                           const char *object_name)
 {
     TALLOC_CTX *tmp_ctx;
     struct sss_domain_info *dom = NULL;
@@ -186,7 +187,7 @@ find_subdomain_by_object_name(struct sss_domain_info *domain,
     if (domainname == NULL) {
         dom = domain;
     } else {
-        dom = find_subdomain_by_name(domain, domainname, true);
+        dom = find_domain_by_name(domain, domainname, true);
     }
 
 done:
@@ -205,6 +206,7 @@ struct sss_domain_info *new_subdomain(TALLOC_CTX *mem_ctx,
                                       const char *forest)
 {
     struct sss_domain_info *dom;
+    bool inherit_option;
 
     DEBUG(SSSDBG_TRACE_FUNC,
           "Creating [%s] as subdomain of [%s]!\n", name, parent->name);
@@ -216,6 +218,17 @@ struct sss_domain_info *new_subdomain(TALLOC_CTX *mem_ctx,
     }
 
     dom->parent = parent;
+
+    /* Sub-domains always have the same view as the parent */
+    dom->has_views = parent->has_views;
+    if (parent->view_name != NULL) {
+        dom->view_name = talloc_strdup(dom, parent->view_name);
+        if (dom->view_name == NULL) {
+            DEBUG(SSSDBG_OP_FAILURE, "Failed to copy parent's view name.\n");
+            goto fail;
+        }
+    }
+
     dom->name = talloc_strdup(dom, name);
     if (dom->name == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "Failed to copy domain name.\n");
@@ -269,6 +282,14 @@ struct sss_domain_info *new_subdomain(TALLOC_CTX *mem_ctx,
     dom->enumerate = enumerate;
     dom->fqnames = true;
     dom->mpg = mpg;
+    /* If the parent domain filters out group members, the subdomain should
+     * as well if configured */
+    inherit_option = string_in_list(CONFDB_DOMAIN_IGNORE_GROUP_MEMBERS,
+                                    parent->sd_inherit, false);
+    if (inherit_option) {
+        dom->ignore_group_members = parent->ignore_group_members;
+    }
+
     /* If the parent domain explicitly limits ID ranges, the subdomain
      * should honour the limits as well.
      */
@@ -360,7 +381,7 @@ sss_krb5_touch_config(void)
 }
 
 errno_t
-sss_write_domain_mappings(struct sss_domain_info *domain, bool add_capaths)
+sss_write_domain_mappings(struct sss_domain_info *domain)
 {
     struct sss_domain_info *dom;
     struct sss_domain_info *parent_dom;
@@ -374,7 +395,7 @@ sss_write_domain_mappings(struct sss_domain_info *domain, bool add_capaths)
     mode_t old_mode;
     FILE *fstream = NULL;
     int i;
-    bool capaths_started;
+    bool capaths_started = false;
     char *uc_forest;
     char *uc_parent;
 
@@ -421,8 +442,9 @@ sss_write_domain_mappings(struct sss_domain_info *domain, bool add_capaths)
     fd = mkstemp(tmp_file);
     umask(old_mode);
     if (fd < 0) {
-        DEBUG(SSSDBG_OP_FAILURE, "creating the temp file [%s] for domain-realm "
-                                  "mappings failed.", tmp_file);
+        DEBUG(SSSDBG_OP_FAILURE,
+              "creating the temp file [%s] for domain-realm mappings failed\n",
+              tmp_file);
         ret = EIO;
         talloc_zfree(tmp_ctx);
         goto done;
@@ -462,48 +484,45 @@ sss_write_domain_mappings(struct sss_domain_info *domain, bool add_capaths)
         }
     }
 
-    if (add_capaths) {
-        capaths_started = false;
-        parent_dom = domain;
-        uc_parent = get_uppercase_realm(tmp_ctx, parent_dom->name);
-        if (uc_parent == NULL) {
+    parent_dom = domain;
+    uc_parent = get_uppercase_realm(tmp_ctx, parent_dom->name);
+    if (uc_parent == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "get_uppercase_realm failed.\n");
+        ret = ENOMEM;
+        goto done;
+    }
+
+    for (dom = get_next_domain(domain, true);
+            dom && IS_SUBDOMAIN(dom); /* if we get back to a parent, stop */
+            dom = get_next_domain(dom, false)) {
+
+        if (dom->forest == NULL) {
+            continue;
+        }
+
+        uc_forest = get_uppercase_realm(tmp_ctx, dom->forest);
+        if (uc_forest == NULL) {
             DEBUG(SSSDBG_OP_FAILURE, "get_uppercase_realm failed.\n");
             ret = ENOMEM;
             goto done;
         }
 
-        for (dom = get_next_domain(domain, true);
-             dom && IS_SUBDOMAIN(dom); /* if we get back to a parent, stop */
-             dom = get_next_domain(dom, false)) {
-
-            if (dom->forest == NULL) {
-                continue;
-            }
-
-            uc_forest = get_uppercase_realm(tmp_ctx, dom->forest);
-            if (uc_forest == NULL) {
-                DEBUG(SSSDBG_OP_FAILURE, "get_uppercase_realm failed.\n");
-                ret = ENOMEM;
-                goto done;
-            }
-
-            if (!capaths_started) {
-                ret = fprintf(fstream, "[capaths]\n");
-                if (ret < 0) {
-                    DEBUG(SSSDBG_OP_FAILURE, "fprintf failed\n");
-                    ret = EIO;
-                    goto done;
-                }
-                capaths_started = true;
-            }
-
-            ret = fprintf(fstream, "%s = {\n  %s = %s\n}\n%s = {\n  %s = %s\n}\n",
-                                   dom->realm, uc_parent, uc_forest,
-                                   uc_parent, dom->realm, uc_forest);
+        if (!capaths_started) {
+            ret = fprintf(fstream, "[capaths]\n");
             if (ret < 0) {
-                DEBUG(SSSDBG_CRIT_FAILURE, "fprintf failed\n");
+                DEBUG(SSSDBG_OP_FAILURE, "fprintf failed\n");
+                ret = EIO;
                 goto done;
             }
+            capaths_started = true;
+        }
+
+        ret = fprintf(fstream, "%s = {\n  %s = %s\n}\n%s = {\n  %s = %s\n}\n",
+                                dom->realm, uc_parent, uc_forest,
+                                uc_parent, dom->realm, uc_forest);
+        if (ret < 0) {
+            DEBUG(SSSDBG_CRIT_FAILURE, "fprintf failed\n");
+            goto done;
         }
     }
 
@@ -558,10 +577,286 @@ done:
         if (err < 0) {
             err = errno;
             DEBUG(SSSDBG_MINOR_FAILURE,
-                  "Could not remove file [%s]: [%d]: %s",
-                   tmp_file, err, strerror(err));
+                  "Could not remove file [%s]: [%d]: %s\n",
+                  tmp_file, err, strerror(err));
         }
     }
     talloc_free(tmp_ctx);
     return ret;
 }
+
+/* Save domain names, do not descend. */
+errno_t get_dom_names(TALLOC_CTX *mem_ctx,
+                      struct sss_domain_info *start_dom,
+                      char ***_dom_names,
+                      int *_dom_names_count)
+{
+    struct sss_domain_info *dom;
+    TALLOC_CTX *tmp_ctx;
+    char **dom_names;
+    size_t count, i;
+    errno_t ret;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    /* get count of domains*/
+    count = 0;
+    dom = start_dom;
+    while (dom) {
+        count++;
+        dom = get_next_domain(dom, false);
+    }
+
+    dom_names = talloc_array(tmp_ctx, char*, count);
+    if (dom_names == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    /* copy names */
+    i = 0;
+    dom = start_dom;
+    while (dom) {
+        dom_names[i] = talloc_strdup(dom_names, dom->name);
+        if (dom_names[i] == NULL) {
+            ret = ENOMEM;
+            goto done;
+        }
+        dom = get_next_domain(dom, false);
+        i++;
+    }
+
+    if (_dom_names != NULL ) {
+        *_dom_names = talloc_steal(mem_ctx, dom_names);
+    }
+
+    if (_dom_names_count != NULL ) {
+        *_dom_names_count =  count;
+    }
+
+    ret = EOK;
+
+done:
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
+#define LOCALAUTH_PLUGIN_CONFIG \
+"[plugins]\n" \
+" localauth = {\n" \
+"  module = sssd:"APP_MODULES_PATH"/sssd_krb5_localauth_plugin.so\n" \
+"  enable_only = sssd\n" \
+" }"
+
+static errno_t sss_write_krb5_localauth_snippet(const char *path)
+{
+#ifdef HAVE_KRB5_LOCALAUTH_PLUGIN
+    int ret;
+    errno_t err;
+    TALLOC_CTX *tmp_ctx = NULL;
+    char *tmp_file = NULL;
+    const char *file_name;
+    int fd = -1;
+    mode_t old_mode;
+    ssize_t written;
+    size_t size;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "talloc_new failed.\n");
+        return ENOMEM;
+    }
+
+    file_name = talloc_asprintf(tmp_ctx, "%s/localauth_plugin", path);
+    if (file_name == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "talloc_asprintf failed.\n");
+        ret = ENOMEM;
+        goto done;
+    }
+
+    DEBUG(SSSDBG_FUNC_DATA, "File for localauth plugin configuration is [%s]\n",
+                             file_name);
+
+    tmp_file = talloc_asprintf(tmp_ctx, "%sXXXXXX", file_name);
+    if (tmp_file == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "talloc_asprintf failed.\n");
+        ret = ENOMEM;
+        goto done;
+    }
+
+    old_mode = umask(077);
+    fd = mkstemp(tmp_file);
+    umask(old_mode);
+    if (fd < 0) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "creating the temp file [%s] for domain-realm mappings failed\n",
+              tmp_file);
+        ret = EIO;
+        talloc_zfree(tmp_ctx);
+        goto done;
+    }
+
+    size = sizeof(LOCALAUTH_PLUGIN_CONFIG) -1;
+    written = sss_atomic_write_s(fd, discard_const(LOCALAUTH_PLUGIN_CONFIG),
+                                 size);
+    close(fd);
+    if (written == -1) {
+        ret = errno;
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "write failed [%d][%s]\n", ret, sss_strerror(ret));
+        goto done;
+    }
+
+    if (written != size) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Wrote %zd bytes expected %zu\n", written, size);
+        ret = EIO;
+        goto done;
+    }
+
+    ret = rename(tmp_file, file_name);
+    if (ret == -1) {
+        ret = errno;
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "rename failed [%d][%s].\n", ret, sss_strerror(ret));
+        goto done;
+    }
+    tmp_file = NULL;
+
+    ret = chmod(file_name, 0644);
+    if (ret == -1) {
+        ret = errno;
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "chmod failed [%d][%s].\n", ret, sss_strerror(ret));
+        goto done;
+    }
+
+done:
+    if (tmp_file != NULL) {
+        err = unlink(tmp_file);
+        if (err == -1) {
+            err = errno;
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  "Could not remove file [%s]: [%d]: %s\n",
+                  tmp_file, err, sss_strerror(err));
+        }
+    }
+
+    talloc_free(tmp_ctx);
+    return ret;
+#else
+    DEBUG(SSSDBG_TRACE_ALL, "Kerberos localauth plugin not available.\n");
+    return EOK;
+#endif
+}
+
+errno_t sss_write_krb5_conf_snippet(const char *path)
+{
+    errno_t ret;
+    errno_t err;
+
+    if (path != NULL && (*path == '\0' || strcasecmp(path, "none") == 0)) {
+        DEBUG(SSSDBG_TRACE_FUNC, "Empty path, nothing to do.\n");
+        return EOK;
+    }
+
+    if (path == NULL || *path != '/') {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Invalid or missing path [%s]-\n",
+                                    path == NULL ? "missing" : path);
+        return EINVAL;
+    }
+
+    ret = sss_write_krb5_localauth_snippet(path);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "sss_write_krb5_localauth_snippet failed.\n");
+        goto done;
+    }
+
+    ret = EOK;
+
+done:
+    err = sss_krb5_touch_config();
+    if (err != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Unable to change last modification time "
+              "of krb5.conf. Created mappings may not be loaded.\n");
+        /* Ignore */
+    }
+
+    return ret;
+}
+
+errno_t fix_domain_in_name_list(TALLOC_CTX *mem_ctx,
+                                struct sss_domain_info *dom,
+                                char **in, char ***_out)
+{
+    int ret;
+    size_t c;
+    TALLOC_CTX *tmp_ctx;
+    char **out;
+    struct sss_domain_info *head;
+    struct sss_domain_info *out_domain;
+    char *in_name;
+    char *in_domain;
+
+    head = get_domains_head(dom);
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "talloc_new failed.\n");
+        return ENOMEM;
+    }
+
+    /* count elements */
+    for (c = 0; in[c] != NULL; c++);
+
+    out = talloc_zero_array(tmp_ctx, char *, c + 1);
+    if (out == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "talloc_array failed.\n");
+        ret = ENOMEM;
+        goto done;
+    }
+
+    for (c = 0; in[c] != NULL; c++) {
+        ret = sss_parse_name(tmp_ctx, head->names, in[c], &in_domain,
+                              &in_name);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_OP_FAILURE, "sss_parse_name failed for [%s].\n",
+                                      in[c]);
+            goto done;
+        }
+
+        if (in_domain == NULL) {
+            out[c] = talloc_strdup(out, in_name);
+        } else {
+            out_domain = find_domain_by_name(head, in_domain, true);
+            if (out_domain == NULL) {
+                DEBUG(SSSDBG_CRIT_FAILURE,
+                      "Cannot find domain with name [%s].\n", in_domain);
+                ret = EINVAL;
+                goto done;
+            }
+
+            out[c] = sss_get_domain_name(out, in_name, out_domain);
+        }
+
+        if (out[c] == NULL) {
+            DEBUG(SSSDBG_OP_FAILURE, "%s failed.\n",
+                  in_domain == NULL ? "talloc_strdup" : "sss_tc_fqname");
+            ret = ENOMEM;
+            goto done;
+        }
+    }
+
+    *_out = talloc_steal(mem_ctx, out);
+
+    ret = EOK;
+
+done:
+    talloc_free(tmp_ctx);
+
+    return ret;
+}
