@@ -28,8 +28,8 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <ldb.h>
 #include "util/util.h"
-#include "ldb.h"
 #include "confdb/confdb.h"
 #include "monitor/monitor_interfaces.h"
 
@@ -194,8 +194,14 @@ int pidfile(const char *path, const char *name)
 
         /* nothing in the file or no process */
         close(fd);
-        unlink(file);
-
+        ret = unlink(file);
+        /* non-fatal failure */
+        if (ret != EOK) {
+            ret = errno;
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  "Failed to remove file: %s - %d [%s]!\n",
+                  file, ret, sss_strerror(ret));
+        }
     } else {
         if (err != ENOENT) {
             talloc_free(file);
@@ -237,24 +243,18 @@ int pidfile(const char *path, const char *name)
     return 0;
 }
 
-static void sig_hup(int sig)
-{
-    /* cycle log/debug files */
-    return;
-}
-
-void sig_term(int sig)
+void orderly_shutdown(int status)
 {
 #if HAVE_GETPGRP
-    static int done_sigterm;
-    if (done_sigterm == 0 && getpgrp() == getpid()) {
+    static int sent_sigterm;
+    if (sent_sigterm == 0 && getpgrp() == getpid()) {
         DEBUG(SSSDBG_FATAL_FAILURE, "SIGTERM: killing children\n");
-        done_sigterm = 1;
+        sent_sigterm = 1;
         kill(-getpgrp(), SIGTERM);
     }
 #endif
-    sss_log(SSS_LOG_INFO, "Shutting down");
-    exit(0);
+    if (status == 0) sss_log(SSS_LOG_INFO, "Shutting down");
+    exit(status);
 }
 
 static void default_quit(struct tevent_context *ev,
@@ -264,31 +264,15 @@ static void default_quit(struct tevent_context *ev,
                          void *siginfo,
                          void *private_data)
 {
-#if HAVE_GETPGRP
-    static int done_sigterm;
-    if (done_sigterm == 0 && getpgrp() == getpid()) {
-        DEBUG(SSSDBG_FATAL_FAILURE, "SIGTERM: killing children\n");
-        done_sigterm = 1;
-        kill(-getpgrp(), SIGTERM);
-    }
-#endif
-    sss_log(SSS_LOG_INFO, "Shutting down");
-    exit(0);
+    orderly_shutdown(0);
 }
 
 #ifndef HAVE_PRCTL
 static void sig_segv_abrt(int sig)
 {
-#if HAVE_GETPGRP
-    static int done;
-    if (done == 0 && getpgrp() == getpid()) {
-        DEBUG(SSSDBG_FATAL_FAILURE, "%s: killing children\n",
-                                     strsignal(sig));
-        done = 1;
-        kill(-getpgrp(), SIGTERM);
-    }
-#endif  /* HAVE_GETPGRP */
-    exit(1);
+    DEBUG(SSSDBG_FATAL_FAILURE,
+          "Received signal %s, shutting down\n", strsignal(sig));
+    orderly_shutdown(1);
 }
 #endif /* HAVE_PRCTL */
 
@@ -321,8 +305,6 @@ static void setup_signals(void)
     BlockSignals(false, SIGHUP);
     BlockSignals(false, SIGTERM);
 
-    CatchSignal(SIGHUP, sig_hup);
-
 #ifndef HAVE_PRCTL
         /* If prctl is not defined on the system, try to handle
          * some common termination signals gracefully */
@@ -368,8 +350,8 @@ int die_if_parent_died(void)
     ret = prctl(PR_SET_PDEATHSIG, SIGTERM, 0, 0, 0);
     if (ret != 0) {
         ret = errno;
-        DEBUG(SSSDBG_OP_FAILURE, "prctl failed [%d]: %s",
-                                  ret, strerror(ret));
+        DEBUG(SSSDBG_OP_FAILURE, "prctl failed [%d]: %s\n",
+                                 ret, strerror(ret));
         return ret;
     }
 #endif
@@ -394,14 +376,75 @@ static void te_server_hup(struct tevent_context *ev,
 
     DEBUG(SSSDBG_CRIT_FAILURE, "Received SIGHUP. Rotating logfiles.\n");
 
-    ret = monitor_common_rotate_logs(lctx->confdb, lctx->confdb_path);
+    ret = server_common_rotate_logs(lctx->confdb, lctx->confdb_path);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE, "Could not reopen log file [%s]\n",
                                      strerror(ret));
     }
 }
 
+errno_t server_common_rotate_logs(struct confdb_ctx *confdb,
+                                  const char *conf_path)
+{
+    errno_t ret;
+    int old_debug_level = debug_level;
+
+    ret = rotate_debug_files();
+    if (ret) {
+        sss_log(SSS_LOG_ALERT, "Could not rotate debug files! [%d][%s]\n",
+                               ret, strerror(ret));
+        return ret;
+    }
+
+    /* Get new debug level from the confdb */
+    ret = confdb_get_int(confdb, conf_path,
+                         CONFDB_SERVICE_DEBUG_LEVEL,
+                         old_debug_level,
+                         &debug_level);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Error reading from confdb (%d) [%s]\n",
+                  ret, strerror(ret));
+        /* Try to proceed with the old value */
+        debug_level = old_debug_level;
+    }
+
+    if (debug_level != old_debug_level) {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              "Debug level changed to %#.4x\n", debug_level);
+        debug_level = debug_convert_old_level(debug_level);
+    }
+
+    return EOK;
+}
+
+static const char *get_db_path(void)
+{
+#ifdef UNIT_TESTING
+#ifdef TEST_DB_PATH
+    return TEST_DB_PATH;
+#else
+    #error "TEST_DB_PATH must be defined when unit testing server.c!"
+#endif /* TEST_DB_PATH */
+#else
+    return DB_PATH;
+#endif /* UNIT_TESTING */
+}
+
+static const char *get_pid_path(void)
+{
+#ifdef UNIT_TESTING
+#ifdef TEST_PID_PATH
+    return TEST_PID_PATH;
+#else
+    #error "TEST_PID_PATH must be defined when unit testing server.c!"
+#endif /* TEST_PID_PATH */
+#else
+    return PID_PATH;
+#endif
+}
+
 int server_setup(const char *name, int flags,
+                 uid_t uid, gid_t gid,
                  const char *conf_entry,
                  struct main_context **main_ctx)
 {
@@ -416,6 +459,19 @@ int server_setup(const char *name, int flags,
     struct tevent_signal *tes;
     struct logrotate_ctx *lctx;
 
+    ret = chown_debug_file(NULL, uid, gid);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "Cannot chown the debug files, debugging might not work!\n");
+    }
+
+    ret = become_user(uid, gid);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FUNC_DATA,
+              "Cannot become user [%"SPRIuid"][%"SPRIgid"].\n", uid, gid);
+        return ret;
+    }
+
     debug_prg_name = strdup(name);
     if (!debug_prg_name) {
         return ENOMEM;
@@ -423,6 +479,15 @@ int server_setup(const char *name, int flags,
 
     setenv("_SSS_LOOPS", "NO", 0);
 
+    /* To make sure the domain cannot be set from the environment, unset the
+     * variable explicitly when setting up any server. Backends later set the
+     * value after reading domain from the configuration */
+    ret = unsetenv(SSS_DOM_ENV);
+    if (ret != 0) {
+        DEBUG(SSSDBG_MINOR_FAILURE, "Unsetting "SSS_DOM_ENV" failed, journald "
+              "logging mightnot work as expected\n");
+    }
+
     setup_signals();
 
     /* we want default permissions on created files to be very strict,
@@ -435,10 +500,10 @@ int server_setup(const char *name, int flags,
     }
 
     if (flags & FLAGS_PID_FILE) {
-        ret = pidfile(PID_PATH, name);
+        ret = pidfile(get_pid_path(), name);
         if (ret != EOK) {
-            DEBUG(SSSDBG_FATAL_FAILURE, "Error creating pidfile: %s/%s! "
-                  "(%d [%s])\n", PID_PATH, name, ret, strerror(ret));
+            DEBUG(SSSDBG_FATAL_FAILURE, "Error creating pidfile: %s/%s.pid! "
+                  "(%d [%s])\n", get_pid_path(), name, ret, strerror(ret));
             return ret;
         }
     }
@@ -480,7 +545,8 @@ int server_setup(const char *name, int flags,
     ctx->parent_pid = getppid();
     ctx->event_ctx = event_ctx;
 
-    conf_db = talloc_asprintf(ctx, "%s/%s", DB_PATH, CONFDB_FILE);
+    conf_db = talloc_asprintf(ctx, "%s/%s",
+                              get_db_path(), CONFDB_FILE);
     if (conf_db == NULL) {
         DEBUG(SSSDBG_FATAL_FAILURE, "Out of memory, aborting!\n");
         return ENOMEM;
