@@ -26,8 +26,10 @@
 #include "util/util.h"
 #include "util/sss_krb5.h"
 
-static char *
-get_primary(TALLOC_CTX *mem_ctx, const char *pattern, const char *hostname)
+char *
+sss_krb5_get_primary(TALLOC_CTX *mem_ctx,
+                     const char *pattern,
+                     const char *hostname)
 {
     char *primary;
     char *dot;
@@ -132,7 +134,9 @@ errno_t select_principal_from_keytab(TALLOC_CTX *mem_ctx,
 
     do {
         if (primary_patterns[i]) {
-            primary = get_primary(tmp_ctx, primary_patterns[i], hostname);
+            primary = sss_krb5_get_primary(tmp_ctx,
+                                           primary_patterns[i],
+                                           hostname);
             if (primary == NULL) {
                 ret = ENOMEM;
                 goto done;
@@ -168,7 +172,7 @@ errno_t select_principal_from_keytab(TALLOC_CTX *mem_ctx,
         if (_principal) {
             kerr = krb5_unparse_name(krb_ctx, client_princ, &principal_string);
             if (kerr) {
-                DEBUG(SSSDBG_CRIT_FAILURE, "krb5_unparse_name failed");
+                DEBUG(SSSDBG_CRIT_FAILURE, "krb5_unparse_name failed\n");
                 ret = EFAULT;
                 goto done;
             }
@@ -176,7 +180,7 @@ errno_t select_principal_from_keytab(TALLOC_CTX *mem_ctx,
             *_principal = talloc_strdup(mem_ctx, principal_string);
             free(principal_string);
             if (!*_principal) {
-                DEBUG(SSSDBG_CRIT_FAILURE, "talloc_strdup failed");
+                DEBUG(SSSDBG_CRIT_FAILURE, "talloc_strdup failed\n");
                 ret = ENOMEM;
                 goto done;
             }
@@ -188,7 +192,7 @@ errno_t select_principal_from_keytab(TALLOC_CTX *mem_ctx,
                                                KRB5_PRINCIPAL_UNPARSE_NO_REALM,
                                                &principal_string);
             if (kerr) {
-                DEBUG(SSSDBG_CRIT_FAILURE, "krb5_unparse_name failed");
+                DEBUG(SSSDBG_CRIT_FAILURE, "krb5_unparse_name failed\n");
                 ret = EFAULT;
                 goto done;
             }
@@ -196,7 +200,7 @@ errno_t select_principal_from_keytab(TALLOC_CTX *mem_ctx,
             *_primary = talloc_strdup(mem_ctx, principal_string);
             free(principal_string);
             if (!*_primary) {
-                DEBUG(SSSDBG_CRIT_FAILURE, "talloc_strdup failed");
+                DEBUG(SSSDBG_CRIT_FAILURE, "talloc_strdup failed\n");
                 if (_principal) talloc_zfree(*_principal);
                 ret = ENOMEM;
                 goto done;
@@ -208,10 +212,18 @@ errno_t select_principal_from_keytab(TALLOC_CTX *mem_ctx,
             sss_krb5_princ_realm(krb_ctx, client_princ,
                                  &realm_name,
                                  &realm_len);
+            if (realm_len == 0) {
+                DEBUG(SSSDBG_CRIT_FAILURE, "sss_krb5_princ_realm failed.\n");
+                if (_principal) talloc_zfree(*_principal);
+                if (_primary) talloc_zfree(*_primary);
+                ret = EINVAL;
+                goto done;
+            }
+
             *_realm = talloc_asprintf(mem_ctx, "%.*s",
                                       realm_len, realm_name);
             if (!*_realm) {
-                DEBUG(SSSDBG_CRIT_FAILURE, "talloc_asprintf failed");
+                DEBUG(SSSDBG_CRIT_FAILURE, "talloc_asprintf failed\n");
                 if (_principal) talloc_zfree(*_principal);
                 if (_primary) talloc_zfree(*_primary);
                 ret = ENOMEM;
@@ -227,6 +239,12 @@ errno_t select_principal_from_keytab(TALLOC_CTX *mem_ctx,
     }
 
 done:
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Failed to read keytab [%s]: %s\n",
+               KEYTAB_CLEAN_NAME, strerror(ret));
+        sss_log(SSS_LOG_ERR, "Failed to read keytab [%s]: %s\n",
+                KEYTAB_CLEAN_NAME, strerror(ret));
+    }
     if (keytab) krb5_kt_close(krb_ctx, keytab);
     if (krb_ctx) krb5_free_context(krb_ctx);
     if (client_princ != NULL) {
@@ -237,82 +255,6 @@ done:
     return ret;
 }
 
-int sss_krb5_verify_keytab_ex(const char *principal, const char *keytab_name,
-                              krb5_context context, krb5_keytab keytab)
-{
-    bool found;
-    char *kt_principal;
-    krb5_error_code krberr;
-    krb5_kt_cursor cursor;
-    krb5_keytab_entry entry;
-
-    krberr = krb5_kt_start_seq_get(context, keytab, &cursor);
-    if (krberr) {
-        DEBUG(SSSDBG_FATAL_FAILURE,
-              "Cannot read keytab [%s].\n", KEYTAB_CLEAN_NAME);
-
-        sss_log(SSS_LOG_ERR, "Error reading keytab file [%s]: [%d][%s]. "
-                             "Unable to create GSSAPI-encrypted LDAP "
-                             "connection.",
-                             KEYTAB_CLEAN_NAME, krberr,
-                             sss_krb5_get_error_message(context, krberr));
-
-        return EIO;
-    }
-
-    found = false;
-    while((krb5_kt_next_entry(context, keytab, &entry, &cursor)) == 0){
-        krberr = krb5_unparse_name(context, entry.principal, &kt_principal);
-        if (krberr) {
-            DEBUG(SSSDBG_FATAL_FAILURE,
-                  "Could not parse keytab entry\n");
-            sss_log(SSS_LOG_ERR, "Could not parse keytab entry\n");
-            return EIO;
-        }
-
-        if (strcmp(principal, kt_principal) == 0) {
-            found = true;
-        }
-        free(kt_principal);
-        krberr = sss_krb5_free_keytab_entry_contents(context, &entry);
-        if (krberr) {
-            /* This should never happen. The API docs for this function
-             * specify only success for this function
-             */
-            DEBUG(SSSDBG_CRIT_FAILURE,"Could not free keytab entry contents\n");
-            /* This is non-fatal, so we'll continue here */
-        }
-
-        if (found) {
-            break;
-        }
-    }
-
-    krberr = krb5_kt_end_seq_get(context, keytab, &cursor);
-    if (krberr) {
-        DEBUG(SSSDBG_FATAL_FAILURE, "Could not close keytab.\n");
-        sss_log(SSS_LOG_ERR, "Could not close keytab file [%s].",
-                             KEYTAB_CLEAN_NAME);
-        return EIO;
-    }
-
-    if (!found) {
-        DEBUG(SSSDBG_FATAL_FAILURE,
-              "Principal [%s] not found in keytab [%s]\n",
-               principal,
-               KEYTAB_CLEAN_NAME);
-        sss_log(SSS_LOG_ERR, "Error processing keytab file [%s]: "
-                             "Principal [%s] was not found. "
-                             "Unable to create GSSAPI-encrypted LDAP connection.",
-                             KEYTAB_CLEAN_NAME, principal);
-
-        return EFAULT;
-    }
-
-    return EOK;
-}
-
-
 enum matching_mode {MODE_NORMAL, MODE_PREFIX, MODE_POSTFIX};
 /**
  * We only have primary and instances stored separately, we need to
@@ -345,6 +287,10 @@ static bool match_principal(krb5_context ctx,
     bool ret = false;
 
     sss_krb5_princ_realm(ctx, principal, &realm_name, &realm_len);
+    if (realm_len == 0) {
+        DEBUG(SSSDBG_MINOR_FAILURE, "sss_krb5_princ_realm failed.\n");
+        return false;
+    }
 
     tmp_ctx = talloc_new(NULL);
     if (!tmp_ctx) {
@@ -579,7 +525,7 @@ errno_t check_fast(const char *str, bool *use_fast)
 
     return EOK;
 #else
-    sss_log(SSS_LOG_ALERT, "This build of sssd done not support FAST. "
+    sss_log(SSS_LOG_ALERT, "This build of sssd does not support FAST. "
                            "Please remove option krb5_use_fast.\n");
     return EINVAL;
 #endif
@@ -824,8 +770,15 @@ void sss_krb5_get_init_creds_opt_set_canonicalize(krb5_get_init_creds_opt *opts,
 void sss_krb5_princ_realm(krb5_context context, krb5_const_principal princ,
                           const char **realm, int *len)
 {
-    *realm = krb5_principal_get_realm(context, princ);
-    *len = strlen(*realm);
+    const char *realm_str = krb5_principal_get_realm(context, princ);
+
+    if (realm_str != NULL) {
+        *realm = realm_str;
+        *len = strlen(realm_str);
+    } else {
+        *realm = NULL;
+        *len = 0;
+    }
 }
 #else
 void sss_krb5_princ_realm(krb5_context context, krb5_const_principal princ,
@@ -1076,3 +1029,43 @@ done:
     return NULL;
 #endif /* HAVE_KRB5_CC_COLLECTION */
 }
+
+krb5_error_code sss_krb5_kt_have_content(krb5_context context,
+                                         krb5_keytab keytab)
+{
+#ifdef HAVE_KRB5_KT_HAVE_CONTENT
+    return krb5_kt_have_content(context, keytab);
+#else
+    krb5_keytab_entry entry;
+    krb5_kt_cursor cursor;
+    krb5_error_code kerr;
+    krb5_error_code kerr_end;
+
+    kerr = krb5_kt_start_seq_get(context, keytab, &cursor);
+    if (kerr != 0) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "krb5_kt_start_seq_get failed, assuming no entries.\n");
+        return KRB5_KT_NOTFOUND;
+    }
+
+    kerr = krb5_kt_next_entry(context, keytab, &entry, &cursor);
+    kerr_end = krb5_kt_end_seq_get(context, keytab, &cursor);
+    if (kerr != 0) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "krb5_kt_next_entry failed, assuming no entries.\n");
+        return KRB5_KT_NOTFOUND;
+    }
+    kerr = krb5_free_keytab_entry_contents(context, &entry);
+
+    if (kerr_end != 0) {
+        DEBUG(SSSDBG_TRACE_FUNC,
+              "krb5_kt_end_seq_get failed, ignored.\n");
+    }
+    if (kerr != 0) {
+        DEBUG(SSSDBG_TRACE_FUNC,
+              "krb5_free_keytab_entry_contents failed, ignored.\n");
+    }
+
+    return 0;
+#endif
+}
