@@ -23,9 +23,14 @@
 #include <pcre.h>
 #include <errno.h>
 #include <talloc.h>
+#include <pwd.h>
+#include <grp.h>
 
 #include "confdb/confdb.h"
+#include "util/strtonum.h"
 #include "util/util.h"
+#include "util/safe-format-string.h"
+#include "responder/common/responder.h"
 
 #ifdef HAVE_LIBPCRE_LESSER_THAN_7
 #define NAME_DOMAIN_PATTERN_OPTIONS (PCRE_EXTENDED)
@@ -33,18 +38,6 @@
 #define NAME_DOMAIN_PATTERN_OPTIONS (PCRE_DUPNAMES | PCRE_EXTENDED)
 #endif
 
-char *get_username_from_uid(TALLOC_CTX *mem_ctx, uid_t uid)
-{
-    char *username;
-    struct passwd *pwd;
-
-    pwd = getpwuid(uid);
-    if (!pwd) return NULL;
-
-    username = talloc_strdup(mem_ctx, pwd->pw_name);
-    return username;
-}
-
 /* Function returns given realm name as new uppercase string */
 char *get_uppercase_realm(TALLOC_CTX *memctx, const char *name)
 {
@@ -137,18 +130,7 @@ done:
 
 static errno_t sss_fqnames_init(struct sss_names_ctx *nctx, const char *fq_fmt)
 {
-    struct pattern_desc {
-        const char *pattern;
-        const char *desc;
-        int flag;
-    };
-
-    struct pattern_desc fqname_patterns[] = {
-        { "%1$s", "user name", FQ_FMT_NAME },
-        { "%2$s", "domain name", FQ_FMT_DOMAIN },
-        { "%3$s", "domain flat name", FQ_FMT_FLAT_NAME },
-        { NULL, NULL, 0 }
-    };
+    char *fq;
 
     nctx->fq_fmt = talloc_strdup(nctx, fq_fmt);
     if (nctx->fq_fmt == NULL) {
@@ -157,34 +139,20 @@ static errno_t sss_fqnames_init(struct sss_names_ctx *nctx, const char *fq_fmt)
 
     DEBUG(SSSDBG_CONF_SETTINGS, "Using fq format [%s].\n", nctx->fq_fmt);
 
-    /* Fail if the name specifier is missing and warn if the domain
-     * specifier is missing
-     */
-    if (strstr(fq_fmt, fqname_patterns[0].pattern) == NULL) {
+    /* Fail if the name specifier is missing, or if the format is
+     * invalid */
+    fq = sss_tc_fqname2 (nctx, nctx, "unused.example.com", "unused", "the-test-user");
+    if (fq == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "The fq format is invalid [%s]\n", nctx->fq_fmt);
+        return EINVAL;
+    } else if (strstr (fq, "the-test-user") == NULL) {
         DEBUG(SSSDBG_OP_FAILURE,
               "Username pattern not found in [%s]\n", nctx->fq_fmt);
         return ENOENT;
     }
-    nctx->fq_flags = FQ_FMT_NAME;
-
-    for (int i = 1; fqname_patterns[i].pattern; i++) {
-        char *s;
-        s = strstr(fq_fmt, fqname_patterns[i].pattern);
-        if (s == NULL) {
-            /* Append the format specifier */
-            nctx->fq_fmt = talloc_strdup_append(nctx->fq_fmt,
-                                                fqname_patterns[i].pattern);
-            if (nctx->fq_fmt == NULL) {
-                return ENOMEM;
-            }
-            continue;
-        }
-
-        DEBUG(SSSDBG_CONF_SETTINGS,
-              "Found the pattern for %s\n", fqname_patterns[i].desc);
-        nctx->fq_flags |= fqname_patterns[i].flag;
-    }
 
+    talloc_free (fq);
     return EOK;
 }
 
@@ -241,9 +209,9 @@ int sss_names_init(TALLOC_CTX *mem_ctx, struct confdb_ctx *cdb,
                    const char *domain, struct sss_names_ctx **out)
 {
     TALLOC_CTX *tmpctx = NULL;
-    char *conf_path;
-    char *re_pattern;
-    char *fq_fmt;
+    char *conf_path = NULL;
+    char *re_pattern = NULL;;
+    char *fq_fmt = NULL;
     int ret;
 
     tmpctx = talloc_new(NULL);
@@ -252,15 +220,17 @@ int sss_names_init(TALLOC_CTX *mem_ctx, struct confdb_ctx *cdb,
         goto done;
     }
 
-    conf_path = talloc_asprintf(tmpctx, CONFDB_DOMAIN_PATH_TMPL, domain);
-    if (conf_path == NULL) {
-        ret = ENOMEM;
-        goto done;
-    }
+    if (domain != NULL) {
+        conf_path = talloc_asprintf(tmpctx, CONFDB_DOMAIN_PATH_TMPL, domain);
+        if (conf_path == NULL) {
+            ret = ENOMEM;
+            goto done;
+        }
 
-    ret = confdb_get_string(cdb, tmpctx, conf_path,
-                            CONFDB_NAME_REGEX, NULL, &re_pattern);
-    if (ret != EOK) goto done;
+        ret = confdb_get_string(cdb, tmpctx, conf_path,
+                                CONFDB_NAME_REGEX, NULL, &re_pattern);
+        if (ret != EOK) goto done;
+    }
 
     /* If not found in the domain, look in globals */
     if (re_pattern == NULL) {
@@ -269,7 +239,7 @@ int sss_names_init(TALLOC_CTX *mem_ctx, struct confdb_ctx *cdb,
         if (ret != EOK) goto done;
     }
 
-    if (re_pattern == NULL) {
+    if (re_pattern == NULL && conf_path != NULL) {
         ret = get_id_provider_default_re(tmpctx, cdb, conf_path, &re_pattern);
         if (ret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE, "Failed to get provider default regular " \
@@ -297,9 +267,11 @@ int sss_names_init(TALLOC_CTX *mem_ctx, struct confdb_ctx *cdb,
 #endif
     }
 
-    ret = confdb_get_string(cdb, tmpctx, conf_path,
-                            CONFDB_FULL_NAME_FORMAT, NULL, &fq_fmt);
-    if (ret != EOK) goto done;
+    if (conf_path != NULL) {
+        ret = confdb_get_string(cdb, tmpctx, conf_path,
+                                CONFDB_FULL_NAME_FORMAT, NULL, &fq_fmt);
+        if (ret != EOK) goto done;
+    }
 
     /* If not found in the domain, look in globals */
     if (fq_fmt == NULL) {
@@ -337,7 +309,7 @@ int sss_parse_name(TALLOC_CTX *memctx,
 
     ret = pcre_exec(re, NULL, orig, origlen, 0, PCRE_NOTEMPTY, ovec, 30);
     if (ret == PCRE_ERROR_NOMATCH) {
-        return EINVAL;
+        return ERR_REGEX_NOMATCH;
     } else if (ret < 0) {
         DEBUG(SSSDBG_MINOR_FAILURE, "PCRE Matching error, %d\n", ret);
         return EINVAL;
@@ -418,7 +390,7 @@ static struct sss_domain_info * match_any_domain_or_subdomain_name(
         return dom;
     }
 
-    return find_subdomain_by_name(dom, dmatch, true);
+    return find_domain_by_name(dom, dmatch, true);
 }
 
 int sss_parse_name_for_domains(TALLOC_CTX *memctx,
@@ -436,8 +408,9 @@ int sss_parse_name_for_domains(TALLOC_CTX *memctx,
     int ret;
 
     tmp_ctx = talloc_new(NULL);
-    if (tmp_ctx == NULL)
+    if (tmp_ctx == NULL) {
         return ENOMEM;
+    }
 
     rname = NULL;
     rdomain = NULL;
@@ -476,7 +449,7 @@ int sss_parse_name_for_domains(TALLOC_CTX *memctx,
                     }
                     rname = nmatch;
                     break;
-                } else if (candidate_name == NULL) {
+                } else if (candidate_domain == NULL) {
                     candidate_domain = dmatch;
                 }
             }
@@ -508,18 +481,22 @@ int sss_parse_name_for_domains(TALLOC_CTX *memctx,
                 }
                 if (match == NULL) {
                     DEBUG(SSSDBG_FUNC_DATA, "default domain [%s] is currently " \
-                                             "not know, trying to look it up.\n",
-                                             rdomain);
+                                            "not known, trying to look it up.\n",
+                                            rdomain);
                     *domain = talloc_steal(memctx, rdomain);
                     ret = EAGAIN;
                     goto done;
                 }
+                DEBUG(SSSDBG_FUNC_DATA, "using default domain [%s]\n", rdomain);
             }
 
-            DEBUG(SSSDBG_FUNC_DATA, "using default domain [%s]\n", rdomain);
-
             rname = candidate_name;
         } else if (candidate_domain) {
+            /* This branch is taken when the input matches the configured
+             * regular expression, but the domain is now known. Normally, this
+             * is the case with a FQDN of a user from subdomain that was not
+             * yet discovered
+             */
             *domain = talloc_steal(memctx, candidate_domain);
             ret = EAGAIN;
             goto done;
@@ -595,19 +572,13 @@ sss_get_cased_name_list(TALLOC_CTX *mem_ctx, const char * const *orig,
 }
 
 static inline const char *
-safe_fq_str(struct sss_names_ctx *nctx, uint8_t part, const char *str)
-{
-    return nctx->fq_flags & part ? str : "";
-}
-
-static inline const char *
-safe_flat_name(struct sss_names_ctx *nctx, struct sss_domain_info *domain)
+calc_flat_name(struct sss_domain_info *domain)
 {
     const char *s;
 
-    s = safe_fq_str(nctx, FQ_FMT_FLAT_NAME, domain->flat_name);
+    s = domain->flat_name;
     if (s == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Flat name requested but domain has no"
+        DEBUG(SSSDBG_MINOR_FAILURE, "Flat name requested but domain has no"
               "flat name set, falling back to domain name\n");
         s = domain->name;
     }
@@ -615,28 +586,42 @@ safe_flat_name(struct sss_names_ctx *nctx, struct sss_domain_info *domain)
     return s;
 }
 
-static inline size_t
-fq_part_len(struct sss_names_ctx *nctx, struct sss_domain_info *dom,
-            uint8_t part, const char *str)
-{
-    const char *s = str;
-
-    if (part == FQ_FMT_FLAT_NAME) {
-        s = safe_flat_name(nctx, dom);
-    }
-    return nctx->fq_flags & part ? strlen(s) : 0;
-}
-
 char *
 sss_tc_fqname(TALLOC_CTX *mem_ctx, struct sss_names_ctx *nctx,
               struct sss_domain_info *domain, const char *name)
 {
     if (domain == NULL || nctx == NULL) return NULL;
 
-    return talloc_asprintf(mem_ctx, nctx->fq_fmt,
-                           safe_fq_str(nctx, FQ_FMT_NAME, name),
-                           safe_fq_str(nctx, FQ_FMT_DOMAIN, domain->name),
-                           safe_flat_name(nctx, domain));
+    return sss_tc_fqname2 (mem_ctx, nctx, domain->name,
+                           calc_flat_name (domain), name);
+}
+
+static void
+safe_talloc_callback (void *data,
+                      const char *piece,
+                      size_t len)
+{
+    char **output = data;
+    if (*output != NULL)
+        *output = talloc_strndup_append(*output, piece, len);
+}
+
+char *
+sss_tc_fqname2(TALLOC_CTX *mem_ctx, struct sss_names_ctx *nctx,
+               const char *domain_name, const char *flat_dom_name,
+               const char *name)
+{
+    const char *args[] = { name, domain_name, flat_dom_name, NULL };
+    char *output;
+
+    if (nctx == NULL) return NULL;
+
+    output = talloc_strdup(mem_ctx, "");
+    if (safe_format_string_cb(safe_talloc_callback, &output, nctx->fq_fmt, args, 3) < 0)
+        output = NULL;
+    else if (output == NULL)
+        errno = ENOMEM;
+    return output;
 }
 
 int
@@ -645,19 +630,8 @@ sss_fqname(char *str, size_t size, struct sss_names_ctx *nctx,
 {
     if (domain == NULL || nctx == NULL) return -EINVAL;
 
-    return snprintf(str, size, nctx->fq_fmt,
-                    safe_fq_str(nctx, FQ_FMT_NAME, name),
-                    safe_fq_str(nctx, FQ_FMT_DOMAIN, domain->name),
-                    safe_flat_name(nctx, domain));
-}
-
-size_t
-sss_fqdom_len(struct sss_names_ctx *nctx,
-              struct sss_domain_info *domain)
-{
-    size_t len = fq_part_len(nctx, domain, FQ_FMT_DOMAIN, domain->name);
-    len += fq_part_len(nctx, domain, FQ_FMT_FLAT_NAME, domain->flat_name);
-    return len;
+    return safe_format_string(str, size, nctx->fq_fmt,
+                              name, domain->name, calc_flat_name (domain), NULL);
 }
 
 char *
@@ -666,13 +640,72 @@ sss_get_domain_name(TALLOC_CTX *mem_ctx,
                     struct sss_domain_info *dom)
 {
     char *user_name;
+    char *domain = NULL;
+    int ret;
 
-    if (IS_SUBDOMAIN(dom) && dom->fqnames) {
+    /* check if the name already contains domain part */
+    if (dom->names != NULL) {
+        ret = sss_parse_name(mem_ctx, dom->names, orig_name, &domain, NULL);
+        if (ret == ERR_REGEX_NOMATCH) {
+            DEBUG(SSSDBG_TRACE_FUNC,
+                  "sss_parse_name could not parse domain from [%s]. "
+                  "Assuming it is not FQDN.\n", orig_name);
+        } else if (ret != EOK) {
+            DEBUG(SSSDBG_TRACE_FUNC,
+                  "sss_parse_name failed [%d]: %s\n", ret, sss_strerror(ret));
+            return NULL;
+        }
+    }
+
+    if (IS_SUBDOMAIN(dom) && dom->fqnames && domain == NULL) {
         /* we always use the fully qualified name for subdomain users */
         user_name = sss_tc_fqname(mem_ctx, dom->names, dom, orig_name);
     } else {
         user_name = talloc_strdup(mem_ctx, orig_name);
     }
 
+    talloc_free(domain);
+
     return user_name;
 }
+
+errno_t sss_user_by_name_or_uid(const char *input, uid_t *_uid, gid_t *_gid)
+{
+    uid_t uid;
+    errno_t ret;
+    char *endptr;
+    struct passwd *pwd;
+
+    /* Try if it's an ID first */
+    errno = 0;
+    uid = strtouint32(input, &endptr, 10);
+    if (errno != 0 || *endptr != '\0') {
+        ret = errno;
+        if (ret == ERANGE) {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  "UID [%s] is out of range.\n", input);
+            return ret;
+        }
+
+        /* Nope, maybe a username? */
+        pwd = getpwnam(input);
+    } else {
+        pwd = getpwuid(uid);
+    }
+
+    if (pwd == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "[%s] is neither a valid UID nor a user name which could be "
+              "resolved by getpwnam().\n", input);
+        return EINVAL;
+    }
+
+    if (_uid) {
+        *_uid = pwd->pw_uid;
+    }
+
+    if (_gid) {
+        *_gid = pwd->pw_gid;
+    }
+    return EOK;
+}
