@@ -18,15 +18,19 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "config.h"
 #include <ctype.h>
 #include <netdb.h>
 #include <poll.h>
+#include <sys/types.h>
 #include <sys/socket.h>
+#include <arpa/inet.h>
+#include <talloc.h>
+#include <dhash.h>
+#include <time.h>
 
-#include "talloc.h"
 #include "util/util.h"
 #include "util/sss_utf8.h"
-#include "dhash.h"
 
 int split_on_separator(TALLOC_CTX *mem_ctx, const char *str,
                        const char sep, bool trim, bool skip_empty,
@@ -152,7 +156,8 @@ char **parse_args(const char *str)
     num = 0;
     i = 0;
     e = false;
-    w = false;
+    /* skip leading whitespaces */
+    w = true;
     p = str;
     while (*p) {
         if (*p == '\\') {
@@ -204,19 +209,18 @@ char **parse_args(const char *str)
             tmp[i] = '\0';
             i++;
         }
-        if (tmp[i-1] != '\0' || strlen(tmp) == 0) {
-            /* check next char and skip multiple spaces */
-            continue;
-        }
 
-        r = realloc(ret, (num + 2) * sizeof(char *));
-        if (!r) goto fail;
-        ret = r;
-        ret[num+1] = NULL;
-        ret[num] = strdup(tmp);
-        if (!ret[num]) goto fail;
-        num++;
-        i = 0;
+        /* save token to result array */
+        if (i > 1 && tmp[i-1] == '\0') {
+            r = realloc(ret, (num + 2) * sizeof(char *));
+            if (!r) goto fail;
+            ret = r;
+            ret[num+1] = NULL;
+            ret[num] = strdup(tmp);
+            if (!ret[num]) goto fail;
+            num++;
+            i = 0;
+        }
     }
 
     free(tmp);
@@ -749,3 +753,183 @@ bool is_host_in_domain(const char *host, const char *domain)
 
     return false;
 }
+
+/* addr is in network order for both IPv4 and IPv6 versions */
+bool check_ipv4_addr(struct in_addr *addr, uint8_t flags)
+{
+    char straddr[INET_ADDRSTRLEN];
+
+    if (inet_ntop(AF_INET, addr, straddr, INET_ADDRSTRLEN) == NULL) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "inet_ntop failed, won't log IP addresses\n");
+        snprintf(straddr, INET_ADDRSTRLEN, "unknown");
+    }
+
+    if ((flags & SSS_NO_MULTICAST) && IN_MULTICAST(ntohl(addr->s_addr))) {
+        DEBUG(SSSDBG_FUNC_DATA, "Multicast IPv4 address %s\n", straddr);
+        return false;
+    } else if ((flags & SSS_NO_LOOPBACK)
+               && inet_netof(*addr) == IN_LOOPBACKNET) {
+        DEBUG(SSSDBG_FUNC_DATA, "Loopback IPv4 address %s\n", straddr);
+        return false;
+    } else if ((flags & SSS_NO_LINKLOCAL)
+               && (addr->s_addr & htonl(0xffff0000)) == htonl(0xa9fe0000)) {
+        /* 169.254.0.0/16 */
+        DEBUG(SSSDBG_FUNC_DATA, "Link-local IPv4 address %s\n", straddr);
+        return false;
+    } else if ((flags & SSS_NO_BROADCAST)
+               && addr->s_addr == htonl(INADDR_BROADCAST)) {
+        DEBUG(SSSDBG_FUNC_DATA, "Broadcast IPv4 address %s\n", straddr);
+        return false;
+    }
+
+    return true;
+}
+
+bool check_ipv6_addr(struct in6_addr *addr, uint8_t flags)
+{
+    char straddr[INET6_ADDRSTRLEN];
+
+    if (inet_ntop(AF_INET6, addr, straddr, INET6_ADDRSTRLEN) == NULL) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "inet_ntop failed, won't log IP addresses\n");
+        snprintf(straddr, INET6_ADDRSTRLEN, "unknown");
+    }
+
+    if ((flags & SSS_NO_LINKLOCAL) && IN6_IS_ADDR_LINKLOCAL(addr)) {
+        DEBUG(SSSDBG_FUNC_DATA, "Link local IPv6 address %s\n", straddr);
+        return false;
+    } else if ((flags & SSS_NO_LOOPBACK) && IN6_IS_ADDR_LOOPBACK(addr)) {
+        DEBUG(SSSDBG_FUNC_DATA, "Loopback IPv6 address %s\n", straddr);
+        return false;
+    } else if ((flags & SSS_NO_MULTICAST) && IN6_IS_ADDR_MULTICAST(addr)) {
+        DEBUG(SSSDBG_FUNC_DATA, "Multicast IPv6 address %s\n", straddr);
+        return false;
+    }
+
+    return true;
+}
+
+const char * const * get_known_services(void)
+{
+    static const char *svc[] = {"nss", "pam", "sudo", "autofs",
+                                "ssh", "pac", "ifp", NULL };
+
+    return svc;
+}
+
+errno_t add_strings_lists(TALLOC_CTX *mem_ctx, const char **l1, const char **l2,
+                          bool copy_strings, char ***_new_list)
+{
+    size_t c;
+    size_t l1_count = 0;
+    size_t l2_count = 0;
+    size_t new_count = 0;
+    char **new;
+    int ret;
+
+    if (l1 != NULL) {
+        for (l1_count = 0; l1[l1_count] != NULL; l1_count++);
+    }
+
+    if (l2 != NULL) {
+        for (l2_count = 0; l2[l2_count] != NULL; l2_count++);
+    }
+
+    new_count = l1_count + l2_count;
+
+    new = talloc_array(mem_ctx, char *, new_count + 1);
+    if (new == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "talloc_array failed.\n");
+        return ENOMEM;
+    }
+    new [new_count] = NULL;
+
+    if (copy_strings) {
+        for(c = 0; c < l1_count; c++) {
+            new[c] = talloc_strdup(new, l1[c]);
+            if (new[c] == NULL) {
+                DEBUG(SSSDBG_OP_FAILURE, "talloc_strdup failed.\n");
+                ret = ENOMEM;
+                goto done;
+            }
+        }
+        for(c = 0; c < l2_count; c++) {
+            new[l1_count + c] = talloc_strdup(new, l2[c]);
+            if (new[l1_count + c] == NULL) {
+                DEBUG(SSSDBG_OP_FAILURE, "talloc_strdup failed.\n");
+                ret = ENOMEM;
+                goto done;
+            }
+        }
+    } else {
+        if (l1 != NULL) {
+            memcpy(new, l1, sizeof(char *) * l1_count);
+        }
+
+        if (l2 != NULL) {
+            memcpy(&new[l1_count], l2, sizeof(char *) * l2_count);
+        }
+    }
+
+    *_new_list = new;
+    ret = EOK;
+
+done:
+    if (ret != EOK) {
+        talloc_free(new);
+    }
+
+    return ret;
+}
+
+/* Convert GeneralizedTime (http://en.wikipedia.org/wiki/GeneralizedTime)
+ * to unix time (seconds since epoch). Use UTC time zone.
+ */
+errno_t sss_utc_to_time_t(const char *str, const char *format, time_t *_unix_time)
+{
+    char *end;
+    struct tm tm;
+    size_t len;
+    time_t ut;
+
+    if (str == NULL) {
+        return EINVAL;
+    }
+
+    len = strlen(str);
+    if (str[len-1] != 'Z') {
+        DEBUG(SSSDBG_TRACE_INTERNAL,
+              "%s does not seem to be in UTZ time zone.\n", str);
+        return ERR_TIMESPEC_NOT_SUPPORTED;
+    }
+
+    memset(&tm, 0, sizeof(tm));
+
+    end = strptime(str, format, &tm);
+    /* not all characters from format were matched */
+    if (end == NULL) {
+        DEBUG(SSSDBG_TRACE_INTERNAL,
+              "String [%s] failed to match format [%s].\n", str, format);
+        return EINVAL;
+    }
+
+    /* str is 'longer' than format */
+    if (*end != '\0') {
+        DEBUG(SSSDBG_TRACE_INTERNAL,
+              "String [%s] is longer than format [%s].\n", str, format);
+        return EINVAL;
+    }
+
+    ut = mktime(&tm);
+    if (ut == -1) {
+        DEBUG(SSSDBG_TRACE_INTERNAL,
+              "mktime failed to convert [%s].\n", str);
+        return EINVAL;
+    }
+
+    tzset();
+    ut -= timezone;
+    *_unix_time = ut;
+    return EOK;
+}
