@@ -37,6 +37,8 @@
 #include <pcre.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <arpa/inet.h>
+#include <netinet/in.h>
 
 #include <talloc.h>
 #include <tevent.h>
@@ -66,12 +68,18 @@ extern int debug_level;
 extern int debug_timestamps;
 extern int debug_microseconds;
 extern int debug_to_file;
+extern int debug_to_stderr;
 extern const char *debug_log_file;
-void debug_fn(const char *function, int level, const char *format, ...)
-                                SSS_ATTRIBUTE_PRINTF(3, 4);
+void debug_fn(const char *file,
+              long line,
+              const char *function,
+              int level,
+              const char *format, ...) SSS_ATTRIBUTE_PRINTF(5,6);
 int debug_convert_old_level(int old_level);
 errno_t set_debug_file_from_fd(const int fd);
 
+#define SSS_DOM_ENV           "_SSS_DOM"
+
 #define SSSDBG_FATAL_FAILURE  0x0010   /* level 0 */
 #define SSSDBG_CRIT_FAILURE   0x0020   /* level 1 */
 #define SSSDBG_OP_FAILURE     0x0040   /* level 2 */
@@ -100,6 +108,8 @@ errno_t set_debug_file_from_fd(const int fd);
          _("Debug level"), NULL}, \
         {"debug-to-files", 'f', POPT_ARG_NONE, &debug_to_file, 0, \
          _("Send the debug output to files instead of stderr"), NULL }, \
+        {"debug-to-stderr", 0, POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN, &debug_to_stderr, 0, \
+         _("Send the debug output to stderr directly."), NULL }, \
         {"debug-timestamps", 0, POPT_ARG_INT, &debug_timestamps, 0, \
          _("Add debug timestamps"), NULL}, \
          {"debug-microseconds", 0, POPT_ARG_INT, &debug_microseconds, 0, \
@@ -115,8 +125,11 @@ errno_t set_debug_file_from_fd(const int fd);
 */
 #define DEBUG(level, format, ...) do { \
     int __debug_macro_level = level; \
-    if (DEBUG_IS_SET(__debug_macro_level)) \
-        debug_fn(__FUNCTION__, __debug_macro_level, format, ##__VA_ARGS__); \
+    if (DEBUG_IS_SET(__debug_macro_level)) { \
+        debug_fn(__FILE__, __LINE__, __FUNCTION__, \
+                 __debug_macro_level, \
+                 format, ##__VA_ARGS__); \
+    } \
 } while (0)
 
 /** \def DEBUG_IS_SET(level)
@@ -139,6 +152,14 @@ errno_t set_debug_file_from_fd(const int fd);
     talloc_set_log_fn(talloc_log_fn); \
 } while (0)
 
+/* CLI tools shall debug to stderr even when SSSD was compiled with journald
+ * support
+ */
+#define DEBUG_CLI_INIT(dbg_lvl) do { \
+    DEBUG_INIT(dbg_lvl);             \
+    debug_to_stderr = 1;             \
+} while (0)
+
 #define PRINT(fmt, ...) fprintf(stdout, gettext(fmt), ##__VA_ARGS__)
 #define ERROR(fmt, ...) fprintf(stderr, gettext(fmt), ##__VA_ARGS__)
 
@@ -154,6 +175,12 @@ errno_t set_debug_file_from_fd(const int fd);
 
 #define SSSD_MAIN_OPTS SSSD_DEBUG_OPTS
 
+#define SSSD_SERVER_OPTS(uid, gid) \
+        {"uid", 0, POPT_ARG_INT, &uid, 0, \
+          _("The user ID to run the server as"), NULL}, \
+        {"gid", 0, POPT_ARG_INT, &gid, 0, \
+          _("The group ID to run the server as"), NULL},
+
 #define FLAGS_NONE 0x0000
 #define FLAGS_DAEMON 0x0001
 #define FLAGS_INTERACTIVE 0x0002
@@ -191,6 +218,7 @@ errno_t set_debug_file_from_fd(const int fd);
 /* From debug.c */
 void ldb_debug_messages(void *context, enum ldb_debug_level level,
                         const char *fmt, va_list ap);
+int chown_debug_file(const char *filename, uid_t uid, gid_t gid);
 int open_debug_file_ex(const char *filename, FILE **filep, bool want_cloexec);
 int open_debug_file(void);
 int rotate_debug_files(void);
@@ -207,6 +235,7 @@ void talloc_log_fn(const char *msg);
 #define SSS_LOG_DEBUG   7   /* debug-level messages */
 
 void sss_log(int priority, const char *format, ...) SSS_ATTRIBUTE_PRINTF(2, 3);
+void sss_log_ext(int priority, int facility, const char *format, ...) SSS_ATTRIBUTE_PRINTF(3, 4);
 
 /* from server.c */
 struct main_context {
@@ -215,13 +244,16 @@ struct main_context {
     pid_t parent_pid;
 };
 
+errno_t server_common_rotate_logs(struct confdb_ctx *confdb,
+                                  const char *conf_entry);
 int die_if_parent_died(void);
 int pidfile(const char *path, const char *name);
 int server_setup(const char *name, int flags,
+                 uid_t uid, gid_t gid,
                  const char *conf_entry,
                  struct main_context **main_ctx);
 void server_loop(struct main_context *main_ctx);
-void sig_term(int sig);
+void orderly_shutdown(int status);
 
 /* from signal.c */
 #include <signal.h>
@@ -243,18 +275,11 @@ void *sss_mem_attach(TALLOC_CTX *mem_ctx,
 int password_destructor(void *memctx);
 
 /* from usertools.c */
-char *get_username_from_uid(TALLOC_CTX *mem_ctx, uid_t uid);
-
 char *get_uppercase_realm(TALLOC_CTX *memctx, const char *name);
 
-#define FQ_FMT_NAME         0x01
-#define FQ_FMT_DOMAIN       0x02
-#define FQ_FMT_FLAT_NAME    0x04
-
 struct sss_names_ctx {
     char *re_pattern;
     char *fq_fmt;
-    uint8_t fq_flags;
 
     pcre *re;
 };
@@ -299,18 +324,26 @@ char *
 sss_tc_fqname(TALLOC_CTX *mem_ctx, struct sss_names_ctx *nctx,
               struct sss_domain_info *domain, const char *name);
 
+/* Return fully-qualified name according to the fq_fmt. The name is allocated using
+ * talloc on top of mem_ctx. In contrast to sss_tc_fqname() sss_tc_fqname2()
+ * expects the domain and flat domain name as separate arguments.
+ */
+char *
+sss_tc_fqname2(TALLOC_CTX *mem_ctx, struct sss_names_ctx *nctx,
+               const char *dom_name, const char *flat_dom_name,
+               const char *name);
+
 /* Return fully-qualified name formatted according to the fq_fmt. The buffer in "str" is
  * "size" bytes long. Returns the number of bytes written on success or a negative
  * value of failure.
+ *
+ * Pass a zero size to calculate the length that would be needed by the fully-qualified
+ * name.
  */
 int
 sss_fqname(char *str, size_t size, struct sss_names_ctx *nctx,
            struct sss_domain_info *domain, const char *name);
 
-size_t
-sss_fqdom_len(struct sss_names_ctx *nctx,
-              struct sss_domain_info *domain);
-
 /* Subdomains use fully qualified names in the cache while primary domains use
  * just the name. Return either of these for a specified domain or subdomain
  */
@@ -321,18 +354,6 @@ sss_get_domain_name(TALLOC_CTX *mem_ctx, const char *orig_name,
 /* from backup-file.c */
 int backup_file(const char *src, int dbglvl);
 
-/* from check_and_open.c */
-enum check_file_type {
-    CHECK_DONT_CHECK_FILE_TYPE = -1,
-    CHECK_REG,
-    CHECK_DIR,
-    CHECK_CHR,
-    CHECK_BLK,
-    CHECK_FIFO,
-    CHECK_LNK,
-    CHECK_SOCK
-};
-
 /* check_file()
  * Verify that a file has certain permissions and/or is of a certain
  * file type. This function can be used to determine if a file is a
@@ -344,8 +365,8 @@ enum check_file_type {
  * inode to minimize impact. Permission changes may have occurred,
  * however.
  */
-errno_t check_file(const char *filename, const int uid, const int gid,
-                   const int mode, enum check_file_type type,
+errno_t check_file(const char *filename,
+                   uid_t uid, gid_t gid, mode_t mode, mode_t mask,
                    struct stat *caller_stat_buf, bool follow_symlink);
 
 /* check_fd()
@@ -355,8 +376,8 @@ errno_t check_file(const char *filename, const int uid, const int gid,
  * is the safer way to perform file checks and should be preferred
  * over check_file for nearly all situations.
  */
-errno_t check_fd(int fd, const int uid, const int gid,
-                 const int mode, enum check_file_type type,
+errno_t check_fd(int fd, uid_t uid, gid_t gid,
+                 mode_t mode, mode_t mask,
                  struct stat *caller_stat_buf);
 
 /* check_and_open_readonly()
@@ -364,11 +385,27 @@ errno_t check_fd(int fd, const int uid, const int gid,
  * permissions and is of a certain file type. This function wraps
  * check_fd(), and is considered race-condition safe.
  */
-errno_t check_and_open_readonly(const char *filename, int *fd, const uid_t uid,
-                               const gid_t gid, const mode_t mode,
-                               enum check_file_type type);
+errno_t check_and_open_readonly(const char *filename, int *fd,
+                                uid_t uid, gid_t gid,
+                                mode_t mode, mode_t mask);
 
 /* from util.c */
+#define SSS_NO_LINKLOCAL 0x01
+#define SSS_NO_LOOPBACK 0x02
+#define SSS_NO_MULTICAST 0x04
+#define SSS_NO_BROADCAST 0x08
+
+#define SSS_NO_SPECIAL \
+        (SSS_NO_LINKLOCAL|SSS_NO_LOOPBACK|SSS_NO_MULTICAST|SSS_NO_BROADCAST)
+
+/* These two functions accept addr in network order */
+bool check_ipv4_addr(struct in_addr *addr, uint8_t check);
+bool check_ipv6_addr(struct in6_addr *addr, uint8_t check);
+
+const char * const * get_known_services(void);
+
+errno_t sss_user_by_name_or_uid(const char *input, uid_t *_uid, gid_t *_gid);
+
 int split_on_separator(TALLOC_CTX *mem_ctx, const char *str,
                        const char sep, bool trim, bool skip_empty,
                        char ***_list, int *size);
@@ -389,6 +426,24 @@ errno_t sss_hash_create_ex(TALLOC_CTX *mem_ctx,
                            hash_delete_callback *delete_callback,
                            void *delete_private_data);
 
+/**
+ * @brief Add two list of strings
+ *
+ * Create a new NULL-termintated list of strings by adding two lists together.
+ *
+ * @param[in] mem_ctx      Talloc memory context for the new list.
+ * @param[in] l1           First NULL-termintated list of strings.
+ * @param[in] l2           Second NULL-termintated list of strings.
+ * @param[in] copy_strings If set to 'true' the list items will be copied
+ *                         otherwise only the pointers to the items are
+ *                         copied.
+ * @param[out] new_list    New NULL-terminated list of strings. Must be freed
+ *                         with talloc_free() by the caller. If copy_strings
+ *                         is 'true' the new elements will be freed as well.
+ */
+errno_t add_strings_lists(TALLOC_CTX *mem_ctx, const char **l1, const char **l2,
+                          bool copy_strings, char ***_new_list);
+
 /* Copy a NULL-terminated string list
  * Returns NULL on out of memory error or invalid input
  */
@@ -484,19 +539,19 @@ struct sss_domain_info *get_domains_head(struct sss_domain_info *domain);
 
 struct sss_domain_info *get_next_domain(struct sss_domain_info *domain,
                                         bool descend);
-struct sss_domain_info *find_subdomain_by_name(struct sss_domain_info *domain,
-                                               const char *name,
-                                               bool match_any);
-struct sss_domain_info *find_subdomain_by_sid(struct sss_domain_info *domain,
-                                              const char *sid);
+struct sss_domain_info *find_domain_by_name(struct sss_domain_info *domain,
+                                            const char *name,
+                                            bool match_any);
+struct sss_domain_info *find_domain_by_sid(struct sss_domain_info *domain,
+                                           const char *sid);
 
 struct sss_domain_info*
 sss_get_domain_by_sid_ldap_fallback(struct sss_domain_info *domain,
                                     const char* sid);
 
 struct sss_domain_info *
-find_subdomain_by_object_name(struct sss_domain_info *domain,
-                              const char *object_name);
+find_domain_by_object_name(struct sss_domain_info *domain,
+                           const char *object_name);
 
 bool subdomain_enumerates(struct sss_domain_info *parent,
                           const char *sd_name);
@@ -519,9 +574,24 @@ errno_t sssd_domain_init(TALLOC_CTX *mem_ctx,
 
 #define IS_SUBDOMAIN(dom) ((dom)->parent != NULL)
 
-errno_t sss_write_domain_mappings(struct sss_domain_info *domain,
-                                  bool add_capaths);
+#define DOM_HAS_VIEWS(dom) ((dom)->has_views)
 
+/* the directory domain - realm mappings and other krb5 config snippers are
+ * written to */
+#define KRB5_MAPPING_DIR PUBCONF_PATH"/krb5.include.d"
+
+errno_t sss_write_domain_mappings(struct sss_domain_info *domain);
+
+errno_t sss_write_krb5_conf_snippet(const char *path);
+
+errno_t get_dom_names(TALLOC_CTX *mem_ctx,
+                      struct sss_domain_info *start_dom,
+                      char ***_dom_names,
+                      int *_dom_names_count);
+
+errno_t fix_domain_in_name_list(TALLOC_CTX *mem_ctx,
+                                struct sss_domain_info *dom,
+                                char **in, char ***_out);
 /* from util_lock.c */
 errno_t sss_br_lock_file(int fd, size_t start, size_t len,
                          int num_tries, useconds_t wait);
@@ -533,6 +603,13 @@ errno_t sss_br_lock_file(int fd, size_t start, size_t len,
 #define BUILD_WITH_PAC_RESPONDER false
 #endif
 
+/* from well_known_sids.c */
+errno_t well_known_sid_to_name(const char *sid, const char **dom,
+                               const char **name);
+
+errno_t name_to_well_known_sid(const char *dom, const char *name,
+                               const char **sid);
+
 /* from string_utils.c */
 char * sss_replace_space(TALLOC_CTX *mem_ctx,
                          const char *orig_name,
@@ -541,4 +618,34 @@ char * sss_reverse_replace_space(TALLOC_CTX *mem_ctx,
                                  const char *orig_name,
                                  const char replace_char);
 
+#define GUID_BIN_LENGTH 16
+/* 16 2-digit hex values + 4 dashes + terminating 0 */
+#define GUID_STR_BUF_SIZE (2 * GUID_BIN_LENGTH + 4 + 1)
+
+errno_t guid_blob_to_string_buf(const uint8_t *blob, char *str_buf,
+                                size_t buf_size);
+
+/* from become_user.c */
+errno_t become_user(uid_t uid, gid_t gid);
+struct sss_creds;
+errno_t switch_creds(TALLOC_CTX *mem_ctx,
+                     uid_t uid, gid_t gid,
+                     int num_gids, gid_t *gids,
+                     struct sss_creds **saved_creds);
+errno_t restore_creds(struct sss_creds *saved_creds);
+
+/* from sss_semanage.c */
+/* Please note that libsemange relies on files and directories created with
+ * certain permissions. Therefore the caller should make sure the umask is
+ * not too restricted (especially when called from the daemon code).
+ */
+int set_seuser(const char *login_name, const char *seuser_name,
+               const char *mlsrange);
+int del_seuser(const char *login_name);
+int get_seuser(TALLOC_CTX *mem_ctx, const char *login_name,
+               char **_seuser, char **_mls_range);
+
+/* convert time from generalized form to unix time */
+errno_t sss_utc_to_time_t(const char *str, const char *format, time_t *unix_time);
+
 #endif /* __SSSD_UTIL_H__ */
